//ClineData.h

//This is a virtual base class used to shunt cline fitting functions
//in a general algorithm that fits clines with different data types.


#ifndef ClineDatah
#define ClineDatah = 1

#include "MemoryManager.h"
#include "aError.h"
#include "aList.cpp"
//#include "StringClass.cpp"
#include "MetropolisHastings.h"
#include <ostream.h>
#include "StringList.h"

class ClineData;
class ClineGeneticData;
class aString;
class StringList;

class ClineData: public MetropolisHastingsData{
  public:
  	StringList dataType;//indexed by trait number; 
  						//"morph" for quantitative traits
  						//"codominant" for allozyme etc. data
  						//"haplotype" for mtDNA or Y data
  public:
	ClineData(void){}
	ClineData(ClineData& cd){dataType = cd.dataType;}
	ClineData(ClineData* cd){dataType = cd->dataType;}
	virtual ~ClineData(void){
		dataType.Clear();
		}
	
	virtual ClineData* Duplicate(void){
		return NEW ClineData(*this);}
  public:
	//in ClineGeneticData
	virtual void 		AddDiagnosticAlleles(int , aString& ){aError("Error: in virtual base class function");}
	virtual void 		CullLoci (StringList& ){aError("Error: in virtual base class function");}
	virtual void		FitAllozymeCline(aString& ){aError("Error: in virtual base class function");}
	virtual ClineGeneticData* PoolDiagnosticAlleles (void){aError("Error: in virtual base class function");return 0;}
	virtual ClineGeneticData* PoolDiagnosticHaplotypes (void){aError("Error: in virtual base class function");return 0;}
	virtual void 		PrintGelJockFile (aString& ){aError("Error: in virtual base class function");}
	virtual void 		PrintGelJockFile (ostream& ){aError("Error: in virtual base class function");}
	virtual void 		ReadGelJockDataFile (aString* ){aError("Error: in virtual base class function");}
	virtual void 		ReadGelJockmtDNADataFile (aString* ){aError("Error: in virtual base class function");}
	virtual void 		RestoreLoci (StringList& ){aError("Error: in virtual base class function");}
	virtual void 		StoreDiagnosticAlleles(void){aError("Error: in virtual base class function");}
	virtual void 		StripDiagnosticAlleles(aString& ){aError("Error: in virtual base class function");}

	//in ClinePhenotypicData
	virtual void 		CullTraits (StringList& ){aError("Error: in virtual base class function");}
	virtual void 		CullSkippedTraits (void){aError("Error: in virtual base class function");}
	virtual void		FitTraitCline(aString& ){aError("Error: in virtual base class function");}
	virtual void 		PrintPhenotypeFile (aString& ){aError("Error: in virtual base class function");}
	virtual void 		PrintPhenotypeFile (ostream& ){aError("Error: in virtual base class function");}
	virtual void 		ReadDataFile (aString* ){aError("Error: in virtual base class function");}
	virtual void		RescoreDiagnosticTraits(void){aError("Error: in virtual base class function");}
	virtual void 		RestoreTraits (StringList& ){aError("Error: in virtual base class function");}
	virtual void 		StoreDiagnosticTraits(void){aError("Error: in virtual base class function");}
	virtual void 		StripDiagnosticTraits(aString& ){aError("Error: in virtual base class function");}


	//in both
	virtual int			pops(void){aError("Error: in virtual base class function");return 0;}
	virtual int			NumTraits(void){aError("Error: in virtual base class function");return 0;}
	virtual void 		CalculateHybridIndex(aList<double>&){aError("Error: in virtual base class function");}
	virtual void 		Clear(void){aError("Error: in virtual base class function");}
	virtual double 		Locality(int ){aError("Error: in virtual base class function");return 0.0;}
	virtual void 		PrintHybridIndexFile(aString& ){aError("Error: in virtual base class function");}
	virtual void		ResetFlags(void){aError("Error: in virtual base class function");}
	virtual void 		SetCenterPopList(aList<int>& ){aError("Error: in virtual base class function");}
	virtual void 		SetDataTypes(void){aError("Error: in virtual base class function");}


	};//end ClineData



#endif
