/***************************************************************
*
*	ClineFitInitializer.h
*
*	This class holds all possible cline shape parameters
*	and is used to initialize particular implementations
*	of cline shape analysis.
*
***************************************************************/

#ifndef ClineFitInitializerh
#define ClineFitInitializerh = 1

#include "Element.cpp"
#include "aList.cpp"
#include "aMatrix.cpp"
#include "MatrixList.h"
#include "StringClass.h"
#include "MemoryManager.h"


class ClineFitInitializer;
class ClineFitInitializer{
  public:
  	double leftEnd_;
  	double rightEnd_;
  	aList<long double> c_;
  	aList<long double> w_;
  	aList<long double> thetaL_;
  	aList<long double> thetaR_;
  	aList<long double> zL_;
  	aList<long double> zR_;
  	aList<long double> aveScoreL_;
  	aList<long double> aveScoreR_;
  	long double D_;
  	long double r_;
  	int covMatrixListType_;//0 for 'populations', 1 for 'cline regions' = left, center, right
	MatrixList covMatrixList_;//symmetrical matrices
	MatrixList covParamsOnOff_;//symmetrical matrices
//  	aList<double> varScoreL_;//quantitative traits
//  	aList<double> varScoreCenter_;//quantitative traits
//  	aList<double> varScoreR_;//quantitative traits
  	ObjList<aList<int> > *paramsOnOff_;//indexed [trait][param]
//  	aList<int> paramsOnOff;//indexed [param];items={c,w,thL,thR,zL,zR,bottomL[trait] array,topR[trait] array}
  	ObjList<aList<int> > *paramIdentities_;//indexed [trait][param]; parameters constrained to change together 
	aString dataFileName_;
	aString outFileName_;
	double harMeanRecombRate_;
	aList<int> centralRegionPops_;
	long repsToSave_;
	long repsPerAnnealStep_;
	
  public:

	ClineFitInitializer(double leftEnd, double rightEnd,
						aList<long double>& c, aList<long double>& w,
  						aList<long double>& thetaL, aList<long double>& thetaR,
  						aList<long double>& zL, aList<long double>& zR,
  						long double D, long double r,
  						aList<long double>& aveScoreL,
  						aList<long double>& aveScoreR,
  						int covMatrixListType,
  						MatrixList& covMatrixList,
						MatrixList& covParamsOnOff,
//  					aList<double>& varScoreL,
//					  	aList<double>& varScoreCenter,
//					  	aList<double>& varScoreR,
//  					aList<int>& onOff,
  						ObjList<aList<int> >& onOff,
  						ObjList<aList<int> >& paramIdentities,
						aString& dataFileName,
						aString& outFileName,
						double harMeanRecombRate,
						aList<int>& centralRegionPops,
						long repsToSave,
						long repsPerAnnealStep
						):
			leftEnd_(leftEnd),rightEnd_(rightEnd),
			c_(c), w_(w),
  			thetaL_(thetaL), thetaR_(thetaR),
  			zL_(zL), zR_(zR),D_(D),r_(r),
  			aveScoreL_(aveScoreL),aveScoreR_(aveScoreR),
  			covMatrixListType_(covMatrixListType),
  			covMatrixList_(covMatrixList),covParamsOnOff_(covParamsOnOff),
//  		varScoreL_(varScoreL),varScoreCenter_(varScoreCenter),varScoreR_(varScoreR),
  			paramsOnOff_(&onOff),
  			paramIdentities_(&paramIdentities),
			dataFileName_(dataFileName),
			outFileName_(outFileName),
			harMeanRecombRate_(harMeanRecombRate),
			centralRegionPops_(centralRegionPops),
			repsToSave_(repsToSave),
			repsPerAnnealStep_(repsPerAnnealStep)
			{}

	ClineFitInitializer(double leftEnd, double rightEnd,
						long double c, long double w,
  						long double thetaL, long double thetaR,
  						long double zL, long double zR,
  						long double D, long double r,
  						aList<long double>& aveScoreL,
  						aList<long double>& aveScoreR,
  						int covMatrixListType,
  						MatrixList& covMatrixList,
						MatrixList& covParamsOnOff,
//  					aList<double>& varScoreL,
//					  	aList<double>& varScoreCenter,
//					  	aList<double>& varScoreR,
//  					aList<int>& onOff,
  						ObjList<aList<int> >& onOff,
  						ObjList<aList<int> >& paramIdentities,
						aString& dataFileName,
						aString& outFileName,
						double harMeanRecombRate,
						aList<int>& centralRegionPops,
						long repsToSave,
						long repsPerAnnealStep
						):
			leftEnd_(leftEnd),rightEnd_(rightEnd),
//			c_(c), w_(w),
//  			thetaL_(thetaL), thetaR_(thetaR),
//  			zL_(zL), zR_(zR),
  			D_(D),r_(r),
  			aveScoreL_(aveScoreL),aveScoreR_(aveScoreR),
  			covMatrixListType_(covMatrixListType),
  			covMatrixList_(covMatrixList),covParamsOnOff_(covParamsOnOff),
//  		varScoreL_(varScoreL),varScoreCenter_(varScoreCenter),varScoreR_(varScoreR),
  			paramsOnOff_(&onOff),
  			paramIdentities_(&paramIdentities),
			dataFileName_(dataFileName),
			outFileName_(outFileName),
			harMeanRecombRate_(harMeanRecombRate),
			centralRegionPops_(centralRegionPops),
			repsToSave_(repsToSave),
			repsPerAnnealStep_(repsPerAnnealStep)
			{
			for(int tr=0;tr<traits();++tr){
				c_.Append(c);
				w_.Append(w);
				thetaL_.Append(thetaL);
				thetaR_.Append(thetaR);
				zL_.Append(zL);
				zR_.Append(zR);
				}//tr
			}

	ClineFitInitializer(double leftEnd, double rightEnd,
						long double c, long double w,
  						long double thetaL, long double thetaR,
  						long double zL, long double zR,
  						long double D, long double r,
  						double aveScoreL,
  						double aveScoreR,
  						int covMatrixListType,
  						aMatrix<long double>& covMatrix,//all the same at start
						aMatrix<long double>& covParamsOnOff,
//  					double varScoreL,
//					  	double varScoreCenter,
//					  	double varScoreR,
//  					aList<int>& onOff,
  						ObjList<aList<int> >& onOff,
  						ObjList<aList<int> >& paramIdentities,
						aString& dataFileName,
						aString& outFileName,
						double harMeanRecombRate,
						aList<int>& centralRegionPops,
						long repsToSave,
						long repsPerAnnealStep
						):
			leftEnd_(leftEnd),rightEnd_(rightEnd),
//			c_(c), w_(w),
//  			thetaL_(thetaL), thetaR_(thetaR),
//  			zL_(zL), zR_(zR),
  			D_(D),r_(r),
  			covMatrixListType_(covMatrixListType),
  			paramsOnOff_(&onOff),
  			paramIdentities_(&paramIdentities),
			dataFileName_(dataFileName),
			outFileName_(outFileName),
			harMeanRecombRate_(harMeanRecombRate),
			centralRegionPops_(centralRegionPops),
			repsToSave_(repsToSave),
			repsPerAnnealStep_(repsPerAnnealStep)
			{
		for(int tr=0;tr<traits();++tr){
				c_.Append(c);
				w_.Append(w);
				thetaL_.Append(thetaL);
				thetaR_.Append(thetaR);
				zL_.Append(zL);
				zR_.Append(zR);
				aveScoreL_.Append(aveScoreL);
				aveScoreR_.Append(aveScoreR);
	//			varScoreL_.Append(varScoreL);
	//			varScoreCenter_.Append(varScoreCenter);
	//			varScoreR_.Append(varScoreR);
				}
		if(covMatrixListType<3){
			for(r=0;r<3;++r){//fitting 3 regions
				covMatrixList_.Append(NEW aMatrix<long double>(covMatrix));
				covParamsOnOff_.Append(NEW aMatrix<long double>(covParamsOnOff));
				}}
			else{//fitting each population; covMatrixListType=number of pops
			for(r=0;r<covMatrixListType;++r){
				covMatrixList_.Append(NEW aMatrix<long double>(covMatrix));
				covParamsOnOff_.Append(NEW aMatrix<long double>(covParamsOnOff));
				}}
			
		}

	~ClineFitInitializer(void){
		aveScoreL_.Clear();
		aveScoreR_.Clear();
		covMatrixList_.Clear();
		covParamsOnOff_.Clear();
//		varScoreL_.Clear();
//		varScoreCenter_.Clear();
//		varScoreR_.Clear();
//		paramsOnOff_.Clear();
//  		paramIdentities_.Clear();
		paramsOnOff_=NULL;//forget here; dispose elsewhere
  		paramIdentities_=NULL;//forget here; dispose elsewhere
		centralRegionPops_.Clear();
		}
	
	
	virtual int traits(void){
//					return aveScoreL_.len();
					if (paramIdentities_==NULL) return 0;
					return paramIdentities_->len();
					}
	};//end class ClineFitInitializer



#endif
