// ClineGeneticData.cpp

#ifndef ClineGeneticDatacpp
#define ClineGeneticDatacpp = 1

#include "ClineGeneticData.h"


#ifdef MemoryLeakDebug
MemLeakIDType  ClineGeneticData::memIDCount_ = (MemLeakIDType) 0;//initialized here in file scope
memAllocList  ClineGeneticData::memIDList_ =  memAllocList();//initialized here in file scope
void  ClineGeneticData::PrintIDList(ostream& outFile){
	memIDList_.Print(outFile);}
void  ClineGeneticData::PrintIDListLen(ostream& outFile){
	memIDList_.PrintLen(outFile);}
#endif



ClineGeneticData::ClineGeneticData(ClineGeneticData& parentData):
			PopulationGeneticData(parentData), 
			diagnosticAllelesR(parentData.diagnosticAllelesR),
			centerPopList_(parentData.centerPopList_),
			locality(parentData.locality),
			localityFlag_(parentData.localityFlag_),
			centerPopFis_(parentData.centerPopFis_),
			centerPopFflag_(parentData.centerPopFflag_)
			{	
		}//end ctor ClineGeneticData(ClineGeneticData&, aList<int>&)


ClineGeneticData::ClineGeneticData(ClineGeneticData& parentData, aList<int>& popSubsetNums):
			PopulationGeneticData(parentData,popSubsetNums), 
			diagnosticAllelesR(parentData.diagnosticAllelesR),
			localityFlag_(0),centerPopFis_(0.0),centerPopFflag_(0)
		{//must set centerPopList_ separately
	}//end ctor ClineGeneticData(ClineGeneticData&, aList<int>&)



void ClineGeneticData::AddDiagnosticAlleles(int locusNum, aString& aLocusName){
	//attaches list of diagnistic alleles to aString
	StringList& diagAllelelist = diagnosticAllelesR[locusNum];
	if (diagAllelelist.len()==0) return;
	aLocusName.cat("(");
	for(int a=0;a<diagAllelelist.len();++a){
		aLocusName.cat(diagAllelelist[a]);
		if(a<diagAllelelist.len()-1){
			aLocusName.cat(";");
			}
		}//end for a
	aLocusName.cat(")");
	}//end ClineGeneticData::AddDiagnosticAlleles(int, aString&)



void ClineGeneticData::CalculateHybridIndex(aList<double>& popLocation){
/*  var
   diagAllelesR: ListOfListsTypePtr;
   loc, locLparen, locRparen, locSemicolon, pop, i, chrom, posGtypeWord, codeLen, nAlleles, indivInData: longint;
   locusList, genotypesList: ListStrStructureTypePtr;
   sizePop: ListStructureType;
   twoNList: ListStructureTypePtr;
   locusWord, alleleWord, popXword, popTitleWord, codeWord, dataStrCopy, gtypeWord: string;
   popX, amongPopHarMean, amongLociHarMean: extended;
   indexVal: extended;
   gene: string;
*/
	hybridIndices.Clear();
	HybridIndicesByLocus.Clear();
	popLocation.Clear();
	double hi,locHI,indexVal,nAlleles,locIndexVal,locPloidy;
	aString endcharList(" ;:,/");
	for(int pop=0;pop<pops();++pop){
		//get the population's location from the end of the title string
		aString popTitleWord(popName[pop]);
	    aString* popXword = popTitleWord.ChopOffLastWord(endcharList);
    	double popX = popXword->toDouble();
    	popLocation.Append(popX);
    	delete popXword; popXword=0;
		//get each individual's genotype and compare to diagnostic alleles
    	for (int i=0;i<popSize[pop];++i){
    		HybridIndicesByLocus.Append(NEW HybridIndexByLocus());
      		indexVal = nAlleles = 0.0;
      		hi = -1.0;//use as error flag
    		for(int loc=0;loc<nloci;++loc){
    			locIndexVal = locPloidy = 0.0;
    			locHI = -1.0;//use as error flag
    			for(int a=0;a<Ploidy(pop,i,loc);++a){
    				aString* gene = GetGenotypeString(pop, i, loc, a);
    				if((gene->len() != 0)&&(*gene != ' ')){
    					++nAlleles;++locPloidy;
    					if(diagnosticAllelesR[loc].Includes(gene)){
    						++indexVal;++locIndexVal;}
    					}
    				delete gene; gene = 0;
    				}//end for a
    			if(locPloidy>0.0) {locHI = locIndexVal/locPloidy;}
    			HybridIndicesByLocus.Last().Append(NEW HybridIndexByLocation(locHI, locPloidy, popX));
    			}//end for loc
    		if(nAlleles >0.0){hi = indexVal/nAlleles;}
    		hybridIndices.Append(NEW HybridIndexByLocation(hi, nAlleles, popX));
    		}//end for i
    	}//end for pop
	}// end ClineGeneticData::CalculateHybridIndex(aList<double>&)



long double ClineGeneticData::CenterPopFis(void){
	if(centerPopFflag_){ return centerPopFis_;}
	FstatisticAnalysis centerData(*this, centerPopList_);
	centerData.WeirTheta();
	centerPopFis_ = centerData.Fis(FstatisticAnalysis::WandC);
	centerPopFflag_=1;
	return centerPopFis_;
	}//end ClineGeneticData::CenterPopFis()


void ClineGeneticData::Clear(void){
	PopulationGeneticData::Clear();
	diagnosticAllelesR.Clear();
	hybridIndices.Clear();
	HybridIndicesByLocus.Clear();
	centerPopList_.Clear();
	locality.Clear();
	}//end ClineGeneticData::Clear()


void ClineGeneticData::CullLoci (StringList& lociToCull){
	//This function moves the designated loci to the list of skipped loci
	if(lociToCull.len()==0){return;}//none to cull
	for(int cLoc=0;cLoc<lociToCull.len();++cLoc){
		aString cLocusname = lociToCull[cLoc];
		aString cLocusnameCopy(cLocusname);
		AddDiagnosticAlleles(cLoc, cLocusname);
		int item = rawLocusName.Loc(&cLocusname);
		if((item>=0)&&(!(skippedLocusName.Includes(&cLocusname)))){
				//the locus exists in the rawlist && isn't already skipped
			skippedLocusName.Append(NEW aString(cLocusname));
			skippedLocusNum.Append(item);
			item = locusName.Loc(&cLocusnameCopy);
			locusName.Delete(item);
			lociSkipped++;--nloci;
//			diagnosticAllelesR.Delete(item);//take from this list too
			dataType.Delete(item);
			}//if item exists
		}//cLoc
	ResetFlags();
	CullSkippedLoci();//this culls skipped loci from genotype strings
	CheckForNewAlleles();
	}//end procedure ClineGeneticData::CullLoci(StringList&)



void ClineGeneticData::CullSkippedLoci (void){
	PopulationGeneticData::CullSkippedLoci();
	}//end procedure ClineGeneticData::CullSkippedLoci()



double ClineGeneticData::Locality (int pop){
	//returns the x-axis coordinates of the locality
	if(localityFlag_){
		return locality[pop];}
	locality.Clear();
	aString endchrs(" \t,;");
	for(int p=0;p<pops();++p){
		aString thisPopName(popName[p]);
		aString* thisLocalityName = thisPopName.ChopOffLastWord(endchrs);
		double thisLocality = thisLocalityName->toDouble();
		locality.Append(thisLocality);
		delete thisLocalityName;thisLocalityName=0;
		}//end for pop
	localityFlag_ = true;
	return locality[pop];
	}//end  PopulationGeneticData::GetLocality




ClineGeneticData* ClineGeneticData::PoolDiagnosticAlleles (void){
	if (diagnosticAllelesR.len()==0){
		StoreDiagnosticAlleles();}
	ClineGeneticData* alloData = NEW ClineGeneticData(*this);
	alloData->ResetFlags();
	alloData->title_.cat(": diagnostic alleles pooled");
	//Go through each individual and change its genotype in alloData
	//An allele diagnostic on the left side is A, and on the right side is B
	//Alleles unique to the central part of the zone are pooled with the left side as A
	for(int pop=0;pop<pops();++pop){
	    for(int i=0;i<alloData->rawGtypes[pop].len();++i){
			aString& dataStr = alloData->rawGtypes[pop][i];
			//first snip off & save the individual' s designation code, then rebuild genotype on it
			aString* newGtypeWord = dataStr.ChopOffFirstWord();
			newGtypeWord->PrependCharToLen(' ',4);
			newGtypeWord->cat(' ');
			for(int loc=0;loc<rawLocusName.len();++loc){
				aString dataStrCopy(dataStr);
				dataStrCopy.cut(4,dataStr.len());//assumes 3 characters designated for each locus
				dataStr.cut(0,3);//assumes 3 characters designated for each locus
				aString lName(rawLocusName[loc]);
				StripDiagnosticAlleles(lName);
				if(!(lName == alloData->rawLocusName[loc])){//it has diagnostic alleles
						for(int a=0;a<dataStrCopy.len();++a){
							if(dataStrCopy[a] == ' '){
									newGtypeWord->cat(' ');
									//break;
									}//escape 'a' loop
								else{
									int found=0;
									for(int b=0;b<diagnosticAllelesR[loc].len();++b){
										if (diagnosticAllelesR[loc][b] == dataStrCopy[a]){
											newGtypeWord->cat('B');
											found=1;
											break;}//escape 'b' loop
										}//end for b
									if(!found){
										newGtypeWord->cat('A');}
									}//end if...else
							}//end for a
						if(dataStrCopy.len()<3){//stretch to length 3
							for(int c=dataStrCopy.len();c<3;++c){
								newGtypeWord->cat(' ');}
							}//end if
						}
					else{//this locus not diagnostic
						newGtypeWord->cat(dataStrCopy);
						}
				}//end for loc	
			alloData->rawGtypes[pop][i] = *newGtypeWord;
			delete newGtypeWord; newGtypeWord=0;
			}//end for i
		}//end for pop					
				
	//now replace the diagnostic alleles list for the revised data
	for (int loc=0;loc<alloData->rawLocusName.len();++loc){
		aString rName(alloData->rawLocusName[loc]);
		StripDiagnosticAlleles(rName);
		if(!(rName == alloData->rawLocusName[loc])){//diagnostic alleles given
			StripDiagnosticAlleles(alloData->rawLocusName[loc]);
			alloData->rawLocusName[loc].cat("(B)");
			}
		}//end for loc
	for (int loc=0;loc<alloData->skippedLocusName.len();++loc){
		aString rName(alloData->skippedLocusName[loc]);
		StripDiagnosticAlleles(rName);
		if(!(rName == alloData->skippedLocusName[loc])){//diagnostic alleles given
			StripDiagnosticAlleles(alloData->skippedLocusName[loc]);
			alloData->skippedLocusName[loc].cat("(B)");
			}
		}//end for loc
	alloData->PopulationGeneticData::CullSkippedLoci();
	//now replace the alleles present list for the revised data
	alloData->CheckForNewAlleles();
	alloData->StoreDiagnosticAlleles();
	for(int loc=alloData->skippedLocusNum.len()-1;loc>=0;--loc){
		int num = alloData->skippedLocusNum[loc];
		alloData->diagnosticAllelesR.Delete(num);//pull out skipped diagnostic loci
		}//end for loc	
	alloData->maxall = 2;
	alloData->dataType = dataType;
	return alloData;
 	}//end ClineGeneticData::PoolDiagnosticAlleles()




ClineGeneticData* ClineGeneticData::PoolDiagnosticHaplotypes (void){
	if (diagnosticAllelesR.len()==0){
		StoreDiagnosticAlleles();}
	ClineGeneticData* halpoData = NEW ClineGeneticData(*this);
	halpoData->ResetFlags();
	halpoData->title_.cat(": diagnostic haplotypes pooled");
	aString endCharList(" \t,");
	//Go through each individual and change its genotype in alloData
	//An allele diagnostic on the left side is A, and on the right side is B
	//Alleles unique to the central part of the zone are pooled with the left side as A
	for(int pop=0;pop<pops();++pop){
	    for(int i=0;i<halpoData->rawGtypes[pop].len();++i){
			aString dataStr(halpoData->rawGtypes[pop][i]);
			//first snip off & save the individual' s designation code, then rebuild haplotype on it
			aString* newGtypeWord = dataStr.ChopOffFirstWord(endCharList);
			for(int loc=0;loc<rawLocusName.len();++loc){
				aString *htype = dataStr.ChopOffFirstWord(endCharList);
				aString lName(rawLocusName[loc]);
				StripDiagnosticAlleles(lName);
				int found=0;
				if(!(lName == halpoData->rawLocusName[loc])){//this locus has diagnostic alleles
						for(int b=0;b<diagnosticAllelesR[loc].len();++b){
							if (diagnosticAllelesR[loc][b] == *htype){//haplotype origin right side
								newGtypeWord->cat(" B ");//the space after allows the diploid functions to read this data
								found=1;
								break;}//escape 'b' loop
							}//end for b
						if(!found){
								newGtypeWord->cat(" A ");}//the space after allows the diploid functions to read this data
					}//end if diagnostic
					else{//this locus not diagnostic
						newGtypeWord->cat(*htype);
						}
				delete htype;htype=0;
				}//end for loc
			if(dataStr.len()>0){
				newGtypeWord->cat(" ");
				newGtypeWord->cat(dataStr);}
			halpoData->rawGtypes[pop][i] = *newGtypeWord;
			delete newGtypeWord; newGtypeWord=0;
			}//end for i
		}//end for pop					
				
	//now replace the diagnostic alleles list for the revised data
	for (int loc=0;loc<halpoData->rawLocusName.len();++loc){
		aString rName(halpoData->rawLocusName[loc]);
		StripDiagnosticAlleles(rName);
		if(!(rName == halpoData->rawLocusName[loc])){//diagnostic alleles given
			StripDiagnosticAlleles(halpoData->rawLocusName[loc]);
			halpoData->rawLocusName[loc].cat("(B)");
			}
		}//end for loc
	for (int loc=0;loc<halpoData->skippedLocusName.len();++loc){
		aString rName(halpoData->skippedLocusName[loc]);
		StripDiagnosticAlleles(rName);
		if(!(rName == halpoData->skippedLocusName[loc])){//diagnostic alleles given
			StripDiagnosticAlleles(halpoData->skippedLocusName[loc]);
			halpoData->skippedLocusName[loc].cat("(B)");
			}
		}//end for loc
	halpoData->PopulationGeneticData::CullSkippedLoci();
	//now replace the alleles present list for the revised data
	halpoData->CheckForNewHaplotypes();
	halpoData->StoreDiagnosticAlleles();
	for(int loc=halpoData->skippedLocusNum.len()-1;loc>=0;--loc){
		int num = halpoData->skippedLocusNum[loc];
		halpoData->diagnosticAllelesR.Delete(num);//pull out skipped diagnostic loci
		}//end for loc	
	halpoData->maxall = 2;
	return halpoData;
 	}//end ClineGeneticData::PoolDiagnosticHaplotypes()




void ClineGeneticData::PrintGelJockFile (aString& outFileName){
	PopulationGeneticData::PrintGelJockFile(outFileName);}



void ClineGeneticData::PrintGelJockFile (ostream& outFile){
	//		theGenotypes: ListStrStructureTypePtr;
	aString dataLine = title_;
	dataLine.cat(":  skipped loci omitted");
	outFile<<dataLine<<endl;
	dataLine = "NPOP=";
	dataLine.cat(pops());
	dataLine.cat(", NLOCI=");
	dataLine.cat(nloci);
	dataLine.cat(", MAXALL=");
	dataLine.cat(maxall);
	dataLine.cat(", NGROUPS=");
	dataLine.cat(ngroups);
	outFile<<dataLine<<endl;
	dataLine.clear();
	for (int loc=0;loc<locusName.len();++loc){
		aString lName(locusName[loc]);
		AddDiagnosticAlleles(loc,lName);
		dataLine.cat(lName);
		if( ( ((double)(loc+1)) / 5.0) - ((loc+1) / 5) == 0){//new line for every 5 loci
				outFile<<dataLine<<endl;
				dataLine.clear();}
			else if (loc+1 < locusName.len()){
				dataLine.cat(", ");}
			else{
				outFile<<dataLine<<endl;
				dataLine.clear();}
		}//end for loc
	dataLine = "SKIP=0";
	outFile<<dataLine<<endl;
	dataLine.clear();
	for (int pop=0;pop<pops();++pop){
		outFile<<popName[pop]<<endl;
		StringList& theGenotypes = gtypes[pop];
		for (int i=0;i<theGenotypes.len();++i){
			dataLine = theGenotypes[i];
			outFile<<dataLine<<endl;
			}//end for i
		outFile<<"NEXT"<<endl;
		}//end for pop
	outFile<<"END"<<endl;
	}//end ClineGeneticData::PrintGelJockFile



void ClineGeneticData::PrintHybridIndexFile(aString& outFileName){
	char* rawfilename = outFileName.raw();
	fstream outputData;
	outputData.open(rawfilename, ios::out /*,filebuf::openprot*/);
	for(int i=0;i<hybridIndices.len();++i){
		aString outline(hybridIndices[i].x(),10,5); outline.cat('\t');	
		outline.cat(hybridIndices[i].hi(),2,6); outline.cat('\t');	
		outline.cat(hybridIndices[i].items());
		outputData<<outline<<endl; 
		}//end for i	
	outputData.close();
	delete [] rawfilename; rawfilename=0;
	}//end ClineGeneticData::PrintHybridIndexFile(aString&)



void ClineGeneticData::ReadGelJockDataFile (aString* dataFileName){
	PopulationGeneticData::ReadGelJockDataFile (dataFileName);
	for (int loc=0;loc<locusName.len();++loc){
		aString& lName = locusName[loc];
		StripDiagnosticAlleles(lName);
		}//end for loc
	StoreDiagnosticAlleles();
	dataType.Clear();
	for(int tr=0;tr<NumTraits();++tr){
		dataType.Append(NEW aString("codominant"));}
	}//end ClineGeneticData::ReadGelJockDataFile()


void ClineGeneticData::ReadGelJockmtDNADataFile (aString* dataFileName){
	PopulationGeneticData::ReadGelJockmtDNADataFile (dataFileName);
	for (int loc=0;loc<locusName.len();++loc){
		aString& lName = locusName[loc];
		StripDiagnosticAlleles(lName);
		}//end for loc
	StoreDiagnosticAlleles();
	}//end ClineGeneticData::ReadGelJockmtDNADataFile()


void ClineGeneticData::ResetFlags(void){
	PopulationGeneticData::ResetFlags();
	localityFlag_=0;
	}//end ClineGeneticData::ResetFlags()


void ClineGeneticData::RestoreLoci (StringList& lociToRestore){
	//This function restores the designated loci to each genotype data string
	//and removes them from the skipped loci
	if(lociToRestore.len()==0){return;}//none to cull
	for(int rLoc=0;rLoc<lociToRestore.len();++rLoc){
		aString rLocusname = lociToRestore[rLoc];
		aString rLocusnameCopy(rLocusname);
		AddDiagnosticAlleles(rLoc, rLocusname);
		int item = rawLocusName.Loc(&rLocusname);
		aString rLnm;//stripped raw name stored here, processed below
		if(item>=0){//the locus exists in the list
			//retore to original location
			int inserted = 0;
			for(int j=locusName.len()-1;j>=0;--j){
				//compare to locations of currently used loci
				aString& currLocusname = locusName[j];
				//find the raw locus name's location
				for(int k=0;k<rawLocusName.len();++k){
					aString& rLocName = rawLocusName[k];
					rLnm = rLocName;
					StripDiagnosticAlleles(rLnm);
					if(rLnm == currLocusname){//insert here
						locusName.Insert(NEW aString(rLnm), j+1);
						dataType.Insert(NEW aString("codominant"), j+1);						
						inserted = 1;
						break;
						}//end if
					}//end for k
				}//j		
			if(! inserted){//goes at location 0
				locusName.Prepend(NEW aString(rLnm));
				dataType.Prepend(NEW aString("codominant"));
				}
							
			int item1 = skippedLocusNum.Loc(item);
			if(item1>=0){//it's in the skipped list
				skippedLocusNum.Delete(item1);
				int item2 = skippedLocusName.Loc(&rLocusname);
				skippedLocusName.Delete(item2);
				--lociSkipped;++nloci;
				}//if item exists in skipped list
			}//if item exists
		}//cLoc
	for(int i=0;i<rawLocusName.len();++i){
		cout<<rawLocusName[i]<<",";}
	cout<<endl<<endl;
	for(int i=0;i<locusName.len();++i){
		cout<<locusName[i]<<",";}
	cout<<endl;
	ResetFlags();
	CullSkippedLoci();
	CheckForNewAlleles();
	}//end procedure ClineGeneticData::RestoreLoci


void ClineGeneticData::SetCenterPopList(aList<int>& centerList){
	centerPopList_.Clear();
	centerPopList_ = centerList;
	}//end ClineGeneticData::SetCenterPopList(aList<double>&)


void ClineGeneticData::SetDataTypes(void){//default
	for(int tr=0;tr<NumTraits();++tr){
		dataType.Append(NEW aString("codominant"));}
	}// ClineGeneticData::SetDataTypes()

void ClineGeneticData::SetDataTypes(aString& theDataType){
	for(int tr=0;tr<NumTraits();++tr){
		dataType.Append(NEW aString(theDataType));}
	}// ClineGeneticData::SetDataTypes()



void ClineGeneticData::StoreDiagnosticAlleles(void){
	diagnosticAllelesR.Clear();
	aString endCharList("()[]{}; ");
	for(int loc=0;loc<rawLocusName.len();++loc){//do for every locus first
		diagnosticAllelesR.Append(NEW StringList());
		aString nameAndDiagAlleles(rawLocusName[loc]);
		aString* lname = nameAndDiagAlleles.ChopOffFirstWord(endCharList);
		delete lname; lname = 0;//throw away
		do{
			aString* allname = nameAndDiagAlleles.ChopOffFirstWord(endCharList);
			allname->CullChar(' ');//cull spaces
			if(allname->len()==0) {
					delete allname; allname = 0;//throw away
					}
				else{
					diagnosticAllelesR[loc].Append(allname);
					allname = 0;//forget this
					}
			}while(nameAndDiagAlleles.len()>0);
		}//end for loc
	}//ClineGeneticData::StoreDiagnosticAlleles()


void ClineGeneticData::StripDiagnosticAlleles(aString& aLocusName){
	//strips list of diagnistic alleles to aString
	aString endCharList("()[]{}");
	int endOfName = aLocusName.posEndFirstWord(endCharList);
	if(endOfName>0){
		aLocusName.trunc(endOfName-1);}
	}//end StripDiagnosticAlleles(aString& aLocusName)


#endif



