// ClineGeneticData.h

#ifndef ClineGeneticDatah
#define ClineGeneticDatah = 1


#include "PopulationGeneticData.h"
#include "FstatisticAnalysis.h"
#include "HybridIndexByLocation.h"
#include "HybridIndexByLocus.h"
#include "MetropolisHastings.h"
#include "ClineData.h"
#include "Element.cpp"
#include "aList.cpp"
#include "MemoryManager.h"

	class ClineGeneticData: public ClineData, public PopulationGeneticData{
#ifdef MemoryLeakDebug
	private:
		MemLeakIDType memID_;
	public:
		static memAllocList memIDList_;
		static  MemLeakIDType memIDCount_;
#endif
		public:
			ObjList<StringList> diagnosticAllelesR;
					//list of loci, each with list of all alleles that are diagnostic on 
					//the right side of the hybrid zone
		//NOTE that StringList rawLocusName will hold the diagnostic alleles as well
		
			ObjList<HybridIndexByLocation> hybridIndices;
			ObjList<HybridIndexByLocus> HybridIndicesByLocus;
			aList<int> centerPopList_;//ordinal pops for disequilibrium stats
			StatList<double> locality;//x-axis localities
			int localityFlag_;
			long double centerPopFis_;
			int centerPopFflag_;
		
		public:
		ClineGeneticData(void):localityFlag_(0),
					centerPopFis_((long double)0.0),centerPopFflag_(0){
			#ifdef MemoryLeakDebug
				memID_=++memIDCount_;
				memIDList_.Append(memID_);
			#endif
			}
		ClineGeneticData(aString& s):PopulationGeneticData(s),
					localityFlag_(0),centerPopFis_((long double)0.0),centerPopFflag_(0){
			#ifdef MemoryLeakDebug
				memID_=++memIDCount_;
				memIDList_.Append(memID_);
			#endif
			}
virtual	~ClineGeneticData(void){
			#ifdef MemoryLeakDebug
				long i=memIDList_.Loc(memID_);
				memIDList_.Delete(i);
			#endif
			Clear();
			}

		ClineGeneticData(ClineGeneticData& parentData);//copy constructor

		ClineGeneticData(ClineGeneticData& parentData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets


	void 		AddDiagnosticAlleles(int locusNum, aString& aLocusName);//attaches list of diagnistic alleles to aString
	void 		CalculateHybridIndex(aList<double>& popLocation);
	long double CenterPopFis(void);
	void 		Clear(void);
	void 		CullLoci (StringList& lociToCull);
	void 		CullSkippedLoci (void);
//	void		FitAllozymeCline(aString& dataFileName);
	double 		Locality(int pop);//returns x-axis locality
	int			NumTraits(void){return loci();}
	ClineGeneticData* PoolDiagnosticAlleles (void);
	ClineGeneticData* PoolDiagnosticHaplotypes (void);
	int			pops(void){return npop_;}
	void 		PrintGelJockFile (aString& outFileName);
	void 		PrintGelJockFile (ostream& outFile);
	void 		PrintHybridIndexFile(aString& outFileName);
	void 		ReadGelJockDataFile (aString* dataFileName);//input file assumed closed
	void 		ReadGelJockmtDNADataFile (aString* dataFileName);//input file assumed closed
	void		ResetFlags(void);
	void 		RestoreLoci (StringList& lociToRestore);
	void 		SetCenterPopList(aList<int>& centerList);
	void 		SetDataTypes(void);
	void 		SetDataTypes(aString& theDataType);
	void 		StoreDiagnosticAlleles(void);
	void 		StripDiagnosticAlleles(aString& aLocusName);//strips list of diagnistic alleles to aString


#ifdef MemoryLeakDebug
	static void PrintIDList(ostream& outFile=cout);
	static void PrintIDListLen(ostream& outFile=cout);
#endif
		};//end class ClineGeneticData
		
		
#endif

