/******************************************************
*
*	ClineMultiCWParamRec.cpp
*
*	Parameters for a multi-trait tanh cline defined by center and width only.
*	Assumes the y-axis value of the cline varies between 0 and 1.
*
*	Use this as a virtual base class for derived classes that add parameters.
*
******************************************************/
#ifndef ClineMultiCWParamReccpp
#define ClineMultiCWParamReccpp = 1

#include "ClineMultiCWParamRec.h"

	
ParamRec* ClineMultiCWParamRec::Duplicate(void){
	ClineMultiCWParamRec* copy = NEW ClineMultiCWParamRec(*this);
	return copy;
	}//end ClineMultiCWParamRec::Duplicate()


ClineMultiCWParamRec* ClineMultiCWParamRec::_Duplicate(void){
	ClineMultiCWParamRec* copy = NEW ClineMultiCWParamRec(*this);
	return copy;
	}//end ClineMultiCWParamRec::_Duplicate()


//void ClineMultiCWParamRec::SetPointerToDerivedSelf(void){
//  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in ClineMultiCWParamRec"<<endl;
//  	}//end

long ClineMultiCWParamRec::GetPointerToDerivedSelf(void){
  	return (long) this;
  	}//end


long double ClineMultiCWParamRec::center(int trait){
	ClineShapeCWParamRec* traitParams = 
			(ClineShapeCWParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->center();
	}


long double ClineMultiCWParamRec::width(int trait){
	ClineShapeCWParamRec* traitParams = 
			(ClineShapeCWParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->width();
	}


int ClineMultiCWParamRec::ChangeCenter(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//change each one with the same paramsShared_ value
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	ClineMultiCWParamRec* lower = (ClineMultiCWParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWParamRec* upper = (ClineMultiCWParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	long double oldCenter = center(trait);
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int centerSet = paramsShared[trait][0];
	do{
		long double newCenter = ran.NormDev(oldCenter, sigma);
		ClineShapeCWParamRec* low = (ClineShapeCWParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWParamRec* high = (ClineShapeCWParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWParamRec* traitParams = 
				(ClineShapeCWParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->center(newCenter);
		validChange = traitParams->MeetsParamCriteria(low, high);
		if(! validChange){
			break;}
		//check for co-varying parameters
		for(int tr=0;tr<traitParamSet_.len();++tr){
			if((tr != trait) && (centerSet == paramsShared[tr][0])){//change it too
				ClineShapeCWParamRec* low = (ClineShapeCWParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
				ClineShapeCWParamRec* high = (ClineShapeCWParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
				ClineShapeCWParamRec* traitParams2 = 
						(ClineShapeCWParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
				traitParams2->center(newCenter);
				validChange = traitParams2->MeetsParamCriteria(low, high);
				if(! validChange){//try again if any traits fail
					break;}
				}
			}//end for tr
		++tries;
		if(tries>20){
//			aString err("can't find a good value for center");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (centerSet == paramsShared[tr][1])){//change it back too
						ClineShapeCWParamRec* traitParams = (ClineShapeCWParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams->center(oldCenter);}
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWParamRec::ChangeCenter(int, Ran3&, double, ParamRec*, ParamRec*)


int ClineMultiCWParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, long double weight){
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
	ClineMultiCWParamRec* deviations = (ClineMultiCWParamRec*) paramDeviations->GetPointerToDerivedSelf();
	int trait=0, param=0, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	switch (param){
	 case 0:
	 	 return ChangeCenter(trait, ran, deviations->center(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 1:
	 	 return ChangeWidth(trait, ran, deviations->width(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 default:
		aString errstr("bug: can't change shape param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 2-parameter cline model");
		aError(errstr);
	 	return 0;
	 	}
	 return 0;
	}//end ClineMultiCWParamRec::ChangeParam(int...)



int ClineMultiCWParamRec::ChangeWidth(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiCWParamRec* lower = (ClineMultiCWParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWParamRec* upper = (ClineMultiCWParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	long double oldWidth = width(trait);
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int widthSet = paramsShared[trait][1];
	do{
		long double newWidth = ran.NormDev(oldWidth, sigma);
		ClineShapeCWParamRec* low = (ClineShapeCWParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWParamRec* high = (ClineShapeCWParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWParamRec* traitParams = 
				(ClineShapeCWParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->width(newWidth);
		validChange = traitParams->MeetsParamCriteria(low, high);
		if(! validChange){
			break;}
		//check for co-varying parameters
		for(int tr=0;tr<traitParamSet_.len();++tr){
			if((tr != trait) && (widthSet == paramsShared[tr][1])){//change it too
				ClineShapeCWParamRec* low = (ClineShapeCWParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
				ClineShapeCWParamRec* high = (ClineShapeCWParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
				ClineShapeCWParamRec* traitParams2 = 
						(ClineShapeCWParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
				traitParams2->width(newWidth);
				validChange = traitParams2->MeetsParamCriteria(low, high);
				if(! validChange){
					break;}
				}
			}//end for tr
		++tries;
		if(tries>20){
//			aString err("can't find a good value for center");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (widthSet == paramsShared[tr][1])){//change it back too
						ClineShapeCWParamRec* traitParams = (ClineShapeCWParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams->width(oldWidth);}
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWParamRec::ChangeWidth(int, Ran3&, double, ParamRec*, ParamRec*)


long double ClineMultiCWParamRec::Cline_at_X(long double x, int trait){
	ClineShapeCWParamRec* traitParams = 
			(ClineShapeCWParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->Cline_at_X(x);
	}//end ClineMultiCWParamRec::Cline_at_X(double, int)


long double ClineMultiCWParamRec::ExpectedValue(aList<long double>& coords){
	return Cline_at_X(coords[0],int(coords[1]));
	}//end ParamRec::ExpectedValue(aList<double>&)



long double ClineMultiCWParamRec::leftVal(int trait){
	ClineShapeCWParamRec* traitParams = 
			(ClineShapeCWParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->leftVal(0);
	}//end ClineMultiCWParamRec::leftVal(int)


void ClineMultiCWParamRec::Initialize(ClineFitInitializer& i){
	lnL_=0.0;
	lnLmaxMinusLnL_=0.0;
	traitParamSet_.Clear();//in case of reinitialization
	for(int tr=0;tr<i.traits();++tr){
		traitParamSet_.Append(NEW ClineShapeCWParamRec(i,tr));}
	paramsOnOff_ = i.paramsOnOff_;//copies pointer
	paramsShared_ = i.paramIdentities_;//copies pointer
	varyingParams_=0;//in case of reinitialization
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	for(int p=0;p<BaseParams();++p){//assumes paramsShared is sorted low to high
		int maxparamID =-1;
		for(int tr=0;tr<traits();++tr){
			if((paramsOnOff[tr][p]==1) && (maxparamID<paramsShared[tr][p])){
				varyingParams_++;
				maxparamID = paramsShared[tr][p];
				}//if
			}//tr
		}//p
	}//end ClineMultiCWParamRec::Initialize(ClineFitInitializer&)


long double ClineMultiCWParamRec::maxSlope(int trait){
	ClineShapeCWParamRec* traitParams = 
			(ClineShapeCWParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->maxSlope();
	}//end ClineMultiCWParamRec::maxSlope(int)


int ClineMultiCWParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
	//use type conversion that circumvents usual virtual base class restrictions
	ClineMultiCWParamRec* lower = (ClineMultiCWParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWParamRec* upper = (ClineMultiCWParamRec*) upperLimits->GetPointerToDerivedSelf();
	for(int tr=0;tr<traits();++tr){
		ClineShapeCWParamRec* traitParams = (ClineShapeCWParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		ParamRec& lowParams = lower->traitParamSet_[tr];//no need to cast these yet
		ParamRec& highParams = upper->traitParamSet_[tr];
		int paramsValid = traitParams->MeetsParamCriteria(&lowParams,&highParams);
		if(! paramsValid) return false;
		}//tr
	return true;
	}//end ClineMultiCWParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)


void ClineMultiCWParamRec::MultiplyParam(int paramNum, long double factor){
	//use to rescale a param deviation
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int trait=0, param=0, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	int covaryingID = paramsShared[trait][param];
	for(int tr=trait;tr<traitParamSet_.len();++tr){//assumes paramsShared_ list is sorted
		if(covaryingID == paramsShared[tr][param]){//change it too
			ClineShapeCWParamRec* traitParams = (ClineShapeCWParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
			traitParams->MultiplyParam(param,factor);
			}
		}//tr
	}//ClineMultiCWParamRec::MultiplyParam(int, long double)


int ClineMultiCWParamRec::NumParams(void){
	return varyingParams_;
	}//end ClineMultiCWParamRec::NumParams()


int ClineMultiCWParamRec::NumDerivedParams(void){
	return 0;
	}//end ClineMultiCWParamRec::NumParams()



void ClineMultiCWParamRec::ParamCoords(int globalParamNum, int& trait, int& traitParam, int& derived){
	//returns values for trait && traitParam
	//if params covary among traits, this returns the first trait
	derived = false;//no derived parameters in this class
	int count=0;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	for(int p=0;p<BaseParams();++p){
		int maxSharedID=-1;
		for(int tr=0;tr<traits();++tr){
			if(paramsOnOff[tr][p]==1){//estimating this one
				if(maxSharedID<paramsShared[tr][p]){//this is a new param to estimate
					if(globalParamNum==count){
						trait=tr; traitParam=p;
						return;}
					++count;
					maxSharedID = paramsShared[tr][p];//assumes these are un ascending order
				}}
			}//tr
		}//p
	aString err("Can\'t find param");
	aError(err);
	}//end ClineMultiCWParamRec::ParamCoords(int, int&, int&)


aString* ClineMultiCWParamRec::ParamName(int paramNum){
	int trait=0, param=0, allShared=true, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	aString trstr,  *newName = NEW aString();
	if(traits()>1){
		trstr.cat("[");trstr.cat(trait+1);
		ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
		ObjList<aList<int> >& paramsShared = *paramsShared_;
		int shareState = paramsShared[trait][param];
		for(int tr=0;tr<traits();++tr){
			if(tr != trait && paramsOnOff[tr][param]){//param is being estimated for this trait
				if(shareState != paramsShared[tr][param]){
						allShared=false;}
					else{
						trstr.cat(';');trstr.cat(tr+1);}//only append those with shared states
				}}
		trstr.cat("]");}
	switch (param){
	 case 0:
		newName->cat("center"); break;
	 case 1:
		newName->cat("width"); break;
	 default:
		aString errstr("bug: can't return name of param #");
		errstr.cat(param+1);
		if(traits()>1){
			errstr.cat(" for trait #");
			errstr.cat(trait+1);}
		aError(errstr);
	 	return newName;//null
	 	}	
	if(traits()>1 && !allShared){
		newName->cat(trstr);}
	return newName;
	}//end ClineMultiCWParamRec::ParamName(int )



void ClineMultiCWParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ParamRec::ParamString(dataline, sepchar);
	//each param will be printed, whether or not it is co-estimated with another
	for(int tr=0;tr<traits();++tr){
		ClineShapeCWParamRec* traitParams = (ClineShapeCWParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		for(int p=0;p<BaseParams();++p){//2 params/trait
			if(paramsOnOff[tr][p]==1){//param is estimated
				long double val = traitParams->ParamValue(p);
				dataline.cat(sepchar);
				dataline.cat(val,6,15);
				}//if
			}//p
		}//tr
	}//end ClineMultiCWParamRec::ParamString(aString&, char)



long double ClineMultiCWParamRec::ParamValue(int paramNum){
	//returns value of parameter; first varyng params, then derived params
	int trait=0, param=0, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	ClineShapeCWParamRec* traitParams = (ClineShapeCWParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->ParamValue(param);
	}//end ClineMultiCWParamRec::ParamValue(int )


void ClineMultiCWParamRec::Print(ostream& outFile, long rep, aString& message){
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ParamRec::Print(outFile, rep, message);
	//for now, each param will be printed, whether or not it is co-estimated with another
	for(int tr=0;tr<traits();++tr){
		aString outStr;
		ClineShapeCWParamRec* traitParams = (ClineShapeCWParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		if(paramsOnOff[tr][0]==1){//center
			outStr.cat("center");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->center();
			outStr.cat(val,6,4);
			outStr.separator(5);
			}
		if(paramsOnOff[tr][1]==1){//width
			outStr.cat("width");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->width();
			outStr.cat(val,6,4);
			}
		outFile<<outStr<<endl;
		}//tr
	}//ClineMultiCWParamRec::Print


long double ClineMultiCWParamRec::rightVal(int trait){
	ClineShapeCWParamRec* traitParams = 
			(ClineShapeCWParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->rightVal(0);
	}//end ClineMultiCWParamRec::rightVal()


void ClineMultiCWParamRec::SetParamValue(int paramNum, long double newVal){
	//set value of parameter, and all those that covary with it
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int trait=0, param=0, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	int covaryingID = paramsShared[trait][param];
	for(int tr=trait;tr<traitParamSet_.len();++tr){//assumes paramsShared_ list is sorted
		if(covaryingID == paramsShared[tr][param]){//change it too
			ClineShapeCWParamRec* traitParams = (ClineShapeCWParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
			traitParams->SetParamValue(param,newVal);
			}
		}//tr
	}//end ClineMultiCWParamRec::SetParamValue(int,long double)



void ClineMultiCWParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ParamRec::TitleString(dataline, sepchar);
	//each param will be printed, whether or not it is co-estimated with another
	for(int tr=0;tr<traits();++tr){
		if(paramsOnOff[tr][0]==1){//center
			dataline.cat(sepchar);
			dataline.cat("center");
			if(traits()>1){
				dataline.cat('[');dataline.cat(tr+1);dataline.cat(']');}
			}
		if(paramsOnOff[tr][1]==1){//width
			dataline.cat(sepchar);
			dataline.cat("width");
			if(traits()>1){
				dataline.cat('[');dataline.cat(tr+1);dataline.cat(']');}
			}
		}//tr
	}//end ClineMultiCWParamRec::TitleString(aString&, char)


int ClineMultiCWParamRec::traits(){
	return traitParamSet_.len();
	}//end ClineMultiCWParamRec::traits()


void ClineMultiCWParamRec::operator=(ParamRec& p){
	ParamRec::operator=(p);
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means p must actually point to an object of class ClineMultiCWParamRec
	ClineMultiCWParamRec* pp = (ClineMultiCWParamRec*) p.GetPointerToDerivedSelf();
	paramsOnOff_ = pp->paramsOnOff_;
	paramsShared_ = pp->paramsShared_;
	varyingParams_ = pp->varyingParams_;
	traitParamSet_.Clear();
	for(int tr=0;tr<pp->traits();++tr){
		ClineShapeCWParamRec* traitParams = 
			(ClineShapeCWParamRec*) pp->traitParamSet_[tr].GetPointerToDerivedSelf();
		traitParamSet_.Append(NEW ClineShapeCWParamRec(*traitParams));
		}//tr
	}//end  ClineMultiCWParamRec::operator=(ParamRec&)





int	operator==(ClineMultiCWParamRec& p, ClineMultiCWParamRec& q){
	if(p.traits()!=q.traits()) return 0;
	for(int tr=0;tr<p.traits();++tr){
		ClineShapeCWParamRec* traitParams1 = 
			(ClineShapeCWParamRec*) p.traitParamSet_[tr].GetPointerToDerivedSelf();
		ClineShapeCWParamRec* traitParams2 = 
			(ClineShapeCWParamRec*) q.traitParamSet_[tr].GetPointerToDerivedSelf();
		if(*traitParams1 != *traitParams2){return false;}
		}
	if(p.paramsOnOff_ != q.paramsOnOff_) return false;//compares pointers
	if(p.paramsShared_ != q.paramsShared_) return false;//compares pointers
	return true;
	}//end operator==(ClineMultiCWParamRec&, ClineMultiCWParamRec&)


int	operator!=(ClineMultiCWParamRec& p, ClineMultiCWParamRec& q){
	return !(p==q);
	}//end operator!=(ClineMultiCWParamRec&, ClineMultiCWParamRec&)



#endif


