/******************************************************
*
*	ClineMultiCWTBParamRec.cpp
*
*	Parameters for a multi-trait tanh cline defined by center,width, 
*	maximum (top) & minimum (bottom) only.
*	Assumes the y-axis value of the cline varies between top and bottom.
*	There may be as many traits as desired, each with their own top & bottom,
*	but sharing a common center and width.  
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/
#ifndef ClineMultiCWTBParamReccpp
#define ClineMultiCWTBParamReccpp = 1

#include "ClineMultiCWTBParamRec.h"

	
ParamRec* ClineMultiCWTBParamRec::Duplicate(void){
	ClineMultiCWTBParamRec* copy = NEW ClineMultiCWTBParamRec(*this);
	return copy;
	}//end ClineMultiCWTlTrParamRec::Duplicate()


ClineMultiCWTBParamRec* ClineMultiCWTBParamRec::_Duplicate(void){
	ClineMultiCWTBParamRec* copy = NEW ClineMultiCWTBParamRec(*this);
	return copy;
	}//end ClineMultiCWTBParamRec::_Duplicate()


//void ClineMultiCWTBParamRec::SetPointerToDerivedSelf(void){
//  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in ClineMultiCWTBParamRec"<<endl;
//  	}//end

long ClineMultiCWTBParamRec::GetPointerToDerivedSelf(void){
  	return (long) this;
  	}//end


long double ClineMultiCWTBParamRec::center(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTBParamRec* traitParams = 
			(ClineShapeCWTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->center();
	}


long double ClineMultiCWTBParamRec::width(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTBParamRec* traitParams = 
			(ClineShapeCWTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->width();
	}



int ClineMultiCWTBParamRec::ChangeCenter(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	ClineMultiCWTBParamRec* lower = (ClineMultiCWTBParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTBParamRec* upper = (ClineMultiCWTBParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	long double oldCenter = center(trait);
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int centerSet = paramsShared[trait][0];
	do{
		long double newCenter = ran.NormDev(oldCenter, sigma);
		ClineShapeCWTBParamRec* low = (ClineShapeCWTBParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTBParamRec* high = (ClineShapeCWTBParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTBParamRec* traitParams = 
				(ClineShapeCWTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->center(newCenter);
		validChange = traitParams->MeetsParamCriteria(low, high);
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (centerSet == paramsShared[tr][0])){//change it too
					low = (ClineShapeCWTBParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
					high = (ClineShapeCWTBParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
					ClineShapeCWTBParamRec* traitParams2 = 
							(ClineShapeCWTBParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
					traitParams2->center(newCenter);
					validChange = traitParams2->MeetsParamCriteria(low, high);
					if(! validChange){//try again if any traits fail
						break;}
					}
				}//end for tr
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for center");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (centerSet == paramsShared[tr][0])){//change it back too
						ClineShapeCWTBParamRec* traitParams3 = (ClineShapeCWTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams3->center(oldCenter);
						}
					}//tr
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWTBParamRec::ChangeCenter(int, Ran3&, double, ParamRec*, ParamRec*)


int ClineMultiCWTBParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, long double weight){
	//typecasting trick, so don't use function from base class
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
	ClineMultiCWTBParamRec* deviations = (ClineMultiCWTBParamRec*) paramDeviations->GetPointerToDerivedSelf();
	int trait=0, param=0, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	switch (param){
	 case 0:
	 	 return ChangeCenter(trait, ran, deviations->center(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 1:
	 	 return ChangeWidth(trait, ran, deviations->width(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 2:
	 	 return ChangeLeftScore(trait, ran, deviations->leftVal(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 3:
	 	 return ChangeRightScore(trait, ran, deviations->rightVal(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 default:
		aString errstr("bug: can't change shape param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a multi-trait, 4-parameter cline model");
		aError(errstr);
	 	return 0;
	 	}
	 return 0;
	}//end ClineMultiCWTBParamRec::ChangeParam(int...)



int ClineMultiCWTBParamRec::ChangeLeftScore(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiCWTBParamRec* lower = (ClineMultiCWTBParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTBParamRec* upper = (ClineMultiCWTBParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	long double oldLeft = leftVal(trait);
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int leftSet = paramsShared[trait][2];
	do{
		long double newLeft = ran.NormDev(oldLeft, sigma);
		ClineShapeCWTBParamRec* low = (ClineShapeCWTBParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTBParamRec* high = (ClineShapeCWTBParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTBParamRec* traitParams = 
				(ClineShapeCWTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->leftVal(newLeft);
		validChange = traitParams->MeetsParamCriteria(low, high);
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (paramsOnOff[tr][2]) && (leftSet == paramsShared[tr][2])){//change it too
					low = (ClineShapeCWTBParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
					high = (ClineShapeCWTBParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
					ClineShapeCWTBParamRec* traitParams2 = 
							(ClineShapeCWTBParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
					traitParams2->leftVal(newLeft);
					validChange = traitParams2->MeetsParamCriteria(low, high);
					if(! validChange){
						break;}
					}
				}//end for tr
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for thetaL");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (paramsOnOff[tr][2]) && (leftSet == paramsShared[tr][2])){//change it back too
						ClineShapeCWTBParamRec* traitParams3 = (ClineShapeCWTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams3->leftVal(oldLeft);
						}
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWTBParamRec::ChangeLeftScore(int, Ran3&, double, ParamRec*, ParamRec*)


int ClineMultiCWTBParamRec::ChangeRightScore(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiCWTBParamRec* lower = (ClineMultiCWTBParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTBParamRec* upper = (ClineMultiCWTBParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	long double oldRight = rightVal(trait);
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int rightSet = paramsShared[trait][3];
	do{
		long double newRight = ran.NormDev(oldRight, sigma);
		ClineShapeCWTBParamRec* low = (ClineShapeCWTBParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTBParamRec* high = (ClineShapeCWTBParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTBParamRec* traitParams = 
				(ClineShapeCWTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->rightVal(newRight);
		validChange = traitParams->MeetsParamCriteria(low, high);
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (paramsOnOff[tr][3]) && (rightSet == paramsShared[tr][3])){//change it too
					low = (ClineShapeCWTBParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
					high = (ClineShapeCWTBParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
					ClineShapeCWTBParamRec* traitParams2 = 
							(ClineShapeCWTBParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
					traitParams2->rightVal(newRight);
					validChange = traitParams2->MeetsParamCriteria(low, high);
					if(! validChange){
						break;}
					}
				}//end for tr
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for thetaR");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (paramsOnOff[tr][3]) && (rightSet == paramsShared[tr][3])){//change it back too
						ClineShapeCWTBParamRec* traitParams3 = (ClineShapeCWTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams3->rightVal(oldRight);
						}
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWTBParamRec::ChangeRightScore(int, Ran3&, double, ParamRec*, ParamRec*)



int ClineMultiCWTBParamRec::ChangeWidth(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiCWTBParamRec* lower = (ClineMultiCWTBParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTBParamRec* upper = (ClineMultiCWTBParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	long double oldWidth = width(trait);
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int widthSet = paramsShared[trait][1];
	do{
		long double newWidth = ran.NormDev(oldWidth, sigma);
		ClineShapeCWTBParamRec* low = (ClineShapeCWTBParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTBParamRec* high = (ClineShapeCWTBParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTBParamRec* traitParams = 
				(ClineShapeCWTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->width(newWidth);
		validChange = traitParams->MeetsParamCriteria(low, high);
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (widthSet == paramsShared[tr][1])){//change it too
					low = (ClineShapeCWTBParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
					high = (ClineShapeCWTBParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
					ClineShapeCWTBParamRec* traitParams2 = 
							(ClineShapeCWTBParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
					traitParams2->width(newWidth);
					validChange = traitParams2->MeetsParamCriteria(low, high);
					if(! validChange){
						break;}
					}
				}//end for tr
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for width");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (widthSet == paramsShared[tr][1])){//change it back too
						ClineShapeCWTBParamRec* traitParams3 = (ClineShapeCWTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams3->width(oldWidth);
						}
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWTBParamRec::ChangeWidth(int, Ran3&, double, ParamRec*, ParamRec*)



long double ClineMultiCWTBParamRec::Cline_at_X(long double x, int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTBParamRec* traitParams = 
			(ClineShapeCWTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->Cline_at_X(x);
	}//end ClineMultiCWTBParamRec::Cline_at_X(double, int)



long double ClineMultiCWTBParamRec::leftVal(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTBParamRec* traitParams = 
			(ClineShapeCWTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->leftVal();
	}//end ClineMultiCWTBParamRec::leftVal(int)


void ClineMultiCWTBParamRec::Initialize(ClineFitInitializer& i){
	lnL_=0.0;
	lnLmaxMinusLnL_=0.0;
	paramsOnOff_ = i.paramsOnOff_;//copies pointer
	paramsShared_ = i.paramIdentities_;//copies pointer
	varyingParams_=0;//in case of reinitialization
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	traitParamSet_.Clear();//in case of reinitialization
	for(int tr=0;tr<i.traits();++tr){//account for on/off parameters here
		traitParamSet_.Append(NEW ClineShapeCWTBParamRec(i,tr));
		ClineShapeCWTBParamRec *traitParams = (ClineShapeCWTBParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		if(paramsOnOff[tr][2]==0){//left side off
			traitParams->leftVal((long double)0.0);
			}
		if(paramsOnOff[tr][3]==0){//right side off
			traitParams->rightVal((long double)1.0);
			}
		}
	//count varying params
	for(int p=0;p<BaseParams();++p){//assumes paramsShared is sorted low to high
		int maxparamID =-1;
		for(int tr=0;tr<traits();++tr){
			if((paramsOnOff[tr][p]==1) && (maxparamID<paramsShared[tr][p])){
				varyingParams_++;
				maxparamID = paramsShared[tr][p];
				}//if
			}//tr
		}//p
	}//end ClineMultiCWTBParamRec::Initialize(ClineFitInitializer&)



long double ClineMultiCWTBParamRec::maxSlope(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTBParamRec* traitParams = 
			(ClineShapeCWTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->maxSlope();
	}//end ClineMultiCWTBParamRec::maxSlope(int)


int ClineMultiCWTBParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//use type conversion that circumvents usual virtual base class restrictions
	ClineMultiCWTBParamRec* lower = (ClineMultiCWTBParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTBParamRec* upper = (ClineMultiCWTBParamRec*) upperLimits->GetPointerToDerivedSelf();
	for(int tr=0;tr<traits();++tr){
		ClineShapeCWTBParamRec* traitParams = (ClineShapeCWTBParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		ParamRec& lowParams = lower->traitParamSet_[tr];//no need to cast these yet
		ParamRec& highParams = upper->traitParamSet_[tr];
		int paramsValid = traitParams->MeetsParamCriteria(&lowParams,&highParams);
		if(! paramsValid) return false;
		}//tr
	return true;
	}//end ClineMultiCWTBParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)


void ClineMultiCWTBParamRec::MultiplyParam(int paramNum, long double factor){
	//typecasting trick, so don't use function from base class
	//use to rescale a param deviation
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int trait=0, param=0, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	int covaryingID = paramsShared[trait][param];
	for(int tr=trait;tr<traitParamSet_.len();++tr){//assumes paramsShared_ list is sorted
		if(covaryingID == paramsShared[tr][param]){//change it too
			ClineShapeCWTBParamRec* traitParams = (ClineShapeCWTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
			traitParams->MultiplyParam(param,factor);
			}
		}//tr
	}//ClineMultiCWTBParamRec::MultiplyParam(int, long double)



int ClineMultiCWTBParamRec::NumDerivedParams(void){
	return 0;
	}//end ClineMultiCWTBParamRec::NumDerivedParams()



aString* ClineMultiCWTBParamRec::ParamName(int paramNum){
	int trait=0, param=0, allShared=true, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	aString trstr,  *newName = NEW aString();
	if(traits()>1){
		trstr.cat("[");trstr.cat(trait+1);
		ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
		ObjList<aList<int> >& paramsShared = *paramsShared_;
		int shareState = paramsShared[trait][param];
		for(int tr=0;tr<traits();++tr){
			if(tr != trait && paramsOnOff[tr][param]){//param is being estimated for this trait
				if(shareState != paramsShared[tr][param]){
						allShared=false;}
					else{
						trstr.cat(';');trstr.cat(tr+1);}//only append those with shared states
				}}
		trstr.cat("]");}
	switch (param){
	 case 0:
		newName->cat("center"); break;
	 case 1:
		newName->cat("width"); break;
	 case 2:
		newName->cat("pL"); break;
	 case 3:
		newName->cat("pR"); break;
	 default:
		aString errstr("bug: can't return name of param #");
		errstr.cat(param+1);
		if(traits()>1){
			errstr.cat(" for trait #");
			errstr.cat(trait+1);}
		aError(errstr);
	 	return newName;//null
	 	}	
	if(traits()>1 && !allShared){
		newName->cat(trstr);}
	return newName;
	}//end ClineMultiCWTBParamRec::ParamName(int )



void ClineMultiCWTBParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ParamRec::ParamString(dataline, sepchar);
	long double val;
	//each param will be printed, whether or not it is co-estimated with another
	for(int tr=0;tr<traits();++tr){
		ClineShapeCWTBParamRec* traitParams = (ClineShapeCWTBParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		if(paramsOnOff[tr][0]==1){//center
			dataline.cat(sepchar);
			val = traitParams->center();
			dataline.cat(val,6,15);
			}
		if(paramsOnOff[tr][1]==1){//width
			dataline.cat(sepchar);
			val = traitParams->width();
			dataline.cat(val,6,15);
			}
		if(paramsOnOff[tr][2]==1){//left
			dataline.cat(sepchar);
			val = traitParams->leftVal();
			dataline.cat(val,6,15);
			}
		if(paramsOnOff[tr][3]==1){//right
			dataline.cat(sepchar);
			val = traitParams->rightVal();
			dataline.cat(val,6,15);
			}
		}//tr
	}//end ClineMultiCWTBParamRec::ParamString(aString&, char)



long double ClineMultiCWTBParamRec::ParamValue(int paramNum){
	//typecasting trick, so don't use function from base class
	//returns value of parameter; first varyng params, then derived params
	int trait=0, param=0, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	ClineShapeCWTBParamRec* traitParams = (ClineShapeCWTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	switch(param){
		case 0://center
			return traitParams->center();break;			
		case 1://width
			return traitParams->width();break;			
		case 2://left
			return traitParams->leftVal();break;			
		case 3://right
			return traitParams->rightVal();break;			
		default: break;			
		}
	return 0.0;
	}//end ClineMultiCWTBParamRec::ParamValue(int )


void ClineMultiCWTBParamRec::Print(ostream& outFile, long rep, aString& message){
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ParamRec::Print(outFile, rep, message);
	//for now, each param will be printed, whether or not it is co-estimated with another
	for(int tr=0;tr<traits();++tr){
		aString outStr;
		ClineShapeCWTBParamRec* traitParams = (ClineShapeCWTBParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		if(paramsOnOff[tr][0]==1){//center
			outStr.cat("center");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->center();
			outStr.cat(val,6,4);
			outStr.separator(5);
			}
		if(paramsOnOff[tr][1]==1){//width
			outStr.cat("width");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->width();
			outStr.cat(val,6,4);
			}
		outFile<<outStr<<endl;
		outStr.clear();
		if(paramsOnOff[tr][2]==1){//left score
			outStr="pL";
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->leftVal();
			outStr.cat(val,6,4);
			}
		if(paramsOnOff[tr][3]==1){//thetaR & zR toggle on/off together
			outStr.separator(2);
			outStr.cat("pR");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->rightVal();
			outStr.cat(val,6,4);
			}
		if(outStr.len()>0){
			outFile<<outStr<<endl;}
		}//tr
	}//ClineMultiCWTBParamRec::Print


long double ClineMultiCWTBParamRec::rightVal(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTBParamRec* traitParams = 
			(ClineShapeCWTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->rightVal();
	}//end ClineMultiCWTBParamRec::rightVal(int)


void ClineMultiCWTBParamRec::SetParamValue(int paramNum, long double newVal){
	//set value of parameter, and all those that covary with it
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int trait=0, param=0, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	int covaryingID = paramsShared[trait][param];
	for(int tr=trait;tr<traitParamSet_.len();++tr){//assumes paramsShared_ list is sorted
		if(covaryingID == paramsShared[tr][param]){//change it too
			ClineShapeCWTBParamRec* traitParams = (ClineShapeCWTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
			traitParams->SetParamValue(param,newVal);
			}
		}//tr
	}//end ClineMultiCWTBParamRec::SetParamValue(int,long double)


void ClineMultiCWTBParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ParamRec::TitleString(dataline, sepchar);
	//each param will be printed, whether or not it is co-estimated with another
	for(int tr=0;tr<traits();++tr){
		if(paramsOnOff[tr][0]==1){//center
			dataline.cat(sepchar);
			dataline.cat("center");
			if(traits()>1){
				dataline.cat('[');dataline.cat(tr+1);dataline.cat(']');}
			}
		if(paramsOnOff[tr][1]==1){//width
			dataline.cat(sepchar);
			dataline.cat("width");
			if(traits()>1){
				dataline.cat('[');dataline.cat(tr+1);dataline.cat(']');}
			}
		if(paramsOnOff[tr][2]==1){//thetaL
			dataline.cat(sepchar);
			dataline.cat("pL");
			if(traits()>1){
				dataline.cat('[');dataline.cat(tr+1);dataline.cat(']');}
			}
		if(paramsOnOff[tr][3]==1){//thetaR
			dataline.cat(sepchar);
			dataline.cat("pR");
			if(traits()>1){
				dataline.cat('[');dataline.cat(tr+1);dataline.cat(']');}
			}
		}//tr
	}//end ClineMultiCWTBParamRec::TitleString(aString&, char)


void ClineMultiCWTBParamRec::operator=(ParamRec& p){
	ParamRec::operator=(p);
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means p must actually point to an object of class ClineMultiCWTlTrParamRec
	ClineMultiCWTBParamRec* pp = (ClineMultiCWTBParamRec*) p.GetPointerToDerivedSelf();
	paramsOnOff_ = pp->paramsOnOff_;
	paramsShared_ = pp->paramsShared_;
	varyingParams_ = pp->varyingParams_;
	traitParamSet_.Clear();
	for(int tr=0;tr<pp->traits();++tr){
		ClineShapeCWTBParamRec* traitParams = 
			(ClineShapeCWTBParamRec*) pp->traitParamSet_[tr].GetPointerToDerivedSelf();
		traitParamSet_.Append(NEW ClineShapeCWTBParamRec(*traitParams));
		}//tr
	}//end  ClineMultiCWTBParamRec::operator=(ParamRec&)





int	operator==(ClineMultiCWTBParamRec& p, ClineMultiCWTBParamRec& q){
	if(p.traits()!=q.traits()) return 0;
	for(int tr=0;tr<p.traits();++tr){
		ClineShapeCWTBParamRec* traitParams1 = 
			(ClineShapeCWTBParamRec*) p.traitParamSet_[tr].GetPointerToDerivedSelf();
		ClineShapeCWTBParamRec* traitParams2 = 
			(ClineShapeCWTBParamRec*) q.traitParamSet_[tr].GetPointerToDerivedSelf();
		if(*traitParams1 != *traitParams2){return false;}
		}
	if(p.paramsOnOff_ != q.paramsOnOff_) return false;//compares pointers
	if(p.paramsShared_ != q.paramsShared_) return false;//compares pointers
	return true;
	}//end operator==(ClineMultiCWTBParamRec&, ClineMultiCWTBParamRec&)


int	operator!=(ClineMultiCWTBParamRec& p, ClineMultiCWTBParamRec& q){
	return !(p==q);
	}//end operator!=(ClineMultiCWTBParamRec&, ClineMultiCWTBParamRec&)



#endif


