/******************************************************
*
*	ClineMultiCWTlTrDParamRec.cpp
*
*	Parameters for a muloti-trait, 3-part composite tanh cline defined by center, width,	
*	thetaL, thetaR, zL & zR.  The latter 4 parameters describe introgression
*	slopes and intercepts on the Left and Right sides of the cline.  Disequilibrium
*	is also estimated and derived parameters calculated.
*	Assumes the y-axis value of the cline varies between 0 and 1.
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/
#ifndef ClineMultiCWTlTrDParamReccpp
#define ClineMultiCWTlTrDParamReccpp = 1

#include "ClineMultiCWTlTrDParamRec.h"

	
ParamRec* ClineMultiCWTlTrDParamRec::Duplicate(void){
	ClineMultiCWTlTrDParamRec* copy = NEW ClineMultiCWTlTrDParamRec(*this);
	return copy;
	}//end ClineMultiCWTlTrDParamRec::Duplicate()


ClineMultiCWTlTrDParamRec* ClineMultiCWTlTrDParamRec::_Duplicate(void){
	ClineMultiCWTlTrDParamRec* copy = NEW ClineMultiCWTlTrDParamRec(*this);
	return copy;
	}//end ClineMultiCWTlTrDParamRec::_Duplicate()


//void ClineMultiCWTlTrDParamRec::SetPointerToDerivedSelf(void){
//  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in ClineMultiCWTlTrDParamRec"<<endl;
//  	}//end

long ClineMultiCWTlTrDParamRec::GetPointerToDerivedSelf(void){
  	return (long) this;
  	}//end





long double ClineMultiCWTlTrDParamRec::aveWidth(void){
	//average width over all traits; unestimated widths treated as zero
	double w=0.0;
	for(int tr=0;tr<traits();++tr){
		ClineShapeCWTlTrParamRec *traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		w += traitParams->width();
		}
	w /= traits();
	return w;//default
	}//end ClineMultiCWTlTrDParamRec::width()



int ClineMultiCWTlTrDParamRec::ChangeCenter(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	ClineMultiCWTlTrDParamRec* lower = (ClineMultiCWTlTrDParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTlTrDParamRec* upper = (ClineMultiCWTlTrDParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	double oldCenter = center(trait), leftEnd, rightEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int centerSet = paramsShared[trait][0];
	do{
		double newCenter = ran.NormDev(oldCenter, sigma);
		ClineShapeCWTlTrParamRec* low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* traitParams = 
				(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->center(newCenter);
		validChange = true;//traitParams->MeetsParamCriteria(low, high);
		if(validChange){
			if(paramsOnOff[trait][2]) {
				validChange = traitParams->LJunction(leftEnd=low->xL_);}}
		if(validChange){
			if(paramsOnOff[trait][3]) {
				validChange = traitParams->RJunction(rightEnd=high->xR_);}}
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (centerSet == paramsShared[tr][0])){//change it too
					low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
					high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
					ClineShapeCWTlTrParamRec* traitParams2 = 
							(ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
					traitParams2->center(newCenter);
					validChange = true;//traitParams2->MeetsParamCriteria(low, high);
					if(validChange){
						if(paramsOnOff[tr][2]) {validChange = traitParams2->LJunction(low->xL_);}}
					if(validChange){
						if(paramsOnOff[tr][3]) {validChange = traitParams2->RJunction(high->xR_);}}
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){//try again if any traits fail
						break;}
					}
				}//end for tr
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for center");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (centerSet == paramsShared[tr][0])){//change it back too
						ClineShapeCWTlTrParamRec* traitParams3 = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams3->center(oldCenter);
						if(paramsOnOff[tr][2]) {traitParams3->LJunction(leftEnd);}
						if(paramsOnOff[tr][3]) {traitParams3->RJunction(rightEnd);}
						}
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWTlTrDParamRec::ChangeCenter(int, Ran3&, double, ParamRec*, ParamRec*)



int ClineMultiCWTlTrDParamRec::ChangeD(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary D
	//return 0 for failure, 1 for success
	int validChange=false, tries=0;
	double oldD = D_;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int DSet = paramsShared[0][6];//all D's either on or off together
	do{
		D_ = ran.NormDev(oldD, sigma);
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for D");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				D_ = oldD;}
			return validChange=0;
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWTlTrDParamRec::ChangeD(Ran3&, double, ParamRec*, ParamRec*)



int ClineMultiCWTlTrDParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, long double weight){
	//typecasting trick, so don't use function from base class
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
	ClineMultiCWTlTrDParamRec* deviations = (ClineMultiCWTlTrDParamRec*) paramDeviations->GetPointerToDerivedSelf();
	int trait=0, param=0, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	switch (param){
	 case 0:
	 	 return ChangeCenter(trait, ran, deviations->center(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 1:
	 	 return ChangeWidth(trait, ran, deviations->width(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 2:
	 	 return ChangeThetaL(trait, ran, deviations->thetaL(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 3:
	 	 return ChangeThetaR(trait, ran, deviations->thetaR(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 4:
	 	 return ChangeZL(trait, ran, deviations->zL(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 5:
	 	 return ChangeZR(trait, ran, deviations->zR(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 6:
	 	 return ChangeD(ran, deviations->D()*weight, lowerLimits, upperLimits);
	 	 break;
	 default:
		aString errstr("bug: can't change shape param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a multi-trait, 7-parameter cline model");
		aError(errstr);
	 	return 0;
	 	}
	 return 0;
	}//end ClineMultiCWTlTrDParamRec::ChangeParam(int...)



int ClineMultiCWTlTrDParamRec::ChangeThetaL(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiCWTlTrDParamRec* lower = (ClineMultiCWTlTrDParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTlTrDParamRec* upper = (ClineMultiCWTlTrDParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	double oldThetaL = thetaL(trait), leftEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int thetaSet = paramsShared[trait][2];
	do{
		double newThetaL = ran.NormDev(oldThetaL, sigma);
		ClineShapeCWTlTrParamRec* low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* traitParams = 
				(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->thetaL(newThetaL);
		validChange = traitParams->LJunction(leftEnd=low->xL_);
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (paramsOnOff[tr][2]) && (thetaSet == paramsShared[tr][2])){//change it too
					low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
					high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
					ClineShapeCWTlTrParamRec* traitParams2 = 
							(ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
					traitParams2->thetaL(newThetaL);
					validChange = traitParams2->LJunction(low->xL_);
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){
						break;}
					}
				}//end for tr
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for thetaL");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (paramsOnOff[tr][2]) && (thetaSet == paramsShared[tr][2])){//change it back too
						ClineShapeCWTlTrParamRec* traitParams3 = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams3->thetaL(oldThetaL);
						traitParams3->LJunction(leftEnd);
						}
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWTlTrDParamRec::ChangeThetaL(int, Ran3&, double, ParamRec*, ParamRec*)


int ClineMultiCWTlTrDParamRec::ChangeThetaR(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiCWTlTrDParamRec* lower = (ClineMultiCWTlTrDParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTlTrDParamRec* upper = (ClineMultiCWTlTrDParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	double oldThetaR = thetaR(trait), rightEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int thetaSet = paramsShared[trait][3];
	do{
		double newThetaR = ran.NormDev(oldThetaR, sigma);
		ClineShapeCWTlTrParamRec* low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* traitParams = 
				(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->thetaR(newThetaR);
		validChange = traitParams->RJunction(rightEnd=high->xR_);
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (paramsOnOff[tr][3]) && (thetaSet == paramsShared[tr][3])){//change it too
					low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
					high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
					ClineShapeCWTlTrParamRec* traitParams2 = 
							(ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
					traitParams2->thetaR(newThetaR);
					validChange = traitParams2->RJunction(high->xR_);
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){
						break;}
					}
				}//end for tr
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for thetaR");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (paramsOnOff[tr][3]) && (thetaSet == paramsShared[tr][3])){//change it back too
						ClineShapeCWTlTrParamRec* traitParams3 = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams3->thetaR(oldThetaR);
						traitParams3->RJunction(rightEnd);
						}
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWTlTrDParamRec::ChangeThetaR(int, Ran3&, double, ParamRec*, ParamRec*)



int ClineMultiCWTlTrDParamRec::ChangeWidth(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiCWTlTrDParamRec* lower = (ClineMultiCWTlTrDParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTlTrDParamRec* upper = (ClineMultiCWTlTrDParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	double oldWidth = width(trait), leftEnd, rightEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int widthSet = paramsShared[trait][1];
	do{
		double newWidth = ran.NormDev(oldWidth, sigma);
		ClineShapeCWTlTrParamRec* low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* traitParams = 
				(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->width(newWidth);
		validChange= traitParams->MeetsParamCriteria(low, high);;
		if(validChange){
			if(paramsOnOff[trait][2]) {
				validChange = traitParams->LJunction(leftEnd=low->xL_);}}
		if(validChange){
			if(paramsOnOff[trait][3]) {
				validChange = traitParams->RJunction(rightEnd=high->xR_);}}
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (widthSet == paramsShared[tr][1])){//change it too
					low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
					high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
					ClineShapeCWTlTrParamRec* traitParams2 = 
							(ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
					traitParams2->width(newWidth);
					validChange= traitParams2->MeetsParamCriteria(low, high);;
					if(validChange){
						if(paramsOnOff[tr][2]) {
							validChange = traitParams2->LJunction(low->xL_);}}
					if(validChange){
						if(paramsOnOff[tr][3]) {
							validChange = traitParams2->RJunction(high->xR_);}}
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){
						break;}
					}
				}//end for tr
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for width");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (widthSet == paramsShared[tr][1])){//change it back too
						ClineShapeCWTlTrParamRec* traitParams3 = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams3->width(oldWidth);
						if(paramsOnOff[tr][2]) {traitParams3->LJunction(leftEnd);}
						if(paramsOnOff[tr][3]) {traitParams3->RJunction(rightEnd);}
						}
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWTlTrDParamRec::ChangeWidth(int, Ran3&, double, ParamRec*, ParamRec*)



int ClineMultiCWTlTrDParamRec::ChangeZL(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiCWTlTrDParamRec* lower = (ClineMultiCWTlTrDParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTlTrDParamRec* upper = (ClineMultiCWTlTrDParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	double oldZL = zL(trait), leftEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int zSet = paramsShared[trait][4];
	do{
		double newZL = ran.NormDev(oldZL, sigma);
		ClineShapeCWTlTrParamRec* low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* traitParams = 
				(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->zL(newZL);
		validChange = traitParams->MeetsParamCriteria(low, high);
		if(validChange){
			validChange = traitParams->LJunction(leftEnd=low->xL_);}
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (paramsOnOff[tr][4]) && (zSet == paramsShared[tr][4])){//change it too
					low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
					high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
					ClineShapeCWTlTrParamRec* traitParams2 = 
							(ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
					traitParams2->zL(newZL);
					validChange = traitParams2->MeetsParamCriteria(low, high);
					if(validChange){
						validChange = traitParams2->LJunction(leftEnd=low->xL_);}
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){
						break;}
					}
				}//end for tr
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for zL");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (paramsOnOff[tr][4]) && (zSet == paramsShared[tr][4])){//change it back too
						ClineShapeCWTlTrParamRec* traitParams3 = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams3->zL(oldZL);
						traitParams3->LJunction(leftEnd);
						}
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWTlTrDParamRec::ChangeZL(int, Ran3&, double, ParamRec*, ParamRec*)



int ClineMultiCWTlTrDParamRec::ChangeZR(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiCWTlTrDParamRec* lower = (ClineMultiCWTlTrDParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTlTrDParamRec* upper = (ClineMultiCWTlTrDParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	double oldZR = zR(trait), rightEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int zSet = paramsShared[trait][5];
	do{
		double newZR = ran.NormDev(oldZR, sigma);
		ClineShapeCWTlTrParamRec* low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* traitParams = 
				(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->zR(newZR);
		validChange = traitParams->MeetsParamCriteria(low, high);
		if(validChange){
			validChange = traitParams->RJunction(rightEnd=high->xR_);}
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (paramsOnOff[tr][5]) && (zSet == paramsShared[tr][5])){//change it too
					low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
					high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
					ClineShapeCWTlTrParamRec* traitParams2 = 
							(ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
					traitParams2->zR(newZR);
					validChange = traitParams2->MeetsParamCriteria(low, high);
					if(validChange){
						validChange = traitParams2->RJunction(high->xR_);}
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){
						break;}
					}
				}//end for tr
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for zR");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (paramsOnOff[tr][5]) && (zSet == paramsShared[tr][5])){//change it back too
						ClineShapeCWTlTrParamRec* traitParams3 = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams3->zR(oldZR);
						traitParams3->RJunction(rightEnd);//reset
						}
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWTlTrDParamRec::ChangeZR(int, Ran3&, double, ParamRec*, ParamRec*)


long double ClineMultiCWTlTrDParamRec::ClineDiseqLikelihood (ClineGeneticData* alloData){
	//sample-size based weighting removed
	long nSample;
	long double lnL=0.0;
	long double adjustedD = 0.0;
	aString blank; blank.AppendCharToLen(' ', alloData->alleleLen());
	aMatrix<long double>* ff = ZeroMatrix((long double)0,2,2);
	aMatrix<long double>* bigXX = ZeroMatrix((long double)0,2,2);
	aMatrix<long double>& f = *ff;
	aMatrix<long double>& bigX = *bigXX;
	aMatrix<long double>* bigNN = ZeroMatrix((long double)0,3,3);
	aMatrix<long double>& bigN = *bigNN;
	int adjByFis = true;//set to false here if pops are in HW proportions
    long double FisCenter = 0.0;
    if (adjByFis){//calculate Fis for the center pops only
		FisCenter = alloData->CenterPopFis();
		}
	for(int popNum=0;popNum<alloData->centerPopList_.len();++popNum){
		int pop = alloData->centerPopList_[popNum];
		long double x = alloData->Locality(pop);//this location
		aList<long double> coords1, coords2; coords1.Append(x);coords2.Append(x);
		for(int loc1=0;loc1<alloData->loci()-1;++loc1){
			coords1.Append(loc1);
			long double pAtCline1 = ExpectedValue(coords1);
			pAtCline1 = MAX(pAtCline1,(long double)10.0e-5);//eliminates 0 expected values in likelihood
			pAtCline1 = MIN(pAtCline1,(long double)(1.0-10e-5));
			StringList& alleleNamesList1 = alloData->allelesPresent[loc1];
			for(int loc2 = (loc1 + 1); loc2<alloData->loci();++loc2){
				coords2.Append(loc2);
				long double pAtCline2 = ExpectedValue(coords2);
				pAtCline2 = MAX(pAtCline2,(long double)10.0e-5);//eliminates 0 expected values in likelihood
				pAtCline2 = MIN(pAtCline2,(long double)(1.0-10e-5));
		       	adjustedD = (D_ / sqrt(pAtCline1 * (1.0 - pAtCline1) * pAtCline2 * (1.0 - pAtCline2))) * (1.0 + FisCenter);
				adjustedD = MIN(adjustedD,(long double)1.0);//at low expected cline freqs, adjustedD can exceed 1.0
				adjustedD = MAX(adjustedD,(long double)-1.0);//and this is impossible


				StringList& alleleNamesList2 = alloData->allelesPresent[loc2];
				for(int allele1=0;allele1<alleleNamesList1.len();++allele1){
					//get freq of allele1 (at loc1) in pop
					long double freq1 = alloData->GetFreq(pop, loc1, allele1);
					if ((freq1 > 0.0) && (freq1 < 1.0)){//otherwise D is undefined
						aString& alleleName1 = alleleNamesList1[allele1];
						for(int allele2=0;allele2<alleleNamesList2.len();++allele2){
							nSample = 0;//set sample size to zero, to count for blanks
							//get freq of allele2 (at loc2) in pop
							long double freq2 = alloData->GetFreq(pop, loc2, allele2);
							if ((freq2 > 0.0) && (freq2 < 1.0)){//otherwise D is undefined
								f[0][0] = freq1 * freq2 + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
       							f[0][1] = freq1 * (1.0 - freq2) + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
       							f[1][0] = (1.0 - freq1) * freq2 + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
       							f[1][1] = (1.0 - freq1) * (1.0 - freq2) + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
								//sometimes these values can be less than zero due to sampling error
								//implying that it is infinitely improbable that an observed value
								//could occur under the model.
								//therefore, values <=0 are set to a very small number, and will
								//give a very low finite likelihood
								for(int ind1=0;ind1<2;++ind1){
									for(int ind2=0;ind2<2;++ind2){
										if(f[ind1][ind2] <= 0.0){
											f[ind1][ind2] = 1.0e-10;}
										}}

								//get genotype strings for each individual and  read joint frequencies
								aString& alleleName2 = alleleNamesList2[allele2];
								bigN[0][0]=bigN[0][1]=bigN[0][2]=bigN[1][0]=bigN[1][1]=bigN[1][2]=bigN[2][0]=bigN[2][1]=bigN[2][2]=0.0;
 								int sizePop = alloData->individuals(pop);
								for(int i=0;i<sizePop;++i){
									StringList spot1 = alloData->Genotype(pop,i,loc1);
									//make sure it doesn't have any blanks
									int blankAt1 = false;
       								if(spot1.len() < 2){
       										blankAt1 = true;//happens sometimes at end of data line
											}//with missing loci
										else{
											if((spot1[0] == blank) || (spot1[1] == blank)){
												blankAt1 = true;}
											}
									if(!blankAt1){
										StringList spot2 = alloData->Genotype(pop,i,loc2);
										//make sure it doesn' t have any blanks
										int blankAt2 = false;
       									if(spot2.len() < 2){
       											blankAt2 = true;//happens sometimes at end of data line
												}//with missing loci
											else{
												if((spot2[0] == blank) || (spot2[1] == blank)){
													blankAt2 = true;}
												}
										if(!blankAt2){
											++nSample;
											//use this indiv in calculation
											//count instances of target alleles at each locus in this indiv
											int i1 = 2;
											for(int chrom=0;chrom<2;++chrom){
												aString alleleVal1(spot1[chrom]);
												if( alleleName1 == alleleVal1){
													--i1;}}
											int i2 = 2;
											for(int chrom=0;chrom<2;++chrom){
												aString alleleVal2(spot2[chrom]);
												if(alleleName2 == alleleVal2){
													--i2;}}
											bigN[i1][i2]++;
											}//end if not blankAt2
										}//end if not blankAt1
									}//end for i
								//calculate Hill's (1974) intermediate variables & iterative method
								bigX[0][0] = 2.0 * bigN[0][0] + bigN[0][1] + bigN[1][0];
								bigX[1][0] = 2.0 * bigN[2][0] + bigN[1][0] + bigN[2][1];
								bigX[0][1] = 2.0 * bigN[0][2] + bigN[0][1] + bigN[1][2];
								bigX[1][1] = 2.0 * bigN[2][2] + bigN[1][2] + bigN[2][1];
								for(int index1=0;index1<2;++index1){
									for(int index2=0;index2<2;++index2){
										if (f[index1][index2] > 0.0){
											lnL += bigX[index1][index2] * ln(f[index1][index2]);
											}}}//end for both indexes
								lnL += bigN[1][1] * ln(f[0][0] * f[1][1] + f[0][1] * f[1][0]);
								}//end if 0<freq2<1
							}//end for allele2
						}//end if 0<freq1<1
					}//end for allele1
				coords2.DeleteLast();
				}//end for loc2
			coords1.DeleteLast();
			}//end for loc1
		}//end for popNum
	delete ff;ff=0;
	delete bigXX;bigXX=0;
	delete bigNN; bigNN=0;
    lnLDiseq(lnL);
    return lnL;
	}//end function ClineMultiCWTlTrDParamRec::ClineDiseqLikelihood






long double ClineMultiCWTlTrDParamRec::D(void){
	return D_;
	}//end ClineMultiCWTlTrDParamRec::D( )


void ClineMultiCWTlTrDParamRec::D(long double dd){
	D_ = dd;
	}//end ClineMultiCWTlTrDParamRec::D( )



long double ClineMultiCWTlTrDParamRec::DispersalSigma(void){
	long double aveSigma=0.0;
	for(int tr=0;tr<traits();++tr){
		aveSigma += DispersalSigma(tr);}
	aveSigma /= (long double)traits();
	return aveSigma;
	}//end ClineMultiCWTlTrDParamRec::DispersalSigma()



long double ClineMultiCWTlTrDParamRec::DispersalSigma(int trait){
	ClineShapeCWTlTrParamRec *traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	long double w = traitParams->width();
    if (D_ > 0.0){
		long double sigma = sqrt((D_ * r_ * w * w) / (1.0 + r_));
		return sigma;}
	return (long double)0.0;//default
	}//end ClineMultiCWTlTrDParamRec::DispersalSigma()



long double ClineMultiCWTlTrDParamRec::distanceBarrierL(void){
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	long double aveBarrier=0.0; long double leftSideCount=0.0;
	for(int tr=0;tr<traits();++tr){
		if(paramsOnOff[tr][2]){//left side estimated
			aveBarrier += distanceBarrierL(tr);
			leftSideCount++;}
		}
	if(leftSideCount>0){aveBarrier /= leftSideCount;}
	return aveBarrier;
	}//end ClineMultiCWTlTrDParamRec::distanceBarrierL()



long double ClineMultiCWTlTrDParamRec::distanceBarrierL(int trait){//barrier to introgression in distance units
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	if(paramsOnOff[trait][2]==0){return 0.0;}
    //derivatives found using Mathematica
	ClineShapeCWTlTrParamRec *traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	long double w = traitParams->width();
    if (w ==0.0){
			return exp((long double)100.0);}//huge and undefined
	double th = traitParams->thetaL();
	double XL = traitParams->xL();
	double c = traitParams->center();
	double ZL = traitParams->zL();
    long double derivFactor = (long double)4.0 * sqrt(th) * (XL - (c + ZL)) / w;
    if (derivFactor <= (long double)-100.0){
			return exp((long double)100.0);}//huge and undefined
    long double dPLdXatxL, beta=(long double)0.0;
    if (derivFactor <= (long double)100.0){
			dPLdXatxL = exp(derivFactor) * sqrt(th) / w;}
    	else{
			dPLdXatxL = exp((long double)100.0);}

    if(dPLdXatxL == (long double)0.0){
			return exp((long double)100.0);}
		else{
    		long double dP = traitParams->DeltaP(0);//derivatives found using Mathematica
			beta = dP / dPLdXatxL;}
	return beta;
	}//end ClineMultiCWTlTrDParamRec::distanceBarrierL()
 


long double ClineMultiCWTlTrDParamRec::distanceBarrierR(void){
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	long double aveBarrier=0.0; long double rightSideCount=0.0;
	for(int tr=0;tr<traits();++tr){
		if(paramsOnOff[tr][3]){//right side estimated
			aveBarrier += distanceBarrierR(tr);
			rightSideCount++;}
		}
	if(rightSideCount>0){aveBarrier /= rightSideCount;}
	return aveBarrier;
	}//end ClineMultiCWTlTrDParamRec::distanceBarrierR()



long double ClineMultiCWTlTrDParamRec::distanceBarrierR(int trait){//barrier to introgression in distance units
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	if(paramsOnOff[trait][3]==0){return 0.0;}
    //derivatives found using Mathematica
	ClineShapeCWTlTrParamRec *traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	long double w = traitParams->width();
    if (w ==(long double)0.0){
			return exp((long double)100.0);}//huge and undefined
	long double th = traitParams->thetaR();
	long double XR = traitParams->xR();
	long double c = traitParams->center();
	long double ZR = traitParams->zR();
    long double derivFactor = (long double)4.0 * sqrt(th) * (XR - (c + ZR)) / w;
    if (derivFactor <= (long double)-100.0){
			return exp((long double)100.0);}//huge and undefined
    long double dPRdXatxR, beta=(long double)0.0;
    if (derivFactor <= (long double)100.0){
			dPRdXatxR = exp(derivFactor) * sqrt(th) / w;}
    	else{
			dPRdXatxR = exp((long double)100.0);}

    if(dPRdXatxR == (long double)0.0){
			return exp((long double)100.0);}
		else{
    		long double dP = traitParams->DeltaP(0);//derivatives found using Mathematica
			beta = dP / dPRdXatxR;}
	return beta;
	}//end ClineMultiCWTlTrDParamRec::distanceBarrierR()
 



void ClineMultiCWTlTrDParamRec::Initialize(ClineFitInitializer& i){
	lnL_=0.0;
	lnLmaxMinusLnL_=0.0;
	lnLDiseq_=0.0;
	paramsOnOff_ = i.paramsOnOff_;//copies pointer
	paramsShared_ = i.paramIdentities_;//copies pointer
	varyingParams_=0;//in case of reinitialization
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	traitParamSet_.Clear();//in case of reinitialization
	for(int tr=0;tr<i.traits();++tr){//account for on/off parameters here
		traitParamSet_.Append(NEW ClineShapeCWTlTrParamRec(i,tr));
		ClineShapeCWTlTrParamRec *traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		if(paramsOnOff[tr][2]==0 || paramsOnOff[tr][4]==0){//thetaL or zL off
			paramsOnOff[tr][2] = paramsOnOff[tr][4]=0;//both off if either is off
			traitParams->thetaL(1.0);
			traitParams->zL(0.0);
			traitParams->xL_=i.leftEnd_;
			}
		if(paramsOnOff[tr][3]==0 || paramsOnOff[tr][5]==0){//thetaR or zL off
			paramsOnOff[tr][3] = paramsOnOff[tr][5]=0;//both off if either is off
			traitParams->thetaR(1.0);
			traitParams->zR(0.0);
			traitParams->xR_=i.rightEnd_;
			}
		}
	D_ = i.D_;
	//count varying params
	for(int p=0;p<BaseParams();++p){//assumes paramsShared is sorted low to high
		int maxparamID =-1;
		for(int tr=0;tr<traits();++tr){
			if((paramsOnOff[tr][p]==1) && (maxparamID<paramsShared[tr][p])){
				varyingParams_++;
				maxparamID = paramsShared[tr][p];
				}//if
			}//tr
		}//p
	}//end ClineMultiCWTlTrDParamRec::Initialize(ClineFitInitializer&)



long double ClineMultiCWTlTrDParamRec::lnLDiseq(void){
		return lnLDiseq_;}

		
void ClineMultiCWTlTrDParamRec::lnLDiseq(long double lnLval){
		lnLDiseq_ = lnLval;}
		
		

int ClineMultiCWTlTrDParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//use type conversion that circumvents usual virtual base class restrictions
	ClineMultiCWTlTrDParamRec* lower = (ClineMultiCWTlTrDParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTlTrDParamRec* upper = (ClineMultiCWTlTrDParamRec*) upperLimits->GetPointerToDerivedSelf();
	if((D_ < lower->D_) || (D_ > upper->D_)){ 
		return false;}
	for(int tr=0;tr<traits();++tr){
		ClineShapeCWTlTrParamRec* traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		ParamRec& lowParams = lower->traitParamSet_[tr];//no need to cast these yet
		ParamRec& highParams = upper->traitParamSet_[tr];
		int paramsValid = traitParams->MeetsParamCriteria(&lowParams,&highParams);
		if(! paramsValid) return false;
		}//tr
	return true;
	}//end ClineMultiCWTlTrDParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)


void ClineMultiCWTlTrDParamRec::MultiplyParam(int paramNum, long double factor){
	//use to rescale a param deviation
	//typecasting trick, so don't use function from base class
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int trait=0, param=0, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	if(param==6){//change D only
		D_ *= factor;
		return;}//else...
	int covaryingID = paramsShared[trait][param];
	for(int tr=trait;tr<traitParamSet_.len();++tr){//assumes paramsShared_ list is sorted
		if(covaryingID == paramsShared[tr][param]){//change it too
			ClineShapeCWTlTrParamRec* traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
			traitParams->MultiplyParam(param,factor);
			}
		}//tr
	}//ClineMultiCWTlTrDParamRec::MultiplyParam(int, long double)



int ClineMultiCWTlTrDParamRec::NumDerivedParams(void){
	if(derivedParams_ > (-1)){//default flag set by constructor
		return derivedParams_;}
	//otherwise calculate it here first
	derivedParams_=0;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	int D_on = false, leftTail=0, rightTail=0;
	if(paramsOnOff[0][6]==1) D_on = true;
	if(D_on){
		derivedParams_++;}//DisersalSigma() estimated
	for(int tr=0;tr<traits();++tr){
		int both=0;
		if(paramsOnOff[tr][2]==1){//thetaL && zL turned on
			derivedParams_+=3;//xL_(tr), P_xL(tr) and distanceBarrierL(tr)
			both++;leftTail++;
			if(D_on) derivedParams_+=2;//timeBarrierL(tr),s_perMarkerLocusL(tr)
			}
		if(paramsOnOff[tr][3]==1){//thetaR && zR turned on
			derivedParams_+=3;//xR_(tr), P_xR(tr) and distanceBarrierR(tr)
			both++;rightTail++;
			if(D_on) derivedParams_+=2;//timeBarrierR(tr), s_perMarkerLocusR(tr)
			}
		if(both>0){//either is turned on
			derivedParams_++;//deltaP
			}
		if(D_on){
			derivedParams_++;}//DisersalSigma(tr) estimated
		}//tr
	if(leftTail>0) derivedParams_++;//distanceBarrierL()
	if(rightTail>0) derivedParams_++;//distanceBarrierR()
	if(D_on){
		if(leftTail){derivedParams_++;}//timeBarrierL()
		if(rightTail){derivedParams_++;}//timeBarrierR()
		derivedParams_+=7;
		//WhybridOverWpure(),S(),s_star_hets(), s_star_ecotone(), 
		//NumSelectedLoci(),s_perReproIsolationLocus,
		//s_star_DomEcotone()
		}
	return derivedParams_;
	}//end ClineMultiCWTlTrDParamRec::NumDerivedParams()



long double ClineMultiCWTlTrDParamRec::NumSelectedLoci(void){
	double s = s_star_vs_Hets();
    if (s == (long double)0.0){
		return (long double)0.0;}//no loci under selection if no selection
	long double ww = WhybridOverWpure();
    if ((s >= (long double)1.0) || (ww <= (long double)0.0)){
		return (long double)1.0e20;}//a really high number if fitness=0
    double nLocSel = (long double)2.0 * ln(ww) / ln((long double)1.0 - s);
    return nLocSel;
    }//end ClineMultiCWTlTrDParamRec::NumSelectedLoci()



void ClineMultiCWTlTrDParamRec::ParamCoords(int globalParamNum, int& trait, int& traitParam, int& derived){
	//returns values for trait && traitParam
	//if params covary among traits, this returns the first trait
	derived = false;//to start
	int count=0;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	for(int p=0;p<BaseParams();++p){//try base params first
		int maxSharedID=-1;
		for(int tr=0;tr<traits();++tr){
			if(paramsOnOff[tr][p]==1){//estimating this one
				if(maxSharedID<paramsShared[tr][p]){//this is a new param to estimate
					if(globalParamNum==count){
						trait=tr; traitParam=p;
						return;}
					++count;
					maxSharedID = paramsShared[tr][p];//assumes these are in ascending order
				}}
			}//tr
		}//p
	//if we're here, it must be a derived parameter
	int dp = globalParamNum - NumParams();
	derived = true;
	int D_on = false, leftTail=0, rightTail=0;
	if(paramsOnOff[0][6]==1) D_on = true;
	int dcount=0;
	for(int tr=0;tr<traits();++tr){
		if(paramsOnOff[tr][2]){
			leftTail++;
			if(dp==dcount){//xL estimated
				trait=tr; traitParam=0;return;}
			dcount++;}
		if(paramsOnOff[tr][2]){
			if(dp==dcount){//p[xL] estimated
				trait=tr; traitParam=1;return;}
			dcount++;}
		if(paramsOnOff[tr][2]){
			if(dp==dcount){//distanceBarrierL estimated
				trait=tr; traitParam=2;return;}
			dcount++;}
		if(paramsOnOff[tr][3]){
			rightTail++;
			if(dp==dcount){//xR estimated
				trait=tr; traitParam=3;return;}
			dcount++;}
		if(paramsOnOff[tr][3]){
			if(dp==dcount){//p[xR] estimated
				trait=tr; traitParam=4;return;}
			dcount++;}
		if(paramsOnOff[tr][3]){
			if(dp==dcount){//distanceBarrierR estimated
				trait=tr; traitParam=5;return;}
			dcount++;}
		if(D_on && paramsOnOff[tr][2]){
			if(dp==dcount){//timeBarrierL estimated
				trait=tr; traitParam=6;return;}
			dcount++;}
		if(D_on && paramsOnOff[tr][2]){
			if(dp==dcount){//s_perMarkerLocusL estimated
				trait=tr; traitParam=7;return;}
			dcount++;}
		if(D_on && paramsOnOff[tr][3]){
			if(dp==dcount){//timeBarrierR estimated
				trait=tr; traitParam=8;return;}
			dcount++;}
		if(D_on && paramsOnOff[tr][3]){
			if(dp==dcount){//s_perMarkerLocusR estimated
				trait=tr; traitParam=9;return;}
			dcount++;}
		if(paramsOnOff[tr][2] || paramsOnOff[tr][3]){
			if(dp==dcount){//deltaP estimated
				trait=tr; traitParam=10;return;}
			dcount++;}
		if(D_on && paramsOnOff[tr][3]){
			if(dp==dcount){//dispersalSigma(tr) estimated
				trait=tr; traitParam=11;return;}
			dcount++;}
		}//tr
	//getting here implies that the parameter must be shared among traits
	int dpshared = dp - dcount;	
	dcount=0;
	trait=-1;//flag for this situation
	if(D_on){
		if(dpshared==dcount){//ave. DispersalSigma estimated
			traitParam=0;return;}
		dcount++;}
	if(leftTail){
		if(dpshared==dcount){//ave. distanceBarrierL estimated
			traitParam=1;return;}
		dcount++;}
	if(D_on && leftTail){
		if(dpshared==dcount){//ave. timeBarrierL estimated
			traitParam=2;return;}
		dcount++;}
	if(rightTail){
		if(dpshared==dcount){//ave. distanceBarrierR estimated
			traitParam=3;return;}
		dcount++;}
	if(D_on && rightTail){
		if(dpshared==dcount){//ave. timeBarrierR estimated
			traitParam=4;return;}
		dcount++;}
	if(D_on){
		if(dpshared==dcount){//S estimated
			traitParam=5;return;}
		dcount++;
		if(dpshared==dcount){//Wh/Wp estimated
			traitParam=6;return;}
		dcount++;
		if(dpshared==dcount){//s*[hets] estimated
			traitParam=7;return;}
		dcount++;
		if(dpshared==dcount){//s*[ecotone] estimated
			traitParam=8;return;}
		dcount++;
		if(dpshared==dcount){//s[reproIsolationLocus] estimated
			traitParam=9;return;}
		dcount++;
		if(dpshared==dcount){//n[selLoci] estimated
			traitParam=10;return;}
		dcount++;
		if(dpshared==dcount){//s*[domEcotone] estimated
			traitParam=11;return;}
		}//D_on
	//exhausted possibilities, so return an error message
	derived=false;
	aString err("Can\'t find param");err.cat(globalParamNum+1);
	aError(err);
	}//end ClineMultiCWTlTrDParamRec::ParamCoords(int, int&, int&, int&)




aString* ClineMultiCWTlTrDParamRec::ParamName(int paramNum){
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	int trait=0, param=0, origparam = paramNum, allShared=true, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	aString trStr,  *newName = NEW aString();
	if(! derived){//it's a primary parameter
			if(traits()>1){
				ObjList<aList<int> >& paramsShared = *paramsShared_;
				int shareState = paramsShared[trait][param];
				trStr="[";trStr.cat(trait+1);
				for(int tr=0;tr<traits();++tr){//append names of any shared traits
					if(tr != trait && paramsOnOff[tr][param]){
						if(paramsShared[tr][param]== shareState){
								trStr.cat(';'); trStr.cat(tr+1);}
							else{allShared=false;}
							}}
				trStr.cat("]");}
			switch (param){
			 case 0:
				newName->cat("center"); break;
			 case 1:
				newName->cat("width"); break;
			 case 2:
				newName->cat("thetaL"); break;
			 case 3:
				newName->cat("thetaR"); break;
			 case 4:
				newName->cat("zL"); break;
			 case 5:
				newName->cat("zR"); break;
			 case 6:
				newName->cat("D"); return newName; break;
			 default:
				aString errstr("bug: can't return name of param #");
				errstr.cat(param+1);
				if(traits()>1){
					errstr.cat(" for trait #");errstr.cat(trait+1);}
				aError(errstr);
			 	return newName;//null
			 	}	
			if(traits()>1 && !allShared){newName->cat(trStr);}
			}//if paramNum
		else{//it's a derived parameter
			if(trait>-1){//it is specific to a trait
					trStr.cat('[');trStr.cat(trait+1);trStr.cat(']');
					switch (param){
						 case 0:
							newName->cat("xL"); break;
						 case 1:
							newName->cat("p[xL]"); break;
						 case 2:
							newName->cat("distBarrierL"); break;
						 case 3:
							newName->cat("xR"); break;
						 case 4:
							newName->cat("p[xR]"); break;
						 case 5:
							newName->cat("distBarrierR"); break;
						 case 6:
							newName->cat("timeBarrierL"); break;
						 case 7:
							newName->cat("s_perMarkerLocusL"); break;
						 case 8:
							newName->cat("timeBarrierR"); break;
						 case 9:
							newName->cat("s_perMarkerLocusR"); break;
						 case 10:
							newName->cat("DeltaP"); break;
						 case 11:
							newName->cat("DispersalSigma"); break;
						 default:
						 	aString errstr("bug: can't return name of param #");
							errstr.cat(param+1);
							if(traits()>1){
								errstr.cat(" for trait #");errstr.cat(trait+1);}
							aError(errstr);
						 	return newName;
						 	break;}
					if(traits()>1){newName->cat(trStr);}
					return newName;
					}
				else{//parameter is commonly estimated
					switch (param){
						 case 0:
							newName->cat("DispersalSigma"); break;
						 case 1:
							newName->cat("distBarrierL"); break;
						 case 2:
							newName->cat("timeBarrierL"); break;
						 case 3:
							newName->cat("distBarrierR"); break;
						 case 4:
							newName->cat("timeBarrierR"); break;
						 case 5:
							newName->cat("S"); break;
						 case 6:
							newName->cat("Wh/Wp"); break;
						 case 7:
							newName->cat("s*[hets]"); break;
						 case 8:
							newName->cat("s*[ecotone]"); break;
						 case 9:
							newName->cat("s[reproIsolationLocus]"); break;
						 case 10:
							newName->cat("n[selectedLoci]"); break;
						 case 11:
							newName->cat("s*[domEcotone]"); break;
						 default:
						 	aString errstr("bug: can't return name of co-estimated derived param #");errstr.cat(param+1);
							aError(errstr);
						 	break;}
					}//else..
			}//derived
	return newName;			
	}//end ClineMultiCWTlTrDParamRec::ParamName(int )



void ClineMultiCWTlTrDParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ParamRec::ParamString(dataline, sepchar);
	long double val=0.0;
	//each param will be printed, whether or not it is co-estimated with another
	int D_on = paramsOnOff[0][6];
	if(D_on){
		val = lnLDiseq();
		dataline.cat(sepchar);dataline.cat(val,6,15);
		val = D();
		dataline.cat(sepchar);dataline.cat(val,6,15);}
	for(int tr=0;tr<traits();++tr){
		ClineShapeCWTlTrParamRec* traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		if(paramsOnOff[tr][0]==1){//center
			val = traitParams->center();
			dataline.cat(sepchar);dataline.cat(val,6,15);
			}
		if(paramsOnOff[tr][1]==1){//width
			val = traitParams->width();
			dataline.cat(sepchar);dataline.cat(val,6,15);
			}
		if(paramsOnOff[tr][2]==1){//thetaL
			val = traitParams->thetaL();
			dataline.cat(sepchar);dataline.cat(val,6,15);
			}
		if(paramsOnOff[tr][3]==1){//thetaR
			val = traitParams->thetaR();
			dataline.cat(sepchar);dataline.cat(val,6,15);
			}
		if(paramsOnOff[tr][4]==1){//zL
			val = traitParams->zL();
			dataline.cat(sepchar);dataline.cat(val,6,15);
			}
		if(paramsOnOff[tr][5]==1){//zR
			val = traitParams->zR();
			dataline.cat(sepchar);dataline.cat(val,6,15);
			}
		if(paramsOnOff[tr][2]==1){//xL
			val = traitParams->xL();
			dataline.cat(sepchar);dataline.cat(val,6,15);
			}
		if(paramsOnOff[tr][3]==1){//xR
			val = traitParams->xR();
			dataline.cat(sepchar);dataline.cat(val,6,15);
			}
		if(paramsOnOff[tr][2]==1){//left side estimated
			val = traitParams->B_L(tr);
			dataline.cat(sepchar);dataline.cat(val,6,15);
			}
		if(paramsOnOff[tr][3]==1){//right side estimated
			val = traitParams->B_R(tr);
			dataline.cat(sepchar);dataline.cat(val,6,15);
			}
		if(paramsOnOff[tr][2]==1 || paramsOnOff[tr][3]==1){//either side estimated
			val = traitParams->DeltaP(tr);
			dataline.cat(sepchar);dataline.cat(val,6,15);
			}
		if(D_on){
			val = DispersalSigma(tr);
			dataline.cat(sepchar);dataline.cat(val,6,15);//estimated for this trait
			}
		if(paramsOnOff[tr][2]==1 ){//left side estimated
			val = distanceBarrierL(tr);
			dataline.cat(sepchar);dataline.cat(val,6,15);
			if(D_on){
				val = timeBarrierL(tr);
				dataline.cat(sepchar);dataline.cat(val,6,15);
				val = s_perMarkerLocusL(tr);
				dataline.cat(sepchar);dataline.cat(val,6,15);
				}
			}//left side
		if(paramsOnOff[tr][3]==1 ){//right side estimated
			val = distanceBarrierR(tr);
			dataline.cat(sepchar);dataline.cat(val,6,15);
			if(D_on){
				val = timeBarrierR(tr);
				dataline.cat(sepchar);dataline.cat(val,6,15);
				val = s_perMarkerLocusR(tr);
				dataline.cat(sepchar);dataline.cat(val,6,15);
				}
			}//right side
		}//tr
	//parameters that average over traits
	if(D_on ){
		val = DispersalSigma();
		dataline.cat(sepchar);dataline.cat(val,6,15);
		val = distanceBarrierL();
		dataline.cat(sepchar);dataline.cat(val,6,15);
		val = timeBarrierL();
		dataline.cat(sepchar);dataline.cat(val,6,15);
		val = distanceBarrierR();
		dataline.cat(sepchar);dataline.cat(val,6,15);
		val = timeBarrierR();
		dataline.cat(sepchar);dataline.cat(val,6,15);
		val = S();
		dataline.cat(sepchar);dataline.cat(val,6,15);
		val = WhybridOverWpure();
		dataline.cat(sepchar);dataline.cat(val,6,15);
		val = s_star_vs_Hets();
		dataline.cat(sepchar);dataline.cat(val,6,15);
		val = s_star_Ecotone();
		dataline.cat(sepchar);dataline.cat(val,6,15);
		val = s_perReproIsolationLocus();
		dataline.cat(sepchar);dataline.cat(val,6,15);
		val = NumSelectedLoci();
		dataline.cat(sepchar);dataline.cat(val,6,15);
		val = s_star_DomEcotone();
		dataline.cat(sepchar);dataline.cat(val,6,15);
		}//D_on

	}//end ClineMultiCWTlTrDParamRec::ParamString(aString&, char)



long double ClineMultiCWTlTrDParamRec::ParamValue(int paramNum){
	//typecasting trick, so don't use function from base class
	//returns value of parameter; first varying params, then derived params
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	int trait=0, param=0, origparam = paramNum, allShared=false, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	if(! derived){//it's a primary parameter
			ClineShapeCWTlTrParamRec* traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
			switch (param){
			 case 0:
				return traitParams->center(); break;
			 case 1:
				return traitParams->width(); break;
			 case 2:
				return traitParams->thetaL(); break;
			 case 3:
				return traitParams->thetaR(); break;
			 case 4:
				return traitParams->zL(); break;
			 case 5:
				return traitParams->zR(); break;
			 case 6:
				return D(); break;
			 default:
				aString errstr("bug: can't return value of param #");
				errstr.cat(param+1);
				if(traits()>1){
					errstr.cat(" for trait #");errstr.cat(trait+1);}
				aError(errstr);
			 	return 0.0; break;
			 	}	
			}//if paramNum
		else{//it's a derived parameter
			if(trait>-1){//it is specific to a trait
					ClineShapeCWTlTrParamRec* traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
					switch (param){
						 case 0:
							return traitParams->xL(); break;
						 case 1:
							return traitParams->B_L(); break;
						 case 2:
							return distanceBarrierL(trait); break;
						 case 3:
							return traitParams->xR(); break;
						 case 4:
							return traitParams->B_R(); break;
						 case 5:
							return distanceBarrierR(trait); break;
						 case 6:
							return timeBarrierL(trait); break;
						 case 7:
							return s_perMarkerLocusL(trait); break;
						 case 8:
							return timeBarrierR(trait); break;
						 case 9:
							return s_perMarkerLocusR(trait); break;
						 case 10:
							return traitParams->DeltaP(); break;
						 case 11:
							return DispersalSigma(trait); break;
						 default:
						 	aString errstr("bug: can't return value of derived param #");
							errstr.cat(param+1);
							if(traits()>1){
								errstr.cat(" for trait #");errstr.cat(trait+1);}
							aError(errstr);
						 	break;}
					}
				else{//parameter is commonly estimated
					switch (param){
						 case 0:
							return DispersalSigma(); break;
						 case 1:
							return distanceBarrierL(); break;
						 case 2:
							return timeBarrierL(); break;
						 case 3:
							return distanceBarrierR(); break;
						 case 4:
							return timeBarrierR(); break;
						 case 5:
							return S(); break;
						 case 6:
							return WhybridOverWpure(); break;
						 case 7:
							return s_star_vs_Hets(); break;
						 case 8:
							return s_star_Ecotone(); break;
						 case 9:
							return s_perReproIsolationLocus(); break;
						 case 10:
							return NumSelectedLoci(); break;
						 case 11:
							return s_star_DomEcotone(); break;
						 default:
						 	aString errstr("bug: can't return value of co-estimated derived param #");errstr.cat(param+1);
							aError(errstr);
							return 0.0;
						 	break;}
					}//else..
			}//derived
	return 0.0;

	}//end ClineMultiCWTlTrDParamRec::ParamValue(int )



void ClineMultiCWTlTrDParamRec::Print(ostream& outFile, long rep, aString& message){
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ParamRec::Print(outFile, rep, message);
	//for now, each param will be printed, whether or not it is co-estimated with another
	aString outStr;
	long double val=(long double)0.0;
	for(int tr=0;tr<traits();++tr){
		outStr.clear();
		ClineShapeCWTlTrParamRec* traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		if(paramsOnOff[tr][0]==1){//center
			outStr.cat("center");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->center();
			outStr.cat(val,6,10);
			outStr.separator(5);
			}
		if(paramsOnOff[tr][1]==1){//width
			outStr.cat("width");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->width();
			outStr.cat(val,6,9);
			}
		outFile<<outStr<<endl;
		outStr.clear();
		if(paramsOnOff[tr][2]==1){//thetaL & zL toggle on/off together
			outStr="thetaL";
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->thetaL();
			outStr.cat(val,6,15);
			outStr.separator(2);
			outStr.cat("zL");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->zL();
			outStr.cat(val,6,15);
			outStr.separator(2);
			outStr.cat("xL");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->xL();
			outStr.cat(val,6,15);
			outFile<<outStr<<endl;
			}
		outStr.clear();
		if(paramsOnOff[tr][3]==1){//thetaR & zR toggle on/off together
			outStr.cat("thetaR");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->thetaR();
			outStr.cat(val,6,15);
			outStr.separator(2);
			outStr.cat("zR");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->zR();
			outStr.cat(val,6,15);
			outStr.separator(2);
			outStr.cat("xR");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->xR();
			outStr.cat(val,6,15);
			outFile<<outStr<<endl;
			}
		outStr.clear();
		if(paramsOnOff[tr][2]==1){//left tail estimated
			outStr.cat("p[xL]");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->B_L(0);
			outStr.cat(val,6,15);
			}
		if(paramsOnOff[tr][3]==1){//right tail estimated
			outStr.separator(2);
			outStr.cat("p[xR]");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->B_R(0);
			outStr.cat(val,6,15);
			}
		if(paramsOnOff[tr][2]==1 || paramsOnOff[tr][3]==1){//either tail estimated
			outStr.separator(2);
			outStr.cat("deltaP");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->DeltaP(0);
			outStr.cat(val,6,4);
			outFile<<outStr<<endl;
			}
		}//tr
	outStr="lnLdiseq: ";outStr.cat(lnLDiseq(),6,15);
	outStr.separator(4);outStr.cat("D: ");outStr.cat(D(),6,15);
	outFile<<outStr<<endl;
	outStr="dispersal sigma: ";outStr.cat(DispersalSigma(),6,15);
	outStr.separator(2);outStr.cat("S: ");outStr.cat(S(),6,9);
	outStr.separator(2);outStr.cat("Wh/Wp: ");outStr.cat(WhybridOverWpure(),6,9);
	outFile<<outStr<<endl;
	outStr="s*[hets]: ";outStr.cat(s_star_vs_Hets(),6,4);
	outStr.separator(2);outStr.cat("s*[ecotone]: ");outStr.cat(s_star_Ecotone(),6,4);
	outFile<<outStr<<endl;
	outStr="s*[domEcotone]: ";outStr.cat(s_star_DomEcotone(),6,4);
	outStr.separator(2);outStr.cat("s[repro_Iso_Locus]: ");outStr.cat(s_perReproIsolationLocus(),6,4);
	outFile<<outStr<<endl;
	outStr="n[selected loci]: ";outStr.cat(NumSelectedLoci(),6,4);
	outStr.separator(2);outStr.cat("s[repro_Iso_Locus]: ");outStr.cat(s_perReproIsolationLocus(),6,4);
	outFile<<outStr<<endl;
	}//ClineMultiCWTlTrDParamRec::Print



long double ClineMultiCWTlTrDParamRec::r(void){
	return r_;}


void ClineMultiCWTlTrDParamRec::r(long double rr){
	r_ = rr;}
	

long double ClineMultiCWTlTrDParamRec::S(void){
    if (WhybridOverWpure() == (long double)0.0){
		return (long double)200.0;}//a high number
    long double bigS = (long double)-2.0 * ln(WhybridOverWpure());
    return bigS;
    }//end ClineMultiCWTlTrDParamRec::S()



void ClineMultiCWTlTrDParamRec::SetParamValue(int paramNum, long double newVal){
	//set value of parameter, and all those that covary with it
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int trait=0, param=0, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	if(param==6){
		D_ = newVal;
		return;}
	int covaryingID = paramsShared[trait][param];
	for(int tr=trait;tr<traitParamSet_.len();++tr){//assumes paramsShared_ list is sorted
		if(covaryingID == paramsShared[tr][param]){//change it too
			ClineShapeCWTlTrParamRec* traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
			traitParams->SetParamValue(param,newVal);
			}
		}//tr
	}//end ClineMultiCWTlTrDParamRec::SetParamValue(int,long double)



long double ClineMultiCWTlTrDParamRec::s_perMarkerLocusL(int trait){
	//direct selection on marker loci in left tail of cline
	//assuming genetic background incompatibilities
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	if(paramsOnOff[trait][2]==0){return (long double) 0.0;}
	ClineShapeCWTlTrParamRec* traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	if(traitParams->width()==0.0){return (long double)0.0;}//undefined
    long double s = traitParams->thetaL() * s_star_vs_Hets();
    return s;
    }//end ClineMultiCWTlTrDParamRec::s_perMarkerLocusL(int)


long double ClineMultiCWTlTrDParamRec::s_perMarkerLocusR(int trait){
	//direct selection on marker loci in right tail of cline
	//assuming genetic background incompatibilities
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	if(paramsOnOff[trait][3]==0){return (long double)0.0;}
	ClineShapeCWTlTrParamRec* traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	if(traitParams->width()==0.0){return (long double)0.0;}//undefined
    long double s = traitParams->thetaR() * s_star_vs_Hets();
    return s;
    }//end ClineMultiCWTlTrDParamRec::s_perMarkerLocusR(int)


long double ClineMultiCWTlTrDParamRec::s_perReproIsolationLocus(void){
	//average selection on a locus contributing to reproductive isolation
	long double n = NumSelectedLoci();
	 if (n == (long double)0.0){
			return (long double)0.0;}
    return S() / n;
    }//end ClineMultiCWTlTrDParamRec::s_perReproIsolationLocus(void)




long double ClineMultiCWTlTrDParamRec::s_star_vs_Hets(void){
	//indirect & direct selection on marker loci in middle of cline
	//assuming genetic background incompatibilities
	long double w = aveWidth();
	if(w==(long double)0.0){return (long double)0.0;}//undefined
    long double s = sqr((long double)2.0 * DispersalSigma() / w);
    return s;
    }//end ClineMultiCWTlTrDParamRec::s_star_vs_Hets()


long double ClineMultiCWTlTrDParamRec::s_star_Ecotone(void){
	//indirect & direct selection on marker loci in middle of cline
	//assuming selection across an ecotone
	long double w = aveWidth();
	if(w==(long double)0.0){return (long double)0.0;}//undefined
    long double s = sqr((long double)1.732 * DispersalSigma() / w);
    return s;
    }//end ClineMultiCWTlTrDParamRec::s_star_Ecotone()


long double ClineMultiCWTlTrDParamRec::s_star_DomEcotone(void){
	//indirect & direct selection on marker loci in middle of cline
	//assuming selection across an ecotone on a dominant marker
	long double w = aveWidth();
	if(w==(long double)0.0){return (long double)0.0;}//undefined
    long double s = sqr((long double)1.782 * DispersalSigma() / w);
    return s;
    }//end ClineMultiCWTlTrDParamRec::s_star_DomEcotone()




long double ClineMultiCWTlTrDParamRec::timeBarrierL(void){
	long double d = DispersalSigma();
    if (d == (long double)0.0){
		return (long double)1.0e20;}//basically forever if no dispersal
	long double b = distanceBarrierL();
    if (b / d > (long double)1.0e40){
		return (long double)1.0e20;}//basically forever if beta/sigma too high
    long double timeL = sqr(b / d);
    return timeL;
}//end ClineMultiCWTlTrDParamRec::timeBarrierL()


long double ClineMultiCWTlTrDParamRec::timeBarrierL(int trait){
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	if(paramsOnOff[trait][2]==0){return (long double)0.0;}
	long double d = DispersalSigma(trait);
    if (d == (long double)0.0){
		return (long double)1.0e20;}//basically forever if no dispersal
	long double b = distanceBarrierL(trait);
    if (b / d > (long double)1.0e40){
		return (long double)1.0e20;}//basically forever if beta/sigma too high
    long double timeL = sqr(b / d);
    return timeL;
}//end ClineMultiCWTlTrDParamRec::timeBarrierL(int)


long double ClineMultiCWTlTrDParamRec::timeBarrierR(void){
	long double d = DispersalSigma();
    if (d == (long double)0.0){
		return (long double)1.0e20;}//basically forever if no dispersal
	long double b = distanceBarrierR();
    if (b / d > (long double)1.0e40){
		return (long double)1.0e20;}//basically forever if beta/sigma too high
    long double timeR = sqr(b / d);
    return timeR;
}//end ClineMultiCWTlTrDParamRec::timeBarrierR()


long double ClineMultiCWTlTrDParamRec::timeBarrierR(int trait){
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	if(paramsOnOff[trait][3]==0){return (long double)0.0;}
	long double d = DispersalSigma(trait);
    if (d == (long double)0.0){
		return (long double)1.0e20;}//basically forever if no dispersal
	long double b = distanceBarrierR(trait);
    if (b / d > (long double)1.0e40){
		return (long double)1.0e20;}//basically forever if beta/sigma too high
    long double timeR = sqr(b / d);
    return timeR;
}//end ClineMultiCWTlTrDParamRec::timeBarrierR(int)


void ClineMultiCWTlTrDParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the names of values that will be printed to a big file
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ParamRec::TitleString(dataline, sepchar);
	//each param will be printed, whether or not it is co-estimated with another
	int D_on = paramsOnOff[0][6];
	if(D_on){
		dataline.cat(sepchar);
		dataline.cat("lnLDiseq");
		dataline.cat(sepchar);
		dataline.cat("D");}
	for(int tr=0;tr<traits();++tr){
		aString trStr('[');trStr.cat(tr+1);trStr.cat(']');
		if(paramsOnOff[tr][0]==1){//center
			dataline.cat(sepchar);dataline.cat("center");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][1]==1){//width
			dataline.cat(sepchar);dataline.cat("width");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][2]==1){//thetaL
			dataline.cat(sepchar);dataline.cat("thetaL");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][3]==1){//thetaR
			dataline.cat(sepchar);dataline.cat("thetaR");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][4]==1){//zL
			dataline.cat(sepchar);dataline.cat("zL");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][5]==1){//zR
			dataline.cat(sepchar);dataline.cat("zR");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][2]==1){//xL
			dataline.cat(sepchar);dataline.cat("xL");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][3]==1){//xR
			dataline.cat(sepchar);dataline.cat("xR");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][2]==1){//left side estimated
			dataline.cat(sepchar);dataline.cat("p[xL]");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][3]==1){//right side estimated
			dataline.cat(sepchar);dataline.cat("p[xR]");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][2]==1 || paramsOnOff[tr][3]==1){//either side estimated
			dataline.cat(sepchar);dataline.cat("deltaP");
			if(traits()>1){dataline.cat(trStr);}
			}
		if(D_on){
			dataline.cat(sepchar);dataline.cat("dispersalSigma");//estimated for this trait
			if(traits()>1){dataline.cat(trStr);}
			}
		if(paramsOnOff[tr][2]==1 ){//left side estimated
			dataline.cat(sepchar);dataline.cat("distBarrierL");
			if(traits()>1){dataline.cat(trStr);}
			if(D_on){
				dataline.cat(sepchar);dataline.cat("timeBarrierL");
				if(traits()>1){dataline.cat(trStr);}
				dataline.cat(sepchar);dataline.cat("s_perMarkerLocusL");
				if(traits()>1){dataline.cat(trStr);}
				}
			}//left side
		if(paramsOnOff[tr][3]==1 ){//right side estimated
			dataline.cat(sepchar);dataline.cat("distBarrierR");
			if(traits()>1){dataline.cat(trStr);}
			if(D_on){
				dataline.cat(sepchar);dataline.cat("timeBarrierR");
				if(traits()>1){dataline.cat(trStr);}
				dataline.cat(sepchar);dataline.cat("s_perMarkerLocusR");
				if(traits()>1){dataline.cat(trStr);}
				}
			}//right side
		}//tr
	//parameters that average over traits
	if(D_on ){
		dataline.cat(sepchar);dataline.cat("dispersalSigma");
		dataline.cat(sepchar);dataline.cat("distBarrierL");
		dataline.cat(sepchar);dataline.cat("timeBarrierL");
		dataline.cat(sepchar);dataline.cat("distBarrierR");
		dataline.cat(sepchar);dataline.cat("timeBarrierR");
		dataline.cat(sepchar);dataline.cat("S");
		dataline.cat(sepchar);dataline.cat("Wh/Wp");
		dataline.cat(sepchar);dataline.cat("s*[hets]");
		dataline.cat(sepchar);dataline.cat("s*[ecotone]");
		dataline.cat(sepchar);dataline.cat("s[reproIsolationLocus]");
		dataline.cat(sepchar);dataline.cat("n[selectedLoci]");
		dataline.cat(sepchar);dataline.cat("s*[domEcotone]");
		}//D_on

	}//end ClineMultiCWTlTrDParamRec::TitleString(aString&, char)



long double ClineMultiCWTlTrDParamRec::WhybridOverWpure(void){
	long double w = aveWidth();
	if(w==0){return (long double)0.0;}//undefined
	long double beta = (distanceBarrierL() + distanceBarrierR()) / (long double)2.0;
    if ((beta / w > (long double)3000.0) || (beta / w == (long double)0.0)){
		return (long double)0.0;}
    long double wratio = exp(- r_ * (ln(beta / w)));
    return wratio;
	}//end ClineMultiCWTlTrDParamRec::WhybridOverWpure()





void ClineMultiCWTlTrDParamRec::operator=(ParamRec& p){
	ParamRec::operator=(p);
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means p must actually point to an object of class ClineMultiCWTlTrDParamRec
	ClineMultiCWTlTrDParamRec* pp = (ClineMultiCWTlTrDParamRec*) p.GetPointerToDerivedSelf();
	paramsOnOff_ = pp->paramsOnOff_;
	paramsShared_ = pp->paramsShared_;
	varyingParams_ = pp->varyingParams_;
	derivedParams_ = pp->derivedParams_;
	D_ = pp->D_;
	lnLDiseq_ = pp->lnLDiseq_;	
	r_ = pp->r_;
	traitParamSet_.Clear();
	for(int tr=0;tr<pp->traits();++tr){
		ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) pp->traitParamSet_[tr].GetPointerToDerivedSelf();
		traitParamSet_.Append(NEW ClineShapeCWTlTrParamRec(*traitParams));
		}//tr
	}//end  ClineMultiCWTlTrDParamRec::operator=(ParamRec&)





int	operator==(ClineMultiCWTlTrDParamRec& p, ClineMultiCWTlTrDParamRec& q){
	if(p.traits()!=q.traits()) return 0;
	if(p.D_ != q.D_) return 0;
	if(p.r_ != q.r_) return 0;
	for(int tr=0;tr<p.traits();++tr){
		ClineShapeCWTlTrParamRec* traitParams1 = 
			(ClineShapeCWTlTrParamRec*) p.traitParamSet_[tr].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* traitParams2 = 
			(ClineShapeCWTlTrParamRec*) q.traitParamSet_[tr].GetPointerToDerivedSelf();
		if(*traitParams1 != *traitParams2){return false;}
		}
	if(p.paramsOnOff_ != q.paramsOnOff_) return false;//compares pointers
	if(p.paramsShared_ != q.paramsShared_) return false;//compares pointers
	return true;
	}//end operator==(ClineMultiCWTlTrDParamRec&, ClineMultiCWTlTrDParamRec&)


int	operator!=(ClineMultiCWTlTrDParamRec& p, ClineMultiCWTlTrDParamRec& q){
	return !(p==q);
	}//end operator!=(ClineMultiCWTlTrDParamRec&, ClineMultiCWTlTrDParamRec&)



#endif


