/******************************************************
*
*	ClineMultiCWTlTrParamRec.cpp
*
*	Parameters for a multi-trait, 3-part composite tanh cline defined by center, width,	
*	thetaL, thetaR, zL & zR.  The latter 4 parameters describe introgression
*	slopes and intercepts on the Left and Right sides of the cline.
*	Assumes the y-axis value of the cline varies between 0 and 1, so this is appropriate
*	for genetic markers and quantitative traits scored as percentiles.
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/
#ifndef ClineMultiCWTlTrParamReccpp
#define ClineMultiCWTlTrParamReccpp = 1

#include "ClineMultiCWTlTrParamRec.h"

	
ParamRec* ClineMultiCWTlTrParamRec::Duplicate(void){
	ClineMultiCWTlTrParamRec* copy = NEW ClineMultiCWTlTrParamRec(*this);
	return copy;
	}//end ClineMultiCWTlTrParamRec::Duplicate()


ClineMultiCWTlTrParamRec* ClineMultiCWTlTrParamRec::_Duplicate(void){
	ClineMultiCWTlTrParamRec* copy = NEW ClineMultiCWTlTrParamRec(*this);
	return copy;
	}//end ClineMultiCWTlTrParamRec::_Duplicate()


//void ClineMultiCWTlTrParamRec::SetPointerToDerivedSelf(void){
//  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in ClineMultiCWTlTrParamRec"<<endl;
//  	}//end

long ClineMultiCWTlTrParamRec::GetPointerToDerivedSelf(void){
  	return (long) this;
  	}//end


long double ClineMultiCWTlTrParamRec::center(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->center();
	}


long double ClineMultiCWTlTrParamRec::thetaL(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->thetaL();
	}


long double ClineMultiCWTlTrParamRec::thetaR(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->thetaR();
	}


long double ClineMultiCWTlTrParamRec::width(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->width();
	}

long double ClineMultiCWTlTrParamRec::zL(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->zL();
	}


long double ClineMultiCWTlTrParamRec::zR(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->zR();
	}


long double ClineMultiCWTlTrParamRec::xL(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->xL();
	}


long double ClineMultiCWTlTrParamRec::xR(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->xR();
	}


long double ClineMultiCWTlTrParamRec::B_L(int trait){//cline frequency at xL
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	if(paramsOnOff[trait][2]){//left side estimable for this trait
		return traitParams->B_L(0);}
	return (long double)0.0;//default when left side is off
	}


long double ClineMultiCWTlTrParamRec::B_R(int trait){//cline frequency at xR
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	if(paramsOnOff[trait][3]){//right side estimable for this trait
		return traitParams->B_R(0);}
	return (long double)1.0;//default when right side off
	}


long double ClineMultiCWTlTrParamRec::DeltaP(int trait){//the frequency step across the center region
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->DeltaP(0);
	}



int ClineMultiCWTlTrParamRec::LJunction(int trait, double leftEnd){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	if(paramsOnOff[trait][2]){//left side estimable for this trait
		return traitParams->LJunction(leftEnd);}
	return false;
	}


int ClineMultiCWTlTrParamRec::RJunction(int trait, double rightEnd){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	if(paramsOnOff[trait][3]){//right side estimable for this trait
		return traitParams->RJunction(rightEnd);}
	return false;
	}





int ClineMultiCWTlTrParamRec::ChangeCenter(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	ClineMultiCWTlTrParamRec* lower = (ClineMultiCWTlTrParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTlTrParamRec* upper = (ClineMultiCWTlTrParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	long double oldCenter = center(trait), leftEnd, rightEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int centerSet = paramsShared[trait][0];
	do{
		long double newCenter = ran.NormDev(oldCenter, sigma);
		ClineShapeCWTlTrParamRec* low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* traitParams = 
				(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->center(newCenter);
		validChange = true;//traitParams->MeetsParamCriteria(low, high);
		if(validChange){
			if(paramsOnOff[trait][2]) {
				validChange = traitParams->LJunction(leftEnd=low->xL_);}}
		if(validChange){
			if(paramsOnOff[trait][3]) {
				validChange = traitParams->RJunction(rightEnd=high->xR_);}}
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (centerSet == paramsShared[tr][0])){//change it too
					low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
					high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
					ClineShapeCWTlTrParamRec* traitParams2 = 
							(ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
					traitParams2->center(newCenter);
					validChange = true;//traitParams2->MeetsParamCriteria(low, high);
					if(validChange){
						if(paramsOnOff[tr][2]) {validChange = traitParams2->LJunction(low->xL_);}}
					if(validChange){
						if(paramsOnOff[tr][3]) {validChange = traitParams2->RJunction(high->xR_);}}
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){//try again if any traits fail
						break;}
					}
				}//end for tr
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for center");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (centerSet == paramsShared[tr][0])){//change it back too
						ClineShapeCWTlTrParamRec* traitParams3 = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams3->center(oldCenter);
						if(paramsOnOff[tr][2]) {traitParams3->LJunction(leftEnd);}
						if(paramsOnOff[tr][3]) {traitParams3->RJunction(rightEnd);}
						}
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWTlTrParamRec::ChangeCenter(int, Ran3&, double, ParamRec*, ParamRec*)


int ClineMultiCWTlTrParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, long double weight){
	//typecasting trick, so don't use function from base class
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
	ClineMultiCWTlTrParamRec* deviations = (ClineMultiCWTlTrParamRec*) paramDeviations->GetPointerToDerivedSelf();
	int trait=0, param=0, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	switch (param){
	 case 0:
	 	 return ChangeCenter(trait, ran, deviations->center(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 1:
	 	 return ChangeWidth(trait, ran, deviations->width(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 2:
	 	 return ChangeThetaL(trait, ran, deviations->thetaL(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 3:
	 	 return ChangeThetaR(trait, ran, deviations->thetaR(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 4:
	 	 return ChangeZL(trait, ran, deviations->zL(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 case 5:
	 	 return ChangeZR(trait, ran, deviations->zR(trait)*weight, lowerLimits, upperLimits);
	 	 break;
	 default:
		aString errstr("bug: can't change shape param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a multi-trait, 6-parameter cline model");
		aError(errstr);
	 	return 0;
	 	}
	 return 0;
	}//end ClineMultiCWTlTrParamRec::ChangeParam(int...)



int ClineMultiCWTlTrParamRec::ChangeThetaL(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiCWTlTrParamRec* lower = (ClineMultiCWTlTrParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTlTrParamRec* upper = (ClineMultiCWTlTrParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	long double oldThetaL = thetaL(trait), leftEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int thetaSet = paramsShared[trait][2];
	do{
		long double newThetaL = ran.NormDev(oldThetaL, sigma);
		ClineShapeCWTlTrParamRec* low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* traitParams = 
				(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->thetaL(newThetaL);
		validChange = traitParams->LJunction(leftEnd=low->xL_);
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (paramsOnOff[tr][2]) && (thetaSet == paramsShared[tr][2])){//change it too
					low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
					high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
					ClineShapeCWTlTrParamRec* traitParams2 = 
							(ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
					traitParams2->thetaL(newThetaL);
					validChange = traitParams2->LJunction(low->xL_);
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){
						break;}
					}
				}//end for tr
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for thetaL");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (paramsOnOff[tr][2]) && (thetaSet == paramsShared[tr][2])){//change it back too
						ClineShapeCWTlTrParamRec* traitParams3 = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams3->thetaL(oldThetaL);
						traitParams3->LJunction(leftEnd);
						}
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWTlTrParamRec::ChangeThetaL(int, Ran3&, double, ParamRec*, ParamRec*)


int ClineMultiCWTlTrParamRec::ChangeThetaR(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiCWTlTrParamRec* lower = (ClineMultiCWTlTrParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTlTrParamRec* upper = (ClineMultiCWTlTrParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	long double oldThetaR = thetaR(trait), rightEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int thetaSet = paramsShared[trait][3];
	do{
		long double newThetaR = ran.NormDev(oldThetaR, sigma);
		ClineShapeCWTlTrParamRec* low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* traitParams = 
				(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->thetaR(newThetaR);
		validChange = traitParams->RJunction(rightEnd=high->xR_);
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (paramsOnOff[tr][3]) && (thetaSet == paramsShared[tr][3])){//change it too
					low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
					high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
					ClineShapeCWTlTrParamRec* traitParams2 = 
							(ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
					traitParams2->thetaR(newThetaR);
					validChange = traitParams2->RJunction(high->xR_);
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){
						break;}
					}
				}//end for tr
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for thetaR");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (paramsOnOff[tr][3]) && (thetaSet == paramsShared[tr][3])){//change it back too
						ClineShapeCWTlTrParamRec* traitParams3 = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams3->thetaR(oldThetaR);
						traitParams3->RJunction(rightEnd);
						}
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWTlTrParamRec::ChangeThetaR(int, Ran3&, double, ParamRec*, ParamRec*)



int ClineMultiCWTlTrParamRec::ChangeWidth(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiCWTlTrParamRec* lower = (ClineMultiCWTlTrParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTlTrParamRec* upper = (ClineMultiCWTlTrParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	long double oldWidth = width(trait), leftEnd, rightEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int widthSet = paramsShared[trait][1];
	do{
		long double newWidth = ran.NormDev(oldWidth, sigma);
		ClineShapeCWTlTrParamRec* low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* traitParams = 
				(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->width(newWidth);
		validChange= traitParams->MeetsParamCriteria(low, high);;
		if(validChange){
			if(paramsOnOff[trait][2]) {
				validChange = traitParams->LJunction(leftEnd=low->xL_);}}
		if(validChange){
			if(paramsOnOff[trait][3]) {
				validChange = traitParams->RJunction(rightEnd=high->xR_);}}
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (widthSet == paramsShared[tr][1])){//change it too
					low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
					high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
					ClineShapeCWTlTrParamRec* traitParams2 = 
							(ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
					traitParams2->width(newWidth);
					validChange= traitParams2->MeetsParamCriteria(low, high);;
					if(validChange){
						if(paramsOnOff[tr][2]) {
							validChange = traitParams2->LJunction(low->xL_);}}
					if(validChange){
						if(paramsOnOff[tr][3]) {
							validChange = traitParams2->RJunction(high->xR_);}}
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){
						break;}
					}
				}//end for tr
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for width");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (widthSet == paramsShared[tr][1])){//change it back too
						ClineShapeCWTlTrParamRec* traitParams3 = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams3->width(oldWidth);
						if(paramsOnOff[tr][2]) {traitParams3->LJunction(leftEnd);}
						if(paramsOnOff[tr][3]) {traitParams3->RJunction(rightEnd);}
						}
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWTlTrParamRec::ChangeWidth(int, Ran3&, double, ParamRec*, ParamRec*)



int ClineMultiCWTlTrParamRec::ChangeZL(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiCWTlTrParamRec* lower = (ClineMultiCWTlTrParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTlTrParamRec* upper = (ClineMultiCWTlTrParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	long double oldZL = zL(trait), leftEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int zSet = paramsShared[trait][4];
	do{
		long double newZL = ran.NormDev(oldZL, sigma);
		ClineShapeCWTlTrParamRec* low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* traitParams = 
				(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->zL(newZL);
		validChange = traitParams->MeetsParamCriteria(low, high);
		if(validChange){
			validChange = traitParams->LJunction(leftEnd=low->xL_);}
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (paramsOnOff[tr][4]) && (zSet == paramsShared[tr][4])){//change it too
					low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
					high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
					ClineShapeCWTlTrParamRec* traitParams2 = 
							(ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
					traitParams2->zL(newZL);
					validChange = traitParams2->MeetsParamCriteria(low, high);
					if(validChange){
						validChange = traitParams2->LJunction(leftEnd=low->xL_);}
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){
						break;}
					}
				}//end for tr
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for zL");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (paramsOnOff[tr][4]) && (zSet == paramsShared[tr][4])){//change it back too
						ClineShapeCWTlTrParamRec* traitParams3 = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams3->zL(oldZL);
						traitParams3->LJunction(leftEnd);
						}
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWTlTrParamRec::ChangeZL(int, Ran3&, double, ParamRec*, ParamRec*)



int ClineMultiCWTlTrParamRec::ChangeZR(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//change each one with the same paramsShared_ value
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineMultiCWTlTrParamRec* lower = (ClineMultiCWTlTrParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTlTrParamRec* upper = (ClineMultiCWTlTrParamRec*) upperLimits->GetPointerToDerivedSelf();
	int validChange=false, tries=0;
	long double oldZR = zR(trait), rightEnd;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int zSet = paramsShared[trait][5];
	do{
		long double newZR = ran.NormDev(oldZR, sigma);
		ClineShapeCWTlTrParamRec* low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[trait].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* traitParams = 
				(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
		traitParams->zR(newZR);
		validChange = traitParams->MeetsParamCriteria(low, high);
		if(validChange){
			validChange = traitParams->RJunction(rightEnd=high->xR_);}
		if(validChange){
			validChange = traitParams->MeetsParamCriteria(low, high);}
		if(validChange){
			//check for co-varying parameters
			for(int tr=0;tr<traitParamSet_.len();++tr){
				if((tr != trait) && (paramsOnOff[tr][5]) && (zSet == paramsShared[tr][5])){//change it too
					low = (ClineShapeCWTlTrParamRec*) lower->traitParamSet_[tr].GetPointerToDerivedSelf();
					high = (ClineShapeCWTlTrParamRec*) upper->traitParamSet_[tr].GetPointerToDerivedSelf();
					ClineShapeCWTlTrParamRec* traitParams2 = 
							(ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
					traitParams2->zR(newZR);
					validChange = traitParams2->MeetsParamCriteria(low, high);
					if(validChange){
						validChange = traitParams2->RJunction(high->xR_);}
					if(validChange){
						validChange = traitParams2->MeetsParamCriteria(low, high);}
					if(! validChange){
						break;}
					}
				}//end for tr
			}
		if(validChange){
			return validChange;}
		++tries;
		if(tries>20){
//			aString err("can't find a good value for zR");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int tr=0;tr<traitParamSet_.len();++tr){
					if((tr != trait) && (paramsOnOff[tr][5]) && (zSet == paramsShared[tr][5])){//change it back too
						ClineShapeCWTlTrParamRec* traitParams3 = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
						traitParams3->zR(oldZR);
						traitParams3->RJunction(rightEnd);//reset
						}
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineMultiCWTlTrParamRec::ChangeZR(int, Ran3&, double, ParamRec*, ParamRec*)




long double ClineMultiCWTlTrParamRec::Cline_at_X(long double x, int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->Cline_at_X(x);
	}//end ClineMultiCWTlTrParamRec::Cline_at_X(double, int)



void ClineMultiCWTlTrParamRec::Initialize(ClineFitInitializer& i){
	lnL_=0.0;
	lnLmaxMinusLnL_=0.0;
	paramsOnOff_ = i.paramsOnOff_;//copies pointer
	paramsShared_ = i.paramIdentities_;//copies pointer
	varyingParams_=0;//in case of reinitialization
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	traitParamSet_.Clear();//in case of reinitialization
	for(int tr=0;tr<i.traits();++tr){//account for on/off parameters here
		traitParamSet_.Append(NEW ClineShapeCWTlTrParamRec(i,tr));
		ClineShapeCWTlTrParamRec *traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		if(paramsOnOff[tr][2]==0 || paramsOnOff[tr][4]==0){//thetaL or zL off
			paramsOnOff[tr][2] = paramsOnOff[tr][4]=0;//both off if either is off
			traitParams->thetaL(1.0);
			traitParams->zL(0.0);
			traitParams->xL_=i.leftEnd_;
			}
		if(paramsOnOff[tr][3]==0 || paramsOnOff[tr][5]==0){//thetaR or zL off
			paramsOnOff[tr][3] = paramsOnOff[tr][5]=0;//both off if either is off
			traitParams->thetaR(1.0);
			traitParams->zR(0.0);
			traitParams->xR_=i.rightEnd_;
			}
		}
	//count varying params
	for(int p=0;p<BaseParams();++p){//assumes paramsShared is sorted low to high
		int maxparamID =-1;
		for(int tr=0;tr<traits();++tr){
			if((paramsOnOff[tr][p]==1) && (maxparamID<paramsShared[tr][p])){
				varyingParams_++;
				maxparamID = paramsShared[tr][p];
				}//if
			}//tr
		}//p
	}//end ClineMultiCWTlTrParamRec::Initialize(ClineFitInitializer&)



long double ClineMultiCWTlTrParamRec::leftVal(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->leftVal(0);
	}//end ClineMultiCWTlTrParamRec::leftVal(int)


long double ClineMultiCWTlTrParamRec::maxSlope(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->maxSlope();
	}//end ClineMultiCWTlTrParamRec::maxSlope(int)


int ClineMultiCWTlTrParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
	//typecasting trick, so don't use function from base class
	//use type conversion that circumvents usual virtual base class restrictions
	ClineMultiCWTlTrParamRec* lower = (ClineMultiCWTlTrParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineMultiCWTlTrParamRec* upper = (ClineMultiCWTlTrParamRec*) upperLimits->GetPointerToDerivedSelf();
	for(int tr=0;tr<traits();++tr){
		ClineShapeCWTlTrParamRec* traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		ParamRec& lowParams = lower->traitParamSet_[tr];//no need to cast these yet
		ParamRec& highParams = upper->traitParamSet_[tr];
		int paramsValid = traitParams->MeetsParamCriteria(&lowParams,&highParams);
		if(! paramsValid) return false;
		}//tr
	return true;
	}//end ClineMultiCWTlTrParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)


void ClineMultiCWTlTrParamRec::MultiplyParam(int paramNum, long double factor){
	//typecasting trick, so don't use function from base class
	//use to rescale a param deviation
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int trait=0, param=0, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	int covaryingID = paramsShared[trait][param];
	for(int tr=trait;tr<traitParamSet_.len();++tr){//assumes paramsShared_ list is sorted
		if(covaryingID == paramsShared[tr][param]){//change it too
			ClineShapeCWTlTrParamRec* traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
			traitParams->MultiplyParam(param,factor);
			}
		}//tr
	}//ClineMultiCWTlTrParamRec::MultiplyParam(int, double)



int ClineMultiCWTlTrParamRec::NumDerivedParams(void){
	if(derivedParams_ > (-1)){//default flag set by constructor
		return derivedParams_;}
	//otherwise calculate it here first
	derivedParams_=0;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	for(int tr=0;tr<traits();++tr){
		int both=0;
		if(paramsOnOff[tr][2]==1){//thetaL && zL turned on
			derivedParams_+=2;//xL_ and barrierL
			both++;}
		if(paramsOnOff[tr][3]==1){//thetaL && zL turned on
			derivedParams_+=2;//xR_ and barrierR
			both++;}
		if(both>0){//either is turned on
			derivedParams_++;//deltaP
			}
		}//tr
	return derivedParams_;
	}//end ClineMultiCWTlTrParamRec::NumDerivedParams()



void ClineMultiCWTlTrParamRec::ParamCoords(int globalParamNum, int& trait, int& traitParam, int& derived){
	//returns values for trait && traitParam
	//if params covary among traits, this returns the first trait
	derived = false;//to start
	int count=0;
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	for(int p=0;p<BaseParams();++p){//try base params first
		int maxSharedID=-1;
		for(int tr=0;tr<traits();++tr){
			if(paramsOnOff[tr][p]==1){//estimating this one
				if(maxSharedID<paramsShared[tr][p]){//this is a new param to estimate
					if(globalParamNum==count){
						trait=tr; traitParam=p;
						return;}
					++count;
					maxSharedID = paramsShared[tr][p];//assumes these are in ascending order
				}}
			}//tr
		}//p
	//if we're here, it must be a derived parameter
	int dp = globalParamNum - NumParams();
	derived = true;
	int dcount=0;
	for(int tr=0;tr<traits();++tr){
		if(paramsOnOff[tr][2]){
			if(dp==dcount){//xL estimated
				trait=tr; traitParam=0;return;}
			dcount++;}
		if(paramsOnOff[tr][3]){
			if(dp==dcount){//xR estimated
				trait=tr; traitParam=1;return;}
			dcount++;}
		if(paramsOnOff[tr][2]){
			if(dp==dcount){//B_L estimated
				trait=tr; traitParam=2;return;}
			dcount++;}
		if(paramsOnOff[tr][3]){
			if(dp==dcount){//B_R estimated
				trait=tr; traitParam=3;return;}
			dcount++;}
		if(paramsOnOff[tr][2] || paramsOnOff[tr][3]){
			if(dp==dcount){//deltaP estimated
				trait=tr; traitParam=4;return;}
			dcount++;}
		}//tr
	derived=false;
	aString err("Can\'t find param");err.cat(globalParamNum+1);
	aError(err);
	}//end ClineMultiCWTlTrParamRec::ParamCoords(int, int&, int&, int&)


aString* ClineMultiCWTlTrParamRec::ParamName(int paramNum){
	int trait=0, param=0, allShared=true, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	aString trstr,  *newName = NEW aString();
	if(! derived){
			if(traits()>1){
				trstr.cat("[");trstr.cat(trait+1);
				ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
				ObjList<aList<int> >& paramsShared = *paramsShared_;
				int shareState = paramsShared[trait][param];
				for(int tr=0;tr<traits();++tr){
					if(tr != trait && paramsOnOff[tr][param]){//param is being estimated for this trait
						if(shareState != paramsShared[tr][param]){
								allShared=false;}
							else{
								trstr.cat(';');trstr.cat(tr+1);}//only append those with shared states
						}}
				trstr.cat("]");}
			switch (param){
			 case 0:
				newName->cat("center"); break;
			 case 1:
				newName->cat("width"); break;
			 case 2:
				newName->cat("thetaL"); break;
			 case 3:
				newName->cat("thetaR"); break;
			 case 4:
				newName->cat("zL"); break;
			 case 5:
				newName->cat("zR"); break;
			 default:
				aString errstr("bug: can't return name of param #");
				errstr.cat(param+1);
				if(traits()>1){
					errstr.cat(" for trait #");
					errstr.cat(trait+1);}
				aError(errstr);
			 	return newName;//null
			 	}	
			if(traits()>1 && !allShared){
				newName->cat(trstr);}
			}//!derived
		else{//it's derived
			if(traits()>1){
				trstr.cat("[");trstr.cat(trait+1);trstr.cat("]");}
			switch (param){
			 case 0:
				newName->cat("xL"); break;
			 case 1:
				newName->cat("xR"); break;
			 case 2:
				newName->cat("p[xL]"); break;
			 case 3:
				newName->cat("p[xR]"); break;
			 case 4:
				newName->cat("deltaP"); break;
			 default:
				aString errstr("bug: can't return name of derived param #");
				errstr.cat(param+1);
				if(traits()>1){
					errstr.cat(" for trait #");
					errstr.cat(trait+1);}
				aError(errstr);
			 	return newName;//null
			 	}	
			if(traits()>1){
				newName->cat(trstr);}
			}//!derived		
	return newName;
	}//end ClineMultiCWTlTrParamRec::ParamName(int )



void ClineMultiCWTlTrParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ParamRec::ParamString(dataline, sepchar);
	long double val;
	//each param will be printed, whether or not it is co-estimated with another
	for(int tr=0;tr<traits();++tr){
		ClineShapeCWTlTrParamRec* traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		for(int p=0;p<BaseParams();++p){//6 params/trait
			if(paramsOnOff[tr][p]==1){//param is estimated
				val = traitParams->ParamValue(p);
				dataline.cat(sepchar);
				dataline.cat(val,6,15);
				}//if
			}//p
		if(paramsOnOff[tr][2]==1){//thetaL & zL toggle on/off together
			val = traitParams->xL();
			dataline.cat(sepchar);
			dataline.cat(val,6,15);
			}
		if(paramsOnOff[tr][3]==1){//thetaR & zR toggle on/off together
			val = traitParams->xR();
			dataline.cat(sepchar);
			dataline.cat(val,6,15);
			}
		if(paramsOnOff[tr][2]==1){//thetaL & zL toggle on/off together
			val = traitParams->B_L(0);
			dataline.cat(sepchar);
			dataline.cat(val,6,15);
			}
		if(paramsOnOff[tr][3]==1){//thetaR & zR toggle on/off together
			val = traitParams->B_R(0);
			dataline.cat(sepchar);
			dataline.cat(val,6,15);
			}
		if(paramsOnOff[tr][2]==1 || paramsOnOff[tr][3]==1){//either
			val = traitParams->DeltaP(0);
			dataline.cat(sepchar);
			dataline.cat(val,6,15);
			}
		}//tr
	}//end ClineMultiCWTlTrParamRec::ParamString(aString&, char)



long double ClineMultiCWTlTrParamRec::ParamValue(int paramNum){
	//typecasting trick, so don't use function from base class
	//returns value of parameter; first varyng params, then derived params
	int trait=0, param=0, allShared=true, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	ClineShapeCWTlTrParamRec* traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	if(! derived){
			switch (param){
			 case 0:
				return traitParams->center(); break;
			 case 1:
				return traitParams->width(); break;
			 case 2:
				return traitParams->thetaL(); break;
			 case 3:
				return traitParams->thetaR(); break;
			 case 4:
				return traitParams->zL(); break;
			 case 5:
				return traitParams->zR(); break;
			 default:
				aString errstr("bug: can't return name of param #");
				errstr.cat(param+1);
				if(traits()>1){
					errstr.cat(" for trait #");
					errstr.cat(trait+1);}
				aError(errstr);
			 	}	
			}//!derived
		else{//it's derived
			switch (param){
			 case 0:
				return traitParams->xL(); break;
			 case 1:
				return traitParams->xR(); break;
			 case 2:
				return traitParams->B_L(); break;
			 case 3:
				return traitParams->B_R(); break;
			 case 4:
				return traitParams->DeltaP(); break;
			 default:
				aString errstr("bug: can't return name of derived param #");
				errstr.cat(param+1);
				if(traits()>1){
					errstr.cat(" for trait #");
					errstr.cat(trait+1);}
				aError(errstr);
			 	}	
			}//!derived	
	return 0.0;//default	
	}//end ClineMultiCWTlTrParamRec::ParamValue(int )


void ClineMultiCWTlTrParamRec::Print(ostream& outFile, long rep, aString& message){
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ParamRec::Print(outFile, rep, message);
	//for now, each param will be printed, whether or not it is co-estimated with another
	for(int tr=0;tr<traits();++tr){
		aString outStr;
		ClineShapeCWTlTrParamRec* traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		if(paramsOnOff[tr][0]==1){//center
			outStr.cat("center");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->center();
			outStr.cat(val,6,4);
			outStr.separator(5);
			}
		if(paramsOnOff[tr][1]==1){//width
			outStr.cat("width");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->width();
			outStr.cat(val,6,4);
			}
		outFile<<outStr<<endl;
		outStr.clear();
		if(paramsOnOff[tr][2]==1){//thetaL & zL toggle on/off together
			outStr="thetaL";
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->thetaL();
			outStr.cat(val,6,4);
			outStr.separator(2);
			outStr.cat("zL");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->zL();
			outStr.cat(val,6,4);
			outStr.separator(2);
			outStr.cat("xL");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->xL();
			outStr.cat(val,6,4);
			outFile<<outStr<<endl;
			}
		outStr.clear();
		if(paramsOnOff[tr][3]==1){//thetaR & zR toggle on/off together
			outStr.cat("thetaR");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->thetaR();
			outStr.cat(val,6,4);
			outStr.separator(2);
			outStr.cat("zR");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->zR();
			outStr.cat(val,6,4);
			outStr.separator(2);
			outStr.cat("xR");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			val = traitParams->xR();
			outStr.cat(val,6,4);
			outFile<<outStr<<endl;
			}
		outStr.clear();
		if(paramsOnOff[tr][2]==1){//left tail estimated
			outStr.cat("P_xL");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->B_L(0);
			outStr.cat(val,6,4);
			}
		if(paramsOnOff[tr][3]==1){//right tail estimated
			outStr.separator(2);
			outStr.cat("P_xR");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->B_R(0);
			outStr.cat(val,6,4);
			}
		if(paramsOnOff[tr][2]==1 || paramsOnOff[tr][3]==1){//either tail estimated
			outStr.separator(2);
			outStr.cat("deltaP");
			if(traits()>1){
				outStr.cat('[');outStr.cat(tr+1);outStr.cat(']');}
			outStr.cat(": ");
			double val = traitParams->DeltaP(0);
			outStr.cat(val,6,4);
			outFile<<outStr<<endl;
			}
		}//tr
	}//ClineMultiCWTlTrParamRec::Print


long double ClineMultiCWTlTrParamRec::rightVal(int trait){
	//typecasting trick, so don't use function from base class
	ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	return traitParams->rightVal(0);
	}//end ClineMultiCWTlTrParamRec::rightVal()


void ClineMultiCWTlTrParamRec::SetParamValue(int paramNum, long double newVal){
	//set value of parameter, and all those that covary with it
	ObjList<aList<int> >& paramsShared = *paramsShared_;
	int trait=0, param=0, derived=false;
	ParamCoords(paramNum, trait, param, derived);
	int covaryingID = paramsShared[trait][param];
	for(int tr=trait;tr<traitParamSet_.len();++tr){//assumes paramsShared_ list is sorted
		if(covaryingID == paramsShared[tr][param]){//change it too
			ClineShapeCWTlTrParamRec* traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
			traitParams->SetParamValue(param,newVal);
			}
		}//tr
	}//end ClineMultiCWTlTrParamRec::SetParamValue(int,long double)


void ClineMultiCWTlTrParamRec::SetXL(long double xLval){//use to override when setting limiting values
	for(int tr=0;tr<traits();++tr){
		ClineShapeCWTlTrParamRec* traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		traitParams->xL_=xLval;
		}//tr
	}//end ClineMultiCWTlTrParamRec::SetXL(double)


void ClineMultiCWTlTrParamRec::SetXR(long double xRval){//use to override when setting limiting values
	for(int tr=0;tr<traits();++tr){
		ClineShapeCWTlTrParamRec* traitParams = (ClineShapeCWTlTrParamRec*) traitParamSet_[tr].GetPointerToDerivedSelf();
		traitParams->xR_=xRval;
		}//tr
	}//end ClineMultiCWTlTrParamRec::SetXR(double)



void ClineMultiCWTlTrParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ObjList<aList<int> >& paramsOnOff = *paramsOnOff_;
	ParamRec::TitleString(dataline, sepchar);
	//each param will be printed, whether or not it is co-estimated with another
	for(int tr=0;tr<traits();++tr){
		if(paramsOnOff[tr][0]==1){//center
			dataline.cat(sepchar);
			dataline.cat("center");
			if(traits()>1){
				dataline.cat('[');dataline.cat(tr+1);dataline.cat(']');}
			}
		if(paramsOnOff[tr][1]==1){//width
			dataline.cat(sepchar);
			dataline.cat("width");
			if(traits()>1){
				dataline.cat('[');dataline.cat(tr+1);dataline.cat(']');}
			}
		if(paramsOnOff[tr][2]==1){//thetaL
			dataline.cat(sepchar);
			dataline.cat("thetaL");
			if(traits()>1){
				dataline.cat('[');dataline.cat(tr+1);dataline.cat(']');}
			}
		if(paramsOnOff[tr][3]==1){//thetaR
			dataline.cat(sepchar);
			dataline.cat("thetaR");
			if(traits()>1){
				dataline.cat('[');dataline.cat(tr+1);dataline.cat(']');}
			}
		if(paramsOnOff[tr][4]==1){//zL
			dataline.cat(sepchar);
			dataline.cat("zL");
			if(traits()>1){
				dataline.cat('[');dataline.cat(tr+1);dataline.cat(']');}
			}
		if(paramsOnOff[tr][5]==1){//zR
			dataline.cat(sepchar);
			dataline.cat("zR");
			if(traits()>1){
				dataline.cat('[');dataline.cat(tr+1);dataline.cat(']');}
			}
		if(paramsOnOff[tr][2]==1){//xL
			dataline.cat(sepchar);
			dataline.cat("xL");
			if(traits()>1){
				dataline.cat('[');dataline.cat(tr+1);dataline.cat(']');}
			}
		if(paramsOnOff[tr][3]==1){//xR
			dataline.cat(sepchar);
			dataline.cat("xR");
			if(traits()>1){
				dataline.cat('[');dataline.cat(tr+1);dataline.cat(']');}
			}
		if(paramsOnOff[tr][2]==1){//left side estimated
			dataline.cat(sepchar);
			dataline.cat("P_at_xL");
			if(traits()>1){
				dataline.cat('[');dataline.cat(tr+1);dataline.cat(']');}
			}
		if(paramsOnOff[tr][3]==1){//right side estimated
			dataline.cat(sepchar);
			dataline.cat("P_at_xR");
			if(traits()>1){
				dataline.cat('[');dataline.cat(tr+1);dataline.cat(']');}
			}
		if(paramsOnOff[tr][2]==1 || paramsOnOff[tr][3]==1){//either side estimated
			dataline.cat(sepchar);
			dataline.cat("deltaP");
			if(traits()>1){
				dataline.cat('[');dataline.cat(tr+1);dataline.cat(']');}
			}
		}//tr
	}//end ClineMultiCWTlTrParamRec::TitleString(aString&, char)


void ClineMultiCWTlTrParamRec::operator=(ParamRec& p){
	ParamRec::operator=(p);
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means p must actually point to an object of class ClineMultiCWTlTrParamRec
	ClineMultiCWTlTrParamRec* pp = (ClineMultiCWTlTrParamRec*) p.GetPointerToDerivedSelf();
	paramsOnOff_ = pp->paramsOnOff_;
	paramsShared_ = pp->paramsShared_;
	varyingParams_ = pp->varyingParams_;
	derivedParams_ = pp->derivedParams_;
	traitParamSet_.Clear();
	for(int tr=0;tr<pp->traits();++tr){
		ClineShapeCWTlTrParamRec* traitParams = 
			(ClineShapeCWTlTrParamRec*) pp->traitParamSet_[tr].GetPointerToDerivedSelf();
		traitParamSet_.Append(NEW ClineShapeCWTlTrParamRec(*traitParams));
		}//tr
	}//end  ClineMultiCWTlTrParamRec::operator=(ParamRec&)





int	operator==(ClineMultiCWTlTrParamRec& p, ClineMultiCWTlTrParamRec& q){
	if(p.traits()!=q.traits()) return 0;
	for(int tr=0;tr<p.traits();++tr){
		ClineShapeCWTlTrParamRec* traitParams1 = 
			(ClineShapeCWTlTrParamRec*) p.traitParamSet_[tr].GetPointerToDerivedSelf();
		ClineShapeCWTlTrParamRec* traitParams2 = 
			(ClineShapeCWTlTrParamRec*) q.traitParamSet_[tr].GetPointerToDerivedSelf();
		if(*traitParams1 != *traitParams2){return false;}
		}
	if(p.paramsOnOff_ != q.paramsOnOff_) return false;//compares pointers
	if(p.paramsShared_ != q.paramsShared_) return false;//compares pointers
	return true;
	}//end operator==(ClineMultiCWTlTrParamRec&, ClineMultiCWTlTrParamRec&)


int	operator!=(ClineMultiCWTlTrParamRec& p, ClineMultiCWTlTrParamRec& q){
	return !(p==q);
	}//end operator!=(ClineMultiCWTlTrParamRec&, ClineMultiCWTlTrParamRec&)



#endif


