/******************************************************
*
*	ClineMultiCWTlTrTBParamRec.h
*
*	Parameters for a multi-trait, 3-part composite tanh cline defined by center, width,	
*	thetaL, thetaR, zL, zR, and scores on the left and right sides.  
*	The middele 4 parameters describe introgression
*	slopes and intercepts on the Left and Right sides of the cline.
*	The mean y-axis value of the cline varies for each trait on each side.
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/

#ifndef ClineMultiCWTlTrTBParamRech
#define ClineMultiCWTlTrTBParamRech = 1


#include "ParamRec.h"
#include "Hyperbolics.h"
#include "InlineTemplates.h"
#include "ClineFitInitializer.h"
#include "ClineMultiCWParamRec.h"
#include "ClineMultiCWTlTrParamRec.h"
#include "ClineShapeCWTlTrTBParamRec.h"
#include "Element.cpp"
#include "aList.cpp"
#include "MemoryManager.h"

class ClineMultiCWTlTrTBParamRec;
class ClineMultiCWTlTrTBParamRec: public ClineMultiCWTlTrParamRec{
  public:
  	//uses variables held in ClineMultiCWParamRec & ClineShapeCWTlTrParamRec
  	//but holds list of class ClineShapeCWTlTrTBParamRec instead of ClineShapeCWTrTlParamRec
//  	int derivedParams_;
  public:
  	ClineMultiCWTlTrTBParamRec(void):ClineMultiCWTlTrParamRec(){
//  				SetPointerToDerivedSelf();//empty parameter set
  				}
  	ClineMultiCWTlTrTBParamRec(ClineFitInitializer& i):
  					ClineMultiCWTlTrParamRec(){//initialize empty, then fill here
//  				SetPointerToDerivedSelf();
  				Initialize(i);
  				}
  	ClineMultiCWTlTrTBParamRec(ClineMultiCWTlTrTBParamRec& p):
  					ClineMultiCWTlTrParamRec(){//initialize empty, then fill here
  				lnL_ = p.lnL_;
  				lnLmaxMinusLnL_ = p.lnLmaxMinusLnL_;
  				paramsOnOff_ = p.paramsOnOff_;
  				paramsShared_ = p.paramsShared_;
  				varyingParams_ = p.varyingParams_;
  				derivedParams_ = p.derivedParams_;
//  				SetPointerToDerivedSelf();
  				for(int tr=0;tr<p.traits();++tr){
  					ClineShapeCWTlTrTBParamRec* traitparams = (ClineShapeCWTlTrTBParamRec*) p.traitParamSet_[tr].GetPointerToDerivedSelf();
  					traitParamSet_.Append(NEW ClineShapeCWTlTrTBParamRec(*traitparams));}
				}
				
  	virtual ~ClineMultiCWTlTrTBParamRec(void){//handled by base class
//  			pointerToDerivedSelf = NULL;//forget
//  			paramsOnOff_=NULL;//forget
//  			paramsShared_=NULL;//forget
//  			traitParamSet_.Clear();
  			}

  	virtual ParamRec* 						Duplicate(void);
  	virtual ClineMultiCWTlTrTBParamRec* 	_Duplicate(void);
//	virtual void 							SetPointerToDerivedSelf(void);//always override in derived
	virtual long 							GetPointerToDerivedSelf(void);//always override in derived
  	


	virtual long double center(int trait);//returns c_ for trait
	virtual long double width(int trait);//returns w_ for trait
	virtual long double leftVal(int trait);//average limiting score on left side
	virtual long double rightVal(int trait);//average limiting score on right side
	virtual long double maxSlope(int trait);//most extreme slope of a cline (positive or negative)
	virtual long double thetaL(int trait);//returns thetaL_
	virtual long double thetaR(int trait);//returns thetaR_
	virtual long double zL(int trait);//returns zL_
	virtual long double zR(int trait);//returns zR_
	virtual long double xL(int trait);//returns xL_ -- derived parameter; same for all traits
	virtual long double xR(int trait);//returns xR_ -- derived parameter; same for all traits
	virtual long double B_L(int trait);//cline frequency at xL
	virtual long double B_R(int trait);//cline frequency at xR
	virtual long double DeltaP(int trait);//the frequency step across the center region
	virtual void 		SetXL(long double xLval);//use to override when setting limiting values
	virtual void 		SetXR(long double xRval);//use to override when setting limiting values

	virtual int 		BaseParams(void){return 8;}
  	
	virtual int 		ChangeCenter(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
									ParamRec* paramDeviations,  
									ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=1.0);
	virtual int 		ChangeLeftScore(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeRightScore(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeThetaL(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeThetaR(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeWidth(int trait, Ran3& ran, double sigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeZL(int trait, Ran3& ran, double sigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeZR(int trait, Ran3& ran, double sigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual long double Cline_at_X(long double x, int trait);
//	virtual long double ExpectedValue(aList<long double>& coords);
	virtual void 		Initialize(ClineFitInitializer& i);
	virtual int 		LJunction(int trait, double leftEnd);//finds point where left and center cline sections meet
	virtual int 		RJunction(int trait, double rightEnd);//finds point where right and center cline sections meet
	virtual int 		MeetsParamCriteria(ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void 		MultiplyParam(int paramNum, long double factor);//use to rescale a param deviation
//	virtual int 		NumParams(void);//number of varying parameters
	virtual int 		NumDerivedParams(void);//number of params that depend on varying params
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamCoords(int globalParamNum, int& trait, int& traitParam, int& derived);
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
	virtual void 		SetParamValue(int paramNum, long double newVal);//sets value of parameter
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file
//	virtual int 		TotalParams(void);//number of parameters that could vary, plus derived paramerters
//	virtual int 		traits(void);//number of traits -- handled by base class
	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

	friend int	operator==(ClineMultiCWTlTrTBParamRec& p, ClineMultiCWTlTrTBParamRec& q);
	friend int	operator!=(ClineMultiCWTlTrTBParamRec& p, ClineMultiCWTlTrTBParamRec& q);

	};//end class ClineMultiCWTlTrTBParamRec



#endif