// ClinePhenotypicData.cpp

#ifndef ClinePhenotypicDatacpp
#define ClinePhenotypicDatacpp = 1

#include "ClinePhenotypicData.h"

/*
#ifdef MemoryLeakDebug
MemLeakIDType  ClinePhenotypicData::memIDCount_ = (MemLeakIDType) 0;//initialized here in file scope
memAllocList  ClinePhenotypicData::memIDList_ =  memAllocList();//initialized here in file scope
void  ClinePhenotypicData::PrintIDList(ostream& outFile){
	memIDList_.Print(outFile);}
void  ClinePhenotypicData::PrintIDListLen(ostream& outFile){
	memIDList_.PrintLen(outFile);}
#endif
*/


ClinePhenotypicData::ClinePhenotypicData(ClinePhenotypicData& parentData):
			PopulationPhenotypicData(parentData), 
			centerPopList_(parentData.centerPopList_),
			locality(parentData.locality),
			localityFlag_(parentData.localityFlag_),
			diagTraitsRecordedFlag_(parentData.diagTraitsRecordedFlag_)
			{	
		}//end ctor ClinePhenotypicData(ClinePhenotypicData&)


ClinePhenotypicData::ClinePhenotypicData(ClinePhenotypicData& parentData, aList<int>& popSubsetNums):
			PopulationPhenotypicData(parentData,popSubsetNums),
			localityFlag_(0),
			diagTraitsRecordedFlag_(parentData.diagTraitsRecordedFlag_)
		{//must set centerPopList_ separately
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
	}//end ctor ClinePhenotypicData(ClinePhenotypicData&, aList<int>&)



void ClinePhenotypicData::CalculateHybridIndex(aList<double>& popLocation){
	hybridIndices.Clear();
	HybridIndicesByTrait.Clear();
	popLocation.Clear();
	double hInd,indexVal,traitIndexVal;
	aString endcharList(" ;:,/");
	for(int pop=0;pop<npop_;++pop){
		//get the population's location from the end of the title string
		aString popTitleWord(popName[pop]);
	    aString* popXword = popTitleWord.ChopOffLastWord(endcharList);
    	double popX = popXword->toDouble();
    	popLocation.Append(popX);
    	delete popXword; popXword=0;
		//get each individual's genotype and compare to diagnostic alleles
    	for (int i=0;i<popSize[pop];++i){
    		HybridIndicesByTrait.Append(NEW HybridIndexByLocus());
      		indexVal = 0.0;
      		double traitsScored = 0.0;//use as error flag
    		for(int tr=0;tr<ntraits;++tr){
    			traitIndexVal = 0.0;
    			aString* traitStr = GetTraitStr(pop, i, tr);
    			if((traitStr->len() != 0)&&(*traitStr != ' ')){
    				++traitsScored;
    				double traitVal = traitStr->toDouble();
    				if(diagnosticTraitsR[tr] == "-"){
    						traitIndexVal -= traitVal;}//subtract if cline is inverted
  						else{
  							traitIndexVal += traitVal;}//add if cline from low to high
    				}
    			delete traitStr; traitStr = 0;
    			HybridIndicesByTrait.Last().Append(NEW HybridIndexByLocation(traitIndexVal, 1, popX));
    			indexVal += traitIndexVal;
    			}//end for tr
    		if(traitsScored >0.0){hInd = indexVal/traitsScored;}//assumes all equal mean & variance
    		hybridIndices.Append(NEW HybridIndexByLocation(hInd, traitsScored, popX));
    		}//end for i
    	}//end for pop
	}// end ClinePhenotypicData::CalculateHybridIndex(aList<double>&)




void ClinePhenotypicData::Clear(void){
	PopulationPhenotypicData::Clear();
	hybridIndices.Clear();
	HybridIndicesByTrait.Clear();
	diagnosticTraitsR.Clear();
	centerPopList_.Clear();
	locality.Clear();
	ResetFlags();
	}//end ClinePhenotypicData::Clear()


void ClinePhenotypicData::CullTraits (StringList& traitsToCull){
	//This function moves the designated traits to the list of skipped traits
	if(traitsToCull.len()==0){return;}//none to cull
	for(int ctrait=0;ctrait<traitsToCull.len();++ctrait){
		aString ctraitname = traitsToCull[ctrait];
		aString ctraitnameCopy(ctraitname);
		int item = rawTraitName.Loc(&ctraitname);
		if((item>=0)&&(!(skippedTraitName.Includes(&ctraitname)))){
				//the locus exists in the rawlist && isn't already skipped
			skippedTraitName.Append(NEW aString(ctraitname));
			skippedTraitNum.Append(item);
			item = traitName.Loc(&ctraitnameCopy);
			traitName.Delete(item);
			traitsSkipped++;--ntraits;
			dataType.Delete(item);
			}//if item exists
		}//cLoc
	ResetFlags();
	CullSkippedTraits();
	}//end procedure ClinePhenotypicData::CullTraits(StringList&)



void ClinePhenotypicData::CullSkippedTraits (void){
	PopulationPhenotypicData::CullSkippedTraits();
	}//end procedure ClinePhenotypicData::CullSkippedTraits()



double ClinePhenotypicData::Locality (int pop){
	//returns the x-axis coordinates of the locality
	if(localityFlag_){
		return locality[pop];}
	locality.Clear();
	aString endchrs(" \t,;");
	for(int p=0;p<npop_;++p){
		aString thisPopName(popName[p]);
		aString* thisLocalityName = thisPopName.ChopOffLastWord(endchrs);
		double thisLocality = thisLocalityName->toDouble();
		locality.Append(thisLocality);
		delete thisLocalityName;thisLocalityName=0;
		}//end for pop
	localityFlag_ = true;
	return locality[pop];
	}//end  ClinePhenotypicData::Locality


int ClinePhenotypicData::LocationRelativeToCenterList(double x){
	StatList<double> centerLocalities;
	for(int p=0;p<centerPopList_.len();++p){
		centerLocalities.Append(Locality(centerPopList_[p]));}
	if(x < centerLocalities.min()){
			return 0;}//it's to the left
		else if(x > centerLocalities.max()){
			return 2;}//it's to the right
		else{}
	return 1;//it's between the bounds of the list
	}//ClinePhenotypicData::LocationRelativeToCenterList(double)



void ClinePhenotypicData::PrintPhenotypeFile (aString& outFileName){
	PopulationPhenotypicData::PrintPhenotypeFile(outFileName);
	}//end ClinePhenotypicData::PrintPhenotypeFile



void ClinePhenotypicData::PrintPhenotypeFile (ostream& outFile){
	PopulationPhenotypicData::PrintPhenotypeFile(outFile);
	}//end ClinePhenotypicData::PrintPhenotypeFile



void ClinePhenotypicData::PrintHybridIndexFile(aString& outFileName){
	char* rawfilename = outFileName.raw();
	fstream outputData;
	outputData.open(rawfilename, ios::out /*,filebuf::openprot*/);
	for(int i=0;i<hybridIndices.len();++i){
		aString outline(hybridIndices[i].x(),10,5); outline.cat('\t');	
		outline.cat(hybridIndices[i].hi(),2,6); outline.cat('\t');	
		outline.cat(hybridIndices[i].items());
		outputData<<outline<<endl; 
		}//end for i	
	outputData.close();
	delete [] rawfilename; rawfilename=0;
	}//end ClinePhenotypicData::PrintHybridIndexFile(aString&)



void ClinePhenotypicData::ReadDataFile (aString* dataFileName){
	PopulationPhenotypicData::ReadDataFile (dataFileName);
	for(int tr=0;tr<NumTraits();++tr){
		dataType.Append(NEW aString("morph"));}
	}//end ClinePhenotypicData::ReadDataFile()


void ClinePhenotypicData::RescoreDiagnosticTraits(void){
	//switches direction of clines originally from high to low
	//by making positive trait values negative and vice versa
	if(diagTraitsRecordedFlag_){return;}
	StoreDiagnosticTraits();
	for(int tr=0;tr<ntraits;++tr){
		if(diagnosticTraitsR[tr]=='-'){
			for(int pop=0;pop<npop_;++pop){
				for(int i=0;i<popSize[pop];++i){
					double z = traits_[pop][i][tr];
					traits_[pop][i][tr] = -z;
					}//i
				}//pop
			}//if
		}//tr
	diagTraitsRecordedFlag_=true;
	}//end ClinePhenotypicData::RescoreDiagnosticTraits()




void ClinePhenotypicData::ResetFlags(void){
	localityFlag_=0;
	diagTraitsRecordedFlag_=0;
	}//end ClinePhenotypicData::ResetFlags()


void ClinePhenotypicData::RestoreTraits (StringList& traitsToRestore){
	//This function restores the designated loci to each genotype data string
	//and removes them from the skipped loci
	PopulationPhenotypicData::RestoreTraits(traitsToRestore);
	dataType.Clear();
	for(int tr=0;tr<NumTraits();++tr){
		dataType.Append(NEW aString("morph"));}
	ResetFlags();
	}//end procedure ClinePhenotypicData::RestoreTraits


void ClinePhenotypicData::SetCenterPopList(aList<int>& centerList){
	centerPopList_.Clear();
	centerPopList_ = centerList;
	}//end ClinePhenotypicData::SetCenterPopList(aList<double>&)


void ClinePhenotypicData::SetDataTypes(void){
	for(int tr=0;tr<traits();++tr){
		dataType.Append(NEW aString("morph"));}
	}//end ClinePhenotypicData::SetDataTypes()


void ClinePhenotypicData::StoreDiagnosticTraits(void){
	//morph traits are '-' if cline is high to low values, and '+' if low to high
	if(diagTraitsRecordedFlag_){return;}
	diagnosticTraitsR.Clear();
	aString endCharList("()[]{}; ");
	for(int tr=0;tr<rawTraitName.len();++tr){//do for every trait first
		aString nameAndDiagTraits(rawTraitName[tr]);
		aString* trname = nameAndDiagTraits.ChopOffFirstWord(endCharList);
		delete trname; trname = 0;//throw away
		do{
			trname = nameAndDiagTraits.ChopOffFirstWord(endCharList);
			trname->CullChar(' ');//cull spaces
			if(trname->len()==0) {
					delete trname; trname = 0;//throw away
					diagnosticTraitsR.Append(NEW aString('+'));// string placeholder
					}
				else{
					diagnosticTraitsR.Append(trname);
					trname = 0;//forget this
					}
			}while(nameAndDiagTraits.len()>0);
		}//end for tr
	for(int tr=skippedTraitNum.len()-1;tr>=0;--tr){//remove skipped loci
		int theTrait = skippedTraitNum[tr];
		diagnosticTraitsR.Delete(theTrait);}
	}//ClinePhenotypicData::StoreDiagnosticTraits()



void ClinePhenotypicData::StripDiagnosticTraits(aString& aTraitName){
	//strips list of trait names of their diagnostic trait notation
	aString endCharList("()[]{}");
	int endOfName = aTraitName.posEndFirstWord(endCharList);
	if(endOfName>0){
		aTraitName.trunc(endOfName-1);}
	}//end ClinePhenotypicData::StripDiagnosticTraits(aString&)



#endif



