// ClinePhenotypicData.h

#ifndef ClinePhenotypicDatah
#define ClinePhenotypicDatah = 1


#include "PopulationPhenotypicData.h"
#include "HybridIndexByLocation.h"
#include "HybridIndexByLocus.h"
#include "MetropolisHastings.h"
#include "ClineData.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"
//#include "AdamLibraryDebug.h"
#include "MemoryManager.h"


	class ClinePhenotypicData: public ClineData, public PopulationPhenotypicData{
/*
#ifdef MemoryLeakDebug
	private:
		MemLeakIDType memID_;
	public:
		static memAllocList memIDList_;
		static  MemLeakIDType memIDCount_;
#endif
*/
		public:
			ObjList<HybridIndexByLocation> hybridIndices;
			ObjList<HybridIndexByLocus> HybridIndicesByTrait;
			StringList diagnosticTraitsR;
			aList<int> centerPopList_;//ordinal pops for disequilibrium stats
			StatList<double> locality;//x-axis localities
			int localityFlag_;
			int diagTraitsRecordedFlag_;
		
		public:
		ClinePhenotypicData(void):localityFlag_(0),
								  diagTraitsRecordedFlag_(0){
			#ifdef MemoryLeakDebug
				memID_=++memIDCount_;
				memIDList_.Append(memID_);
			#endif
			}
		ClinePhenotypicData(aString& s):PopulationPhenotypicData(s),
									    localityFlag_(0),
									    diagTraitsRecordedFlag_(0){
			#ifdef MemoryLeakDebug
				memID_=++memIDCount_;
				memIDList_.Append(memID_);
			#endif
			}
virtual	~ClinePhenotypicData(void){
			#ifdef MemoryLeakDebug
				long i=memIDList_.Loc(memID_);
				memIDList_.Delete(i);
			#endif
			Clear();
			}

		ClinePhenotypicData(ClinePhenotypicData& parentData);//copy constructor

		ClinePhenotypicData(ClinePhenotypicData& parentData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets

	void 		CalculateHybridIndex(aList<double>& popLocation);
	void 		Clear(void);
	void 		CullTraits (StringList& traitsToCull);
	void 		CullSkippedTraits (void);
//	void		FitTraitCline(aString& dataFileName);
	double 		Locality(int pop);//returns x-axis locality
	int 		LocationRelativeToCenterList(double x);//0=left, 1=center, 2=right side of cline
	int			NumTraits(void){return ntraits;}
	int 		pops(void){return PopulationPhenotypicData::npop();}
	void 		PrintPhenotypeFile (aString& outFileName);
	void 		PrintPhenotypeFile (ostream& outFile);
	void 		PrintHybridIndexFile(aString& outFileName);
	void 		ReadDataFile (aString* dataFileName);//input file assumed closed
	void		RescoreDiagnosticTraits(void);//switches direction of clines originally from high to low
	void		ResetFlags(void);
	void 		RestoreTraits (StringList& traitsToRestore);
	void 		SetCenterPopList(aList<int>& centerList);
	void 		SetDataTypes(void);
	void 		StoreDiagnosticTraits(void);//append (-) to end of trait name if cline goes high to low
	void 		StripDiagnosticTraits(aString& aTraitName);


#ifdef MemoryLeakDebug
	static void PrintIDList(ostream& outFile=cout);
	static void PrintIDListLen(ostream& outFile=cout);
#endif
		};//end class ClinePhenotypicData
		
		
#endif

