/******************************************************
*
*	ClineShapeCWParamRec.cpp
*
*	Parameters for a tanh cline defined by center and width only.
*	Assumes the y-axis value of the cline varies between 0 and 1.
*
*	Use this as a virtual base class for derived classes that add parameters.
*
******************************************************/
#ifndef ClineShapeCWParamReccpp
#define ClineShapeCWParamReccpp = 1

#include "ClineShapeCWParamRec.h"

	
ParamRec* ClineShapeCWParamRec::Duplicate(void){
	ClineShapeCWParamRec* copy = NEW ClineShapeCWParamRec(*this);
	return copy;
	}//end ClineShapeCWParamRec::Duplicate()


ClineShapeCWParamRec* ClineShapeCWParamRec::_Duplicate(void){
	ClineShapeCWParamRec* copy = NEW ClineShapeCWParamRec(*this);
	return copy;
	}//end ClineShapeCWParamRec::_Duplicate()


//void ClineShapeCWParamRec::SetPointerToDerivedSelf(void){
//  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in ClineShapeCWParamRec"<<endl;
//  	}//end

long ClineShapeCWParamRec::GetPointerToDerivedSelf(void){
  	return (long) this;
  	}//end


long double ClineShapeCWParamRec::center(void){
	return c_;}


void ClineShapeCWParamRec::center(long double c){
	c_ = c;}


int ClineShapeCWParamRec::ChangeCenter(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false;
	long double oldCenter = c_;
	do{
		c_ = ran.NormDev(oldCenter, sigma);
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWParamRec::ChangeCenter(Ran3&, double, ParamRec*, ParamRec*)


int ClineShapeCWParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, long double weight){
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
//	ClineShapeCWParamRec* deviations = (ClineShapeCWParamRec*) paramDeviations->pointerToDerivedSelf;
	ClineShapeCWParamRec* deviations = (ClineShapeCWParamRec*) paramDeviations->GetPointerToDerivedSelf();
	switch (paramNum){
	 case 0:
	 	 return ChangeCenter(ran, deviations->center()*weight, lowerLimits, upperLimits);
	 	 break;
	 case 1:
	 	 return ChangeWidth(ran, deviations->width()*weight, lowerLimits, upperLimits);
	 	 break;
	 default:
		aString errstr("bug: can't change shape param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 2-parameter cline model");
		aError(errstr);
	 	return 0;
	 	}
	 return 0;
	}//end ClineShapeCWParamRec::ChangeParam(int)

/*
int ClineShapeCWParamRec::ChangeParams(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	int validchange = false;
	int paramToChange = RandomParameter(ran);
	do{
		validchange = ChangeParam(paramToChange, ran, 
										paramDeviations, lowerLimits, upperLimits);
		}while(!(validchange));
	return paramToChange;
	}//end ClineShapeCWParamRec::ChangeParams(Ran3&, ParamRec*, ParamRec*, ParamRec*)
*/

int ClineShapeCWParamRec::ChangeWidth(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false;
	long double oldWidth = w_;
	do{
		w_ = ran.NormDev(oldWidth, sigma);
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWParamRec::ChangeWidth(Ran3&, double, ParamRec*, ParamRec*)



long double ClineShapeCWParamRec::Cline_at_X(long double x, int trait){
    if(w_ == (long double)0.0){
    	if(x==c_){return (long double)0.5;}
    		else if(x<c_){return leftVal(trait);}
    		else{//x>c_
    			return rightVal(trait);}
    	}
    long double p  =  ((long double)1.0 + Tanh((long double)2.0 * (x - c_) / w_)) / (long double)2.0;
	long double pp = leftVal(trait) + ( rightVal(trait) - leftVal(trait)) * p;
	return pp;
	}//end ClineShapeCWParamRec::Cline_at_X(double, int)


long double ClineShapeCWParamRec::ExpectedValue(aList<long double>& coords){
	return Cline_at_X(coords[0],0);
	}//end ParamRec::ExpectedValue(aList<double>&)



long double ClineShapeCWParamRec::leftVal(int /*trait*/){
	//average limiting score on left side
	if(w_ > (long double) 0.0){
		return (long double)0.0;}
	if(w_ == (long double)0.0){//could introduce a bug, if it's a perfect step cline with left high
		return (long double)0.0;}//but a decision has to be made
	return 1.0;
	}//end ClineShapeCWParamRec::leftVal()


long double ClineShapeCWParamRec::maxSlope(void){
	if(w_==0.0){return 0.0;}
	return 1.0/w_;
	}//end ClineShapeCWParamRec::maxSlope()


void ClineShapeCWParamRec::Initialize(ClineFitInitializer& i, int trait_){
	lnL_=0.0;
	lnLmaxMinusLnL_=0.0;
	c_= i.c_[trait_];
	w_ = i.w_[trait_];
	}//end ClineShapeCWParamRec::Initialize(ClineFitInitializer&)


int ClineShapeCWParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
	//use type conversion that circumvents usual virtual base class restrictions
//	ClineShapeCWParamRec* lower = (ClineShapeCWParamRec*) lowerLimits->pointerToDerivedSelf;
//	ClineShapeCWParamRec* upper = (ClineShapeCWParamRec*) upperLimits->pointerToDerivedSelf;
	ClineShapeCWParamRec* lower = (ClineShapeCWParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineShapeCWParamRec* upper = (ClineShapeCWParamRec*) upperLimits->GetPointerToDerivedSelf();
	if(c_ <lower->c_){return 0;}
		else if (c_ > upper->c_){return 0;}
		else {}
	if(w_ == 0.0){return 0;}//a singularity in the model
	return 1;
	}//end ClineShapeCWParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)


void ClineShapeCWParamRec::MultiplyParam(int paramNum, long double factor){
	//use to rescale a param deviation
	switch (paramNum){
	 case 0:
	 	 c_ *= factor;
	 	 break;
	 case 1:
	 	 w_ *= factor;
	 	 break;
	 default:
		aString errstr("bug: can't rescale param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 2-parameter cline model");
		aError(errstr);
	 	}
	}//ClineShapeCWParamRec::MultiplyParam(int, double)


int ClineShapeCWParamRec::NumParams(void){
	return 2;
	}//end ClineShapeCWParamRec::NumParams()


int ClineShapeCWParamRec::NumDerivedParams(void){
	return 0;
	}//end ClineShapeCWParamRec::NumParams()



aString* ClineShapeCWParamRec::ParamName(int paramNum){
	switch (paramNum){
	 case 0:
	 	 return NEW aString("center");
	 	 break;
	 case 1:
	 	 return NEW aString("width");
	 	 break;
	 default:
		aString errstr("bug: can't return name of param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 2-parameter cline model");
		aError(errstr);
	 	}
	return NEW aString();
	}//end ClineShapeCWParamRec::ParamName(int )



void ClineShapeCWParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::ParamString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat(c_,6,15);
	dataline.cat(sepchar);
	dataline.cat(w_,6,15);
	}//end ClineShape2ParamRec::ParamString(aString&, char)



long double ClineShapeCWParamRec::ParamValue(int paramNum){
	//returns value of parameter; first varyng params, then derived params
	switch (paramNum){
	 case 0:
	 	 return center();
	 	 break;
	 case 1:
	 	 return width();
	 	 break;
	 default:
		aString errstr("bug: can't return value of param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 2-parameter cline model");
		aError(errstr);
	 	}
	return 0.0;
	}//end ClineShapeCWParamRec::ParamValue(int )


void ClineShapeCWParamRec::Print(ostream& outFile, long rep, aString& message){
	ParamRec::Print(outFile, rep, message);
	aString outStr("center: ");
	outStr.cat(center(),6,4);
	outStr.separator(5);
	outStr.cat("width: ");
	outStr.cat(width(),6,4);
	outFile<<outStr<<endl;
	}//ClineShapeCWParamRec::Print


long double ClineShapeCWParamRec::rightVal(int /*trait*/){
	//average limiting score on right side
	if(w_ > (long double)0.0){
		return (long double)1.0;}
	if(w_ == (long double)0.0){//could introduce a bug, if it's a perfect step cline with left high
		return (long double)1.0;}//but a decision has to be made
	return (long double)0.0;
	}//end ClineShape2ParamRec::rightVal()


void ClineShapeCWParamRec::SetParamValue(int paramNum, long double newVal){
	//set value of parameter
	switch (paramNum){
	 case 0:
	 	 center(newVal);
	 	 break;
	 case 1:
	 	 width(newVal);
	 	 break;
	 default:
		aString errstr("bug: can't set value of param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 2-parameter cline model");
		aError(errstr);
	 	}
	}//end ClineShapeCWParamRec::SetParamValue(int,double)



void ClineShapeCWParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::TitleString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat("center");
	dataline.cat(sepchar);
	dataline.cat("width");
	}//end ClineShapeCWParamRec::TitleString(aString&, char)


//int ClineShapeCWParamRec::TotalParams(void){
//	return NumParams();}



long double ClineShapeCWParamRec::width(void){
	return w_;}	


void ClineShapeCWParamRec::width(long double w){
	w_ = w;}	




void ClineShapeCWParamRec::operator=(ParamRec& p){
	ParamRec::operator=(p);
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means p must actually point to an object of class ClineShape2ParamRec
//	ClineShapeCWParamRec* pp = (ClineShapeCWParamRec*) p.pointerToDerivedSelf;
	ClineShapeCWParamRec* pp = (ClineShapeCWParamRec*) p.GetPointerToDerivedSelf();
	c_= pp->c_;
	w_ = pp->w_;
	}//end  ClineShapeCWParamRec::operator=(ParamRec&)





int	operator==(ClineShapeCWParamRec& p, ClineShapeCWParamRec& q){
	ParamRec& a = p;
	ParamRec& b = q;
	return ((a == b) && (p.c_ == q.c_) && (p.w_ == q.w_));
	}//end operator==(ClineShapeCWParamRec&, ClineShapeCWParamRec&)


int	operator!=(ClineShapeCWParamRec& p, ClineShapeCWParamRec& q){
	return !(p==q);
	}//end operator!=(ClineShapeCWParamRec&, ClineShapeCWParamRec&)



#endif


