/******************************************************
*
*	ClineShapeCWTBParamRec.cpp
*
*	Parameters for a tanh cline defined by center,width, 
*	maximum (top) & minimum (bottom) only.
*	Assumes the y-axis value of the cline varies between top and bottom.
*	There may be as many traits as desired, each with their own top & bottom,
*	but sharing a common center and width.  
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/
#ifndef ClineShapeCWTBParamReccpp
#define ClineShapeCWTBParamReccpp = 1

#include "ClineShapeCWTBParamRec.h"

ClineShapeCWTBParamRec::ClineShapeCWTBParamRec(void):ClineShapeCWParamRec(){
//  				SetPointerToDerivedSelf();
//  	cout<<"constructing ClineShapeCWTBParamRec()"<<endl;
  	}


ClineShapeCWTBParamRec::ClineShapeCWTBParamRec(long double c, long double w,
  						long double leftSideScore, long double rightSideScore):
  					ClineShapeCWParamRec(c,w),
  					pL_(leftSideScore), 
  					pR_(rightSideScore){
  	c_=c;
  	w_=w;
//  	SetPointerToDerivedSelf();
//  	cout<<"constructing ClineShapeCWTBParamRec(double...)"<<endl;
  	}


ClineShapeCWTBParamRec::ClineShapeCWTBParamRec(ClineFitInitializer& i, int trait_):
  					pL_(i.aveScoreL_[trait_]), 
  					pR_(i.aveScoreR_[trait_]){
  	c_=i.c_[trait_];
  	w_=i.w_[trait_];
//  	SetPointerToDerivedSelf();
//  	cout<<"constructing ClineShapeCWTBParamRec(double...)"<<endl;
  	}


ClineShapeCWTBParamRec::ClineShapeCWTBParamRec(ClineShapeCWTBParamRec& p):
  					pL_(p.pL_), 
  					pR_(p.pR_){
	lnL_ = p.lnL_;
	lnLmaxMinusLnL_ = p.lnLmaxMinusLnL_;
  	c_=p.c_;
  	w_=p.w_;
//  	SetPointerToDerivedSelf();
	}


ClineShapeCWTBParamRec::~ClineShapeCWTBParamRec(void){
//  	pointerToDerivedSelf = NULL;//forget
  	Clear();
  	}



ParamRec* ClineShapeCWTBParamRec::Duplicate(void){
	ClineShapeCWTBParamRec* copy = NEW ClineShapeCWTBParamRec(*this);
	return copy;
	}//end ClineShapeCWTBParamRec::Duplicate()


ClineShapeCWTBParamRec* ClineShapeCWTBParamRec::_Duplicate(void){
	ClineShapeCWTBParamRec* copy = NEW ClineShapeCWTBParamRec(*this);
	return copy;
	}//end ClineShapeCWTBParamRec::_Duplicate()


//void ClineShapeCWTBParamRec::SetPointerToDerivedSelf(void){
//  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in ClineShapeCWTBParamRec"<<endl;
//  	}//end

long ClineShapeCWTBParamRec::GetPointerToDerivedSelf(void){
  	return (long) this;
  	}//end


int ClineShapeCWTBParamRec::ChangeLeftScore(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, long double weight){
	//use normal distribution to vary score
	//return 0 for failure, 1 for success
	ClineShapeCWTBParamRec* deviations = (ClineShapeCWTBParamRec*) paramDeviations->GetPointerToDerivedSelf();
	int validChange=false;
	long double oldScore = pL_;
	long double sigma = deviations->pL_;
	do{
		pL_ = ran.NormDev(oldScore, sigma*weight);
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTBParamRec::ChangeLeftScore(Ran3&, double, ParamRec*, ParamRec*)


int ClineShapeCWTBParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, long double weight){
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
	int n = ClineShapeCWParamRec::NumParams();
	if(paramNum < n){
		return ClineShapeCWParamRec::ChangeParam(paramNum, ran, paramDeviations, 
							lowerLimits, upperLimits,weight);}
	paramNum -= n;
	if(paramNum == 0){
	 	return ChangeLeftScore(ran, paramDeviations, lowerLimits, upperLimits, weight);
	 	}
	 else if(paramNum == 1){
	 	return ChangeRightScore(ran, paramDeviations, lowerLimits, upperLimits, weight);
	 	}
	//default 	
	aString errstr("bug: can't change shape param #");
	errstr.cat(paramNum+1);
	errstr.cat(" in a ");
	errstr.cat(NumParams()+1);
	errstr.cat("-parameter cline model");
	aError(errstr);
	return 0;
	}//end ClineShapeCWTBParamRec::ChangeParam(int)



int ClineShapeCWTBParamRec::ChangeRightScore(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, long double weight){
	//use normal distribution to vary score
	//return 0 for failure, 1 for success
	ClineShapeCWTBParamRec* deviations = (ClineShapeCWTBParamRec*) paramDeviations->GetPointerToDerivedSelf();
	int validChange=false;
	long double oldScore = pR_;
	long double sigma = deviations->pR_;
	do{
		pR_ = ran.NormDev(oldScore, sigma*weight);
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTBParamRec::ChangeRightScore(Ran3&, double, ParamRec*, ParamRec*)



void ClineShapeCWTBParamRec::Clear(void){
  	}//end ClineShapeCWTBParamRec::Clear()


long double ClineShapeCWTBParamRec::Cline_at_X(long double x, int trait){
	long double diff = rightVal(trait) - leftVal(trait);
	long double score = ABS(diff);
	long double val = (long double)2.0 * (x - c_) / w_;
	val = MAX(val, (long double)-69.0);//numbers beyond these will give overrun errors
	val = MIN(val,  (long double)69.0);//and be equivalent to 0 and 1
	long double tanh = Tanh(val);
    long double p = diff * (1.0 + tanh) / (long double)2.0;
	return leftVal(trait) + p;
	}//end ClineShapeCWTBParamRec::Cline_at_X(double, int)


long double ClineShapeCWTBParamRec::ExpectedValue(aList<long double>& coords){
	return Cline_at_X(coords[0]);
	}//end ClineShapeCWTBParamRec::ExpectedValue(aList<double>&)



long double ClineShapeCWTBParamRec::leftVal(int /*trait*/){
	//average limiting score on left side
	return pL_;
	}//end ClineShapeCWTBParamRec::leftVal(int)


void ClineShapeCWTBParamRec::leftVal(long double newVal, int /*trait*/){
	//average limiting score on left side
	pL_=newVal;
	}//end ClineShapeCWTBParamRec::leftVal(double,int)



int ClineShapeCWTBParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
	//use type conversion that circumvents usual virtual base class restrictions
	ClineShapeCWTBParamRec* lower = (ClineShapeCWTBParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineShapeCWTBParamRec* upper = (ClineShapeCWTBParamRec*) upperLimits->GetPointerToDerivedSelf();
	if(pL_ <lower->pL_){return 0;}
		else if (pL_ > upper->pL_){return 0;}
		else {}
	if(pR_ <lower->pR_){return 0;}
		else if (pR_ > upper->pR_){return 0;}
		else {}
	return ClineShapeCWParamRec::MeetsParamCriteria (lowerLimits, upperLimits);
	}//end ClineShapeCWTBParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)


void ClineShapeCWTBParamRec::MultiplyParam(int paramNum, long double factor){
	//use to rescale a param deviation
	int n = ClineShapeCWParamRec::NumParams();
	if(paramNum<n){
		ClineShapeCWParamRec::MultiplyParam(paramNum, factor);
		return;
		}
	paramNum -= n;
	if(paramNum ==0){
		 	pL_ *= factor;
		 	return;
		 	}
		else if(paramNum ==1){
		 	pR_ *= factor;
		 	return;
		 	}
		else{}
	//default 	
	aString errstr("bug: can't rescale param #");
	errstr.cat(paramNum+1);
	errstr.cat(" in a ");
	errstr.cat(NumParams()+1);
	errstr.cat("-parameter cline model");
	aError(errstr);
	}//ClineShapeCWTBParamRec::MultiplyParam(int, double)


int ClineShapeCWTBParamRec::NumParams(void){
	int n = ClineShapeCWParamRec::NumParams() + 2;//left, right scores
	return n;
	}//end ClineShapeCWTBParamRec::NumParams()



int ClineShapeCWTBParamRec::NumDerivedParams(void){
	return 0;
	}//end ClineShapeCWTBParamRec::NumDerivedParams()



aString* ClineShapeCWTBParamRec::ParamName(int paramNum){
	//returns value of parameter; first varying params, then derived params
	int n = ClineShapeCWParamRec::NumParams();
	if(paramNum<n){
		return ClineShapeCWParamRec::ParamName(paramNum);
		}
	paramNum -= n;
	aString* newStr = NEW aString();
	if(paramNum==0){
			newStr->cat("pL");
			}
		else if(paramNum==1){
			newStr->cat("pR");
			}
		else{
			//default
			aString errstr("bug: can't return name of param #");
			errstr.cat(paramNum+1);
			errstr.cat(" in a ");
			errstr.cat(NumParams()+1);
			errstr.cat("-parameter cline model");
			aError(errstr);
			newStr->clear();
			}
	return newStr;
	}//end ClineShapeCWTBParamRec::ParamName(int )



void ClineShapeCWTBParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ClineShapeCWParamRec::ParamString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat(pL_,6,15);
	dataline.cat(sepchar);
	dataline.cat(pR_,6,15);
	}//end ClineShapeCWTBParamRec::ParamString(aString&, char)



long double ClineShapeCWTBParamRec::ParamValue(int paramNum){
	//returns value of parameter; first varying params, then derived params
	int n = ClineShapeCWParamRec::NumParams();
	if(paramNum<n){
		return ClineShapeCWParamRec::ParamValue(paramNum);}
	paramNum -= n;
	if(paramNum==0){
			return pL_;}
		else if(paramNum==1){
			return pR_;}
		else{}
	//default
	aString errstr("bug: can't return value of param #");
	errstr.cat(paramNum+1);
	errstr.cat(" in a ");
	errstr.cat(NumParams()+1);
	errstr.cat("-parameter cline model");
	aError(errstr);
	return 0.0;
	}//end ClineShapeCWTBParamRec::ParamValue(int )


void ClineShapeCWTBParamRec::Print(ostream& outFile, long rep, aString& message){
	ClineShapeCWParamRec::Print(outFile, rep, message);
	aString outStr("pL: ");
	outStr.cat((double)leftVal(),6,9);
	outStr.separator(5);
	outStr.cat("pR: ");
	outStr.cat((double)rightVal(),6,9);
	outFile<<outStr<<endl;
	}//ClineShapeCWParamRec::Print


long double ClineShapeCWTBParamRec::rightVal(int /*trait*/){
	//average limiting score on right side
	return pR_;
	}//end ClineShapeCWTBParamRec::rightVal()



void ClineShapeCWTBParamRec::rightVal(long double newVal, int /*trait*/){
	//sets average limiting score on right side
	pR_=newVal;
	}//end ClineShapeCWTBParamRec::rightVal()



void ClineShapeCWTBParamRec::SetParamValue(int paramNum, long double newVal){
	int n = ClineShapeCWParamRec::NumParams(), orig = paramNum;
	if(paramNum<n){
		ClineShapeCWParamRec::SetParamValue(paramNum,newVal);
		return;
		}
	paramNum -= n;
	if(paramNum ==0){
			pL_ = newVal;}
		else if(paramNum==1){
			pR_ = newVal;}
		else{
			aString errstr("bug: can't set value of param #");
			errstr.cat(orig+1);
			errstr.cat(" in a 4-parameter cline model");
			aError(errstr);
			}
	}//end ClineShapeCWTBParamRec::SetParamValue(int,double)



void ClineShapeCWTBParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ClineShapeCWParamRec::TitleString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat("pL");
	dataline.cat(sepchar);
	dataline.cat("pR");
	}//end ClineShapeCWTBParamRec::TitleString(aString&, char)


//int ClineShapeCWTBParamRec::TotalParams(void){
//	return NumParams();}

/*  left over from defunct multi-trait parameter records
int ClineShapeCWTBParamRec::trait(int paramNum){
	int n = ClineShapeCWParamRec::NumParams();
	if(paramNum<n){
		aString errstr("bug: can't return trait #");
		errstr.cat(paramNum-n);
		errstr.cat(" in a ");
		errstr.cat(traits());
		errstr.cat("-trait cline model");
		aError(errstr);
		return 0;}
	paramNum -= n;
	if(paramNum<pL_.len()){
		return paramNum;}
	paramNum -= pL_.len();
	if(paramNum<pR_.len()){
		return paramNum;}
	//too high
	aString errstr("bug: can't return trait #");
	errstr.cat(paramNum+1);
	errstr.cat(" in a ");
	errstr.cat(traits());
	errstr.cat("-trait cline model");
	aError(errstr);
	return 0;
	}//end ClineShapeCWTBParamRec::trait(int)
*/	
/*
int ClineShapeCWTBParamRec::traits(void){
	return pL_.len();
	}//end  ClineShapeCWTBParamRec::traits()
*/


void ClineShapeCWTBParamRec::operator=(ParamRec& p){
//	ClineShapeCWParamRec::operator=(p);
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means pp must actually point to an object of class ClineShapeCWTBParamRec
	ClineShapeCWTBParamRec* pp = (ClineShapeCWTBParamRec*) p.GetPointerToDerivedSelf();
	lnL_= pp->lnL_;
	lnLmaxMinusLnL_ = pp->lnLmaxMinusLnL_;	
	c_= pp->c_;
	w_ = pp->w_;
	pL_= pp->pL_;
	pR_ = pp->pR_;
	}//end  ClineShapeCWTBParamRec::operator=(ParamRec&)





int	operator==(ClineShapeCWTBParamRec& p, ClineShapeCWTBParamRec& q){
	ClineShapeCWParamRec& a = p;
	ClineShapeCWParamRec& b = q;
	return ((a == b) && 
			(p.pL_ == q.pL_) && 
			(p.pR_ == q.pR_));
	}//end operator==(ClineShapeCWParamRec&, ClineShapeCWParamRec&)


int	operator!=(ClineShapeCWTBParamRec& p, ClineShapeCWTBParamRec& q){
	return !(p==q);
	}//end operator!=(ClineShapeCWParamRec&, ClineShapeCWParamRec&)



#endif


