/******************************************************
*
*	ClineShapeCWTlTrDParamRec.h
*
*	Parameters for a 3-part composite tanh cline defined by center, width,	
*	thetaL, thetaR, zL & zR.  The latter 4 parameters describe introgression
*	slopes and intercepts on the Left and Right sides of the cline.  Disequilibrium
*	is also estimated and derived parameters calculated.
*	Assumes the y-axis value of the cline varies between 0 and 1.
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/

#ifndef ClineShapeCWTlTrDParamRech
#define ClineShapeCWTlTrDParamRech = 1


#include "ParamRec.h"
#include "Hyperbolics.h"
#include "InlineTemplates.h"
#include "ClineFitInitializer.h"
#include "ClineShapeCWParamRec.h"
#include "ClineShapeCWTlTrParamRec.h"
#include "ClineDiseqParamRec.h"
#include "MemoryManager.h"

class ClineShapeCWTlTrDParamRec;
class ClineShapeCWTlTrDParamRec: 
						virtual public ParamRec, 
						virtual public ClineShapeCWTlTrParamRec,
						virtual public ClineDiseqParamRec
						{
  public:
	 long double r_;//harmonic mean recombination rate 	
  public:
  	ClineShapeCWTlTrDParamRec(void);
  	ClineShapeCWTlTrDParamRec(long double c, long double w,//does not set xL_ or xR_; must do this separately
							 long double tL, long double tR,//use for high/low limits and deviations
							 long double zL, long double zR,
							 long double r, long double D);
  	ClineShapeCWTlTrDParamRec(long double c, long double w,//sets xL_ and xR_
							  long double tL, long double tR,
							  long double zL, long double zR,
							  long double r, long double D,							 
							 double leftEnd, double rightEnd);
  	ClineShapeCWTlTrDParamRec(ClineFitInitializer& i);
  	ClineShapeCWTlTrDParamRec(ClineShapeCWTlTrDParamRec& p);
  	virtual ~ClineShapeCWTlTrDParamRec(void){
//  			pointerToDerivedSelf = NULL;//forget
  				}

  	virtual ParamRec* 				Duplicate(void);
  	virtual ClineShapeCWTlTrDParamRec* 	_Duplicate(void);
//	virtual void 					SetPointerToDerivedSelf(void);//always override in derived
	virtual long 					GetPointerToDerivedSelf(void);//always override in derived
  	


			long double r(void);//returns r_
			void 		r(long double rr);//sets r_
	virtual long double distanceBarrierL(void);//barrier to introgression from left to right in distance units
	virtual long double distanceBarrierR(void);//barrier to introgression from right to left in distance units

	virtual int 		ChangeCenter(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeWidth(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeThetaL(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeThetaR(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeZL(Ran3& ran, double sigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeZR(Ran3& ran, double sigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);

	virtual int 		ChangeParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
									ParamRec* paramDeviations,  
									ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=1.0);
	virtual long double DispersalSigma();
	virtual void 		Initialize(ClineFitInitializer& i);
	virtual int 		MeetsParamCriteria(ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void 		MultiplyParam(int paramNum, long double factor);//use to rescale a param deviation
	virtual int 		NumParams(void);//number of parameters
	virtual int 		NumDerivedParams(void);//number of parameters that are functions of varying parameters
	virtual long double NumSelectedLoci(void);//assumes many loci and selection via genetic background
//	virtual void 		ParamCoords(int globalParamNum, int& trait, int& traitParam);
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
	virtual long double S(void);//effective selection against hybrids in center as if hybrids were single-locus heterozygotes
	virtual void 		SetParamValue(int paramNum, long double newVal);//sets value of parameter

	virtual long double s_perMarkerLocusL(void);//ave. direct selection on marker loci in left tail of cline
	virtual long double s_perMarkerLocusR(void);//ave. direct selection on marker loci in right tail of cline
	virtual long double s_perReproIsolationLocus(void);//average selection on a locus contributing to reproductive isolation
	
						//indirect & direct selection on marker loci in middle of cline
	virtual long double s_star_vs_Hets(void);//assuming genetic background incompatibilities
	virtual long double s_star_Ecotone(void);//assuming selection across an ecotone
	virtual long double s_star_DomEcotone(void);//assuming selection across an ecotone on a dominant marker

	virtual long double timeBarrierL(void);
	virtual long double timeBarrierR(void);
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file
	virtual	long double WhybridOverWpure(void);//ratio in cline center

	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

	friend int	operator==(ClineShapeCWTlTrDParamRec& p, ClineShapeCWTlTrDParamRec& q);
	friend int	operator!=(ClineShapeCWTlTrDParamRec& p, ClineShapeCWTlTrDParamRec& q);

	};//end class ClineShapeCWTlTrDParamRec



#endif