/******************************************************
*
*	ClineShapeCWTlTrParamRec.cpp
*
*	Parameters for a 3-part composite tanh cline defined by center, width,	
*	thetaL, thetaR, zL & zR.  The latter 4 parameters describe introgression
*	slopes and intercepts on the Left and Right sides of the cline.
*	Assumes the y-axis value of the cline varies between 0 and 1.
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/

#ifndef ClineShapeCWTlTrParamReccpp
#define ClineShapeCWTlTrParamReccpp = 1

#include "ClineShapeCWTlTrParamRec.h"

	
ClineShapeCWTlTrParamRec::ClineShapeCWTlTrParamRec(void):
				thetaL_((long double)1.0),thetaR_((long double)1.0),
				zL_((long double)0.0),zR_((long double)0.0),
				xL_(c_),xR_(c_){
//  	SetPointerToDerivedSelf();
  	}


ClineShapeCWTlTrParamRec::ClineShapeCWTlTrParamRec(long double c, long double w,
												   long double tL, long double tR,
												   long double zL, long double zR):
					thetaL_(tL),thetaR_(tR),zL_(zL),zR_(zR){
//  	SetPointerToDerivedSelf();
	lnL_= 0.0;
	lnLmaxMinusLnL_ = 0.0;	
  	c_=c;
  	w_=w;
  	}


ClineShapeCWTlTrParamRec::ClineShapeCWTlTrParamRec( long double c, long double w,
													long double tL, long double tR,
													long double zL, long double zR,
													double leftEnd, double rightEnd):
					thetaL_(tL),thetaR_(tR),zL_(zL),zR_(zR){
//  	SetPointerToDerivedSelf();
	lnL_= 0.0;
	lnLmaxMinusLnL_ = 0.0;	
  	c_=c;
  	w_=w;
  	LJunction(leftEnd);
  	RJunction(rightEnd);
  	}


ClineShapeCWTlTrParamRec::ClineShapeCWTlTrParamRec(ClineFitInitializer& i, int trait_):
					thetaL_(i.thetaL_[trait_]),thetaR_(i.thetaR_[trait_]),
					zL_(i.zL_[trait_]),zR_(i.zR_[trait_]){
//  	SetPointerToDerivedSelf();
	lnL_= 0.0;
	lnLmaxMinusLnL_ = 0.0;	
  	c_=i.c_[trait_];
  	w_=i.w_[trait_];
  	LJunction(i.leftEnd_);
  	RJunction(i.rightEnd_);
  	}


ClineShapeCWTlTrParamRec::ClineShapeCWTlTrParamRec(ClineShapeCWTlTrParamRec& p):
					thetaL_(p.thetaL_),thetaR_(p.thetaR_),
					zL_(p.zL_),zR_(p.zR_),xL_(p.xL_),xR_(p.xR_){
//  	SetPointerToDerivedSelf();
	lnL_= p.lnL_;
	lnLmaxMinusLnL_ = p.lnLmaxMinusLnL_;	
  	c_=p.c_;
  	w_=p.w_;
	}



ParamRec* ClineShapeCWTlTrParamRec::Duplicate(void){
	ClineShapeCWTlTrParamRec* copy = NEW ClineShapeCWTlTrParamRec(*this);
	return copy;
	}//end ClineShapeCWTlTrParamRec::Duplicate()


ClineShapeCWTlTrParamRec* ClineShapeCWTlTrParamRec::_Duplicate(void){
	ClineShapeCWTlTrParamRec* copy = NEW ClineShapeCWTlTrParamRec(*this);
	return copy;
	}//end ClineShapeCWTlTrParamRec::_Duplicate()


//void ClineShapeCWTlTrParamRec::SetPointerToDerivedSelf(void){
//  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in ClineShapeCWTlTrParamRec"<<endl;
//  	}//end ClineShapeCWTlTrParamRec::SetPointerToDerivedSelf()

long ClineShapeCWTlTrParamRec::GetPointerToDerivedSelf(void){
  	return (long) this;
  	}//end ClineShapeCWTlTrParamRec::SetPointerToDerivedSelf()


long double ClineShapeCWTlTrParamRec::thetaL(void){
	return thetaL_;}


void ClineShapeCWTlTrParamRec::thetaL(long double th){
	thetaL_ = th;}


long double ClineShapeCWTlTrParamRec::thetaR(void){
	return thetaR_;}


void ClineShapeCWTlTrParamRec::thetaR(long double th){
	thetaR_ = th;}


long double ClineShapeCWTlTrParamRec::zL(void){
	return zL_;}


void ClineShapeCWTlTrParamRec::zL(long double z){
	zL_ = z;}


long double ClineShapeCWTlTrParamRec::zR(void){
	return zR_;}


void ClineShapeCWTlTrParamRec::zR(long double z){
	zR_ = z;}


long double ClineShapeCWTlTrParamRec::xL(void){
	return xL_;}


long double ClineShapeCWTlTrParamRec::xR(void){
	return xR_;}


long double ClineShapeCWTlTrParamRec::B_L(int trait){//the frequency at the left junction
	return Cline_at_X(xL_,trait);}


long double ClineShapeCWTlTrParamRec::B_R(int trait){//the frequency at the right junction
	return Cline_at_X(xR_,trait);}

long double ClineShapeCWTlTrParamRec::DeltaP(int trait){//the frequency step across the center region
	return B_R(trait)-B_L(trait);}



int ClineShapeCWTlTrParamRec::ChangeCenter(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false, changeTries=0;
	long double oldCenter = c_;
	ClineShapeCWTlTrParamRec* lower = (ClineShapeCWTlTrParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineShapeCWTlTrParamRec* upper = (ClineShapeCWTlTrParamRec*) upperLimits->GetPointerToDerivedSelf();
	do{
		c_ = ran.NormDev(oldCenter, sigma);
		++changeTries;
		validChange = LJunction(lower->xL_);
		if(validChange){
			validChange = RJunction(upper->xR_);}
		if(validChange){
			validChange = MeetsParamCriteria(lowerLimits, upperLimits);}
		if((!(validChange ))&&(changeTries>100)){
//			aString err("Can\'t change center after 100 tries");
//			aError(err);
			c_= oldCenter;
			LJunction(lower->xL_);
			RJunction(upper->xR_);
			return false;
			}
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrParamRec::ChangeCenter(Ran3&, double, ParamRec*, ParamRec*)


int ClineShapeCWTlTrParamRec::ChangeThetaL(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false, changeTries=0;
	long double oldThL = thetaL_;
	ClineShapeCWTlTrParamRec* lower = (ClineShapeCWTlTrParamRec*) lowerLimits->GetPointerToDerivedSelf();
	do{
		thetaL_ = ran.NormDev(oldThL, sigma);
		++changeTries;
		validChange = LJunction(lower->xL_);
		if(validChange){
			validChange = MeetsParamCriteria(lowerLimits, upperLimits);}
		if((!(validChange ))&&(changeTries>100)){
//			aString err("Can\'t change thetaL after 100 tries");
//			aError(err);
			thetaL_ = oldThL;
			LJunction(lower->xL_);
			return false;
			}
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrParamRec::ChangeThetaL(Ran3&, double, ParamRec*, ParamRec*)


int ClineShapeCWTlTrParamRec::ChangeThetaR(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false, changeTries=0;
	long double oldThR = thetaR_;
	ClineShapeCWTlTrParamRec* upper = (ClineShapeCWTlTrParamRec*) upperLimits->GetPointerToDerivedSelf();
	do{
		thetaR_ = ran.NormDev(oldThR, sigma);
		++changeTries;
		validChange = RJunction(upper->xR_);
		if(validChange){
			validChange = MeetsParamCriteria(lowerLimits, upperLimits);}
		if((!(validChange ))&&(changeTries>100)){
//			aString err("Can\'t change thetaR after 100 tries");
//			aError(err);
			thetaR_ = oldThR;
			RJunction(upper->xR_);
			return false;
			}
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrParamRec::ChangeThetaR(Ran3&, double, ParamRec*, ParamRec*)


int ClineShapeCWTlTrParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, long double weight){
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
	ClineShapeCWTlTrParamRec* deviations = (ClineShapeCWTlTrParamRec*) paramDeviations->GetPointerToDerivedSelf();
	int success = 0;
	switch (paramNum){
	 case 0:
	 	  success =  ChangeCenter(ran, deviations->center()*weight, lowerLimits, upperLimits);
	 	 break;
	 case 1:
	 	  success =  ChangeWidth(ran, deviations->width()*weight, lowerLimits, upperLimits);
	 	 break;
	 case 2:
	 	  success =  ChangeThetaL(ran, deviations->thetaL()*weight, lowerLimits, upperLimits);
	 	 break;
	 case 3:
	 	  success =  ChangeThetaR(ran, deviations->thetaR()*weight, lowerLimits, upperLimits);
	 	 break;
	 case 4:
	 	  success =  ChangeZL(ran, deviations->zL()*weight, lowerLimits, upperLimits);
	 	 break;
	 case 5:
	 	  success =  ChangeZR(ran, deviations->zR()*weight, lowerLimits, upperLimits);
	 	 break;
	 default:
		aString errstr("bug: can't change shape param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 6-parameter cline model");
		aError(errstr);
	 	}
	return  success;
	}//end ClineShapeCWTlTrParamRec::ChangeParam(int,Ran3&,ParamRec*,ParamRec*,ParamRec*)


int ClineShapeCWTlTrParamRec::ChangeWidth(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false, changeTries=0;
	long double oldWidth = w_;
	ClineShapeCWTlTrParamRec* lower = (ClineShapeCWTlTrParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineShapeCWTlTrParamRec* upper = (ClineShapeCWTlTrParamRec*) upperLimits->GetPointerToDerivedSelf();
	do{
		w_ = ran.NormDev(oldWidth, sigma);
		++changeTries;
		validChange = LJunction(lower->xL_);
		if(validChange){
			validChange = RJunction(upper->xR_);}
		if(validChange){
			validChange = MeetsParamCriteria(lowerLimits, upperLimits);}
		if((!(validChange ))&&(changeTries>100)){
//			aString err("Can\'t change width after 100 tries");
//			aError(err);
			w_ = oldWidth;
			LJunction(lower->xL_);
			RJunction(upper->xR_);
			return false;
			}
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrParamRec::ChangeWidth(Ran3&, double, ParamRec*, ParamRec*)


int ClineShapeCWTlTrParamRec::ChangeZL(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false, changeTries=0;
	long double oldZL = zL_;
	ClineShapeCWTlTrParamRec* lower = (ClineShapeCWTlTrParamRec*) lowerLimits->GetPointerToDerivedSelf();
	do{
		zL_ = ran.NormDev(oldZL, sigma);
		++changeTries;
		validChange = LJunction(lower->xL_);
		if(validChange){
			validChange = MeetsParamCriteria(lowerLimits, upperLimits);}
		if((!(validChange ))&&(changeTries>100)){
//			aString err("Can\'t change zL after 100 tries");
//			aError(err);
			zL_ = oldZL;
			LJunction(lower->xL_);
			return false;
			}
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrParamRec::ChangeZL(Ran3&, double, ParamRec*, ParamRec*)


int ClineShapeCWTlTrParamRec::ChangeZR(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false, changeTries=0;
	long double oldZR = zR_;
	ClineShapeCWTlTrParamRec* upper = (ClineShapeCWTlTrParamRec*) upperLimits->GetPointerToDerivedSelf();
	do{
		zR_ = ran.NormDev(oldZR, sigma);
		++changeTries;
		validChange = RJunction(upper->xR_);
		if(validChange){
			validChange = MeetsParamCriteria(lowerLimits, upperLimits);}
		if((!(validChange ))&&(changeTries>100)){
//			aString err("Can\'t change zR after 100 tries");
//			aError(err);
			zR_ = oldZR;
			RJunction(upper->xR_);
			return false;
			}
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrParamRec::ChangeZR(Ran3&, double, ParamRec*, ParamRec*)




long double ClineShapeCWTlTrParamRec::Cline_at_X(long double x, int trait){
    long double p=0.0;
    long double t= rightVal(trait),b=leftVal(trait);
    if(w_ == (long double)0.0){
    	if(x==c_){return leftVal(trait) + ( rightVal(trait) - leftVal(trait))*(xR_+xL_)/(long double)2.0;}}
    if(x <= xL_){//left section of cline
    		p = exp((long double)4.0 * (x - (c_ + zL_)) * exp(ln(thetaL_)/(long double)2.0) / w_);//bug if w==0
    		}
    	else if( x < xR_){//center section
    		p = ((long double)1.0 + Tanh((long double)2.0 * (x - c_) / w_)) / 2.0;
    		}
    	else{//x>= xR_; right section of cline
    		p = (long double)1.0 - exp((long double)(-4.0) * (x - (c_ - zR_)) * exp(ln(thetaR_)/(long double)2.0) / w_);//bug if w==0
//    		return t - (t-b)*p;
    		}
	long double pp = b + (t - b) * p;
	return pp;
	}//end ClineShapeCWTlTrParamRec::Cline_at_X(double, int)



long double ClineShapeCWTlTrParamRec::ExpectedValue(aList<long double>& coords){
	return Cline_at_X(coords[0],0);
	}//end ClineShapeCWTlTrParamRec::ExpectedValue(aList<double>&)


void ClineShapeCWTlTrParamRec::Initialize(ClineFitInitializer& i, int trait_){
	lnL_=0.0;
	lnLmaxMinusLnL_=0.0;
	c_= i.c_[trait_];
	w_ = i.w_[trait_];
	thetaL_ = i.thetaL_[trait_];
	thetaR_ = i.thetaR_[trait_];
	zL_ = i.zL_[trait_];
	zR_ = i.zR_[trait_];
	LJunction(i.leftEnd_);
	RJunction(i.rightEnd_);
	}//end ClineShapeCWTlTrParamRec::Initialize(ClineFitInitializer&)


int ClineShapeCWTlTrParamRec::LJunction (double leftEnd){
	//finds the point where the left and center functions converge
	if(w_==0.0){xL_=c_;return 1;}
	if(thetaL_<0.0){return 0;}
	if(thetaL_==1.0){xL_=leftEnd; return 1;}
	double x1, x2, xLogP, xTanhP, diff1, diff2;
    int exitLoop, maxedOut;
    double t = rightVal(0), b = leftVal(0); 
	// they might not ever intersect, so 
	//strategy is to iterate down the curves and see where the difference converges
	int LJunctionFound = false;
	x1 = center();	//start at the center and find the difference between curves
	double incrementPower = 0.0;
	xLogP = b + (t - b) * exp(4.0 * (x1 - (c_ + zL_)) * exp(ln(thetaL_)/2.0) / w_);
	xTanhP = b + (t - b) * (1.0 + Tanh(2.0 * (x1 - c_) / w_)) / 2.0;
	diff1 = xTanhP - xLogP;
	if (diff1 > 0.0){//continue only if left function is below the center function
		exitLoop = false;
		do{//get function values at 2 locations and compare them
			maxedOut = false;
			x2 = x1 - exp(incrementPower * ln(10.0));//=1*10^incrementPower
     		xLogP = b + (t - b) * exp(4.0 * (x2 - (c_ + zL_)) * exp(ln(thetaL_)/2.0) / w_);
			if (ABS(2.0 * (x2 - c_) / w_) >= 6000.0){//maximum double
					xTanhP = b + (t - b) * 0.5;
					maxedOut = true;
					}
				else{
       				xTanhP = b + (t - b) * (1.0 + Tanh(2.0 * (x2 - c_) / w_)) / 2.0;
					}
			diff2 = xTanhP - xLogP;
			if (diff2 <= 0.0){//convergence point surrounded; check for convergence in middle
       				incrementPower--;
					}
				else{    
					if (diff2 > diff1){return 0;}   
					if (diff2 <= diff1){// probably converging to a point
						//so move to the left one step
						diff1 = diff2;
       					if (!maxedOut){
								x1 = x2;}
							else{
								incrementPower--;}
						}//end if diff2...
					if(x2 < leftEnd){//ran off the end
						xL_ = leftEnd;
						return 0;//LJunction not found
						}
					}//end if diff2<0 else
			if (incrementPower < -3.0){//close enough
				exitLoop = true;
				if (x2 < leftEnd){
						xL_ = leftEnd;}
					else{//just assign a value at this closest point
						xL_ = x2;}
       			LJunctionFound = true;
				}//end if incrementPower
			}while(! exitLoop);
		}//end if diff1>0
	return LJunctionFound;
	}//end ClineShapeCWTlTrParamRec::LJunction (void)




int ClineShapeCWTlTrParamRec::RJunction (double rightEnd){
	//finds the point where the right and center functions converge
	if(w_==0.0){xR_=c_;return 1;}
	if(thetaR_<0.0){return 0;}
	if(thetaR_==1.0){xR_=rightEnd;return 1;}
	double x1, x2, xLogP, xTanhP, diff1, diff2;
    int exitLoop, maxedOut;
    double t = rightVal(0), b = leftVal(0); 
	// they might not ever intersect, so 
	//strategy is to iterate down the curves and see where the difference converges
	int RJunctionFound = false;
	x1 = center();	//start at the center and find the difference between curves
	double incrementPower = 0.0;
	xLogP = t - (t - b) * exp(-4.0 * (x1 - (c_ - zR_)) * exp(ln(thetaR_)/2.0) / w_);
	xTanhP = b + (t - b) * (1.0 + Tanh(2.0 * (x1 - c_) / w_)) / 2.0;
	diff1 = xTanhP - xLogP;
	if (diff1 < 0.0){//continue only if right function is below the center function
		exitLoop = false;
		do{//get function values at 2 locations and compare them
			maxedOut = false;
			x2 = x1 + exp(incrementPower * ln(10.0));//=1*10^incrementPower
     		xLogP = t - (t - b) * exp(-4.0 * (x2 - (c_ - zR_)) * exp(ln(thetaR_)/2.0) / w_);
			if (ABS(2.0 * (x2 - c_) / w_) >= 3200.0){//maximum exp double
					xTanhP = b + (t - b) * 0.5;
					maxedOut = true;
					}
				else{
       				xTanhP = b + (t - b) * (1.0 + Tanh(2.0 * (x2 - c_) / w_)) / 2.0;
					}
			diff2 = xTanhP - xLogP;
			if (diff2 >= 0.0){//convergence point surrounded; check for convergence in middle
       				incrementPower--;
					}
				else{ 
					if (diff2 < diff1){return 0;}   
					if (diff2 >= diff1){// probably converging to a point
						//so move to the right one step
						diff1 = diff2;
       					if (!maxedOut){
								x1 = x2;}
							else{
								incrementPower--;}
						}//end if diff2...
					if(x2 > rightEnd){
						xR_ = rightEnd;
						return 0;// RJunction not found
						}
					}//end if diff2<0 else
			if (incrementPower < -3.0){//close enough
				exitLoop = true;
				if (x2 > rightEnd){
						xR_ = rightEnd;}
					else{//just assign a value at this closest point
						xR_ = x2;}
       			RJunctionFound = true;
				}//end if incrementPower
			}while(! exitLoop);
		}//end if diff1>0
	return RJunctionFound;
	}//end ClineShapeCWTlTrParamRec::RJunction (double)


int ClineShapeCWTlTrParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
	//use type conversion that circumvents usual virtual base class restrictions
	ClineShapeCWTlTrParamRec* lower = (ClineShapeCWTlTrParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineShapeCWTlTrParamRec* upper = (ClineShapeCWTlTrParamRec*) upperLimits->GetPointerToDerivedSelf();
	if(c_ <lower->c_){return 0;}
		else if (c_ > upper->c_){return 0;}
		else {}
	if(w_ <= 0.0){return 0;}
		else if (w_ > upper->w_){return 0;}
		else {}
	if(thetaL_ <lower->thetaL_){return 0;}
		else if (thetaL_ > upper->thetaL_){return 0;}
		else {}
	if(thetaR_ <lower->thetaR_){return 0;}
		else if (thetaR_ > upper->thetaR_){return 0;}
		else {}
	if(zL_ <lower->zL_){return 0;}
		else if (zL_ > upper->zL_){return 0;}
		else {}
	if(zR_ <lower->zR_){return 0;}
		else if (zR_ > upper->zR_){return 0;}
		else {}
	if((xL_ > xR_)||(xL_>c_)||(xR_<c_)){return 0;}
		else if(xL_ < lower->xL_){return 0;}
		else if(xR_ > upper->xR_){return 0;}
		else {}
		
	return 1;
	}//end ClineShapeCWTlTrParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)


void ClineShapeCWTlTrParamRec::MultiplyParam(int paramNum, long double factor){
	//use to rescale a param deviation
	switch (paramNum){
	 case 0:
	 	 c_ *= factor;
	 	 break;
	 case 1:
	 	 w_ *= factor;
	 	 break;
	 case 2:
	 	 thetaL_ *= factor;
	 	 break;
	 case 3:
	 	 thetaR_ *= factor;
	 	 break;
	 case 4:
	 	 zL_ *= factor;
	 	 break;
	 case 5:
	 	 zR_ *= factor;
	 	 break;
	 default:
		aString errstr("bug: can't rescale param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 6-parameter cline model");
		aError(errstr);
	 	}
	}//ClineShapeCWTlTrParamRec::MultiplyParam(int, double)


int ClineShapeCWTlTrParamRec::NumParams(void){
	return 6;
	}//end ClineShapeCWTlTrParamRec::NumParams()


int ClineShapeCWTlTrParamRec::NumDerivedParams(void){
	return 5;
	}//end ClineShapeCWTlTrParamRec::NumParams()



aString* ClineShapeCWTlTrParamRec::ParamName(int paramNum){
	//returns name of parameter; first varyng params, then derived params
	switch (paramNum){
	 case 0:
	 	 return NEW aString("center");
	 	 break;
	 case 1:
	 	 return NEW aString("width");
	 	 break;
	 case 2:
	 	 return NEW aString("thetaL");
	 	 break;
	 case 3:
	 	 return NEW aString("thetaR");
	 	 break;
	 case 4:
	 	 return NEW aString("zL");
	 	 break;
	 case 5:
	 	 return NEW aString("zR");
	 	 break;
	 case 6:
	 	 return NEW aString("xL");
	 	 break;
	 case 7:
	 	 return NEW aString("xR");
	 	 break;
	 case 8:
	 	 return NEW aString("B_L");
	 	 break;
	 case 9:
	 	 return NEW aString("B_R");
	 	 break;
	 case 10:
	 	 return NEW aString("deltaP");
	 	 break;
	 default:
		aString errstr("bug: can't return name of param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 6-parameter cline model");
		aError(errstr);
	 	}
	return NEW aString();
	}//end ClineShapeCWTlTrParamRec::ParamName(int )



void ClineShapeCWTlTrParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ClineShapeCWParamRec::ParamString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat(thetaL_,6,15);
	dataline.cat(sepchar);
	dataline.cat(zL_,6,15);
	dataline.cat(sepchar);
	dataline.cat(xL_,6,15);
	dataline.cat(sepchar);
	dataline.cat(thetaR_,6,15);
	dataline.cat(sepchar);
	dataline.cat(zR_,6,15);
	dataline.cat(sepchar);
	dataline.cat(xR_,6,15);
	dataline.cat(sepchar);
	dataline.cat(B_L(0),2,15);//base class assumes one trait
	dataline.cat(sepchar);
	dataline.cat(B_R(0),2,15);//base class assumes one trait
	dataline.cat(sepchar);
	dataline.cat(DeltaP(0),2,15);
	}//end ClineShapeCWTlTrParamRec::ParamString(aString&, char)



long double ClineShapeCWTlTrParamRec::ParamValue(int paramNum){
	//returns value of parameter; first varyng params, then derived params
	switch (paramNum){
	 case 0:
	 	 return center();
	 	 break;
	 case 1:
	 	 return width();
	 	 break;
	 case 2:
	 	 return thetaL();
	 	 break;
	 case 3:
	 	 return thetaR();
	 	 break;
	 case 4:
	 	 return zL();
	 	 break;
	 case 5:
	 	 return zR();
	 	 break;
	 case 6:
	 	 return xL();
	 	 break;
	 case 7:
	 	 return xR();
	 	 break;
	 case 8:
	 	 return B_L(0);//base class assumes one trait
	 	 break;
	 case 9:
	 	 return B_R(0);//base class assumes one trait
	 	 break;
	 case 10:
	 	 return DeltaP(0);//base class assumes one trait
	 	 break;
	 default:
		aString errstr("bug: can't return value of param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 6-parameter cline model");
		aError(errstr);
	 	}
	return 0.0;
	}//end ClineShapeCWTlTrParamRec::ParamValue(int )


void ClineShapeCWTlTrParamRec::Print(ostream& outFile, long rep, aString& message){
	ClineShapeCWParamRec::Print(outFile, rep, message);
	aString outStr("thetaL: ");
	outStr.cat(thetaL(),2,9);
	outStr.separator(2);
	outStr.cat("zL: ");
	outStr.cat(zL(),6,4);
	outStr.separator(2);
	outStr.cat("xL: ");
	outStr.cat(xL(),6,4);
	outFile<<outStr<<endl;
	outStr="thetaR: ";
	outStr.cat(thetaR(),2,9);
	outStr.separator(2);
	outStr.cat("zR: ");
	outStr.cat(zR(),6,4);
	outStr.separator(2);
	outStr.cat("xR: ");
	outStr.cat(xR(),6,4);
	outFile<<outStr<<endl;
	outStr="P_xL: ";
	outStr.cat(B_L(0),2,9);//base class assumes one trait
	outStr.separator(2);
	outStr.cat("P_xR: ");
	outStr.cat(B_R(0),2,9);//base class assumes one trait
	outStr.separator(2);
	outStr.cat("deltaP: ");
	outStr.cat(DeltaP(0),2,9);//base class assumes one trait
	outFile<<outStr<<endl;
	}//ClineShapeCWTlTrParamRec::Print(ostream&, long, aString&)



void ClineShapeCWTlTrParamRec::SetParamValue(int paramNum, long double newVal){
	//set value of parameter; only for varying parameters
	switch (paramNum){
	 case 0:
	 	 center(newVal);
	 	 break;
	 case 1:
	 	 width(newVal);
	 	 break;
	 case 2:
	 	 thetaL(newVal);
	 	 break;
	 case 3:
	 	 thetaR(newVal);
	 	 break;
	 case 4:
	 	 zL(newVal);
	 	 break;
	 case 5:
	 	 zR(newVal);
	 	 break;
	 default:
		aString errstr("bug: can't set value of param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 6-parameter cline model");
		aError(errstr);
	 	}
	}//end ClineShapeCWTlTrParamRec::SetParamValue(int,long double)



void ClineShapeCWTlTrParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ClineShapeCWParamRec::TitleString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat("thetaL");
	dataline.cat(sepchar);
	dataline.cat("zL");
	dataline.cat(sepchar);
	dataline.cat("xL");
	dataline.cat(sepchar);
	dataline.cat("thetaR");
	dataline.cat(sepchar);
	dataline.cat("zR");
	dataline.cat(sepchar);
	dataline.cat("xR");
	dataline.cat(sepchar);
	dataline.cat("P_xL");
	dataline.cat(sepchar);
	dataline.cat("P_xR");
	dataline.cat(sepchar);
	dataline.cat("delta_P");
	}//end ClineShapeCWTlTrParamRec::TitleString(aString&, char)




void ClineShapeCWTlTrParamRec::operator=(ParamRec& p){
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means p must actually point to an object of class ClineShapeCWTlTrParamRec
	ClineShapeCWTlTrParamRec* pp = (ClineShapeCWTlTrParamRec*) p.GetPointerToDerivedSelf();
	lnL_= pp->lnL_;
	lnLmaxMinusLnL_ = pp->lnLmaxMinusLnL_;	
  	c_ = pp->c_;
  	w_ = pp->w_;
	thetaL_ = pp->thetaL_;
	thetaR_ = pp->thetaR_;
	zL_ = pp->zL_;
	zR_ = pp->zR_;
	xL_ = pp->xL_;
	xR_ = pp->xR_;
	}//end  ClineShapeCWTlTrParamRec::operator=(ParamRec&)





int	operator==(ClineShapeCWTlTrParamRec& p, ClineShapeCWTlTrParamRec& q){
	return ((p.lnL_ == q.lnL_) &&
			(p.c_ == q.c_) &&
			(p.w_ == q.w_) &&
			(p.thetaL_ == q.thetaL_) &&
			(p.thetaR_ == q.thetaR_) &&
			(p.zL_ == q.zL_) &&
			(p.zR_ == q.zR_)
			 );
	}//end operator==(ClineShapeCWTlTrParamRec&, ClineShapeCWTlTrParamRec&)


int	operator!=(ClineShapeCWTlTrParamRec& p, ClineShapeCWTlTrParamRec& q){
	return !(p==q);
	}//end operator!=(ClineShapeCWTlTrParamRec&, ClineShapeCWTlTrParamRec&)



#endif


