/******************************************************
*
*	ClineShapeCWTlTrParamRec.h
*
*	Parameters for a 3-part composite tanh cline defined by center, width,	
*	thetaL, thetaR, zL & zR.  The latter 4 parameters describe introgression
*	slopes and intercepts on the Left and Right sides of the cline.
*	Assumes the y-axis value of the cline varies between 0 and 1.
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/

#ifndef ClineShapeCWTlTrParamRech
#define ClineShapeCWTlTrParamRech = 1


#include "ParamRec.h"
#include "Hyperbolics.h"
#include "InlineTemplates.h"
#include "ClineShapeCWParamRec.h"
#include "ClineFitInitializer.h"
#include "MemoryManager.h"

class ClineShapeCWTlTrParamRec;
class ClineShapeCWTlTrParamRec: 
						virtual public ParamRec, 
						virtual public ClineShapeCWParamRec{
  public:
	long double thetaL_;//decay on the left
	long double thetaR_;//decay on the right
	long double zL_;//asymptote on the left
	long double zR_;//asymptote on the right
	long double xL_;//derived parameter--junction point of left and center regions of cline
	long double xR_;//derived parameter--junction point of right and center regions of cline
  public:
  	ClineShapeCWTlTrParamRec(void);
  	ClineShapeCWTlTrParamRec(long double c, long double w,//does not set xL_ or xR_; must do this separately
							 long double tL, long double tR,//use for high/low limits and deviations
							 long double zL, long double zR);
  	ClineShapeCWTlTrParamRec(long double c, long double w,//sets xL_ and xR_
							 long double tL, long double tR,
							 long double zL, long double zR,
							 double leftEnd, double rightEnd);
  	ClineShapeCWTlTrParamRec(ClineFitInitializer& i, int trait_);
  	ClineShapeCWTlTrParamRec(ClineShapeCWTlTrParamRec& p);
  	virtual ~ClineShapeCWTlTrParamRec(void){
//  			pointerToDerivedSelf = NULL;//forget
  				}

  	virtual ParamRec* 				Duplicate(void);
  	virtual ClineShapeCWTlTrParamRec* 	_Duplicate(void);
//	virtual void 					SetPointerToDerivedSelf(void);//always override in derived
	virtual long 					GetPointerToDerivedSelf(void);//always override in derived
  	


	virtual long double thetaL(void);//returns thetaL_
	virtual void 		thetaL(long double th);//sets thetaL_
	virtual long double thetaR(void);//returns thetaR_
	virtual void 		thetaR(long double th);//sets thetaR_
	virtual long double zL(void);//returns zL_
	virtual void 		zL(long double th);//sets zL_
	virtual long double zR(void);//returns zR_
	virtual void 		zR(long double th);//sets zR_
	virtual long double xL(void);//returns xL_ -- derived parameter; same for all traits
	virtual long double xR(void);//returns xR_ -- derived parameter; same for all traits
	virtual long double B_L(int trait=0);//cline frequency at xL
	virtual long double B_R(int trait=0);//cline frequency at xR
	virtual long double DeltaP(int trait=0);//the frequency step across the center region

	virtual long double Cline_at_X(long double x, int trait=0);//cline frequency
  	
	virtual int 		ChangeCenter(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeWidth(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeThetaL(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeThetaR(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
									ParamRec* paramDeviations,  
									ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=1.0);
	virtual int 		ChangeZL(Ran3& ran, double sigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeZR(Ran3& ran, double sigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual long double ExpectedValue(aList<long double>& coords);
	virtual int 		LJunction(double leftEnd);//finds point where left and center cline sections meet
	virtual int 		RJunction(double rightEnd);//finds point where right and center cline sections meet
	virtual void 		Initialize(ClineFitInitializer& i, int trait_);
	virtual int 		MeetsParamCriteria(ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void 		MultiplyParam(int paramNum, long double factor);//use to rescale a param deviation
	virtual int 		NumParams(void);//number of parameters
	virtual int 		NumDerivedParams(void);//number of parameters that are functions of varying parameters
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
	virtual void 		SetParamValue(int paramNum, long double newVal);//sets value of parameter
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file

	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

	friend int	operator==(ClineShapeCWTlTrParamRec& p, ClineShapeCWTlTrParamRec& q);
	friend int	operator!=(ClineShapeCWTlTrParamRec& p, ClineShapeCWTlTrParamRec& q);

	};//end class ClineShapeCWTlTrParamRec



#endif