/******************************************************
*
*	ClineShapeCWTlTrTBDParamRec.cpp
*
*	Parameters for a 3-part composite tanh cline defined by center, width,	
*	thetaL, thetaR, zL, zR, and the mean frequencies on each side of the cline.  
*	The latter theta and z parameters describe introgression
*	slopes and intercepts on the Left and Right sides of the cline.
*	The mean y-axis value of the cline varies for each trait on each side.
*	Disequilibrium is also estimated and derived parameters calculated.
*	The widths of each trait's cline are a shared common value and all other
*	parameters vary independenty.
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/

#ifndef ClineShapeCWTlTrTBDParamReccpp
#define ClineShapeCWTlTrTBDParamReccpp = 1

#include "ClineShapeCWTlTrTBDParamRec.h"

	
ClineShapeCWTlTrTBDParamRec::ClineShapeCWTlTrTBDParamRec(void):r_((long double)0.5){
  	SetPointerToDerivedSelf();
  	}




ClineShapeCWTlTrTBDParamRec::ClineShapeCWTlTrTBDParamRec(ClineFitInitializer& i){
  	SetPointerToDerivedSelf();
	lnL_= 0.0;
	lnLmaxMinusLnL_ = 0.0;
	for(int trait=0;trait<i.aveScoreL_.len();++trait){
		ClineShapeCWTlTrTBParamRec* traitParams = NEW ClineShapeCWTlTrTBParamRec(i,trait);
		traitParams->paramOnOffList_[1]=0;//width off locally; handled by this holding class
		traitParamSet_.Append(traitParams);
		}
	r_=i.r_;
	D_=i.D_;
	lnLDiseq_=0.0;
  	}


ClineShapeCWTlTrTBDParamRec::ClineShapeCWTlTrTBDParamRec(ClineShapeCWTlTrTBDParamRec& p){
  	SetPointerToDerivedSelf();
	lnL_ = p.lnL_;
	lnLmaxMinusLnL_ = p.lnLmaxMinusLnL_;
	traitParamSet_ = p.traitParamSet_;	
	r_ = p.r_;
	D_ = p.D_;
	lnLDiseq_ = p.lnLDiseq_;
	}



ParamRec* ClineShapeCWTlTrTBDParamRec::Duplicate(void){
	ClineShapeCWTlTrTBDParamRec* copy = NEW ClineShapeCWTlTrTBDParamRec(*this);
	return copy;
	}//end ClineShapeCWTlTrTBDParamRec::Duplicate()


ClineShapeCWTlTrTBDParamRec* ClineShapeCWTlTrTBDParamRec::_Duplicate(void){
	ClineShapeCWTlTrTBDParamRec* copy = NEW ClineShapeCWTlTrTBDParamRec(*this);
	return copy;
	}//end ClineShapeCWTlTrTBDParamRec::_Duplicate()


void ClineShapeCWTlTrTBDParamRec::SetPointerToDerivedSelf(void){
  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in ClineShapeCWTlTrTBDParamRec"<<endl;
  	}//end ClineShapeCWTlTrTBDParamRec::SetPointerToDerivedSelf()



long double ClineShapeCWTlTrTBDParamRec::B_L(int trait){
	//cline frequency at xL
	ClineShapeCWTlTrTBParamRec* params = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
	return params->B_L();
	}//end ClineShapeCWTlTrTBDParamRec::B_L(int)


long double ClineShapeCWTlTrTBDParamRec::B_R(int trait){
	//cline frequency at xR
	ClineShapeCWTlTrTBParamRec* params = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
	return params->B_R();
	}//end ClineShapeCWTlTrTBDParamRec::B_R(void)



int ClineShapeCWTlTrTBDParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, double weight){
	//uses type conversion that circumvents usual virtual base class restrictions
	//return 0 for failure, 1 for success
	ClineShapeCWTlTrTBDParamRec* deviations = (ClineShapeCWTlTrTBDParamRec*) paramDeviations->pointerToDerivedSelf;
	int paramSetToAlter, paramCounter=-1, origParam = paramNum;
	for(int t=0;t<traitParamSet_.len();++t){
		int nFort = traitParamSet_[t].NumParams()-1;//don't change width here
		paramCounter += nFort;
		if(paramCounter >= paramNum) {//change a subset parameter
			paramSetToAlter=t;
			int paramToAlter = paramNum - (paramCounter - nFort + 1);
			if(paramToAlter>0){paramToAlter++;}//skips paramToAlter=1 which is width
			ClineShapeCWTlTrTBDParamRec* lower = (ClineShapeCWTlTrTBDParamRec*) lowerLimits->pointerToDerivedSelf;
			ClineShapeCWTlTrTBDParamRec* upper = (ClineShapeCWTlTrTBDParamRec*) upperLimits->pointerToDerivedSelf;
			return traitParamSet_[t].ChangeParam(paramToAlter, ran, &(deviations->traitParamSet_[t]),
							&(lower->traitParamSet_[t]), &(upper->traitParamSet_[t]), weight);
			}
		}//end for t
	//otherwise vary width or D
	paramNum -= paramCounter+1;
	switch (paramNum){
	   case 0:
		  return ChangeWidth(ran, deviations->width()*weight, lowerLimits, upperLimits);
		  break;
	   case 1:
		   return ChangeD(ran, deviations->D()*weight, lowerLimits, upperLimits);
		   break;
	   default:
		 aString errstr("bug: can't change shape param #");
		errstr.cat(origParam+1);
		errstr.cat(" in a ");errstr.cat(NumParams());errstr.cat("-parameter cline model");
		aError(errstr);
		 }//end switch
	return  0;
	}//end ClineShapeCWTlTrTBDParamRec::ChangeParam(int,Ran3&,ParamRec*,ParamRec*,ParamRec*)




int ClineShapeCWTlTrTBDParamRec::ChangeWidth(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary width
	//return 0 for failure, 1 for success
	ClineShapeCWTlTrTBDParamRec* lower = (ClineShapeCWTlTrTBDParamRec*) lowerLimits->pointerToDerivedSelf;
	ClineShapeCWTlTrTBDParamRec* upper = (ClineShapeCWTlTrTBDParamRec*) upperLimits->pointerToDerivedSelf;
	int validChange=false,tries=0;
	long double oldWidth = width();
	do{
		long double newWidth = ran.NormDev(oldWidth, sigma);
		for(int t=0;t<traitParamSet_.len();++t){
			ClineShapeCWTlTrTBParamRec* low = (ClineShapeCWTlTrTBParamRec*) lower->traitParamSet_[t].pointerToDerivedSelf;
			ClineShapeCWTlTrTBParamRec* high = (ClineShapeCWTlTrTBParamRec*) upper->traitParamSet_[t].pointerToDerivedSelf;
			ClineShapeCWTlTrTBParamRec* traitParams = 
					(ClineShapeCWTlTrTBParamRec*) traitParamSet_[t].pointerToDerivedSelf;
			traitParams->width(newWidth);
			validChange = traitParams->LJunction(low->xL_);
			if(validChange){
				validChange = traitParams->RJunction(high->xR_);}
			if(validChange){
				validChange = traitParams->MeetsParamCriteria(low, high);}
			if(! validChange){
				break;}
			}//end for t
		++tries;
		if(tries>20){
//			aString err("can't find a good value for width");
//			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-5){///bail out
				for(int t=0;t<traitParamSet_.len();++t){
					ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[t].pointerToDerivedSelf;
					traitParams->width(oldWidth);
					}//t
				return validChange=0;}
			}//tries			
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrTBDParamRec::ChangeWidth(Ran3&, double, ParamRec*, ParamRec*)


long double ClineShapeCWTlTrTBDParamRec::Cline_at_X(double x, int trait){//location, trait
	ClineShapeCWTlTrTBParamRec* traitParams = 
			(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
	return traitParams->Cline_at_X(x);
	}//end ClineShapeCWTlTrTBDParamRec::Cline_at_X(double, int)


long double ClineShapeCWTlTrTBDParamRec::D(void){//returns D_
	return D_;}
	
	
void ClineShapeCWTlTrTBDParamRec::D(long double d){//sets D_
	D_ = d;}

long double ClineShapeCWTlTrTBDParamRec::ClineDiseqLikelihood (ClineGeneticData* alloData){
	return ClineDiseqParamRec::ClineDiseqLikelihood (alloData);
/*
	//sample-size based weighting removed
	long nSample;
	double lnL=0.0;
	double adjustedD = 0.0;
	aMatrix<double>* ff = ZeroMatrix((double)0,2,2);
	aMatrix<double>* bigXX = ZeroMatrix((double)0,2,2);
	aMatrix<double>& f = *ff;
	aMatrix<double>& bigX = *bigXX;
	int adjByFis = true;//set to false here if pops are in HW proportions
    double FisCenter = 0.0;
    if (adjByFis){//calculate Fis for the center pops only
		FstatisticAnalysis centerPopData(*alloData,alloData->centerPopList_);
		centerPopData.WeirTheta();
		FisCenter = centerPopData.Fis(FstatisticAnalysis::WandC);
		}
	for(int popNum=0;popNum<alloData->centerPopList_.len();++popNum){
		int pop = alloData->centerPopList_[popNum];
		double x = alloData->Locality(pop);//this location
		for(int loc1=0;loc1<alloData->loci()-1;++loc1){
			StringList& alleleNamesList1 = alloData->allelesPresent[loc1];
			aList<double> coords; coords.Append(x); coords.Append(loc1);
			double p1AtCline = ExpectedValue(coords);
			p1AtCline = MAX(p1AtCline,10.0e-5);//eliminates 0 expected values in likelihood
			p1AtCline = MIN(p1AtCline,1.0-10e-5);
			for(int loc2 = (loc1 + 1); loc2<alloData->loci();++loc2){
				StringList& alleleNamesList2 = alloData->allelesPresent[loc2];
				aList<double> coords2; coords2.Append(x); coords2.Append(loc2);
				double p2AtCline = ExpectedValue(coords2);
				p2AtCline = MAX(p2AtCline,10.0e-5);//eliminates 0 expected values in likelihood
				p2AtCline = MIN(p2AtCline,1.0-10e-5);
    	   		adjustedD = (D_ / sqrt((p1AtCline * (1.0 - p1AtCline))*(p2AtCline * (1.0 - p2AtCline)))) * (1.0 + FisCenter);
				adjustedD = MIN(adjustedD,1.0);//at low expected cline freqs, adjustedD can exceed 1.0
				adjustedD = MAX(adjustedD,-1.0);//and this is impossible
				for(int allele1=0;allele1<alleleNamesList1.len();++allele1){
					//get freq of allele1 (at loc1) in pop
					double freq1 = alloData->GetFreq(pop, loc1, allele1);
					if ((freq1 > 0.0) && (freq1 < 1.0)){//otherwise D is undefined
						aString& alleleName1 = alleleNamesList1[allele1];
						for(int allele2=0;allele2<alleleNamesList2.len();++allele2){
							nSample = 0;//set sample size to zero, to count for blanks
							//get freq of allele2 (at loc2) in pop
							double freq2 = alloData->GetFreq(pop, loc2, allele2);
							if ((freq2 > 0.0) && (freq2 < 1.0)){//otherwise D is undefined
								f[0][0] = freq1 * freq2 + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
       							f[0][1] = freq1 * (1.0 - freq2) + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
       							f[1][0] = (1.0 - freq1) * freq2 + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
       							f[1][1] = (1.0 - freq1) * (1.0 - freq2) + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
								//sometimes these values can be less than zero due to sampling error
								//implying that it is infinitely improbable that an observed value
								//could occur under the model.
								//therefore, values <=0 are set to a very small number, and will
								//give a very low finite likelihood
								for(int ind1=0;ind1<2;++ind1){
									for(int ind2=0;ind2<2;++ind2){
										if(f[ind1][ind2] <= 0.0){
											f[ind1][ind2] = 1.0e-10;}
										}}

								//get genotype strings for each individual and  read joint frequencies
								aString& alleleName2 = alleleNamesList2[allele2];
								aMatrix<double>* bigNN = ZeroMatrix((double)0,3,3);
								aMatrix<double>& bigN = *bigNN;
 								int sizePop = alloData->individuals(pop);
								for(int i=0;i<sizePop;++i){
									aString* spot1 = alloData->GetGenotypeString(pop,i,loc1);
									//make sure it doesn't have any blanks
									int blankAt1 = false;
       								if(spot1->len() < 2){
       										blankAt1 = true;//happens sometimes at end of data line
											}//with missing loci
										else{
											if(((*spot1)[0] == ' ') || ((*spot1)[1] == ' ')){
												blankAt1 = true;}
											}
									if(!blankAt1){
										aString* spot2 = alloData->GetGenotypeString(pop,i,loc2);
										//make sure it doesn' t have any blanks
										int blankAt2 = false;
       									if(spot2->len() < 2){
       											blankAt2 = true;//happens sometimes at end of data line
												}//with missing loci
											else{
												if(((*spot2)[0] == ' ') || ((*spot2)[1] == ' ')){
													blankAt2 = true;}
												}
										if(!blankAt2){
											++nSample;
											//use this indiv in calculation
											//count instances of target alleles at each locus in this indiv
											int i1 = 2;
											for(int chrom=0;chrom<2;++chrom){
												aString alleleVal1((*spot1)[chrom]);
												if( alleleName1 == alleleVal1){
													--i1;}}
											int i2 = 2;
											for(int chrom=0;chrom<2;++chrom){
												aString alleleVal2((*spot2)[chrom]);
												if(alleleName2 == alleleVal2){
													--i2;}}
											bigN[i1][i2]++;
											}//end if not blankAt2
										delete spot2; spot2=0;
										}//end if not blankAt1
									delete spot1; spot1=0;
									}//end for i
								//calculate Hill's (1974) intermediate variables & iterative method
								bigX[0][0] = 2.0 * bigN[0][0] + bigN[0][1] + bigN[1][0];
								bigX[1][0] = 2.0 * bigN[2][0] + bigN[1][0] + bigN[2][1];
								bigX[0][1] = 2.0 * bigN[0][2] + bigN[0][1] + bigN[1][2];
								bigX[1][1] = 2.0 * bigN[2][2] + bigN[1][2] + bigN[2][1];
								for(int index1=0;index1<2;++index1){
									for(int index2=0;index2<2;++index2){
										if (f[index1][index2] > 0.0){
											lnL += bigX[index1][index2] * ln(f[index1][index2]);
											}}}//end for both indexes
								lnL += bigN[1][1] * ln(f[0][0] * f[1][1] + f[0][1] * f[1][0]);
								delete bigNN; bigNN=0;
								}//end if 0<freq2<1
							}//end for allele2
						}//end if 0<freq1<1
					}//end for allele1
				}//end for loc2
			}//end for loc1
		}//end for popNum
	delete ff;ff=0;
	delete bigXX;bigXX=0;
    lnLDiseq(lnL);
    return lnL;
*/
	}//end function ClineShapeCWTlTrTBDParamRec::ClineDiseqLikelihood


long double ClineShapeCWTlTrTBDParamRec::DispersalSigma(void){
    if (D_ > 0.0){
    	long double w = width();
		long double sigma = sqrt((D_ * r_ * w * w) / ((long double)1.0 + r_));
		return sigma;}
	return (long double)0.0;//default
	}//end ClineShapeCWTlTrTBDParamRec::DispersalSigma()


long double ClineShapeCWTlTrTBDParamRec::distanceBarrierL(int trait){
	//barrier to introgression in distance units
    //derivatives found using Mathematica
    long double w = width();
	ClineShapeCWTlTrTBParamRec* traitParams = 
			(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
    if (!(traitParams->paramsOn_.Includes(2))||!(traitParams->paramsOn_.Includes(4))){
			return exp((long double)100.0);}//huge and undefined
    long double derivFactor = (long double)4.0 * sqrt(traitParams->thetaL()) 
    		* (traitParams->xL() - (traitParams->center() + traitParams->zL())) / w;
    long double dPLdXatxL, beta;
    if ((w ==(long double)0.0)||(derivFactor <= (long double)-100.0)){
			return exp((long double)100.0);}//huge and undefined
    if (derivFactor <= (long double)100.0){
			dPLdXatxL = exp(derivFactor) * sqrt(traitParams->thetaL()) / w;}
    	else{
			dPLdXatxL = exp((long double)100.0);}

    if(dPLdXatxL == (long double)0.0){
			return exp((long double)100.0);}
		else{
//    		long double dP = traitParams->DeltaP(trait);//derivatives found using Mathematica
    		long double dP = traitParams->DeltaP(0);//derivatives found using Mathematica
			beta = dP / dPLdXatxL;}
	return beta;
	}//end ClineShapeCWTlTrTBDParamRec::distanceBarrierL(int)
 


long double ClineShapeCWTlTrTBDParamRec::distanceBarrierR(int trait){//barrier to introgression in distance units
    //derivatives found using Mathematica
    long double w = width();
	ClineShapeCWTlTrTBParamRec* traitParams = 
			(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
    if (!(traitParams->paramsOn_.Includes(3))||!(traitParams->paramsOn_.Includes(5))){
			return exp(100.0);}//huge and undefined
    long double derivFactor = (long double)-4.0 * sqrt(traitParams->thetaR()) 
    		* (traitParams->xR() - (traitParams->center() - traitParams->zR())) / w;
    long double dPRdXatxR, beta;
    if ((w ==(long double)0.0)||(derivFactor <= (long double)-100.0)){
			return exp((long double)100.0);}//huge and undefined
    if (derivFactor <= (long double)100.0){
			dPRdXatxR = exp(derivFactor) * sqrt(traitParams->thetaR()) / w;}
    	else{
			dPRdXatxR = exp((long double)100.0);}

    if(dPRdXatxR ==(long double) 0.0){
			return exp((long double)100.0);}
		else{
//    		long double dP = traitParams->DeltaP(trait);//derivatives found using Mathematica
    		long double dP = traitParams->DeltaP(0);//derivatives found using Mathematica
			beta = dP / dPRdXatxR;}
	return beta;
	}//end ClineShapeCWTlTrTBDParamRec::distanceBarrierR(int)
 

long double ClineShapeCWTlTrTBDParamRec::ExpectedValue(aList<double>& coords){
	int trait = (int) (coords[1]);
	return Cline_at_X(coords[0],trait);//location, trait
	}//end ClineShapeCWTlTrTBDParamRec::ExpectedValue(aList<double>&)


void ClineShapeCWTlTrTBDParamRec::Initialize(ClineFitInitializer& i){
	traitParamSet_.Clear();
	lnL_= 0.0;
	lnLmaxMinusLnL_ = 0.0;
	for(int trait=0;trait<i.aveScoreL_.len();++trait){
		ClineShapeCWTlTrTBParamRec* traitParams = NEW ClineShapeCWTlTrTBParamRec(i,trait);
		traitParams->paramOnOffList_[1]=0;//width off locally; handled by this holding class
		traitParamSet_.Append(traitParams);
		}
	r_=i.r_;
	D_=i.D_;
	lnLDiseq_=0.0;
	}//end ClineShapeCWTlTrTBDParamRec::Initialize(ClineFitInitializer&)


long double ClineShapeCWTlTrTBDParamRec::leftVal(int trait){
	ClineShapeCWTlTrTBParamRec* traitParams = 
					(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
	return traitParams->leftVal(0);}


long double ClineShapeCWTlTrTBDParamRec::lnLDiseq(void){//returns lnLDiseq_
	return lnLDiseq_;}
	
	
void ClineShapeCWTlTrTBDParamRec::lnLDiseq(long double lnD){//sets lnLDiseq_
	lnLDiseq_ = lnD;}


int ClineShapeCWTlTrTBDParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
	//use type conversion that circumvents usual virtual base class restrictions
	ClineShapeCWTlTrTBDParamRec* lower = (ClineShapeCWTlTrTBDParamRec*) lowerLimits->pointerToDerivedSelf;
	ClineShapeCWTlTrTBDParamRec* upper = (ClineShapeCWTlTrTBDParamRec*) upperLimits->pointerToDerivedSelf;
	if((D_<lower->D_) || (D_>upper->D_)){return 0;}
	for(int t=0;t<traitParamSet_.len();++t){
		ParamRec& low = lower->traitParamSet_[t];
		ParamRec& high = upper->traitParamSet_[t];
		ClineShapeCWTlTrTBParamRec* traitParams = 
					(ClineShapeCWTlTrTBParamRec*) traitParamSet_[t].pointerToDerivedSelf;
		int success = traitParams->MeetsParamCriteria(&low, &high);
		if(!success){return 0;}
		}//end for t
	return 1;
	}//end ClineShapeCWTlTrTBDParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)


void ClineShapeCWTlTrTBDParamRec::MultiplyParam(int paramNum, long double factor){
	//use to rescale a param deviation
	int paramSetToAlter, paramCounter=-1, n = NumParams(), origParam = paramNum;
	for(int t=0;t<traitParamSet_.len();++t){
		int nFort = traitParamSet_[t].NumParams();
		paramCounter += nFort;
		if(paramCounter >= paramNum) {//change a subset parameter
			paramSetToAlter=t;
			int paramToAlter = paramNum - (paramCounter - nFort + 1);
			return traitParamSet_[t].MultiplyParam(paramToAlter, factor);
			}
		}//end for t
	//otherwise scale width or D
	paramNum -= paramCounter+1;
	switch (paramNum){
		case 0:
			for(int t=0;t<traitParamSet_.len();++t){//change all widths
				ClineShapeCWTlTrTBParamRec* traitParams = 
								(ClineShapeCWTlTrTBParamRec*) traitParamSet_[t].pointerToDerivedSelf;
				traitParams->w_ *= factor;
				}
		 	 break;
		 case 1:
		 	 D_ *= factor;
		 	 break;
		 default:
			aString errstr("bug: can't rescale param #");
			errstr.cat(origParam+1);
			errstr.cat(" in a ");errstr.cat(NumParams());errstr.cat("-parameter cline model");
			aError(errstr);
		 }//end switch
	}//ClineShapeCWTlTrTBDParamRec::MultiplyParam(int, double)


int ClineShapeCWTlTrTBDParamRec::NumParams(void){
	int sum=2;//width & D
	for(int t=0;t<traitParamSet_.len();++t){//params of each trait
		sum += traitParamSet_[t].NumParams()-1;}
	return sum;
	}//end ClineShapeCWTlTrTBDParamRec::NumParams()


int ClineShapeCWTlTrTBDParamRec::NumDerivedParams(void){
	int sum=8;
	for(int t=0;t<traitParamSet_.len();++t){//params of each trait
		sum += traitParamSet_[t].NumDerivedParams();
		sum += 6;//barrier strengths & individual locus selection
		}
	return sum;
	}//end ClineShapeCWTlTrTBDParamRec::NumDerivedParams()



long double ClineShapeCWTlTrTBDParamRec::NumSelectedLoci(void){
	long double s = s_star_vs_Hets();
    if (s == (long double)0.0){
		return (long double)0.0;}//no loci under selection if no selection
	long double ww = WhybridOverWpure();
    if ((s >= (long double)1.0) || (ww <= (long double) 0.0)){
		return (long double)1.0e20;}//a really high number if fitness=0
    double nLocSel = (long double)2.0 * ln(ww) / ln((long double)1.0 - s);
    return nLocSel;
    }//end ClineShapeCWTlTrTBDParamRec::NumSelectedLoci()




aString* ClineShapeCWTlTrTBDParamRec::ParamName(int paramNum){
	//returns name of parameter; first varying params, then derived params
	int paramToGet, paramCounter=-1, n = NumParams(), origParam = paramNum;
	for(int t=0;t<traitParamSet_.len();++t){
		int nFort = traitParamSet_[t].NumParams();
		paramCounter += nFort;
		if(paramCounter >= paramNum) {//get a trait parameter
			paramToGet = paramNum - (paramCounter - nFort + 1);
			aString* pName = traitParamSet_[t].ParamName(paramToGet);
			pName->cat("[");pName->cat(t+1);pName->cat("]");
			return pName;
			}
		}//end for t
	//otherwise get width or D
	paramNum -= paramCounter+1;
	switch (paramNum){
		case 0:
			 return NEW aString("width");
		 	 break;
		 case 1:
			 return NEW aString("D");
		 	 break;
		 default:break;
		 }//end switch 
	paramNum -= 2;
	//it must be a derived parameter
	paramCounter=-1;
	n = NumDerivedParams();
	for(int tt=0;tt<traitParamSet_.len();++tt){
		int nFort = traitParamSet_[tt].NumDerivedParams();
		paramCounter += nFort;
		if(paramCounter >= paramNum) {//get a trait derived parameter
			int nn = traitParamSet_[tt].NumParams();
			paramToGet = paramNum - (paramCounter - nFort + 1) + nn;
			aString* pName = traitParamSet_[tt].ParamName(paramToGet);
			pName->cat("[");pName->cat(tt+1);pName->cat("]");
			return pName;
			}
		}//end for tt
	paramNum -= paramCounter+1;
	switch (paramNum){
		case 0:
	 	 return NEW aString("sigma");//dispersal
	 	 break;
	 	case 1:
	 	 return NEW aString("s*[hets]");
	 	 break;
	 	case 2:
	 	 return NEW aString("s*[ecotone]");
	 	 break;
	 	case 3:
	 	 return NEW aString("s*[dom&ecotone]");
	 	 break;
	 	case 4:
	 	 return NEW aString("W[hybrid]/[W[pure]");
	 	 break;
	 	case 5:
	 	 return NEW aString("S");
	 	 break;
	 	case 6:
	 	 return NEW aString("num loci selected");
	 	 break;
	 	case 7:
	 	 return NEW aString("s per selected locus");
	 	 break;
	 	default: break;
	 	}//end switch
	paramNum -= 8;
	aString derivedName;
	int foundIt = false;
	for(int ttt=0;ttt<traitParamSet_.len();++ttt){
		switch(paramNum){
	 		case 0:
	 	 		derivedName = "barrierL[dist]";
	 	 		foundIt = true; break;
	 		case 1:
	 			derivedName = "barrierR[dist]";
	 	 		foundIt = true; break;
	 		case 2:
	 	 		derivedName = "barrierL[time]";
	 	 		foundIt = true; break;
	 		case 3:
	 	 		derivedName = "barrierR[time]";
	 	 		foundIt = true; break;
	 		case 4:
	 	 		derivedName = "sL[marker]";
	 	 		foundIt = true; break;
	 		case 5:
	 	 		derivedName = "sR[marker]";
	 	 		foundIt = true; break;
	 		default:break;
	 		}//end switch
	 	if(foundIt){
	 		derivedName.cat("[");derivedName.cat(ttt+1);derivedName.cat("]");
	 		return NEW aString(derivedName);
	 		}
	 	paramNum -= 6;
	 	}//end for ttt
	aString errstr("bug: can't return name of param #");
	errstr.cat(origParam+1);
	errstr.cat(" in a ");errstr.cat(TotalParams());errstr.cat("-parameter cline model");
	aError(errstr);
	return NEW aString();
	}//end ClineShapeCWTlTrTBDParamRec::ParamName(int )



void ClineShapeCWTlTrTBDParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::ParamString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat(width(),6,15);
	dataline.cat(sepchar);
	dataline.cat(D_,2,15);
	dataline.cat(sepchar);
	dataline.cat(lnLDiseq_,6,15);
	dataline.cat(sepchar);
	dataline.cat(DispersalSigma(),6,15);
	dataline.cat(sepchar);
	dataline.cat(s_star_vs_Hets(),2,15);
	dataline.cat(sepchar);
	dataline.cat(s_star_Ecotone(),2,15);
	dataline.cat(sepchar);
	dataline.cat(s_star_DomEcotone(),2,15);
	dataline.cat(sepchar);
	dataline.cat(WhybridOverWpure(),6,15);
	dataline.cat(sepchar);
	dataline.cat(S(),6,15);
	dataline.cat(sepchar);
	dataline.cat(NumSelectedLoci(),6,15);
	dataline.cat(sepchar);
	dataline.cat(s_perReproIsolationLocus(),2,15);
	for(int t=0;t<traitParamSet_.len();++t){
		int n=traitParamSet_[t].TotalParams();
		for(int p=0;p<n;++p){			
			dataline.cat(sepchar);
			dataline.cat(traitParamSet_[t].ParamValue(p),6,15);
			}
		dataline.cat(sepchar);
		dataline.cat((double)distanceBarrierL(t),9,15);
		dataline.cat(sepchar);
		dataline.cat((double)distanceBarrierR(t),9,15);
		dataline.cat(sepchar);
		dataline.cat((double)timeBarrierL(t),9,15);
		dataline.cat(sepchar);
		dataline.cat((double)timeBarrierR(t),9,15);
		dataline.cat(sepchar);
		dataline.cat(s_perMarkerLocusL(t),2,15);
		dataline.cat(sepchar);
		dataline.cat(s_perMarkerLocusR(t),2,15);
		}//end for t
	}//end ClineShapeCWTlTrTBDParamRec::ParamString(aString&, char)



long double ClineShapeCWTlTrTBDParamRec::ParamValue(int paramNum){
	//returns value of parameter; first varying params, then derived params
	int paramToGet, paramCounter=-1, n = NumParams(), origParam = paramNum;
	for(int t=0;t<traitParamSet_.len();++t){
		int nFort = traitParamSet_[t].NumParams();
		paramCounter += nFort;
		if(paramCounter >= paramNum) {//get a trait parameter
			paramToGet = paramNum - (paramCounter - nFort + 1);
			return traitParamSet_[t].ParamValue(paramToGet);
			}
		}//end for t
	//otherwise get width or D
	paramNum -= paramCounter+1;
	switch (paramNum){
		case 0:
			 return width();
		 	 break;
		 case 1:
			 return D();
		 	 break;
		 default:break;
		 }//end switch 
	paramNum -= 2;
	//it must be a derived parameter
	paramCounter=-1;
	n = NumDerivedParams();
	for(int tt=0;tt<traitParamSet_.len();++tt){
		int nFort = traitParamSet_[tt].NumDerivedParams();
		paramCounter += nFort;
		if(paramCounter >= paramNum) {//get a trait derived parameter
			int nn = traitParamSet_[tt].NumParams();
			paramToGet = paramNum - (paramCounter - nFort + 1) + nn;
			return traitParamSet_[tt].ParamValue(paramToGet);
			}
		}//end for tt
	paramNum -= paramCounter+1;
	switch (paramNum){
		case 0:
	 	 return DispersalSigma();//dispersal
	 	 break;
	 	case 1:
	 	 return s_star_vs_Hets();
	 	 break;
	 	case 2:
	 	 return s_star_Ecotone();
	 	 break;
	 	case 3:
	 	 return s_star_DomEcotone();
	 	 break;
	 	case 4:
	 	 return WhybridOverWpure();
	 	 break;
	 	case 5:
	 	 return S();
	 	 break;
	 	case 6:
	 	 return NumSelectedLoci();
	 	 break;
	 	case 7:
	 	 return s_perReproIsolationLocus();
	 	 break;
	 	default: break;
	 	}//end switch
	paramNum -= 8;
	aString derivedName;
	int foundIt = false;
	for(int ttt=0;ttt<traitParamSet_.len();++ttt){
		switch(paramNum){
	 		case 0:
	 	 		return distanceBarrierL(ttt);
	 	 		break;
	 		case 1:
	 			return distanceBarrierR(ttt);
	 	 		break;
	 		case 2:
	 	 		return timeBarrierL(ttt);
	 	 		break;
	 		case 3:
	 	 		return timeBarrierR(ttt);
	 	 		break;
	 		case 4:
	 	 		return s_perMarkerLocusL(ttt);
	 	 		break;
	 		case 5:
	 	 		return s_perMarkerLocusR(ttt);
	 	 		break;
	 		default:break;
	 		}//end switch
	 	paramNum -= 6;
	 	}//end for ttt
	aString errstr("bug: can't return value of param #");
	errstr.cat(origParam+1);
	errstr.cat(" in a ");errstr.cat(TotalParams());errstr.cat("-parameter cline model");
	aError(errstr);
	return 0.0;
	}//end ClineShapeCWTlTrTBParamRec::ParamValue(int )


void ClineShapeCWTlTrTBDParamRec::Print(ostream& outFile, long rep, aString& message){
	ParamRec::Print(outFile, rep, message);
	for(int t=0;t<traitParamSet_.len();++t){
		ClineShapeCWTlTrTBParamRec* traitParams = 
						(ClineShapeCWTlTrTBParamRec*) traitParamSet_[t].pointerToDerivedSelf;
		int pCount = 0;
		aString outLine;
		for(int p=0;p<traitParams->TotalParams();++p){
			aString* pName = traitParams->ParamName(p);
			double pVal = traitParams->ParamValue(p);
			pName->cat("[");pName->cat(t+1);pName->cat("]: ");
			if((pVal>-2.0)&&(pVal<2.0)){
					pName->cat(pVal,2,9);}
				else{
					pName->cat(pVal,6,9);}
			++pCount;
			outLine.cat(*pName); 
			delete pName; pName=0;
			if(MOD(pCount,3)){
					outLine.separator(2);}
				else{
					cout<<outLine<<endl;
					outLine = "";}
			}
		if(outLine.len()>0){
			cout<<outLine<<endl;
			outLine = "";}
		}//end for t
	aString outStr("w: ");
	outStr.cat(width(),6,9);
	outStr.separator(2);
	outStr.cat("D: ");
	outStr.cat(D(),2,9);
	outStr.separator(2);
	outStr.cat("lnLDiseq: ");
	outStr.cat(lnLDiseq(),6,9);
	outFile<<outStr<<endl;
	outStr = "sigma: ";
	outStr.cat(DispersalSigma(),6,9);
	outStr.separator(2);
	outStr = "W[h]/W[p]: ";
	outStr.cat(WhybridOverWpure(),6,9);
	outStr.separator(2);
	outStr.cat("S: ");
	outStr.cat(S(),6,9);
	outFile<<outStr<<endl;
	outStr = "s*[hets]: ";
	outStr.cat(s_star_vs_Hets(),2,9);
	outStr.separator(2);
	outStr.cat("s*[ecotone]: ");
	outStr.cat(s_star_Ecotone(),2,9);
	outStr.separator(2);
	outStr.cat("s*[dom,ecotone]: ");
	outStr.cat(s_star_DomEcotone(),2,9);
	outFile<<outStr<<endl;
	outStr = "n loci selected: ";
	outStr.cat(NumSelectedLoci(),6,9);
	outStr.separator(2);
	outStr.cat("s per RI locus: ");
	outStr.cat(s_perReproIsolationLocus(),2,9);
	outFile<<outStr<<endl;
	for(int t=0;t<traitParamSet_.len();++t){
		outStr = "barrierL[dist]["; outStr.cat(t+1);outStr.cat("]: ");
		outStr.cat((double)distanceBarrierL(t),9,9);
		outStr.separator(2);
		outStr.cat("barrierR[dist]["); outStr.cat(t+1);outStr.cat("]: ");
		outStr.cat((double)distanceBarrierR(t),9,9);
		outFile<<outStr<<endl;
		outStr = "barrierL[time]["; outStr.cat(t+1);outStr.cat("]: ");
		outStr.cat((double)timeBarrierL(t),9,9);
		outStr.separator(2);
		outStr.cat("barrierR[time]["); outStr.cat(t+1);outStr.cat("]: ");
		outStr.cat((double)timeBarrierR(t),9,9);
		outFile<<outStr<<endl;
		outStr = "sL[marker]["; outStr.cat(t+1);outStr.cat("]: ");
		outStr.cat(s_perMarkerLocusL(t),2,9);
		outStr.separator(2);
		outStr.cat("sR[marker]["); outStr.cat(t+1);outStr.cat("]: ");
		outStr.cat(s_perMarkerLocusR(t),2,9);
		outFile<<outStr<<endl;
		}//end for t
	}//ClineShapeCWTlTrTBDParamRec::Print(ostream&, long, aString&)



long double ClineShapeCWTlTrTBDParamRec::rightVal(int trait){
	ClineShapeCWTlTrTBParamRec* traitParams = 
					(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
	return traitParams->rightVal(0);}



long double ClineShapeCWTlTrTBDParamRec::S(void){
	long double WhWp = WhybridOverWpure();
    if (WhWp == (long double)0.0){
		return (long double)200.0;}//an arbitrary high number
    long double bigS = (long double)-2.0 * ln(WhWp);
    return bigS;
    }//end ClineShapeCWTlTrTBDParamRec::S()


void ClineShapeCWTlTrTBDParamRec::SetParamValue(int paramNum, long double newVal){
	//set value of parameter; only for varying parameters
	int paramSetToAlter, paramCounter=-1, n = NumParams(), origParam = paramNum;
	for(int t=0;t<traitParamSet_.len();++t){
		int nFort = traitParamSet_[t].NumParams();
		paramCounter += nFort;
		if(paramCounter >= paramNum) {//change a subset parameter
			paramSetToAlter=t;
			int paramToAlter = paramNum - (paramCounter - nFort + 1);
			traitParamSet_[t].SetParamValue(paramToAlter, newVal);
			return;
			}
		}//end for t
	//otherwise scale width or D
	paramNum -= paramCounter+1;
	switch (paramNum){
		case 0:
			for(int t=0;t<traitParamSet_.len();++t){//change all widths
				ClineShapeCWTlTrTBParamRec* traitParams = 
								(ClineShapeCWTlTrTBParamRec*) traitParamSet_[t].pointerToDerivedSelf;
				traitParams->width(newVal);
				}//end for t
		 	 break;
		 case 1:
		 	 D_ = newVal;
		 	 break;
		 default:
			aString errstr("bug: can't change param #");
			errstr.cat(origParam+1);
			errstr.cat(" in a ");errstr.cat(NumParams());errstr.cat("-parameter cline model");
			aError(errstr);
		 }//end switch
	}//end ClineShapeCWTlTrTBDParamRec::SetParamValue(int,long double)


long double ClineShapeCWTlTrTBDParamRec::s_perMarkerLocusL(int trait){
	//direct selection on marker loci in left tail of cline
	//assuming genetic background incompatibilities
	if(width()==0.0){return (long double) 0.0;}//undefined
	ClineShapeCWTlTrTBParamRec* traitParams = 
					(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
    long double s = traitParams->thetaL() * s_star_vs_Hets();
    return s;
    }//end ClineShapeCWTlTrTBDParamRec::s_perMarkerLocusL(int)


long double ClineShapeCWTlTrTBDParamRec::s_perMarkerLocusR(int trait){
	//direct selection on marker loci in right tail of cline
	//assuming genetic background incompatibilities
	if(width()==0.0){return (long double)0.0;}//undefined
	ClineShapeCWTlTrTBParamRec* traitParams = 
					(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].pointerToDerivedSelf;
    long double s = traitParams->thetaR() * s_star_vs_Hets();
    return s;
    }//end ClineShapeCWTlTrTBDParamRec::s_perMarkerLocusL(int)


long double ClineShapeCWTlTrTBDParamRec::s_perReproIsolationLocus(void){
	//average selection on a locus contributing to reproductive isolation
	long double n = NumSelectedLoci();
	 if (n == 0.0){
			return (long double)0.0;}
    return S() / n;
    }//end ClineShapeCWTlTrTBDParamRec::s_perReproIsolationLocus(void)




long double ClineShapeCWTlTrTBDParamRec::s_star_vs_Hets(void){
	//indirect & direct selection on marker loci in middle of cline
	//assuming genetic background incompatibilities
	if(width()==0.0){return (long double)0.0;}//undefined
    long double s = sqr((long double)2.0 * DispersalSigma() / width());
    return s;
    }//end ClineShapeCWTlTrTBDParamRec::s_star_vs_Hets()


long double ClineShapeCWTlTrTBDParamRec::s_star_Ecotone(void){
	//indirect & direct selection on marker loci in middle of cline
	//assuming selection across an ecotone
	if(width()==0.0){return (long double)0.0;}//undefined
    long double s = sqr((long double)1.732 * DispersalSigma() / width());
    return s;
    }//end ClineShapeCWTlTrTBDParamRec::s_star_Ecotone()


long double ClineShapeCWTlTrTBDParamRec::s_star_DomEcotone(void){
	//indirect & direct selection on marker loci in middle of cline
	//assuming selection across an ecotone on a dominant marker
	if(width()==0.0){return (long double)0.0;}//undefined
    long double s = sqr((long double)1.782 * DispersalSigma() / width());
    return s;
    }//end ClineShapeCWTlTrTBDParamRec::s_star_DomEcotone()




long double ClineShapeCWTlTrTBDParamRec::timeBarrierL(int trait){
	long double d = DispersalSigma();
    if (d == 0.0){
		return (long double)1.0e20;}//basically forever if no dispersal
	long double b = distanceBarrierL(trait);
    if (b / d > (long double)1.0e40){
		return (long double)1.0e20;}//basically forever if beta/sigma too high
    long double timeL = sqr(b / d);
    return timeL;
}//end ClineShapeCWTlTrTBDParamRec::timeBarrierL(int)


long double ClineShapeCWTlTrTBDParamRec::timeBarrierR(int trait){
	long double d = DispersalSigma();
    if (d == 0.0){
		return (long double)1.0e20;}//basically forever if no dispersal
	long double b = distanceBarrierR(trait);
    if (b / d > (long double)1.0e40){
		return (long double)1.0e20;}//basically forever if beta/sigma too high
    long double timeR = sqr(b / d);
    return timeR;
}//end ClineShapeCWTlTrTBDParamRec::timeBarrierR(int)




void ClineShapeCWTlTrTBDParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::TitleString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat("width");
	dataline.cat(sepchar);
	dataline.cat("D");
	dataline.cat(sepchar);
	dataline.cat("lnLDiseq");
	dataline.cat(sepchar);
	dataline.cat("dispersal_sigma");
	dataline.cat(sepchar);
	dataline.cat("s*[hets]");
	dataline.cat(sepchar);
	dataline.cat("s*[ecotone]");
	dataline.cat(sepchar);
	dataline.cat("s*[dom,ecotone]");
	dataline.cat(sepchar);
	dataline.cat("W[hyb]/W[pure]");
	dataline.cat(sepchar);
	dataline.cat("S");
	dataline.cat(sepchar);
	dataline.cat("n_loci_selected");
	dataline.cat(sepchar);
	dataline.cat("s_per_Repro_Isolation_Locus");
	for(int t=0;t<traitParamSet_.len();++t){
		aString tStr;
		int n = traitParamSet_[t].TotalParams();
		for(int p=0;p<n;++p){
			aString* pName = traitParamSet_[t].ParamName(p);
			pName->cat("[");pName->cat(t+1);pName->cat("]");
			dataline.cat(sepchar);
			dataline.cat(*pName);
			delete pName;pName=0;
			}//end for p			
		dataline.cat(sepchar);
		dataline.cat("barrierL[dist][");dataline.cat(t+1);dataline.cat("]");
		dataline.cat(sepchar);
		dataline.cat("barrierR[dist][");dataline.cat(t+1);dataline.cat("]");
		dataline.cat(sepchar);
		dataline.cat("barrierL[time][");dataline.cat(t+1);dataline.cat("]");
		dataline.cat(sepchar);
		dataline.cat("barrierR[time][");dataline.cat(t+1);dataline.cat("]");
		dataline.cat(sepchar);
		dataline.cat("sL_marker[");dataline.cat(t+1);dataline.cat("]");
		dataline.cat(sepchar);
		dataline.cat("sR_marker[");dataline.cat(t+1);dataline.cat("]");
		}//end for t	
	}//end ClineShapeCWTlTrTBDParamRec::TitleString(aString&, char)


long double ClineShapeCWTlTrTBDParamRec::WhybridOverWpure(void){
	long double w = width();
	if(w==0){return (long double) 0.0;}//undefined
	long double beta = 0.0;
	for(int t=0;t<traitParamSet_.len();++t){
		beta += distanceBarrierL(t) + distanceBarrierR(t);
		}
	beta /= ((long double)2.0* traitParamSet_.len());//average
    if ((beta / w > (long double)3000.0) || (beta / w == (long double)0.0)){
		return (long double)0.0;}
    double wratio = exp(- r_ * (ln(beta / w)));
    return wratio;
	}//end ClineShapeCWTlTrTBDParamRec::WhybridOverWpure()


long double ClineShapeCWTlTrTBDParamRec::width(void){//returns width
	ClineShapeCWTlTrTBParamRec* traitParams = 
			(ClineShapeCWTlTrTBParamRec*) traitParamSet_[0].pointerToDerivedSelf;
	return traitParams->width();
	}//end ClineShapeCWTlTrTBDParamRec::width()


void ClineShapeCWTlTrTBDParamRec::operator=(ParamRec& p){
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means p must actually point to an object of class ClineShapeCWTlTrParamRec
	ClineShapeCWTlTrTBDParamRec* pp = (ClineShapeCWTlTrTBDParamRec*) p.pointerToDerivedSelf;
	traitParamSet_.Clear();
	for(int t=0;t<pp->traitParamSet_.len();++t){
		ClineShapeCWTlTrTBParamRec* ppTrait = 
					(ClineShapeCWTlTrTBParamRec*) pp->traitParamSet_[t].pointerToDerivedSelf;
		traitParamSet_.Append(NEW ClineShapeCWTlTrTBParamRec(*ppTrait));
		}
	lnL_= pp->lnL_;
	lnLmaxMinusLnL_ = pp->lnLmaxMinusLnL_;	
  	r_ = pp->r_;
  	D_ = pp->D_;
  	lnLDiseq_ = pp->lnLDiseq_;
	}//end  ClineShapeCWTlTrTBDParamRec::operator=(ParamRec&)





int	operator==(ClineShapeCWTlTrTBDParamRec& p, ClineShapeCWTlTrTBDParamRec& q){
	if ((p.lnL_ != q.lnL_) ||
			(p.r_ != q.r_) ||
			(p.D_ != q.D_)
			 ){return 0;}
	if(p.traitParamSet_.len() != q.traitParamSet_.len()){return 0;}
	for(int t=0;t<p.traitParamSet_.len();++t){
		ClineShapeCWTlTrTBParamRec* pTrait = 
					(ClineShapeCWTlTrTBParamRec*) p.traitParamSet_[t].pointerToDerivedSelf;
		ClineShapeCWTlTrTBParamRec* qTrait = 
					(ClineShapeCWTlTrTBParamRec*) q.traitParamSet_[t].pointerToDerivedSelf;
		if(pTrait != qTrait){return 0;}
		}//end for t
	return 1;
	}//end operator==(ClineShapeCWTlTrTBDParamRec&, ClineShapeCWTlTrTBDParamRec&)


int	operator!=(ClineShapeCWTlTrTBDParamRec& p, ClineShapeCWTlTrTBDParamRec& q){
	return !(p==q);
	}//end operator!=(ClineShapeCWTlTrTBDParamRec&, ClineShapeCWTlTrTBDParamRec&)



#endif


