/******************************************************
*
*	ClineShapeCWTlTrTBParamRec.h
*
*	Parameters for a 3-part composite tanh cline defined by center, width,	
*	thetaL, thetaR, zL & zR.  The latter 4 parameters describe introgression
*	slopes and intercepts on the Left and Right sides of the cline.
*	The mean y-axis value of the cline varies for each trait on each side.
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/

#ifndef ClineShapeCWTlTrTBParamRech
#define ClineShapeCWTlTrTBParamRech = 1


#include "ParamRec.h"
#include "Hyperbolics.h"
#include "InlineTemplates.h"
#include "ClineShapeCWTlTrParamRec.h"
#include "MemoryManager.h"

class ClineShapeCWTlTrTBParamRec;
class ClineShapeCWTlTrTBParamRec: public ClineShapeCWTlTrParamRec{
  public:
  	long double pL_;//can fit more than one marker
  	long double pR_;
//  	int pLOn_, pROn_;
//  	aList<int> paramOnOffList_;//allows each shape parameter to vary if on (1), but not if off (0)
//  	aList<int> paramsOn_;
  		/*********Parameter order in the on/off lists
  		c_,w_,tL_,tR_,zL_,zR_,
  		p[L,trait]*ntraits,   
  		p[R,trait]*ntraits, 
  		xL_,xR_,
  		bL_*ntraits,
  		bR_*ntraits,  
  		deltaP[trait]*ntraits
  		Thus, the list has variable length that depends on the number of traits or loci being fit at one time
  		**********/
  public:
  	ClineShapeCWTlTrTBParamRec(void);
  	ClineShapeCWTlTrTBParamRec(long double c, long double w,//does not set xL_ or xR_; must do this separately
							 long double tL, long double tR,//use for high/low limits and deviations
							 long double zL, long double zR,
							 long double leftSideScore, long double rightSideScore
							 );
  	ClineShapeCWTlTrTBParamRec(long double c, long double w,//sets xL_ and xR_
							 long double tL, long double tR,
							 long double zL, long double zR,
							 long double leftSideScore, long double rightSideScore,
							 double leftEnd, double rightEnd
							 );
//  	ClineShapeCWTlTrTBParamRec(ClineFitInitializer& i);//all traits
  	ClineShapeCWTlTrTBParamRec(ClineFitInitializer& i, int trait_);//just this trait
  	ClineShapeCWTlTrTBParamRec(ClineShapeCWTlTrTBParamRec& p);
  	virtual ~ClineShapeCWTlTrTBParamRec(void){
//  			pointerToDerivedSelf = NULL;//forget
  				}

  	virtual ParamRec* 				Duplicate(void);
  	virtual ClineShapeCWTlTrTBParamRec* 	_Duplicate(void);
//	virtual void 					SetPointerToDerivedSelf(void);//always override in derived
	virtual long 					GetPointerToDerivedSelf(void);//always override in derived
  	


	virtual long double B_L(int trait=0);//cline frequency at xL; standardized to {0,1} unless left side not estimated
	virtual long double B_R(int trait=0);//cline frequency at xR; standardized to {0,1} unless right side not estimated
	virtual int 		ChangeCenter(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeLeftScore(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
/*debug*/	virtual int 		ChangeParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
									ParamRec* paramDeviations,  
									ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=1.0);
	virtual int 		ChangeRightScore(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeThetaL(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeThetaR(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeWidth(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeZL(Ran3& ran, double sigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeZR(Ran3& ran, double sigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual long double ExpectedValue(aList<long double>& coords);
	virtual void 		Initialize(ClineFitInitializer& i){Initialize(i,0);}
	virtual void 		Initialize(ClineFitInitializer& i, int trait_);//single-trait cline on this trait
	virtual long double leftVal(int trait=0);//returns value
			void 		leftVal(long double lft,int trait=0);//sets value
	virtual int			LJunction(double leftEnd);
	virtual int 		MeetsParamCriteria(ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void 		MultiplyParam(int paramNum, long double factor);//use to rescale a param deviation
	virtual int 		NumParams(void);//number of parameters
	virtual int 		NumDerivedParams(void);//number of parameters that are functions of varying parameters
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
	virtual long double rightVal(int trait=0);//returns value
			void 		rightVal(long double rt,int trait=0);//sets value
	virtual int			RJunction(double rightEnd);
	virtual void 		SetParamValue(int paramNum, long double newVal);//sets value of parameter
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file
	virtual int			traits(void){return 1;}
	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

	friend int	operator==(ClineShapeCWTlTrTBParamRec& p, ClineShapeCWTlTrTBParamRec& q);
	friend int	operator!=(ClineShapeCWTlTrTBParamRec& p, ClineShapeCWTlTrTBParamRec& q);

	};//end class ClineShapeCWTlTrTBParamRec



#endif