/******************************************************
*
*	ClineShapeCWTlTrTBQ2ParamRec.cpp
*
*	Quantitative trait parameters for a 3-part composite tanh cline defined by center, width,	
*	thetaL, thetaR, zL & zR.  The latter 4 parameters describe introgression
*	slopes and intercepts on the Left and Right sides of the cline.
*	The mean y-axis value of the cline varies for each trait on each side.
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/

#ifndef ClineShapeCWTlTrTBQ2ParamReccpp
#define ClineShapeCWTlTrTBQ2ParamReccpp = 1

#include "ClineShapeCWTlTrTBQ2ParamRec.h"

	
ClineShapeCWTlTrTBQ2ParamRec::ClineShapeCWTlTrTBQ2ParamRec(void){
//  	SetPointerToDerivedSelf();
  	}




ClineShapeCWTlTrTBQ2ParamRec::ClineShapeCWTlTrTBQ2ParamRec(ClineFitInitializer& i){
	//assumes the parameter analysis is turned on
// 	SetPointerToDerivedSelf();
 	Initialize(i);
  	}



ClineShapeCWTlTrTBQ2ParamRec::ClineShapeCWTlTrTBQ2ParamRec(ClineShapeCWTlTrTBQ2ParamRec& p){
//  	SetPointerToDerivedSelf();
	lnL_= p.lnL_;
	lnLmaxMinusLnL_ = p.lnLmaxMinusLnL_;	
  	traitParamSet_=p.traitParamSet_;
	r_=p.r_;
	covMatrixList_ = p.covMatrixList_;
	covMatrixListType_ = p.covMatrixListType_;
	invCovMatrixList_ = p.invCovMatrixList_;
	detCovMatrixList_ = p.detCovMatrixList_;
	invCovMatrixFlags_ = p.invCovMatrixFlags_;
	detCovMatrixFlags_ = p.detCovMatrixFlags_;
	nparams_=p.nparams_;
	nshparams_=p.nshparams_;
	ndparams_=p.ndparams_;
	paramIdentities_ = p.paramIdentities_;
	varyingParams_ = p.varyingParams_;
	D_=p.D_;
	Dcalculated_=p.Dcalculated_;
	}



ParamRec* ClineShapeCWTlTrTBQ2ParamRec::Duplicate(void){
	ClineShapeCWTlTrTBQ2ParamRec* copy = NEW ClineShapeCWTlTrTBQ2ParamRec(*this);
	return copy;
	}//end ClineShapeCWTlTrTBQ2ParamRec::Duplicate()


ClineShapeCWTlTrTBQ2ParamRec* ClineShapeCWTlTrTBQ2ParamRec::_Duplicate(void){
	ClineShapeCWTlTrTBQ2ParamRec* copy = NEW ClineShapeCWTlTrTBQ2ParamRec(*this);
	return copy;
	}//end ClineShapeCWTlTrTBQ2ParamRec::_Duplicate()


//void ClineShapeCWTlTrTBQ2ParamRec::SetPointerToDerivedSelf(void){
//  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in ClineShapeCWTlTrTBQ2ParamRec"<<endl;
//  	}//end ClineShapeCWTlTrTBQ2ParamRec::SetPointerToDerivedSelf()

long ClineShapeCWTlTrTBQ2ParamRec::GetPointerToDerivedSelf(void){
  	return (long) this;
  	}//end ClineShapeCWTlTrTBQ2ParamRec::SetPointerToDerivedSelf()






long double ClineShapeCWTlTrTBQ2ParamRec::B_L(int trait){//returns value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
    return traitParams->B_L(0);
}//end ClineShapeCWTlTrTBQ2ParamRec::B_L(int)


long double ClineShapeCWTlTrTBQ2ParamRec::B_R(int trait){//returns value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
    return traitParams->B_R(0);
}//end ClineShapeCWTlTrTBQ2ParamRec::B_R(int)


long double ClineShapeCWTlTrTBQ2ParamRec::center(int trait){//returns value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
    return traitParams->center();
}//end ClineShapeCWTlTrTBQ2ParamRec::center(int)


void ClineShapeCWTlTrTBQ2ParamRec::center(int trait, long double value){//sets value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
//    if (!(traitParams->paramsOn_.Includes(0))){//assumes 1 trait/paramset in this configuration
//    if (traitParams->paramsOn_.Includes(0)){//assumes 1 trait/paramset in this configuration
			traitParams->center(value);
//	}//otherwise ignore
}//end ClineShapeCWTlTrTBQ2ParamRec::center(int, double)



int ClineShapeCWTlTrTBQ2ParamRec::ChangeCenter(Ran3& ran, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& covaryingParamList, long double weight){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false, tries=0;
	int tr0 = covaryingParamList[0];
	long double oldCenter = center(tr0);
	ClineShapeCWTlTrTBQ2ParamRec* devs = (ClineShapeCWTlTrTBQ2ParamRec*) paramDeviations->GetPointerToDerivedSelf();
	long double sigma = devs->center(tr0);
	do{
		long double newCenter = ran.NormDev(oldCenter, sigma*weight);
		for(int tr=0;tr<covaryingParamList.len();++tr){//set all co-varying centers
			center(covaryingParamList[tr],newCenter);}
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		++tries;
		if((! validChange) && tries>20){
			for(int tr=0;tr<covaryingParamList.len();++tr){//set all co-varying centers
				center(covaryingParamList[tr],oldCenter);}
			return 0;//can't find a good value, so bail out
			}			
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrTBQ2ParamRec::ChangeCenter(Ran3&, ParamRec*, ParamRec*, ParamRec*, aList<int>&)




int ClineShapeCWTlTrTBQ2ParamRec::ChangeCovMatrix(Ran3& ran, int paramNum, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, double weight){
	//use normal distribution to vary score
	//limit covariance to give -1correlation1
	//return 0 for failure, 1 for success
	int validChange=false, tries=0;
	ClineShapeCWTlTrTBQ2ParamRec* devs = (ClineShapeCWTlTrTBQ2ParamRec*) paramDeviations->GetPointerToDerivedSelf();
	double n = traits();
	int paramsPerRegion = n + n*(n-1.0)/2.0;
	int region = DIV(paramNum,paramsPerRegion);//corresponding region
	int param = MOD(paramNum,paramsPerRegion);//paramnum within the right region
	int pcount=0, tr1, tr2, gotit=false;
	for(int t1=0;t1<traits();++t1){
		for(int t2=0;t2<=t1;++t2){
			if(pcount==param){
				gotit=true;
				tr1=t1; tr2=t2;
				break;}
			pcount++;
			}//t2
		if(gotit){break;}
		}//t1
	if (!gotit){//debugging
			aString err("not counting parameters right in ClineShapeCWTlTrTBQ2ParamRec::ChangeCovMatrix()");
			aError(err);}
	long double oldScore = covMatrixList_[region][tr1][tr2];
	long double sigma = devs->covMatrixList_[region][tr1][tr2];
	do{
		covMatrixList_[region][tr1][tr2] = covMatrixList_[region][tr2][tr1] = ran.NormDev(oldScore, sigma*weight);
		invCovMatrixFlags_[region] = detCovMatrixFlags_[region] = 0;//recalculate these
//		if(covMatrixList_[region].Determinant()<0){
//			aError("change makes det <0");}
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		++tries;
		if((! validChange) && tries>20){
			aString err("can't find a good value for covMatrixList_[");
			err.cat(region);err.cat("][");err.cat(tr1);err.cat("][");err.cat(tr2);err.cat("]");
			aError(err);
			tries = 0;
			sigma *= 0.9;
			if(sigma<1.0e-4){///bail out
				covMatrixList_[region][tr1][tr2] = covMatrixList_[region][tr2][tr1]=oldScore;
				validChange=1;}
			}
		}while(!(validChange));
	if(tr1 != tr2){Dcalculated_=false;}
	return validChange;
	}//end ClineShapeCWTlTrTBQ2ParamRec::ChangeCovMatrix(Ran3&, ParamRec*, ParamRec*, ParamRec*)



int ClineShapeCWTlTrTBQ2ParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, long double weight){
	//return 0 for failure, 1 for success
	int paramToAlter = paramNum;
	aList<int> covaryingParamList;//traits that will vary together for a given parameter
	if(paramNum<NumShapeParams()){
		ShapeParamCoords(paramNum, paramToAlter, covaryingParamList);}
	//uses type conversion that circumvents usual virtual base class restrictions
	ClineShapeCWTlTrTBQ2ParamRec* deviations = (ClineShapeCWTlTrTBQ2ParamRec*) paramDeviations->GetPointerToDerivedSelf();
	int success = 0;
	switch (paramToAlter){
	   case 0:
		  return ChangeCenter(ran, paramDeviations, lowerLimits, upperLimits, covaryingParamList, weight);
		  break;
		case 1:
		  return ChangeWidth(ran, paramDeviations, lowerLimits, upperLimits, covaryingParamList, weight);
		  break;
		case 2:
		   return ChangeThetaL(ran, paramDeviations, lowerLimits, upperLimits, covaryingParamList, weight);
		   break;
		 case 3:
			 return ChangeThetaR(ran, paramDeviations, lowerLimits, upperLimits, covaryingParamList, weight);
			 break;
		 case 4:
		 	 return ChangeZL(ran, paramDeviations, lowerLimits, upperLimits, covaryingParamList, weight);
		 	 break;
		 case 5:
		 	 return ChangeZR(ran, paramDeviations, lowerLimits, upperLimits, covaryingParamList, weight);
		 	 break;
		 case 6:
		 	 return ChangeLeftScore(ran, paramDeviations, lowerLimits, upperLimits, covaryingParamList, weight);
		 	 break;
		 case 7:
		 	 return ChangeRightScore(ran, paramDeviations, lowerLimits, upperLimits, covaryingParamList, weight);
		 	 break;
		 default:
		 	 break;//do next step
		 }//end switch
	paramToAlter -= NumShapeParams();
	if(paramToAlter<NumCovMatrixParams()){
		return ChangeCovMatrix(ran, paramToAlter, paramDeviations, lowerLimits, upperLimits, weight);
		}
	aString errstr("bug: can't change shape param #");
	errstr.cat(paramNum+1);
	errstr.cat(" in a ");errstr.cat(NumParams());errstr.cat("-parameter cline model");
	aError(errstr);
	return  success;
	}//end ClineShapeCWTlTrTBQ2ParamRec::ChangeParam(int,Ran3&,ParamRec*,ParamRec*,ParamRec*)



int ClineShapeCWTlTrTBQ2ParamRec::ChangeWidth(Ran3& ran, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& covaryingParamList, long double weight){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false, tries=0;
	int tr0 = covaryingParamList[0];
	long double oldWidth = width(tr0);
	ClineShapeCWTlTrTBQ2ParamRec* devs = (ClineShapeCWTlTrTBQ2ParamRec*) paramDeviations->GetPointerToDerivedSelf();
	double sigma = devs->width(tr0);
	do{
		long double newWidth = ran.NormDev(oldWidth, sigma*weight);
		for(int tr=0;tr<covaryingParamList.len();++tr){//set all co-varying centers
			width(covaryingParamList[tr],newWidth);}
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		++tries;
		if((! validChange) && tries>20){
			for(int tr=0;tr<covaryingParamList.len();++tr){//set all co-varying centers
				width(covaryingParamList[tr],oldWidth);}
			return 0;//can't find a good value, so bail out
			}			
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrTBQ2ParamRec::ChangeWidth(Ran3&, ParamRec*, ParamRec*, ParamRec*, aList<int>&)




int ClineShapeCWTlTrTBQ2ParamRec::ChangeLeftScore(Ran3& ran, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& covaryingParamList, long double weight){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false, tries=0;
	int tr0 = covaryingParamList[0];
	long double oldLeftVal = leftVal(tr0);
	ClineShapeCWTlTrTBQ2ParamRec* devs = (ClineShapeCWTlTrTBQ2ParamRec*) paramDeviations->GetPointerToDerivedSelf();
	double sigma = devs->leftVal(tr0);
	do{
		long double newLeftVal = ran.NormDev(oldLeftVal, sigma*weight);
		for(int tr=0;tr<covaryingParamList.len();++tr){//set all co-varying centers
			leftVal(covaryingParamList[tr],newLeftVal);}
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		++tries;
		if((! validChange) && tries>20){
			for(int tr=0;tr<covaryingParamList.len();++tr){//set all co-varying centers
				leftVal(covaryingParamList[tr],oldLeftVal);}
			return 0;//can't find a good value, so bail out
			}			
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrTBQ2ParamRec::ChangeLeftScore(Ran3&, ParamRec*, ParamRec*, ParamRec*, aList<int>&)




int ClineShapeCWTlTrTBQ2ParamRec::ChangeRightScore(Ran3& ran, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& covaryingParamList, long double weight){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false, tries=0;
	int tr0 = covaryingParamList[0];
	long double oldRightVal = rightVal(tr0);
	ClineShapeCWTlTrTBQ2ParamRec* devs = (ClineShapeCWTlTrTBQ2ParamRec*) paramDeviations->GetPointerToDerivedSelf();
	double sigma = devs->rightVal(tr0);
	do{
		long double newRightVal = ran.NormDev(oldRightVal, sigma*weight);
		for(int tr=0;tr<covaryingParamList.len();++tr){//set all co-varying centers
			rightVal(covaryingParamList[tr],newRightVal);}
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		++tries;
		if((! validChange) && tries>20){
 			for(int tr=0;tr<covaryingParamList.len();++tr){//set all co-varying centers
				rightVal(covaryingParamList[tr],oldRightVal);}
			return 0;//can't find a good value, so bail out
			}			
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrTBQ2ParamRec::ChangeRightScore(Ran3&, ParamRec*, ParamRec*, ParamRec*, aList<int>&)




int ClineShapeCWTlTrTBQ2ParamRec::ChangeThetaL(Ran3& ran, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& covaryingParamList, long double weight){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false, tries=0;
	int tr0 = covaryingParamList[0];
	long double oldThetaL = thetaL(tr0);
	ClineShapeCWTlTrTBQ2ParamRec* devs = (ClineShapeCWTlTrTBQ2ParamRec*) paramDeviations->GetPointerToDerivedSelf();
	double sigma = devs->thetaL(tr0);
	do{
		long double newThetaL = ran.NormDev(oldThetaL, sigma*weight);
		for(int tr=0;tr<covaryingParamList.len();++tr){//set all co-varying centers
			thetaL(covaryingParamList[tr],newThetaL);}
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		++tries;
		if((! validChange) && tries>20){
 			for(int tr=0;tr<covaryingParamList.len();++tr){//set all co-varying centers
				thetaL(covaryingParamList[tr],oldThetaL);}
			return 0;//can't find a good value, so bail out
			}			
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrTBQ2ParamRec::ChangeThetaL(Ran3&, ParamRec*, ParamRec*, ParamRec*, aList<int>&)




int ClineShapeCWTlTrTBQ2ParamRec::ChangeThetaR(Ran3& ran, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& covaryingParamList, long double weight){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false, tries=0;
	int tr0 = covaryingParamList[0];
	long double oldThetaR = thetaR(tr0);
	ClineShapeCWTlTrTBQ2ParamRec* devs = (ClineShapeCWTlTrTBQ2ParamRec*) paramDeviations->GetPointerToDerivedSelf();
	double sigma = devs->thetaR(tr0);
	do{
		long double newThetaR = ran.NormDev(oldThetaR, sigma*weight);
		for(int tr=0;tr<covaryingParamList.len();++tr){//set all co-varying centers
			thetaR(covaryingParamList[tr],newThetaR);}
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		++tries;
		if((! validChange) && tries>20){
 			for(int tr=0;tr<covaryingParamList.len();++tr){//set all co-varying centers
				thetaR(covaryingParamList[tr],oldThetaR);}
			return 0;//can't find a good value, so bail out
			}			
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrTBQ2ParamRec::ChangeThetaR(Ran3&, ParamRec*, ParamRec*, ParamRec*, aList<int>&)




int ClineShapeCWTlTrTBQ2ParamRec::ChangeZL(Ran3& ran, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& covaryingParamList, long double weight){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false, tries=0;
	int tr0 = covaryingParamList[0];
	long double oldZL = zL(tr0);
	ClineShapeCWTlTrTBQ2ParamRec* devs = (ClineShapeCWTlTrTBQ2ParamRec*) paramDeviations->GetPointerToDerivedSelf();
	double sigma = devs->zL(tr0);
	do{
		long double newZL = ran.NormDev(oldZL, sigma*weight);
		for(int tr=0;tr<covaryingParamList.len();++tr){//set all co-varying centers
			zL(covaryingParamList[tr],newZL);}
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		++tries;
		if((! validChange) && tries>20){
 			for(int tr=0;tr<covaryingParamList.len();++tr){//set all co-varying centers
				zL(covaryingParamList[tr],oldZL);}
			return 0;//can't find a good value, so bail out
			}			
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrTBQ2ParamRec::ChangeZL(Ran3&, ParamRec*, ParamRec*, ParamRec*, aList<int>&)




int ClineShapeCWTlTrTBQ2ParamRec::ChangeZR(Ran3& ran, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, aList<int>& covaryingParamList, long double weight){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false, tries=0;
	int tr0 = covaryingParamList[0];
	long double oldZR = zR(tr0);
	ClineShapeCWTlTrTBQ2ParamRec* devs = (ClineShapeCWTlTrTBQ2ParamRec*) paramDeviations->GetPointerToDerivedSelf();
	double sigma = devs->zR(tr0);
	do{
		long double newZR = ran.NormDev(oldZR, sigma*weight);
		for(int tr=0;tr<covaryingParamList.len();++tr){//set all co-varying centers
			zR(covaryingParamList[tr],newZR);}
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		++tries;
		if((! validChange) && tries>20){
 			for(int tr=0;tr<covaryingParamList.len();++tr){//set all co-varying centers
				zR(covaryingParamList[tr],oldZR);}
			return 0;//can't find a good value, so bail out
			}			
		}while(!(validChange));
	return validChange;
	}//end ClineShapeCWTlTrTBQ2ParamRec::ChangeZR(Ran3&, ParamRec*, ParamRec*, ParamRec*, aList<int>&)




long double ClineShapeCWTlTrTBQ2ParamRec::Cov(int trait1, int trait2, int region){
	return covMatrixList_[region][trait1][trait2];
	}//end ClineShapeCWTlTrTBQ2ParamRec::Cov(int, int, int)


void ClineShapeCWTlTrTBQ2ParamRec::CovParamCoords(int paramNum, int& region, int& atTrait1, int& atTrait2){
	//replaces values of region, atTrait1 & atTrait2 with corresponding values
	int origParamNum=paramNum;
	for(int r=0;r<regions();++r){
		if(paramNum<NumCovMatrixParams()/regions()){
			//we're in the right region
			region = r;
			int p=0;
			for(int tr1=0;tr1<traits();++tr1){
				for(int tr2=0;tr2<=tr1;++tr2){
					if(p==paramNum){
						//this is the parameter we want
						atTrait1 = tr1;
						atTrait2 = tr2;
						return;}
					++p;
					}//tr2
				}//tr1
			}//if in right region
		paramNum -= NumCovMatrixParams()/regions();
		}//r
	//shouldn't get here
	aString e("Error in ClineShapeCWTlTrTBQ2ParamRec::CovParamCoords; can't find value for param ");
	e.cat(origParamNum);
	aError(e);
	}//ClineShapeCWTlTrTBQ2ParamRec::CovParamCoords(int, int&, int&, int&)



long double ClineShapeCWTlTrTBQ2ParamRec::CovTraitDeterminant(int region){
	if(detCovMatrixFlags_[region]==1){
		return detCovMatrixList_[region];}
	long double det = covMatrixList_[region].Determinant();
	detCovMatrixList_[region] = det;
	detCovMatrixFlags_[region] = 1;
	return det;
	}//ClineShapeCWTlTrTBQ2ParamRec::CovTraitDeterminant(int)



long double ClineShapeCWTlTrTBQ2ParamRec::D(void){//returns disequilibrium from covariance structure
	if(Dcalculated_){return D_;}
    D_=0.0;
    int count = 0;
    for(int t1=0; t1<traits();++t1){//rows
    	long double dz1 = ABS(B_R(t1) - B_L(t1));
    	for(int t2=0;t2<t1;++t2){//loop through all covariance pairs
    		long double dz2 = ABS(B_R(t2) - B_L(t2));
    		long double cov = Cov(t1,t2,1);//bug here if pops used as regions
    		if(dz1>0.0 && dz2>0.0 && cov>=0.0){
		    	D_ += (2.0 *(1.0 - r_) * cov) / (dz1 * dz2);
		    	++count;}
		    }}
	if(count>0){
		D_ /= (long double) count;
		}//simple mean
	Dcalculated_=true;
	return D_;//default
	}//end ClineShapeCWTlTrTBDParamRec::D()


long double ClineShapeCWTlTrTBQ2ParamRec::DeltaP(int trait){//the frequency step across the center region
	return B_R(trait)-B_L(trait);}


long double ClineShapeCWTlTrTBQ2ParamRec::DispersalSigma(void){
    if (D() > 0.0){
    	long double w = width();
		long double sigma = sqrt((D() * r_ * w * w) / (1.0 + r_));
		return sigma;}
	return (long double) 0.0;//default
	}//end ClineShapeCWTlTrTBDParamRec::DispersalSigma()


long double ClineShapeCWTlTrTBQ2ParamRec::distanceBarrierL(int trait){
	//barrier to introgression in distance units
    //derivatives found using Mathematica
    long double w = width(trait);
	ClineShapeCWTlTrTBParamRec* traitParams = 
			(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
//    if (!(traitParams->paramsOn_.Includes(2))||!(traitParams->paramsOn_.Includes(4))){
//			return exp(100.0);}//huge and undefined
    long double derivFactor = (long double) 4.0 * sqrt(traitParams->thetaL()) 
    		* (traitParams->xL() - (traitParams->center() + traitParams->zL())) / w;
    long double dPLdXatxL, beta;
    if ((w ==(long double)0.0)||(derivFactor <= -100.0)){
			return exp((long double)100.0);}//huge and undefined
    if (derivFactor <= (long double)100.0){
			dPLdXatxL = exp(derivFactor) * sqrt(traitParams->thetaL()) / w;}
    	else{
			dPLdXatxL = exp((long double)100.0);}

    if(dPLdXatxL == (long double)0.0){
			return exp((long double)100.0);}
		else{
    		long double dP = traitParams->DeltaP(0);//derivatives found using Mathematica
    		long double spanP = traitParams->rightVal(0) - traitParams->leftVal(0);
			beta = (dP/spanP) / dPLdXatxL;}
	return beta;
	}//end ClineShapeCWTlTrTBQ2ParamRec::distanceBarrierL(int)
 


long double ClineShapeCWTlTrTBQ2ParamRec::distanceBarrierR(int trait){//barrier to introgression in distance units
    //derivatives found using Mathematica
    long double w = width(trait);
	ClineShapeCWTlTrTBParamRec* traitParams = 
			(ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
//    if (!(traitParams->paramsOn_.Includes(3))||!(traitParams->paramsOn_.Includes(5))){
//			return exp(100.0);}//huge and undefined
    long double derivFactor = (long double)-4.0 * sqrt(traitParams->thetaR()) 
    		* (traitParams->xR() - (traitParams->center() - traitParams->zR())) / w;
    long double dPRdXatxR, beta;
    if ((w ==(long double)0.0)||(derivFactor <= (long double)-100.0)){
			return exp((long double)100.0);}//huge and undefined
    if (derivFactor <= (long double)100.0){
			dPRdXatxR = exp(derivFactor) * sqrt(traitParams->thetaR()) / w;}
    	else{
			dPRdXatxR = exp((long double)100.0);}

    if(dPRdXatxR == (long double)0.0){
			return exp((long double)100.0);}
		else{
    		long double dP = traitParams->DeltaP(0);//derivatives found using Mathematica
    		long double spanP = traitParams->rightVal(0) - traitParams->leftVal(0);
			beta = (dP/spanP) / dPRdXatxR;}
	return beta;
	}//end ClineShapeCWTlTrTBQ2ParamRec::distanceBarrierR(int)
 



long double ClineShapeCWTlTrTBQ2ParamRec::ExpectedValue(aList<long double>& coords){
	//assumes coords[0]=x, coords[1]=trait
	ClineShapeCWTlTrTBParamRec* p = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[coords[1]].GetPointerToDerivedSelf();
	aList<long double> newCoords;
	newCoords.Append(coords[0]);
	newCoords.Append(0);
	return p->ExpectedValue(newCoords);
	}//end ClineShapeCWTlTrTBQ2ParamRec::ExpectedValue(aList<double>&)




void ClineShapeCWTlTrTBQ2ParamRec::Initialize(ClineFitInitializer& i){
	lnL_=0.0;
	lnLmaxMinusLnL_=0.0;
	nparams_=nshparams_=ndparams_=0;
	traitParamSet_.Clear();//in case of reinitialization
	covMatrixList_.Clear();
	invCovMatrixList_.Clear();
	detCovMatrixList_.Clear();
	invCovMatrixFlags_.Clear();
	detCovMatrixFlags_.Clear();
	paramIdentities_.Clear();
	varyingParams_.Clear();
	for(int t=0;t<i.traits();++t){
		ClineShapeCWTlTrTBParamRec* pp = NEW ClineShapeCWTlTrTBParamRec(i,t);//each holds one trait
		traitParamSet_.Append(pp);}
	r_ = i.r_;
	covMatrixList_ = i.covMatrixList_;
	covMatrixListType_ = regions();
	for(int r=0;r<regions();++r){
		invCovMatrixList_.Append(NEW aMatrix<long double>());//fill with placeholders
		detCovMatrixList_.Append(0.0);
		invCovMatrixFlags_.Append(0);
		detCovMatrixFlags_.Append(0);
		}//r
	D_=0.0;
	Dcalculated_=false;
	ObjList<aList<int> >& pident = *(i.paramIdentities_);

	for(int p=0;p<i.paramIdentities_[0].len();++p){//must transpose this matrix
		varyingParams_.Append(0);
		paramIdentities_.Append(NEW aList<int>());
		for(int tr=0;tr<traits();++tr){
			int ident = pident[tr][p];
			if(!(paramIdentities_[p].Includes(ident)) && ident!=0){//check before appending
				varyingParams_[p]++;}
			paramIdentities_[p].Append(ident);
			}//tr
		}//p
	
	}//end ClineShapeCWTlTrTBQParamRec::Initialize(ClineFitInitializer&)




aMatrix<long double>* ClineShapeCWTlTrTBQ2ParamRec::InverseCovTraitMatrix(int region){
	if(invCovMatrixFlags_[region]==1){
		return invCovMatrixList_[region].Duplicate();}
	aMatrix<long double>* inv = covMatrixList_[region].Inverse();
	aMatrix<long double>& oldInv = invCovMatrixList_[region];
	invCovMatrixList_.Replace(inv,region);
	aMatrix<long double>* oldInvPtr = &oldInv;
	delete oldInvPtr; oldInvPtr=0;
	invCovMatrixFlags_[region] = 1;
	return inv->Duplicate();
	}//ClineShapeCWTlTrTBQ2ParamRec::InverseCovTraitMatrix(int)
	
	

long double ClineShapeCWTlTrTBQ2ParamRec::leftVal(int trait){//returns value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
    return traitParams->leftVal(0);//assumes 1 trait/paramset in this configuration
	}//end ClineShapeCWTlTrTBQ2ParamRec::leftVal(int)


void ClineShapeCWTlTrTBQ2ParamRec::leftVal(long double value,int trait){//sets value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
//    if (!(traitParams->paramsOn_.Includes(6))){//assumes 1 trait/paramset in this configuration
//    if (traitParams->paramsOn_.Includes(6)){//assumes 1 trait/paramset in this configuration
			traitParams->leftVal(value,0);//}//otherwise ignore
	}//end ClineShapeCWTlTrTBQ2ParamRec::leftVal(int, double)



int ClineShapeCWTlTrTBQ2ParamRec::LJunction(int trait, double leftEnd){//sets xL and return 0 of the cline parts don't touch
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
//    if (traitParams->paramsOn_.Includes(2)){//left side section fit separately
			return traitParams->LJunction(leftEnd);//}//otherwise ignore
//	return 1;//default
	}//end ClineShapeCWTlTrTBQ2ParamRec::LJunction(int, double)




int ClineShapeCWTlTrTBQ2ParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
	//use type conversion that circumvents usual virtual base class restrictions
	ClineShapeCWTlTrTBQ2ParamRec* lower = (ClineShapeCWTlTrTBQ2ParamRec*) lowerLimits->GetPointerToDerivedSelf();
	ClineShapeCWTlTrTBQ2ParamRec* upper = (ClineShapeCWTlTrTBQ2ParamRec*) upperLimits->GetPointerToDerivedSelf();
	for(int tr=0;tr<traits();++tr){
		if(center(tr) <lower->center(tr)){return 0;}
			else if (center(tr) > upper->center(tr)){return 0;}
			else {}
		if(width(tr) <= 0.0){return 0;}
			else if (width(tr) > upper->width(tr)){return 0;}
			else {}
		if(thetaL(tr) <lower->thetaL(tr)){return 0;}
			else if (thetaL(tr) > upper->thetaL(tr)){return 0;}
			else {}
		if(thetaR(tr) <lower->thetaR(tr)){return 0;}
			else if (thetaR(tr) > upper->thetaR(tr)){return 0;}
			else {}
		if(zL(tr) <lower->zL(tr)){return 0;}
			else if (zL(tr) > upper->zL(tr)){return 0;}
			else {}
		if(zR(tr) <lower->zR(tr)){return 0;}
			else if (zR(tr) > upper->zR(tr)){return 0;}
			else {}
		if(LJunction(tr,lower->center(tr))==0){return 0;}
		if(RJunction(tr,upper->center(tr))==0){return 0;}
		if((xL(tr) > xR(tr))||(xL(tr)>center(tr))||(xR(tr)<center(tr))){return 0;}
			else if(xL(tr) < lower->xL(tr)){return 0;}
			else if(xR(tr) > upper->xR(tr)){return 0;}
			else {}
		if(leftVal(tr) < lower->leftVal(tr)){return 0;}
			else if (leftVal(tr) > upper->leftVal(tr)){return 0;}
			else {}
		if(rightVal(tr) <lower->rightVal(tr)){return 0;}
			else if (rightVal(tr) > upper->rightVal(tr)){return 0;}
			else {}
		}//end for tr
	for(int r=0;r<regions();++r){
		for(int t1=0;t1<traits();++t1){//first check bounds on variances
			if(covMatrixList_[r][t1][t1] <= lower->covMatrixList_[r][t1][t1]){return 0;}
				else if (covMatrixList_[r][t1][t1] > upper->covMatrixList_[r][t1][t1]){return 0;}
				else{}
			}//t1
		//determinant of cov matrix must be >0 or multinormal likelihood function crashes
		long double det = covMatrixList_[r].Determinant();
		if(det <= 0.0){
//			aError("determinant <0");
			return 0;}

				/*	//this is all covered by determinant criterion
						aMatrix<double>* inv = InverseCovTraitMatrix(r);
						long double detinv = inv->Determinant();
						if(detinv <= 0.0){
							//aError("inv determinant <0");
							delete inv; inv=0;
							return 0;}
						delete inv; inv=0;
						
						for(int t1=0;t1<traits()-1;++t1){//covariances must satisfy -1correlation1
							for(int t2=0;t2<t1;++t2){
								double denom = sqrt(covMatrixList_[r][t1][t1])*sqrt(covMatrixList_[r][t2][t2]);
								double corr = covMatrixList_[r][t1][t2]/denom;
								if((corr<-1.0)||(corr>1.0)){return 0;}
								}//t2
							}//t1
				*/
		}//r
	return 1;
	}//end ClineShapeCWTlTrTBQ2ParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)


void ClineShapeCWTlTrTBQ2ParamRec::MultiplyParam(int paramNum, long double factor){
	//use to rescale a param deviation
	int paramToAlter = paramNum,tr1,tr2,reg,atParam;
	if(paramToAlter<NumShapeParams()){
		aList<int> covaryingTraitParams;
		ShapeParamCoords(paramToAlter,atParam,covaryingTraitParams);
		long double val;
		switch (atParam){
			case 0:
			 	 val = center(covaryingTraitParams[0]);
			 	 for(int tr=0;tr<covaryingTraitParams.len();++tr){
			 	 	center(covaryingTraitParams[tr],val*factor);}
				 return;  break;
			 case 1:
			 	 val = width(covaryingTraitParams[0]);
			 	 for(int tr=0;tr<covaryingTraitParams.len();++tr){
			 	 	width(covaryingTraitParams[tr],val*factor);}
				 return;  break;
			 case 2:
			 	 val = thetaL(covaryingTraitParams[0]);
			 	 for(int tr=0;tr<covaryingTraitParams.len();++tr){
			 	 	thetaL(covaryingTraitParams[tr],val*factor);}
				 return;  break;
			 case 3:
			 	 val = thetaR(covaryingTraitParams[0]);
			 	 for(int tr=0;tr<covaryingTraitParams.len();++tr){
			 	 	thetaR(covaryingTraitParams[tr],val*factor);}
				 return;  break;
			 case 4:
			 	 val = zL(covaryingTraitParams[0]);
			 	 for(int tr=0;tr<covaryingTraitParams.len();++tr){
			 	 	zL(covaryingTraitParams[tr],val*factor);}
				 return;  break;
			 case 5:
			 	 val = zR(covaryingTraitParams[0]);
			 	 for(int tr=0;tr<covaryingTraitParams.len();++tr){
			 	 	zR(covaryingTraitParams[tr],val*factor);}
				 return;  break;
			 case 6:
			 	 val = leftVal(covaryingTraitParams[0]);
			 	 for(int tr=0;tr<covaryingTraitParams.len();++tr){
			 	 	leftVal(covaryingTraitParams[tr],val*factor);}
				 return;  break;
			 case 7:
			 	 val = rightVal(covaryingTraitParams[0]);
			 	 for(int tr=0;tr<covaryingTraitParams.len();++tr){
			 	 	rightVal(covaryingTraitParams[tr],val*factor);}
				 return;  break;
			 default:
			 	 break;//do below
			 }//end switch
		}//if < NumShapeParams()

	paramToAlter -= NumShapeParams();
	if(paramToAlter<NumCovMatrixParams()){
		CovParamCoords(paramToAlter, reg, tr1, tr2);
		covMatrixList_[reg][tr1][tr2] *= factor;
		if(tr1!=tr2){covMatrixList_[reg][tr2][tr1] *= factor;}//symmetry
		return;
		}

	aString errstr("bug: can't rescale param #");
	errstr.cat(paramNum+1);
	errstr.cat(" in a ");errstr.cat(NumParams());errstr.cat("-parameter cline model");
	aError(errstr);
	}//ClineShapeCWTlTrTBQ2ParamRec::MultiplyParam(int, double)


int ClineShapeCWTlTrTBQ2ParamRec::NumCovMatrixParams(void){//number of params estimated in covariance matrix
	int tr=traits();
	int n = ((tr-1) * tr)/2 + tr;//integer arithmetic
	return regions()*n;
	}//ClineShapeCWTlTrTBQ2ParamRec::NumCovMatrixParams()


int ClineShapeCWTlTrTBQ2ParamRec::NumDerivedParams(void){//fix this later
	if(ndparams_>0){return ndparams_;}
	ndparams_ = 5*traits();//xL,xR,bL[trait],bR[trait],deltaP[trait];
	if(traits()>1){
		//derived params that depend on D, in this order
		ndparams_ += 2;// sigma, D_
		ndparams_ += 3;// s_star_Ecotone, s_star_vs_Hets, s_perReproIsolationLocus
		ndparams_ += 3;// WhybridWpure, S, NumSelectedLoci
		for(int tr=0;tr<traits();++tr){
			ndparams_ += 2;// distanceBarrerL, distanceBarrierR
			ndparams_ += 2;// timeBarrerL, timeBarrierR
			ndparams_ += 2;//s_perMarkerLocusL, s_perMarkerLocusR
			}
		}
	return ndparams_;
	}//end ClineShapeCWTlTrTBQ2ParamRec::NumParams()


int ClineShapeCWTlTrTBQ2ParamRec::NumParams(void){
	if(nparams_>0){return nparams_;}
	nparams_ = NumShapeParams() + NumCovMatrixParams();
	return nparams_;
	}//end ClineShapeCWTlTrTBQ2ParamRec::NumParams()



int ClineShapeCWTlTrTBQ2ParamRec::NumShapeParams(void){
	//parameters can be shared (=vary together) among traits
	if(nshparams_>0){return nshparams_;}
	nshparams_ = 0;
	for(int p=0;p<8;++p){
		int max=paramIdentities_[p][0];
		for(int tr=1;tr<traits();++tr){//assumes paramIdentities_ holds sorted values
			max=MAX(max,paramIdentities_[p][tr]);}
		nshparams_ += max;
		}
	return nshparams_;
	}//end ClineShapeCWTlTrTBQ2ParamRec::NumShapeParams()



long double ClineShapeCWTlTrTBQ2ParamRec::NumSelectedLoci(void){
	long double s = s_star_vs_Hets();
    if (s == (long double)0.0){
		return 0.0;}//no loci under selection if no selection
	long double ww = WhybridOverWpure();
    if ((s >= 1.0) || (ww <= 0.0)){
		return 1.0e20;}//a really high number if hybrid fitness = 0
    long double nLocSel = 2.0 * ln(ww) / ln(1.0 - s);
    aString a(nLocSel);
    if(a=="undefined"){
    	aError(a);}//debug trap
    return nLocSel;
    }//end ClineShapeCWTlTrTBQ2ParamRec::NumSelectedLoci()



aString* ClineShapeCWTlTrTBQ2ParamRec::ParamName(int paramNum){
	//returns name of parameter; first the varying params, then derived params
	int paramToReport = paramNum;
	int tr1,tr2,reg,atParam;
	aString *theName;
	if(paramToReport<NumShapeParams()){
		aList<int> covaryingTraitParams;
		ShapeParamCoords(paramToReport,atParam,covaryingTraitParams);
		switch (atParam){
			case 0:
			  theName = NEW aString("center[");
			  break;
			case 1:
			  theName =  NEW aString("width[");
			  break;
			case 2:
			  theName =  NEW aString("thetaL[");
			  break;
			case 3:
			  theName =  NEW aString("thetaR[");
			  break;
			case 4:
			  theName =  NEW aString("zL[");
			  break;
			case 5:
			  theName =  NEW aString("zR[");
			  break;
			case 6:
			  theName =  NEW aString("p[L,");
			  break;
			case 7:
			  theName =  NEW aString("p[R,");
			  break;
			default:
			  theName =  NEW aString("error in ParamName() ");
			 break;//do below
			}//end switch
		for(int tr=0;tr<covaryingTraitParams.len();++tr){
		theName->cat(covaryingTraitParams[tr]+1);
		if(tr<covaryingTraitParams.len()-1){
		theName->cat(',');}
		}//tr
		theName->cat(']');
		return theName;
		}//if < NumShapeParams()
	paramToReport -= NumShapeParams();

	if(paramToReport<NumCovMatrixParams()){
		CovParamCoords(paramToReport, reg, tr1, tr2);
		if(tr1==tr2){//variance
				theName=NEW aString("var[region");theName->cat(reg+1); 
				theName->cat("][");theName->cat(tr1+1); theName->cat("]");}
			else{//(tr1!=tr2) --> covariance
				theName=NEW aString("cov[region");theName->cat(reg+1); 
				theName->cat("][");theName->cat(tr1+1); theName->cat("][");
				theName->cat(tr2+1); theName->cat("]");}
		return theName;
		}
	paramToReport -= NumCovMatrixParams();

	//it's a derived parameter
	if(paramToReport<traits()){
		 aString *str = NEW aString("p[xL,"); str->cat(paramToReport+1); str->cat("]");
		 return str;}
	paramToReport -= traits();
	if(paramToReport<traits()){
		 aString *str = NEW aString("p[xR,"); str->cat(paramToReport+1); str->cat("]");
		 return str;}
	paramToReport -= traits();
	if(paramToReport<traits()){
		 aString *str = NEW aString("deltaP["); str->cat(paramToReport+1); str->cat("]");
		 return str;}
	paramToReport -= traits();
	if(paramToReport<traits()){
		 aString *str = NEW aString("xL["); str->cat(paramToReport+1); str->cat("]");
		 return str;}
	paramToReport -= traits();
	if(paramToReport<traits()){
		 aString *str = NEW aString("xR["); str->cat(paramToReport+1); str->cat("]");
		 return str;}
	if(traits()>1){
		paramToReport -= traits();
		switch (paramToReport){
		 	case 0:
		 	 return NEW aString("sigma");
		 	 break;
		 	case 1:
		 	 return NEW aString("D");
		 	 break;
		 	case 2:
		 	 return NEW aString("s*[ecotone]");
		 	 break;
		 	case 3:
		 	 return NEW aString("s*[hets]");
		 	 break;
		 	case 4:
		 	 return NEW aString("s/[reproIsoLocus]");
		 	 break;
		 	case 5:
		 	 return NEW aString("W[h]/W[p]");
		 	 break;
		 	case 6:
		 	 return NEW aString("S");
		 	 break;
		 	case 7:
		 	 return NEW aString("n[selLoci]");
		 	 break;
		    default:
		     break;
		 }//end switch
		paramToReport -= 8;
		if(paramToReport<traits()){
			 aString *str = NEW aString("barrierL[dist,"); str->cat(paramToReport+1); str->cat("]");
			 return str;}
		paramToReport -= traits();
		if(paramToReport<traits()){
			 aString *str = NEW aString("barrierR[dist,"); str->cat(paramToReport+1); str->cat("]");
			 return str;}
		paramToReport -= traits();
		if(paramToReport<traits()){
			 aString *str = NEW aString("barrierL[time,"); str->cat(paramToReport+1); str->cat("]");
			 return str;}
		paramToReport -= traits();
		if(paramToReport<traits()){
			 aString *str = NEW aString("barrierR[time,"); str->cat(paramToReport+1); str->cat("]");
			 return str;}
		paramToReport -= traits();
		if(paramToReport<traits()){
			 aString *str = NEW aString("s/markerL["); str->cat(paramToReport+1); str->cat("]");
			 return str;}
		paramToReport -= traits();
		if(paramToReport<traits()){
			 aString *str = NEW aString("s/markerR["); str->cat(paramToReport+1); str->cat("]");
			 return str;}
		}//if traits()>1

	aString errstr("bug: can't return name of param #");
	errstr.cat(paramNum+1);
	errstr.cat(" in a ");errstr.cat(TotalParams());errstr.cat("-parameter cline model");
	aError(errstr);
	return NEW aString();//default shouldn't get here
	}//end ClineShapeCWTlTrTBQ2ParamRec::ParamName(int )



void ClineShapeCWTlTrTBQ2ParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::ParamString(dataline, sepchar);
	long double val=0.0;
//	for(int p=0;p<NumParams();++p){
	long np=TotalParams();
	for(int p=0;p<np;++p){
		val = ParamValue(p);
		dataline.cat(sepchar);
		dataline.cat(val,6,15);
		}
	//still need derived parameters
	}//end ClineShapeCWTlTrTBQ2ParamRec::ParamString(aString&, char)



long double ClineShapeCWTlTrTBQ2ParamRec::ParamValue(int paramNum){
	//returns value of parameter; first the varying params, then derived params
	int paramToReport = paramNum,tr1,tr2,reg,atParam;
	if(paramToReport<NumShapeParams()){
		aList<int> covaryingTraitParams;
		ShapeParamCoords(paramToReport,atParam,covaryingTraitParams);
		switch (atParam){
			case 0:
			 	 return center(covaryingTraitParams[0]);
			 	 break;
			 case 1:
			 	 return width(covaryingTraitParams[0]);
			 	 break;
			 case 2:
			 	 return thetaL(covaryingTraitParams[0]);
			 	 break;
			 case 3:
			 	 return thetaR(covaryingTraitParams[0]);
			 	 break;
			 case 4:
			 	 return zL(covaryingTraitParams[0]);
			 	 break;
			 case 5:
			 	 return zR(covaryingTraitParams[0]);
			 	 break;
			 case 6:
			 	 return leftVal(covaryingTraitParams[0]);
			 	 break;
			 case 7:
			 	 return rightVal(covaryingTraitParams[0]);
			 	 break;
			 default:
			 	 break;//do below
			 }//end switch
		}//if < NumShapeParams()

	paramToReport -= NumShapeParams();
	if(paramToReport<NumCovMatrixParams()){
		CovParamCoords(paramToReport, reg, tr1, tr2);
		return covMatrixList_[reg][tr1][tr2];
		}
	paramToReport -= NumCovMatrixParams();


	//derived parameters
	if(paramToReport<traits()){
		 return B_L(paramToReport);}
	paramToReport -= traits();
	if(paramToReport<traits()){
		 return B_R(paramToReport);}
	paramToReport -= traits();
	if(paramToReport<traits()){
		 return DeltaP(paramToReport);}
	paramToReport -= traits();
	if(paramToReport<traits()){
		 return xL(paramToReport);}
	paramToReport -= traits();
	if(paramToReport<traits()){
		 return xR(paramToReport);}
	if(traits()>1){
		paramToReport -= traits();
		switch (paramToReport){
		 	case 0:
		 	 return DispersalSigma();//NEW aString("sigma");
		 	 break;
		 	case 1:
		 	 return D();//NEW aString("D");
		 	 break;
		 	case 2:
		 	 return s_star_Ecotone();//NEW aString("s*[ecotone]");
		 	 break;
		 	case 3:
		 	 return s_star_vs_Hets();//NEW aString("s*[hets]");
		 	 break;
		 	case 4:
		 	 return s_perReproIsolationLocus();//NEW aString("s/[reproIsoLocus]");
		 	 break;
		 	case 5:
		 	 return WhybridOverWpure();//NEW aString("W[h]/W[p]");
		 	 break;
		 	case 6:
		 	 return S();//NEW aString("S");
		 	 break;
		 	case 7:
		 	 return NumSelectedLoci();//NEW aString("n[selLoci]");
		 	 break;
		    default:
		     break;
		 }//end switch
		paramToReport -= 8;
		if(paramToReport<traits()){
			 return distanceBarrierL(paramToReport);}
		paramToReport -= traits();
		if(paramToReport<traits()){
			 return distanceBarrierR(paramToReport);}
		paramToReport -= traits();
		if(paramToReport<traits()){
			 return timeBarrierL(paramToReport);}
		paramToReport -= traits();
		if(paramToReport<traits()){
			 return timeBarrierR(paramToReport);}
		paramToReport -= traits();
		if(paramToReport<traits()){
			 return s_perMarkerLocusL(paramToReport);}
		paramToReport -= traits();
		if(paramToReport<traits()){
			 return s_perMarkerLocusR(paramToReport);}
		}//if traits()>1
	
	aString errstr("bug: can't return value of param #");
	errstr.cat(paramNum);
	errstr.cat(" in a ");errstr.cat(TotalParams());errstr.cat("-parameter cline model");
	aError(errstr);
	return 0.0;//shouldn't get here
	}//end ClineShapeCWTlTrTBQ2ParamRec::ParamValue(int )




void ClineShapeCWTlTrTBQ2ParamRec::Print(ostream& outFile, long rep, aString& message){
	//usually to screen
	ParamRec::Print(outFile, rep, message);
	aString outStr;
	for(int tr=0;tr<traits();++tr){
		outStr="trait "; outStr.cat(tr+1);
		outStr.cat(":   c: ");outStr.cat(center(tr),6,15);
		outStr.separator(2);
		outStr.cat("w: ");outStr.cat(width(tr),6,15);
		outStr.separator(2);
		outFile<<outStr<<endl;
		outStr="thetaL: ";outStr.cat(thetaL(tr),2,15);
		outStr.separator(2);
		outStr.cat("zL: ");outStr.cat(zL(tr),6,15);
		outStr.separator(2);
		outStr.cat("xL: ");outStr.cat(xL(tr),6,15);
		outFile<<outStr<<endl;
		outStr="thetaR: ";outStr.cat(thetaR(tr),2,15);
		outStr.separator(2);
		outStr.cat("zR: ");outStr.cat(zR(tr),6,15);
		outStr.separator(2);
		outStr.cat("xR: ");outStr.cat(xR(tr),6,15);
		outFile<<outStr<<endl;
		outStr="P[L]: ";
		outStr.cat(leftVal(tr),2,15);
		outStr.separator(2);
		outStr.cat("P[xL]: ");outStr.cat(B_L(tr),2,15);
		outFile<<outStr<<endl;
		outStr="P[R]: ";outStr.cat(rightVal(tr),2,15);
		outStr.separator(2);
		outStr.cat("P[xR]: ");outStr.cat(B_R(tr),2,15);
		outStr.separator(2);
		outStr.cat("deltaP: ");outStr.cat(DeltaP(tr),2,15);
		outFile<<outStr<<endl;
		if(traits()>1){
			outStr="s[L]: ";outStr.cat(s_perMarkerLocusL(tr),6,15);
			outStr.separator(2);
			outStr.cat("s[R]: ");outStr.cat(s_perMarkerLocusR(tr),6,15);
			outFile<<outStr<<endl;
			}//traits>1
		}//tr
	if(traits()>1){
		outStr="sigma: ";outStr.cat(DispersalSigma(),6,15);outStr.separator(2);
		outStr.cat("D: ");outStr.cat(D(),2,15);
		outFile<<outStr<<endl;
		outStr="s*(ecotone): ";outStr.cat(s_star_Ecotone(),6,15);outStr.separator(2);
		outStr.cat("s*(hybrids): ");outStr.cat(s_star_vs_Hets(),6,15);
		outFile<<outStr<<endl;
		outStr="W(h)/W(p): ";outStr.cat(WhybridOverWpure(),6,15);outStr.separator(2);
		outStr.cat("S: ");outStr.cat(S(),6,15);
		outFile<<outStr<<endl;
		outStr="n[sel loci]: ";outStr.cat(NumSelectedLoci(),6,15);outStr.separator(2);
		outStr.cat("s/[repro isol locus]: ");outStr.cat(s_perReproIsolationLocus(),6,15);
		outFile<<outStr<<endl;
		}//traits>1
	for(int r=0;r<regions();++r){
		outStr="Var-Cov["; 
		if(regions()==3){
				if(r==0){outStr.cat("L]:");}
					else if(r==1){outStr.cat("C]:");}
					else{outStr.cat("R]:");}
				}
			else{
				outStr.cat("pop ");outStr.cat(r+1);outStr.cat("]:");}
		outFile<<outStr<<endl;outStr.clear();
		for(int t1=0;t1<traits();++t1){
			for(int t2=0;t2<=t1;++t2){
				outStr.cat(covMatrixList_[r][t2][t1],6,15);
				outStr.separator(2);}
			outFile<<outStr<<endl;
			outStr.clear();}
		}//r
	}//ClineShapeCWTlTrTBQ2ParamRec::Print(ostream&, long, aString&)


long double ClineShapeCWTlTrTBQ2ParamRec::r(void){//returns recombination rate
	return r_;
	}//end ClineShapeCWTlTrTBQ2ParamRec::r()


void ClineShapeCWTlTrTBQ2ParamRec::r(long double newVal){//returns recombination rate
	r_ = newVal;
	}//end ClineShapeCWTlTrTBQ2ParamRec::r(long double)




long double ClineShapeCWTlTrTBQ2ParamRec::rightVal(int trait){//returns value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
    return traitParams->rightVal(0);//assumes 1 trait/paramset in this configuration
	}//end ClineShapeCWTlTrTBQ2ParamRec::rightVal(int)


void ClineShapeCWTlTrTBQ2ParamRec::rightVal(long double value,int trait){//sets value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
//    if (!(traitParams->paramsOn_.Includes(7))){//assumes 1 trait/paramset in this configuration
 //   if (traitParams->paramsOn_.Includes(7)){//assumes 1 trait/paramset in this configuration
			traitParams->rightVal(value,0);//}//otherwise ignore
	}//end ClineShapeCWTlTrTBQ2ParamRec::rightVal(int, double)



int ClineShapeCWTlTrTBQ2ParamRec::RJunction(int trait, double rightEnd){//sets xL and return 0 of the cline parts don't touch
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
//    if (traitParams->paramsOn_.Includes(3)){//right side section fit separately
			return traitParams->RJunction(rightEnd);//}//otherwise ignore
//	return 1;//default
	}//end ClineShapeCWTlTrTBQ2ParamRec::RJunction(int, double)




long double ClineShapeCWTlTrTBQ2ParamRec::S(void){
	long double WhWp = WhybridOverWpure();
    if (WhWp == (long double)0.0){
		return (long double)200.0;}//an arbitrary high number
    long double bigS = (long double)-2.0 * ln(WhWp);
    return bigS;
    }//end ClineShapeCWTlTrTBQ2ParamRec::S()



void ClineShapeCWTlTrTBQ2ParamRec::SetParamValue(int paramNum, long double newVal){
	//set value of parameter; only for varying parameters
	int paramToAlter = paramNum,tr1,tr2,reg,atParam;
	if(paramToAlter<NumShapeParams()){
		aList<int> covaryingTraitParams;
		ShapeParamCoords(paramToAlter,atParam,covaryingTraitParams);
		switch (atParam){
			case 0:
			 	 for(int tr=0;tr<covaryingTraitParams.len();++tr){
			 	 	center(covaryingTraitParams[tr],newVal);}
				 return;  break;
			 case 1:
			 	 for(int tr=0;tr<covaryingTraitParams.len();++tr){
			 	 	width(covaryingTraitParams[tr],newVal);}
				 return;  break;
			 case 2:
			 	 for(int tr=0;tr<covaryingTraitParams.len();++tr){
			 	 	thetaL(covaryingTraitParams[tr],newVal);}
				 return;  break;
			 case 3:
			 	 for(int tr=0;tr<covaryingTraitParams.len();++tr){
			 	 	thetaR(covaryingTraitParams[tr],newVal);}
				 return;  break;
			 case 4:
			 	 for(int tr=0;tr<covaryingTraitParams.len();++tr){
			 	 	zL(covaryingTraitParams[tr],newVal);}
				 return;  break;
			 case 5:
			 	 for(int tr=0;tr<covaryingTraitParams.len();++tr){
			 	 	zR(covaryingTraitParams[tr],newVal);}
				 return;  break;
			 case 6:
			 	 for(int tr=0;tr<covaryingTraitParams.len();++tr){
			 	 	leftVal(covaryingTraitParams[tr],newVal);}
				 return;  break;
			 case 7:
			 	 for(int tr=0;tr<covaryingTraitParams.len();++tr){
			 	 	rightVal(covaryingTraitParams[tr],newVal);}
				 return;  break;
			 default:
			 	 break;//do below
			 }//end switch
		}//if < NumShapeParams()

	paramToAlter -= NumShapeParams();
	if(paramToAlter<NumCovMatrixParams()){
		CovParamCoords(paramToAlter, reg, tr1, tr2);
		covMatrixList_[reg][tr1][tr2] = (double) newVal;
		if(tr1!=tr2){covMatrixList_[reg][tr2][tr1] = (double) newVal;}//symmetry
		return;
		}


	aString errstr("bug: can't set value of param #");  errstr.cat(paramNum+1);
	errstr.cat(" in a ");  errstr.cat(NumParams());  errstr.cat("-parameter cline model");
	aError(errstr);
	}//end ClineShapeCWTlTrTBQ2ParamRec::SetParamValue(int,long double)




void ClineShapeCWTlTrTBQ2ParamRec::ShapeParamCoords(int paramNum, int& atParam, aList<int>& traitList){
	//replaces value of atParam and fills traitList with identities of co-estimated trait parameters
	traitList.Clear();
	int p = 0, origParamNum = paramNum;
	while(paramNum>=varyingParams_[p]){
		paramNum -= varyingParams_[p];
		++p;
		}//while
	atParam = p;
	for(int tr=0;tr<traits();++tr){
		if(paramIdentities_[p][tr]==paramNum+1){//identities aren't 0-indexed
			traitList.Append(tr);}}
	if(traitList.len()==0){
		aString e("Error in ClineShapeCWTlTrTBQ2ParamRec::ShapeParamCoords(); can't find any traits for param ");
		e.cat(origParamNum);
		aError(e);}
	}//end ClineShapeCWTlTrTBQ2ParamRec::NumShapeParams()



long double ClineShapeCWTlTrTBQ2ParamRec::s_perMarkerLocusL(int trait){
	//direct selection on marker loci in left tail of cline
	//assuming genetic background incompatibilities
	if(width()==(long double)0.0){return (long double)0.0;}//undefined
    long double s = thetaL(trait) * s_star_vs_Hets();
    return s;
    }//end ClineShapeCWTlTrTBQ2ParamRec::s_perMarkerLocusL(int)


long double ClineShapeCWTlTrTBQ2ParamRec::s_perMarkerLocusR(int trait){
	//direct selection on marker loci in right tail of cline
	//assuming genetic background incompatibilities
	if(width()==0.0){return (long double)0.0;}//undefined
    long double s = thetaR(trait) * s_star_vs_Hets();
    return s;
    }//end ClineShapeCWTlTrTBQ2ParamRec::s_perMarkerLocusR(int)


long double ClineShapeCWTlTrTBQ2ParamRec::s_perReproIsolationLocus(void){
	//average selection on a locus contributing to reproductive isolation
	long double n = NumSelectedLoci();
	 if (n == (long double)0.0){
			return (long double)0.0;}
    return S() / n;
    }//end ClineShapeCWTlTrTBQ2ParamRec::s_perReproIsolationLocus(void)



long double ClineShapeCWTlTrTBQ2ParamRec::s_star_vs_Hets(void){
	//indirect & direct selection on marker loci in middle of cline
	//assuming genetic background incompatibilities
	if(width()==0.0){return 0.0;}//undefined
    long double s = sqr((long double)2.0 * DispersalSigma() / width());//square not square root
    return s;
    }//end ClineShapeCWTlTrTBQ2ParamRec::s_star_vs_Hets()


long double ClineShapeCWTlTrTBQ2ParamRec::s_star_Ecotone(void){
	//indirect & direct selection on marker loci in middle of cline
	//assuming selection across an ecotone
	if(width()==(long double)0.0){return (long double)0.0;}//undefined
    long double s = sqr((long double)1.732 * DispersalSigma() / width());//square not square root
    return s;
    }//end ClineShapeCWTlTrTBQ2ParamRec::s_star_Ecotone()



long double ClineShapeCWTlTrTBQ2ParamRec::thetaL(int trait){//returns value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
    return traitParams->thetaL();//assumes 1 trait/paramset in this configuration
	}//end ClineShapeCWTlTrTBQ2ParamRec::thetaL(int)


void ClineShapeCWTlTrTBQ2ParamRec::thetaL(int trait, long double value){//sets value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
//    if (!(traitParams->paramsOn_.Includes(2))){//assumes 1 trait/paramset in this configuration
//    if (traitParams->paramsOn_.Includes(2)){//assumes 1 trait/paramset in this configuration
			traitParams->thetaL(value);//}//otherwise ignore
	}//end ClineShapeCWTlTrTBQ2ParamRec::thetaL(int, double)



long double ClineShapeCWTlTrTBQ2ParamRec::thetaR(int trait){//returns value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
    return traitParams->thetaR();//assumes 1 trait/paramset in this configuration
	}//end ClineShapeCWTlTrTBQ2ParamRec::thetaR(int)


void ClineShapeCWTlTrTBQ2ParamRec::thetaR(int trait, long double value){//sets value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
//    if (!(traitParams->paramsOn_.Includes(3))){//assumes 1 trait/paramset in this configuration
//    if (traitParams->paramsOn_.Includes(3)){//assumes 1 trait/paramset in this configuration
			traitParams->thetaR(value);//}//otherwise ignore
	}//end ClineShapeCWTlTrTBQ2ParamRec::thetaR(int, double)




long double ClineShapeCWTlTrTBQ2ParamRec::timeBarrierL(int trait){
	long double d = DispersalSigma();
    if (d == (long double)0.0){
		return (long double)1.0e20;}//basically forever if no dispersal
	long double b = distanceBarrierL(trait);
    if (b / d > (long double)1.0e40){
		return (long double)1.0e20;}//basically forever if beta/sigma too high
    long double timeL = sqr(b / d);//square not square root
    return timeL;
}//end ClineShapeCWTlTrTBQ2ParamRec::timeBarrierL(int)


long double ClineShapeCWTlTrTBQ2ParamRec::timeBarrierR(int trait){
	long double d = DispersalSigma();
    if (d == (long double)0.0){
		return (long double)1.0e20;}//basically forever if no dispersal
	long double b = distanceBarrierR(trait);
    if (b / d > (long double)1.0e40){
		return (long double)1.0e20;}//basically forever if beta/sigma too high
    long double timeR = sqr(b / d);//square not square root
    return timeR;
}//end ClineShapeCWTlTrTBQ2ParamRec::timeBarrierR(int)



long double ClineShapeCWTlTrTBQ2ParamRec::traitCov(int trait1, int trait2, int pop){
	//returns variance for location x
	return (long double) covMatrixList_[pop][trait1][trait2];
	}

	
int ClineShapeCWTlTrTBQ2ParamRec::traits(void){
	//returns number of traits
	return traitParamSet_.len();
	}

	
long double ClineShapeCWTlTrTBQ2ParamRec::traitVar(int trait, int pop){
	//returns variance for location x
	return (long double)covMatrixList_[pop][trait][trait];
	}



void ClineShapeCWTlTrTBQ2ParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::TitleString(dataline, sepchar);
	//one for each independent primary parameter
	//shape params first
	aList<int> covaryingTraitParams;
	int atParam=0, oldParam=0, paramClassCount=0;
	for(int p=0;p<NumShapeParams();++p){
		ShapeParamCoords(p,atParam,covaryingTraitParams);
		if(atParam==oldParam){//no change
				++paramClassCount;}
			else{
				oldParam=atParam;
				paramClassCount=0;}
		switch (atParam){
			case 0:
			 	 dataline.cat(sepchar);
			 	 dataline.cat("center[");dataline.cat(paramClassCount+1);dataline.cat(']');
			 	 break;
			 case 1:
			 	 dataline.cat(sepchar);
			 	 dataline.cat("width[");dataline.cat(paramClassCount+1);dataline.cat(']');
			 	 break;
			 case 2:
			 	 dataline.cat(sepchar);
			 	 dataline.cat("thetaL[");dataline.cat(paramClassCount+1);dataline.cat(']');
			 	 break;
			 case 3:
			 	 dataline.cat(sepchar);
			 	 dataline.cat("thetaR[");dataline.cat(paramClassCount+1);dataline.cat(']');
			 	 break;
			 case 4:
			 	 dataline.cat(sepchar);
			 	 dataline.cat("zL[");dataline.cat(paramClassCount+1);dataline.cat(']');
			 	 break;
			 case 5:
			 	 dataline.cat(sepchar);
			 	 dataline.cat("zR[");dataline.cat(paramClassCount+1);dataline.cat(']');
			 	 break;
			 case 6:
			 	 dataline.cat(sepchar);
			 	 dataline.cat("leftVal[");dataline.cat(paramClassCount+1);dataline.cat(']');
			 	 break;
			 case 7:
			 	 dataline.cat(sepchar);
			 	 dataline.cat("rightVal[");dataline.cat(paramClassCount+1);dataline.cat(']');
			 	 break;
			 default:
			 	 break;//do below
			 }//end switch
		}//p

	for(int r=0;r<regions();++r){
		for(int t1=0;t1<traits();++t1){
			for(int t2=0;t2<=t1;++t2){
				dataline.cat(sepchar);
				if(t1==t2){dataline.cat("var[");}
					else {dataline.cat("cov[");}
				dataline.cat(r+1);dataline.cat("][");dataline.cat(t1+1);
				if(t1!=t2){dataline.cat("][");dataline.cat(t2+1);}
				dataline.cat("]");
				}}}//t2,t1,r
	
	//derived parameters next
	
	int tr;
	for(tr=0;tr<traits();++tr){
		dataline.cat(sepchar);
		 dataline.cat("p[xL,"); dataline.cat(tr+1); dataline.cat("]");
		 }
	for(tr=0;tr<traits();++tr){
		dataline.cat(sepchar);
		 dataline.cat("p[xR,"); dataline.cat(tr+1); dataline.cat("]");
		 }
	for(tr=0;tr<traits();++tr){
		dataline.cat(sepchar);
		 dataline.cat("deltaP["); dataline.cat(tr+1); dataline.cat("]");
		 }
	for(tr=0;tr<traits();++tr){
		dataline.cat(sepchar);
		 dataline.cat("xL["); dataline.cat(tr+1); dataline.cat("]");
		 }
	for(tr=0;tr<traits();++tr){
		dataline.cat(sepchar);
		 dataline.cat("xR["); dataline.cat(tr+1); dataline.cat("]");
		 }
	if(traits()>1){
		dataline.cat(sepchar);
		dataline.cat("sigma");dataline.cat(sepchar);
		dataline.cat("D");dataline.cat(sepchar);
		dataline.cat("s*[ecotone]");dataline.cat(sepchar);
		dataline.cat("s*[hets]");dataline.cat(sepchar);
		dataline.cat("s/[reproIsoLocus]");dataline.cat(sepchar);
		dataline.cat("W[h]/W[p]");dataline.cat(sepchar);
		dataline.cat("S");dataline.cat(sepchar);
		dataline.cat("n[selLoci]");
		for(tr=0;tr<traits();++tr){
			dataline.cat(sepchar);
			 dataline.cat("barrierL[dist,"); dataline.cat(tr+1); dataline.cat("]");
			 }
		for(tr=0;tr<traits();++tr){
			dataline.cat(sepchar);
			 dataline.cat("barrierR[dist,"); dataline.cat(tr+1); dataline.cat("]");
			 }
		for(tr=0;tr<traits();++tr){
			dataline.cat(sepchar);
			 dataline.cat("barrierL[time,"); dataline.cat(tr+1); dataline.cat("]");
			 }
		for(tr=0;tr<traits();++tr){
			dataline.cat(sepchar);
			 dataline.cat("barrierR[time,"); dataline.cat(tr+1); dataline.cat("]");
			 }
		for(tr=0;tr<traits();++tr){
			dataline.cat(sepchar);
			 dataline.cat("s/markerL["); dataline.cat(tr+1); dataline.cat("]");
			 }
		for(tr=0;tr<traits();++tr){
			dataline.cat(sepchar);
			 dataline.cat("s/markerR["); dataline.cat(tr+1); dataline.cat("]");
			 }
		}//if traits()>1
		
	}//end ClineShapeCWTlTrTBQ2ParamRec::TitleString(aString&, char)



long double ClineShapeCWTlTrTBQ2ParamRec::WhybridOverWpure(void){
	long double w = width(), beta = 0.0;
	if(w==(long double)0.0){return (long double)0.0;}//undefined
	for(int t=0;t<traits();++t){
		beta += distanceBarrierL(t) + distanceBarrierR(t);
		}
	beta /= ((long double)2.0 * traits());//average over left & right sides
//    if ((beta / w > 3000.0) || (beta == 0.0)){
    if ((w / beta > (long double)3000.0) || (beta == (long double)0.0)){
		return (long double)0.0;}
//    double wratio = exp((-1.0/r()) * (ln(beta / w)));//weighted average to -1/r power
    double wratio = exp((long double)r() * ln(w / beta));//weighted average to r power
    return wratio;
	}//end ClineShapeCWTlTrTBQ2ParamRec::WhybridOverWpure()



long double ClineShapeCWTlTrTBQ2ParamRec::width(void){//returns mean value
	long double w=0.0;
	for(int t=0;t<traits();++t){
		w += width(t);}
	return w/(long double)traits();
	}//end ClineShapeCWTlTrTBQ2ParamRec::width()


long double ClineShapeCWTlTrTBQ2ParamRec::width(int trait){//returns value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
    return traitParams->width();//assumes 1 trait/paramset in this configuration
	}//end ClineShapeCWTlTrTBQ2ParamRec::width(int)


void ClineShapeCWTlTrTBQ2ParamRec::width(int trait, long double value){//sets value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
//    if (!(traitParams->paramsOn_.Includes(1))){//assumes 1 trait/paramset in this configuration
//    if (traitParams->paramsOn_.Includes(1)){//assumes 1 trait/paramset in this configuration
			traitParams->width(value);//}//otherwise ignore
	}//end ClineShapeCWTlTrTBQ2ParamRec::width(int, double)


long double ClineShapeCWTlTrTBQ2ParamRec::xL(int trait){//returns value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
    return traitParams->xL();//assumes 1 trait/paramset in this configuration
	}//end ClineShapeCWTlTrTBQ2ParamRec::xL(int)


void ClineShapeCWTlTrTBQ2ParamRec::xL(int trait, long double value){//sets value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	traitParams->xL_=value;//only set 
	}//end ClineShapeCWTlTrTBQ2ParamRec::xL(int, double)


long double ClineShapeCWTlTrTBQ2ParamRec::xR(int trait){//returns value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
    return traitParams->xR();//assumes 1 trait/paramset in this configuration
	}//end ClineShapeCWTlTrTBQ2ParamRec::xR(int)


void ClineShapeCWTlTrTBQ2ParamRec::xR(int trait, long double value){//sets value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
	traitParams->xR_=value;//only set 
	}//end ClineShapeCWTlTrTBQ2ParamRec::xR(int, double)



long double ClineShapeCWTlTrTBQ2ParamRec::zL(int trait){//returns value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
    return traitParams->zL();//assumes 1 trait/paramset in this configuration
	}//end ClineShapeCWTlTrTBQ2ParamRec::zL(int)


void ClineShapeCWTlTrTBQ2ParamRec::zL(int trait, long double value){//sets value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
//    if (!(traitParams->paramsOn_.Includes(4))){//assumes 1 trait/paramset in this configuration
//    if (traitParams->paramsOn_.Includes(4)){//assumes 1 trait/paramset in this configuration
			traitParams->zL(value);//}//otherwise ignore
	}//end ClineShapeCWTlTrTBQ2ParamRec::zL(int, double)



long double ClineShapeCWTlTrTBQ2ParamRec::zR(int trait){//returns value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
    return traitParams->zR();//assumes 1 trait/paramset in this configuration
	}//end ClineShapeCWTlTrTBQ2ParamRec::zR(int)


void ClineShapeCWTlTrTBQ2ParamRec::zR(int trait, long double value){//sets value
	ClineShapeCWTlTrTBParamRec* traitParams = (ClineShapeCWTlTrTBParamRec*) traitParamSet_[trait].GetPointerToDerivedSelf();
//    if (!(traitParams->paramsOn_.Includes(5))){//assumes 1 trait/paramset in this configuration
//    if (traitParams->paramsOn_.Includes(5)){//assumes 1 trait/paramset in this configuration
			traitParams->zR(value);//}//otherwise ignore
	}//end ClineShapeCWTlTrTBQ2ParamRec::zR(int, double)



aVector<long double>* ClineShapeCWTlTrTBQ2ParamRec::ZonCline(long double x){
	//x=location; returns new vector
	aVector<long double> *zz = NEW aVector<long double>((long double)0.0,(long)traits());
	aVector<long double>& z = *zz;
	aList<long double> coords; coords.Append(x);
	for(int tr=0;tr<traits();++tr){
		coords.Append(tr);
		z[tr] = ExpectedValue(coords);
		coords.DeleteLast();
		}//tr
	return zz;
	}//ClineShapeCWTlTrTBQ2ParamRec::ZonCline(long double)




void ClineShapeCWTlTrTBQ2ParamRec::operator=(ParamRec& p){
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means p must actually point to an object of class ClineShapeCWTlTrTBQ2ParamRec
	ClineShapeCWTlTrTBQ2ParamRec* pp = (ClineShapeCWTlTrTBQ2ParamRec*) p.GetPointerToDerivedSelf();
	lnL_= pp->lnL_;
	lnLmaxMinusLnL_ = pp->lnLmaxMinusLnL_;	
  	traitParamSet_=pp->traitParamSet_;
	r_=pp->r_;
	covMatrixList_ = pp->covMatrixList_;
	covMatrixListType_ = pp->covMatrixListType_;
	invCovMatrixList_ = pp->invCovMatrixList_;
	detCovMatrixList_ = pp->detCovMatrixList_;
	invCovMatrixFlags_ = pp->invCovMatrixFlags_;
	detCovMatrixFlags_ = pp->detCovMatrixFlags_;
	nparams_=pp->nparams_;
	nshparams_=pp->nshparams_;
	ndparams_=pp->ndparams_;
	paramIdentities_ = pp->paramIdentities_;
	varyingParams_ = pp->varyingParams_;
	D_=pp->D_;
	Dcalculated_=pp->Dcalculated_;
	}//end  ClineShapeCWTlTrTBQ2ParamRec::operator=(ParamRec&)



int	operator==(ClineShapeCWTlTrTBQ2ParamRec& p, ClineShapeCWTlTrTBQ2ParamRec& q){
	return ((p.lnL_ == q.lnL_) 
			&&(p.r_ == q.r_)
			&&(p.varyingParams_ == q.varyingParams_) 
			&&(p.paramIdentities_ == q.paramIdentities_) 
			&&(p.traitParamSet_ == q.traitParamSet_) 
			&&(p.covMatrixList_ == q.covMatrixList_)
			 );
	}//end operator==(ClineShapeCWTlTrTBQ2ParamRec&, ClineShapeCWTlTrTBQ2ParamRec&)


int	operator!=(ClineShapeCWTlTrTBQ2ParamRec& p, ClineShapeCWTlTrTBQ2ParamRec& q){
	return !(p==q);
	}//end operator!=(ClineShapeCWTlTrTBQ2ParamRec&, ClineShapeCWTlTrTBQ2ParamRec&)



#endif


