/****************************************************
*
*	DiseqLikelihood.cpp
*
*	This calculates the likelihood of a particular value
*	of D, the disequilibrium coefficient, from population genetic data,
*	using a multinomial probability function, for pairs of
*	loci.
*
*****************************************************/

#ifndef DiseqLikelihoodcpp
#define DiseqLikelihoodcpp = 1

#include "DiseqLikelihood.h"


double ClineDiseqLikelihood (ClineDiseqParamRec& diseqParam, MetropolisHastingsData* data){
	//sample-size based weighting removed
	ClineGeneticData* alloData = (ClineGeneticData*) data;
	long nSample;
	double lnL=0.0;
	double adjustedD = 0.0;
	aMatrix<double> f = ZeroMatrix((double)0,2,2);
	aMatrix<double> bigX = ZeroMatrix((double)0,2,2);
	int adjByFis = true;//set to false here if pops are in HW proportions
    double FisCenter = 0.0;
    if (adjByFis){//calculate Fis for the center pops only
		PopulationGeneticData centerPopData(*alloData,alloData->centerPopList_);
		centerPopData.WeirTheta();
		FisCenter = centerPopData.Fis(WandC);
		}
	for(int popNum=0;popNum<alloData->centerPopList_.len();++popNum){
		int pop = alloData->centerPopList_[popNum];
		double x = alloData->GetLocality(pop);//this location
		aList<double> coords; coords.Append(x);
		double pAtCline = diseqParam.ClineDiseqParamRec::ExpectedValue(coords);
		if ((pAtCline == 0.0) || (pAtCline == 1.0) || (FisCenter >= 1.0)) {
				adjustedD = 1.0;}
      		else{
       			adjustedD = (diseqParam.D_ / (pAtCline * (1.0 - pAtCline))) * (1.0 + FisCenter);
				}
		adjustedD = MAX(adjustedD,1.0);
		for(int loc1=0;loc1<alloData->loci()-1;++loc1){
			StringList& alleleNamesList1 = alloData->allelesPresent[loc1];
			for(int loc2 = (loc1 + 1); loc2<alloData->loci();++loc2){
				StringList& alleleNamesList2 = alloData->allelesPresent[loc2];
				for(int allele1=0;allele1<alleleNamesList1.len();++allele1){
					//get freq of allele1 (at loc1) in pop
					double freq1 = alloData->GetFreq(pop, loc1, allele1);
					if ((freq1 > 0.0) && (freq1 < 1.0)){//otherwise D is undefined
						aString& alleleName1 = alleleNamesList1[allele1];
						for(int allele2=0;allele2<alleleNamesList2.len();++allele2){
							nSample = 0;//set sample size to zero, to count for blanks
							//get freq of allele2 (at loc2) in pop
							double freq2 = alloData->GetFreq(pop, loc2, allele2);
							if ((freq2 > 0.0) && (freq2 < 1.0)){//otherwise D is undefined
								f[0][0] = freq1 * freq2 + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
       							f[0][1] = freq1 * (1.0 - freq2) + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
       							f[1][0] = (1.0 - freq1) * freq2 + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
       							f[1][1] = (1.0 - freq1) * (1.0 - freq2) + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
								//sometimes these values can be less than zero due to sampling error
								//implying that it is infinitely improbable that an observed value
								//could occur under the model.
								//therefore, values <=0 are set to a very small number, and will
								//give a very low finite likelihood
								for(int index1=0;index1<2;++index1){
									for(int index2=0;index2<2;++index2){
										if(f[index1][index2] <= 0.0){
											f[index1, index2] = 1.0e-10;}
										}}

								//get genotype strings for each individual and  read joint frequencies
								aString& alleleName2 = alleleNamesList2[allele2];
								aMatrix<double> bigN = ZeroMatrix((double)0,3,3);
 								int sizePop = alloData->individuals(pop);
								for(int i=0;i<sizePop;++i){
									aString* spot1 = alloData->GetGenotypeString(pop,i,loc1);
									//make sure it doesn't have any blanks
									int blankAt1 = false;
       								if(spot1->len() < 2){
       										blankAt1 = true;//happens sometimes at end of data line
											}//with missing loci
										else{
											if((spot1[0] == ' ') || (spot1[1] == ' ')){
												blankAt1 = true;}
											}
									if(!blankAt1){
										aString* spot2 = alloData->GetGenotypeString(pop,i,loc2);
										//make sure it doesn' t have any blanks
										int blankAt2 = false;
       									if(spot2->len() < 2){
       											blankAt2 = true;//happens sometimes at end of data line
												}//with missing loci
											else{
												if((spot2[0] == ' ') || (spot2[1] == ' ')){
													blankAt2 = true;}
												}
										if(!blankAt2){
											++nSample;
											//use this indiv in calculation
											//count instances of target alleles at each locus in this indiv
											int i1 = 2;
											for(int chrom=0;chrom<2;++chrom){
												if(spot1[chrom] == alleleName1){
													--i1;}}
											int i2 = 2;
											for(int chrom=0;chrom<2;++chrom){
												if(spot2[chrom] == alleleName2){
													--i2;}}
											bigN[i1][i2]++;
											}//end if not blankAt2
										}//end if not blankAt1
									}//end for i
								//calculate Hill's (1974) intermediate variables & iterative method
								bigX[0][0] = 2.0 * bigN[0][0] + bigN[0][1] + bigN[1][0];
								bigX[1][0] = 2.0 * bigN[2][0] + bigN[1][0] + bigN[2][1];
								bigX[0][1] = 2.0 * bigN[0][2] + bigN[0][1] + bigN[1][2];
								bigX[1][1] = 2.0 * bigN[2][2] + bigN[1][2] + bigN[2][1];
								for(int index1=0;index1<2;++index1){
									for(int index2=0;index2<2;++index2){
										if (f[index1][index2] > 0.0){
											lnL += bigX[index1][index2] * ln(f[index1][index2]);
											lnL += bigN[1][1] * ln(f[0][0] * f[1][1] + f[0][1] * f[1][0]);
										}}}//end for both indexes
								}//end if 0<freq2<1
							}//end for allele2
						}//end if 0<freq1<1
					}//end for allele1
				}//end for loc2
			}//end for loc1
		}//end for popNum
    diseqParam.lnLDiseq(lnL);
    return lnL;
	}//end function ClineDiseqLikelihood

#endif
