/***************************
*
*
*	FitClineShapes.cpp
*
*	These functions fit genetic marker data to specific
*	Barton cline shape models using likelihood functions maximized
*	via a Metropolis-Hastings algorithm.
*
**************************************************/

#ifndef FitClineShapescpp
#define FitClineShapescpp = 1

#include "FitClineShapes.h"

void FitClineCW(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				){//2-parameter fit
	double temperature = 0.009;
	double lowTemp = 0.0000001;	//best annealing temp 

	//***these parameter records always vary among shapes fitted
	ClineShapeCWParamRec initialParams(startingParamInitializer,0);
	ClineShapeCWParamRec paramDeviations(devParamInitializer,0);
	ClineShapeCWParamRec lowerParamLimits(minParamInitializer,0);
	ClineShapeCWParamRec upperParamLimits(maxParamInitializer,0);
	//***
	
	if(testingFitToGivenShape){
		initialParams.Initialize(testParamInitializer,0);}
		
	MetropolisHastings likelihoodAssessor(initialParams, paramDeviations, 
											lowerParamLimits, upperParamLimits, 
											alloData);
	//***this function may vary among shapes fitted
	likelihoodAssessor.SetLikelihoodFunction(MaxLikeDevFromCline);
	//***

	if(testingFitToGivenShape){
		likelihoodAssessor.FitOfTestParams(outputFileName);
		return;}
	likelihoodAssessor.Anneal(ran, 100.0, 10.0, true, 0.9, repsPerAnnealStep);
	likelihoodAssessor.SampleForSupportLimits(ran, 1.0, true, savedReps, repsBetweenSaves);
	likelihoodAssessor.PrintSavedParams(outputFileName);
	aString outFileSummmaryName(outputFileName);
	outFileSummmaryName.trunc(28);
	outFileSummmaryName.cat(".sum");
	aString emptyString;
	likelihoodAssessor.PrintParamEstimates(outFileSummmaryName, seed, 1, 1, emptyString);//1 for tab separators
	likelihoodAssessor.PrintParamEstimates(cout, seed, 0, 1, emptyString);//0 for no tab separators

	return;
	}//end FitClineCW(...)


void FitClineCWMulti(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				){//2-parameter fit
	double temperature = 0.009;
	double lowTemp = 0.0000001;	//best annealing temp 

	//***these parameter records always vary among shapes fitted
	ClineMultiCWParamRec initialParams(startingParamInitializer);
	ClineMultiCWParamRec paramDeviations(devParamInitializer);
	ClineMultiCWParamRec lowerParamLimits(minParamInitializer);
	ClineMultiCWParamRec upperParamLimits(maxParamInitializer);
	//***
	
	if(testingFitToGivenShape){
		initialParams.Initialize(testParamInitializer);}
		
	MetropolisHastings likelihoodAssessor(initialParams, paramDeviations, 
											lowerParamLimits, upperParamLimits, 
											alloData);
	//***this function may vary among shapes fitted
	likelihoodAssessor.SetLikelihoodFunction(MaxLikeDevFromCline);
	//***

	if(testingFitToGivenShape){
		likelihoodAssessor.FitOfTestParams(outputFileName);
		return;}
	likelihoodAssessor.Anneal(ran, 100.0, 10.0, true, 0.9, repsPerAnnealStep);
	likelihoodAssessor.SampleForSupportLimits(ran, 1.0, true, savedReps, repsBetweenSaves);
	likelihoodAssessor.PrintSavedParams(outputFileName);
	aString outFileSummmaryName(outputFileName);
	outFileSummmaryName.trunc(28);
	outFileSummmaryName.cat(".sum");
	aString emptyString;
	likelihoodAssessor.PrintParamEstimates(outFileSummmaryName, seed, 1, 1, emptyString);//1 for tab separators
	likelihoodAssessor.PrintParamEstimates(cout, seed, 0, 1, emptyString);//0 for no tab separators

	return;
	}//end ClineMultiCWParamRec(...)



void FitClineCWTB(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				){//2-parameter fit
	double temperature = 0.009;
	double lowTemp = 0.0000001;	//best annealing temp 

	//***these parameter records always vary among shapes fitted
	ClineShapeCWTBParamRec initialParams(startingParamInitializer,0);
	ClineShapeCWTBParamRec paramDeviations(devParamInitializer,0);
	ClineShapeCWTBParamRec lowerParamLimits(minParamInitializer,0);
	ClineShapeCWTBParamRec upperParamLimits(maxParamInitializer,0);
	//***
	
	if(testingFitToGivenShape){
		initialParams.Initialize(testParamInitializer,0);}
		
	MetropolisHastings likelihoodAssessor(initialParams, paramDeviations, 
											lowerParamLimits, upperParamLimits, 
											alloData);
	//***this function may vary among shapes fitted
	likelihoodAssessor.SetLikelihoodFunction(MaxLikeDevFromCline);
	//***

	if(testingFitToGivenShape){
		likelihoodAssessor.FitOfTestParams(outputFileName);
		return;}
	likelihoodAssessor.Anneal(ran, 100.0, 10.0, true, 0.9, repsPerAnnealStep);
	likelihoodAssessor.SampleForSupportLimits(ran, 1.0, true, savedReps, repsBetweenSaves);
	likelihoodAssessor.PrintSavedParams(outputFileName);
	aString outFileSummmaryName(outputFileName);
	outFileSummmaryName.trunc(28);
	outFileSummmaryName.cat(".sum");
	aString emptyString;
	likelihoodAssessor.PrintParamEstimates(outFileSummmaryName, seed, 1, 1, emptyString);//1 for tab separators
	likelihoodAssessor.PrintParamEstimates(cout, seed, 0, 1, emptyString);//0 for no tab separators

	return;
	}//end FitClineCWTB(...)



void FitClineCWTBMulti(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				){//2-parameter fit
	double temperature = 0.009;
	double lowTemp = 0.0000001;	//best annealing temp 

	//***these parameter records always vary among shapes fitted
	ClineMultiCWTBParamRec initialParams(startingParamInitializer);
	ClineMultiCWTBParamRec paramDeviations(devParamInitializer);
	ClineMultiCWTBParamRec lowerParamLimits(minParamInitializer);
	ClineMultiCWTBParamRec upperParamLimits(maxParamInitializer);
	//***
	
	if(testingFitToGivenShape){
		initialParams.Initialize(testParamInitializer);}
		
	MetropolisHastings likelihoodAssessor(initialParams, paramDeviations, 
											lowerParamLimits, upperParamLimits, 
											alloData);
	//***this function may vary among shapes fitted
	likelihoodAssessor.SetLikelihoodFunction(MaxLikeDevFromCline);
	//***

	if(testingFitToGivenShape){
		likelihoodAssessor.FitOfTestParams(outputFileName);
		return;}
	likelihoodAssessor.Anneal(ran, 100.0, 10.0, true, 0.9, repsPerAnnealStep);
	likelihoodAssessor.SampleForSupportLimits(ran, 1.0, true, savedReps, repsBetweenSaves);
	likelihoodAssessor.PrintSavedParams(outputFileName);
	aString outFileSummmaryName(outputFileName);
	outFileSummmaryName.trunc(28);
	outFileSummmaryName.cat(".sum");
	aString emptyString;
	likelihoodAssessor.PrintParamEstimates(outFileSummmaryName, seed, 1, 1, emptyString);//1 for tab separators
	likelihoodAssessor.PrintParamEstimates(cout, seed, 0, 1, emptyString);//0 for no tab separators

	return;
	}//end FitClineCWTBMulti(...)



//6-PARAMETER CLINE FIT FUNCTIONS
void FitClineCWTlTrZlZr(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				){//6-parameter fit
	double temperature = 0.009;
	double lowTemp = 0.0000001;	//best annealing temp 

	//***these parameter records always vary among shapes fitted
	ClineShapeCWTlTrParamRec initialParams(startingParamInitializer,0);
	ClineShapeCWTlTrParamRec paramDeviations(devParamInitializer,0);
	ClineShapeCWTlTrParamRec lowerParamLimits(minParamInitializer,0);
	ClineShapeCWTlTrParamRec upperParamLimits(maxParamInitializer,0);
	upperParamLimits.xL_ = upperParamLimits.xR_ = maxParamInitializer.rightEnd_;
	//***


	if(testingFitToGivenShape){
		initialParams.Initialize(testParamInitializer,0);}
		
	MetropolisHastings likelihoodAssessor(initialParams, paramDeviations, 
											lowerParamLimits, upperParamLimits, 
											alloData);
	//***this function may vary among shapes fitted
	likelihoodAssessor.SetLikelihoodFunction(MaxLikeDevFromCline);


	if(testingFitToGivenShape){
		likelihoodAssessor.FitOfTestParams(outputFileName);
		return;}
	likelihoodAssessor.Anneal(ran, 100.0, 10.0, true, 0.9, repsPerAnnealStep);
	likelihoodAssessor.SampleForSupportLimits(ran, 1.0, true, savedReps, repsBetweenSaves);
	likelihoodAssessor.PrintSavedParams(outputFileName);
	aString outFileSummmaryName(outputFileName);
	outFileSummmaryName.trunc(28);
	outFileSummmaryName.cat(".sum");
	aString emptyString;
	likelihoodAssessor.PrintParamEstimates(outFileSummmaryName, seed, 1, 1, emptyString);//1 for tab separators
	likelihoodAssessor.PrintParamEstimates(cout, seed, 0, 1, emptyString);//0 for no tab separators

	return;
	}//end FitClineCWTlTrZlZr(...)




void FitClineCWTlTrZlZrMulti(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				){//6-parameter fit
	double temperature = 0.009;
	double lowTemp = 0.0000001;	//best annealing temp 

	//***these parameter records always vary among shapes fitted
	ClineMultiCWTlTrParamRec initialParams(startingParamInitializer);
	ClineMultiCWTlTrParamRec paramDeviations(devParamInitializer);
	ClineMultiCWTlTrParamRec lowerParamLimits(minParamInitializer);
	ClineMultiCWTlTrParamRec upperParamLimits(maxParamInitializer);
//	lowerParamLimits.SetXL(maxParamInitializer.leftEnd_);
//	lowerParamLimits.SetXR(maxParamInitializer.leftEnd_);
	upperParamLimits.SetXL(maxParamInitializer.rightEnd_);
	upperParamLimits.SetXR(maxParamInitializer.rightEnd_);
	//***


	if(testingFitToGivenShape){
		initialParams.Initialize(testParamInitializer);}
		
	MetropolisHastings likelihoodAssessor(initialParams, paramDeviations, 
											lowerParamLimits, upperParamLimits, 
											alloData);
	//***this function may vary among shapes fitted
	likelihoodAssessor.SetLikelihoodFunction(MaxLikeDevFromCline);


	if(testingFitToGivenShape){
		likelihoodAssessor.FitOfTestParams(outputFileName);
		return;}
	likelihoodAssessor.Anneal(ran, 100.0, 10.0, true, 0.9, repsPerAnnealStep);
	likelihoodAssessor.SampleForSupportLimits(ran, 1.0, true, savedReps, repsBetweenSaves);
	likelihoodAssessor.PrintSavedParams(outputFileName);
	aString outFileSummmaryName(outputFileName);
	outFileSummmaryName.trunc(28);
	outFileSummmaryName.cat(".sum");
	aString emptyString;
	likelihoodAssessor.PrintParamEstimates(outFileSummmaryName, seed, 1, 1, emptyString);//1 for tab separators
	likelihoodAssessor.PrintParamEstimates(cout, seed, 0, 1, emptyString);//0 for no tab separators

	return;
	}//end FitClineCWTlTrZlZrMulti(...)





//8-PARAMETER FIT:  can use for fitting multiple traits
void FitClineCWTlTrZlZrTB(Ran3& ran, ClineData* theData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				){//8-parameter fit
	double temperature = 0.009;
	double lowTemp = 0.0000001;	//best annealing temp 

	//***these parameter records always vary among shapes fitted
	ParamRec *initialParams;
	ParamRec *paramDeviations;
	ParamRec *lowerParamLimits;
	ParamRec *upperParamLimits;
	//***
	//***these parameter records always vary among shapes fitted; assumes all data of common type
	if((theData->dataType[0] == "codominant")||(theData->dataType[0] == "haplotype")){
			ClineShapeCWTlTrTBParamRec* ip = NEW ClineShapeCWTlTrTBParamRec(startingParamInitializer,0);
			if(testingFitToGivenShape){
				ip->Initialize(testParamInitializer,0);}
			paramDeviations = NEW ClineShapeCWTlTrTBParamRec(devParamInitializer,0);
			lowerParamLimits = NEW ClineShapeCWTlTrTBParamRec(minParamInitializer,0);
			ClineShapeCWTlTrTBParamRec* ul = NEW ClineShapeCWTlTrTBParamRec(maxParamInitializer,0);
			ul->xL_=maxParamInitializer.rightEnd_;
			ul->xR_=maxParamInitializer.rightEnd_;
			initialParams = ip;
			upperParamLimits = ul;
			}
		else if(theData->dataType[0] == "morph"){
			ClineShapeCWTlTrTBQ2ParamRec* ip = NEW ClineShapeCWTlTrTBQ2ParamRec(startingParamInitializer);
			if(testingFitToGivenShape){
				ip->Initialize(testParamInitializer);}
			paramDeviations = NEW ClineShapeCWTlTrTBQ2ParamRec(devParamInitializer);
			lowerParamLimits = NEW ClineShapeCWTlTrTBQ2ParamRec(minParamInitializer);
			ClineShapeCWTlTrTBQ2ParamRec* ul = NEW ClineShapeCWTlTrTBQ2ParamRec(maxParamInitializer);
			for(int t=0;t<maxParamInitializer.traits();++t){
				ul->xL(t,maxParamInitializer.rightEnd_);
				ul->xR(t,maxParamInitializer.rightEnd_);}
			initialParams = ip;
			upperParamLimits = ul;
			}
	//***

		
	MetropolisHastings likelihoodAssessor(*initialParams, *paramDeviations, 
											*lowerParamLimits, *upperParamLimits, 
											theData);
	//***this function may vary among shapes fitted
	likelihoodAssessor.SetLikelihoodFunction(MaxLikeDevFromCline);


	if(testingFitToGivenShape){
		likelihoodAssessor.FitOfTestParams(outputFileName);
		return;}
	likelihoodAssessor.Anneal(ran, 100.0, 10.0, true, 0.9, repsPerAnnealStep);
	likelihoodAssessor.SampleForSupportLimits(ran, 1.0, true, savedReps, repsBetweenSaves);
	likelihoodAssessor.PrintSavedParams(outputFileName);
	aString outFileSummmaryName(outputFileName);
	outFileSummmaryName.trunc(28);
	outFileSummmaryName.cat(".sum");
	aString emptyString;
	likelihoodAssessor.PrintParamEstimates(outFileSummmaryName, seed, 1, 1, emptyString);//1 for tab separators
	likelihoodAssessor.PrintParamEstimates(cout, seed, 0, 1, emptyString);//0 for no tab separators

	//clean up
	delete initialParams;		initialParams=0;
	delete paramDeviations;		paramDeviations=0;
	delete lowerParamLimits;	lowerParamLimits=0;
	delete upperParamLimits;	upperParamLimits=0;
	}//end FitClineCWTlTrZlZrTB(...)




void FitClineCWTlTrZlZrTBMulti(Ran3& ran, ClineData* theData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				){//8-parameter fit
	double temperature = 0.009;
	double lowTemp = 0.0000001;	//best annealing temp 

	//***these parameter records always vary among shapes fitted
	ParamRec *initialParams;
	ParamRec *paramDeviations;
	ParamRec *lowerParamLimits;
	ParamRec *upperParamLimits;
	//***
	//***these parameter records always vary among shapes fitted; assumes all data of common type
//	if((theData->dataType[0] == "codominant")||(theData->dataType[0] == "haplotype")){
			ClineMultiCWTlTrTBParamRec* ip = NEW ClineMultiCWTlTrTBParamRec(startingParamInitializer);
			if(testingFitToGivenShape){
				ip->Initialize(testParamInitializer);}
			paramDeviations = NEW ClineMultiCWTlTrTBParamRec(devParamInitializer);
			lowerParamLimits = NEW ClineMultiCWTlTrTBParamRec(minParamInitializer);
			ClineMultiCWTlTrTBParamRec* ul = NEW ClineMultiCWTlTrTBParamRec(maxParamInitializer);
			ul->SetXL(maxParamInitializer.rightEnd_);
			ul->SetXR(maxParamInitializer.rightEnd_);
			initialParams = ip;
			upperParamLimits = ul;
//			}
/*		else if(theData->dataType[0] == "morph"){
			ClineShapeCWTlTrTBQ2ParamRec* ip = NEW ClineShapeCWTlTrTBQ2ParamRec(startingParamInitializer);
			if(testingFitToGivenShape){
				ip->Initialize(testParamInitializer);}
			paramDeviations = NEW ClineShapeCWTlTrTBQ2ParamRec(devParamInitializer);
			lowerParamLimits = NEW ClineShapeCWTlTrTBQ2ParamRec(minParamInitializer);
			ClineShapeCWTlTrTBQ2ParamRec* ul = NEW ClineShapeCWTlTrTBQ2ParamRec(maxParamInitializer);
			for(int t=0;t<maxParamInitializer.traits();++t){
				ul->xL(t,maxParamInitializer.rightEnd_);
				ul->xR(t,maxParamInitializer.rightEnd_);}
			initialParams = ip;
			upperParamLimits = ul;
			}
	//***
*/
		
	MetropolisHastings likelihoodAssessor(*initialParams, *paramDeviations, 
											*lowerParamLimits, *upperParamLimits, 
											theData);
	//***this function may vary among shapes fitted
	likelihoodAssessor.SetLikelihoodFunction(MaxLikeDevFromCline);


	if(testingFitToGivenShape){
		likelihoodAssessor.FitOfTestParams(outputFileName);
		return;}
	likelihoodAssessor.Anneal(ran, 100.0, 10.0, true, 0.9, repsPerAnnealStep);
	likelihoodAssessor.SampleForSupportLimits(ran, 1.0, true, savedReps, repsBetweenSaves);
	likelihoodAssessor.PrintSavedParams(outputFileName);
	aString outFileSummmaryName(outputFileName);
	outFileSummmaryName.trunc(28);
	outFileSummmaryName.cat(".sum");
	aString emptyString;
	likelihoodAssessor.PrintParamEstimates(outFileSummmaryName, seed, 1, 1, emptyString);//1 for tab separators
	likelihoodAssessor.PrintParamEstimates(cout, seed, 0, 1, emptyString);//0 for no tab separators

	//clean up
	delete initialParams;		initialParams=0;
	delete paramDeviations;		paramDeviations=0;
	delete lowerParamLimits;	lowerParamLimits=0;
	delete upperParamLimits;	upperParamLimits=0;
	}//end FitClineCWTlTrZlZrTBMulti(...)



/*  THIS IS DEFUNCT
void FitClineCWTlTrZlZrD(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed
				){//6-parameter fit
	double temperature = 0.009;
	double lowTemp = 0.0000001;	//best annealing temp 

	//***these parameter records always vary among shapes fitted
	ClineShapeCWTlTrDParamRec initialParams(startingParamInitializer);
	ClineShapeCWTlTrDParamRec paramDeviations(devParamInitializer);
	ClineShapeCWTlTrDParamRec lowerParamLimits(minParamInitializer);
	ClineShapeCWTlTrDParamRec upperParamLimits(maxParamInitializer);
	upperParamLimits.xL_ = upperParamLimits.xR_ = maxParamInitializer.rightEnd_;
	//***


	if(testingFitToGivenShape){
		initialParams.Initialize(testParamInitializer);}
		
	MetropolisHastings likelihoodAssessor(initialParams, paramDeviations, 
											lowerParamLimits, upperParamLimits, 
											alloData);
	//***this function may vary among shapes fitted
	likelihoodAssessor.SetLikelihoodFunction(MaxLikeDevFromClineAndDiseq6);


	if(testingFitToGivenShape){
		likelihoodAssessor.FitOfTestParams(outputFileName);
		return;}
	likelihoodAssessor.Anneal(ran, 100.0, 10.0, true, 0.9, repsPerAnnealStep);
	likelihoodAssessor.SampleForSupportLimits(ran, 1.0, true, savedReps, repsBetweenSaves);
	likelihoodAssessor.PrintSavedParams(outputFileName);
	aString outFileSummmaryName(outputFileName);
	outFileSummmaryName.trunc(28);
	outFileSummmaryName.cat(".sum");
	likelihoodAssessor.PrintParamEstimates(outFileSummmaryName, seed, 1);//1 for tab separators
	likelihoodAssessor.PrintParamEstimates(cout, seed, 0);//0 for no tab separators

	return;
	}//end FitClineCWTlTrZlZrD(...)
*/


void FitClineCWTlTrZlZrDMulti(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				){//6-parameter fit
	double temperature = 0.009;
	double lowTemp = 0.0000001;	//best annealing temp 

	//***these parameter records always vary among shapes fitted
	ClineMultiCWTlTrDParamRec initialParams(startingParamInitializer);
	ClineMultiCWTlTrDParamRec paramDeviations(devParamInitializer);
	ClineMultiCWTlTrDParamRec lowerParamLimits(minParamInitializer);
	ClineMultiCWTlTrDParamRec upperParamLimits(maxParamInitializer);
	upperParamLimits.SetXL(maxParamInitializer.rightEnd_);
	upperParamLimits.SetXR(maxParamInitializer.rightEnd_);
	//***


	if(testingFitToGivenShape){
		initialParams.Initialize(testParamInitializer);}
		
	MetropolisHastings likelihoodAssessor(initialParams, paramDeviations, 
											lowerParamLimits, upperParamLimits, 
											alloData);
	//***this function may vary among shapes fitted
	likelihoodAssessor.SetLikelihoodFunction(MaxLikeDevFromClineAndDiseq7);


	if(testingFitToGivenShape){
		likelihoodAssessor.FitOfTestParams(outputFileName);
		return;}
	likelihoodAssessor.Anneal(ran, 100.0, 10.0, true, 0.9, repsPerAnnealStep);
	likelihoodAssessor.SampleForSupportLimits(ran, 1.0, true, savedReps, repsBetweenSaves);
	likelihoodAssessor.PrintSavedParams(outputFileName);
	aString outFileSummmaryName(outputFileName);
	outFileSummmaryName.trunc(28);
	outFileSummmaryName.cat(".sum");
	aString emptyString;
	likelihoodAssessor.PrintParamEstimates(outFileSummmaryName, seed, 1, 1, emptyString);//1 for tab separators
	likelihoodAssessor.PrintParamEstimates(cout, seed, 0, 1, emptyString);//0 for no tab separators

	return;
	}//end FitClineCWTlTrZlZrDMulti(...)


/* THIS IS DEFUNCT
void FitClineCWTlTrZlZrTBD(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				){//6-parameter fit
	double temperature = 0.009;
	double lowTemp = 0.0000001;	//best annealing temp 

	//***these parameter records always vary among shapes fitted
	ClineShapeCWTlTrTBDParamRec initialParams(startingParamInitializer);
	ClineShapeCWTlTrTBDParamRec paramDeviations(devParamInitializer);
	ClineShapeCWTlTrTBDParamRec lowerParamLimits(minParamInitializer);
	ClineShapeCWTlTrTBDParamRec upperParamLimits(maxParamInitializer);
	for(int t=0;t<alloData->loci();++t){
		ClineShapeCWTlTrTBParamRec* uParams = 
				(ClineShapeCWTlTrTBParamRec*) upperParamLimits.traitParamSet_[t].pointerToDerivedSelf;
		uParams->xL_ = uParams->xR_ = maxParamInitializer.rightEnd_;
		if((uParams->paramsOn_.Includes(2)) && (uParams->paramsOn_.Includes(4)) && (uParams->zL_==0.0)){
			uParams->zL_ = 1000.0; uParams->thetaL_=1.0;}//avoids a parameter incompatibility situation
		if((uParams->paramsOn_.Includes(3)) && (uParams->paramsOn_.Includes(5)) && (uParams->zR_==0.0)){
			uParams->zR_ = 1000.0; uParams->thetaR_=1.0;}//avoids a parameter incompatibility situation
		ClineShapeCWTlTrTBParamRec* lowParams = 
				(ClineShapeCWTlTrTBParamRec*) lowerParamLimits.traitParamSet_[t].pointerToDerivedSelf;
		lowParams->xL_ = lowParams->xR_ = maxParamInitializer.leftEnd_;
		if((lowParams->paramsOn_.Includes(2)) && (lowParams->paramsOn_.Includes(4)) && (lowParams->zL_==0.0)){
			lowParams->zL_ = 0.0; lowParams->thetaL_=0.0;}//avoids a parameter incompatibility situation
		if((lowParams->paramsOn_.Includes(3)) && (lowParams->paramsOn_.Includes(5)) && (lowParams->zR_==0.0)){
			lowParams->zR_ = 0.0; lowParams->thetaR_=0.0;}//avoids a parameter incompatibility situation
		ClineShapeCWTlTrTBParamRec* traitParams = 
				(ClineShapeCWTlTrTBParamRec*) initialParams.traitParamSet_[t].pointerToDerivedSelf;
		if((traitParams->paramsOn_.Includes(2)) && (traitParams->paramsOn_.Includes(4)) && (traitParams->zL_==0.0)){
			traitParams->zL_ = traitParams->w_+1.0; traitParams->thetaL_=0.05;}//avoids a parameter incompatibility situation
		if((traitParams->paramsOn_.Includes(3)) && (traitParams->paramsOn_.Includes(5)) && (traitParams->zR_==0.0)){
			traitParams->zR_ = traitParams->w_+1.0; traitParams->thetaR_=0.05;}//avoids a parameter incompatibility situation
		}//end for t
	//***


	if(testingFitToGivenShape){
		initialParams.Initialize(testParamInitializer);}
		
	MetropolisHastings likelihoodAssessor(initialParams, paramDeviations, 
											lowerParamLimits, upperParamLimits, 
											alloData);
	//***this function may vary among shapes fitted
	likelihoodAssessor.SetLikelihoodFunction(MaxLikeDevFromClineAndDiseq9);


	if(testingFitToGivenShape){
		likelihoodAssessor.FitOfTestParams(outputFileName);
		return;}
	likelihoodAssessor.Anneal(ran, 100.0, 10.0, true, 0.9, repsPerAnnealStep);
	likelihoodAssessor.SampleForSupportLimits(ran, 1.0, true, savedReps, repsBetweenSaves);
	likelihoodAssessor.PrintSavedParams(outputFileName);
	aString outFileSummmaryName(outputFileName);
	outFileSummmaryName.trunc(28);
	outFileSummmaryName.cat(".sum");
	likelihoodAssessor.PrintParamEstimates(outFileSummmaryName, seed, 1);//1 for tab separators
	likelihoodAssessor.PrintParamEstimates(cout, seed, 0);//0 for no tab separators

	return;
	}//end FitClineCWTlTrZlZrTBD(...)
*/



void FitClineCWTlTrZlZrTBDMulti(Ran3& ran, ClineGeneticData* theData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				){//8-parameter fit
	double temperature = 0.009;
	double lowTemp = 0.0000001;	//best annealing temp 

	//***these parameter records always vary among shapes fitted
	ParamRec *initialParams;
	ParamRec *paramDeviations;
	ParamRec *lowerParamLimits;
	ParamRec *upperParamLimits;
	//***
	//***these parameter records always vary among shapes fitted; assumes all data of common type
//	if((theData->dataType[0] == "codominant")||(theData->dataType[0] == "haplotype")){
			ClineMultiCWTlTrTBDParamRec* ip = NEW ClineMultiCWTlTrTBDParamRec(startingParamInitializer);
			if(testingFitToGivenShape){
				ip->Initialize(testParamInitializer);}
			paramDeviations = NEW ClineMultiCWTlTrTBDParamRec(devParamInitializer);
			lowerParamLimits = NEW ClineMultiCWTlTrTBDParamRec(minParamInitializer);
			ClineMultiCWTlTrTBDParamRec* ul = NEW ClineMultiCWTlTrTBDParamRec(maxParamInitializer);
			ul->SetXL(maxParamInitializer.rightEnd_);
			ul->SetXR(maxParamInitializer.rightEnd_);
			initialParams = ip;
			upperParamLimits = ul;
//			}
/*		else if(theData->dataType[0] == "morph"){
			ClineShapeCWTlTrTBQ2ParamRec* ip = NEW ClineShapeCWTlTrTBQ2ParamRec(startingParamInitializer);
			if(testingFitToGivenShape){
				ip->Initialize(testParamInitializer);}
			paramDeviations = NEW ClineShapeCWTlTrTBQ2ParamRec(devParamInitializer);
			lowerParamLimits = NEW ClineShapeCWTlTrTBQ2ParamRec(minParamInitializer);
			ClineShapeCWTlTrTBQ2ParamRec* ul = NEW ClineShapeCWTlTrTBQ2ParamRec(maxParamInitializer);
			for(int t=0;t<maxParamInitializer.traits();++t){
				ul->xL(t,maxParamInitializer.rightEnd_);
				ul->xR(t,maxParamInitializer.rightEnd_);}
			initialParams = ip;
			upperParamLimits = ul;
			}
	//***
*/
		
	MetropolisHastings likelihoodAssessor(*initialParams, *paramDeviations, 
											*lowerParamLimits, *upperParamLimits, 
											theData);
	//***this function may vary among shapes fitted
	likelihoodAssessor.SetLikelihoodFunction(MaxLikeDevFromClineAndDiseq9);


	if(testingFitToGivenShape){
		likelihoodAssessor.FitOfTestParams(outputFileName);
		return;}
	likelihoodAssessor.Anneal(ran, 100.0, 10.0, true, 0.9, repsPerAnnealStep);
	likelihoodAssessor.SampleForSupportLimits(ran, 1.0, true, savedReps, repsBetweenSaves);
	likelihoodAssessor.PrintSavedParams(outputFileName);
	aString outFileSummmaryName(outputFileName);
	outFileSummmaryName.trunc(28);
	outFileSummmaryName.cat(".sum");
	aString emptyString;
	likelihoodAssessor.PrintParamEstimates(outFileSummmaryName, seed, 1, 1, emptyString);//1 for tab separators
	likelihoodAssessor.PrintParamEstimates(cout, seed, 0, 1, emptyString);//0 for no tab separators

	//clean up
	delete initialParams;		initialParams=0;
	delete paramDeviations;		paramDeviations=0;
	delete lowerParamLimits;	lowerParamLimits=0;
	delete upperParamLimits;	upperParamLimits=0;
	}//end FitClineCWTlTrZlZrTBDMulti(...)



#endif


