/***************************
*
*
*	FitClineShapes.h
*
*	These functions fit genetic marker data to specific
*	Barton cline shape models using likelihood functions maximized
*	via a Metropolis-Hastings algorithm.
*
**************************************************/

#ifndef FitClineShapesh
#define FitClineShapesh = 1

#include "Element.cpp"
#include "aList.cpp"
#include "StringClass.h"
#include "RandomList.h"
#include "ClineFitGlobals.h"
#include "HarmonicMeanDiseq.h"
#include "FstatisticAnalysis.h"
#include "ParamRec.h"
#include "ClineShapeCWParamRec.h"
#include "ClineMultiCWParamRec.h"
#include "ClineShapeCWTBParamRec.h"
#include "ClineMultiCWTBParamRec.h"
#include "ClineShapeCWTlTrParamRec.h"
#include "ClineMultiCWTlTrParamRec.h"
#include "ClineMultiCWTlTrDParamRec.h"
#include "ClineShapeCWTlTrTBParamRec.h"
#include "ClineMultiCWTlTrTBParamRec.h"
//#include "ClineShapeCWTlTrTBDParamRec.h"//DEFUNCT
#include "ClineMultiCWTlTrTBDParamRec.h"
#include "ClineShapeCWTlTrTBQ2ParamRec.h"
#include "ClineGeneticData.h"
#include "MetropolisHastings.h"
#include "ClineFitFunctions.h"
#include "ClineFitInitializer.h"
#include "MemoryManager.h"


//function to fit 2 params: center, width
void FitClineCW(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				);//2-parameter fit

//function to fit 2 params for multiple traits: center, width
void FitClineCWMulti(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				);//2-parameter fit

//function to fit 4 params: center, width, left mean score, right means score
void FitClineCWTB(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				);//4-parameter fit


void FitClineCWTBMulti(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				);//4-parameter fit


//function to fit 6 params: center, width, thetaL, thetaR, zL & zR
void FitClineCWTlTrZlZr(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				);//6-parameter fit

void FitClineCWTlTrZlZrMulti(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				);//6-parameter fit

//function to fit 8 genetic params: center, width, thetaL, thetaR, zL, zR, p[L] & p[R]
	//can use with multiple loci
void FitClineCWTlTrZlZrTB(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				);//8-parameter fit

//function to fit 8 params: center, width, thetaL, thetaR, zL, zR, p[L] & p[R]
	//can use with multiple traits

void FitClineCWTlTrZlZrTB(Ran3& ran, ClineData* theData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				);//8-parameter fit


void FitClineCWTlTrZlZrTBMulti(Ran3& ran, ClineData* theData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				);//8-parameter fit



//function to fit 7 genetic params: center, width, thetaL, thetaR, zL, zR & D
	//fits shape, disequilibrium, dispersal, selection, etc.
void FitClineCWTlTrZlZrDMulti(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				);//7-parameter fit

/*  THIS IS DEFUNCT
void FitClineCWTlTrZlZrD(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				);//7-parameter fit
*/

//function to fit 9 genetic params: center, width, thetaL, thetaR, zL, zR, max & min scores, & D
	//fits shape, disequilibrium, dispersal, selection, etc.
	//allows specificity of models for individuals traits
	//must be used with multiple diallelic loci
void FitClineCWTlTrZlZrTBDMulti(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				);//9-parameter fit


/*  THIS IS DEFUNCT
void FitClineCWTlTrZlZrTBD(Ran3& ran, ClineGeneticData* alloData,
				ClineFitInitializer& startingParamInitializer,
				ClineFitInitializer& testParamInitializer,
				ClineFitInitializer& devParamInitializer,
				ClineFitInitializer& minParamInitializer,
				ClineFitInitializer& maxParamInitializer,
				aString& outputFileName,int testingFitToGivenShape,
				long savedReps, long repsPerAnnealStep,long seed,int repsBetweenSaves
				);//9-parameter fit
*/




#endif
