/*********************************************
*
*	ClineDiseqParamRec.h
*
*	This base class holds a set of parameter values that
*	are used in a metropolis algorithm for numerically 
*	maximizing an estimate of gametic phase disequilibrium D.
*
*	Always use this in combination with a cline shape class,
*	and always derive functions to vary these values and check
*	their limits.
*
*********************************************/

#ifndef ClineDiseqParamRech
#define ClineDiseqParamRech = 1

#include <math.h>
#include <ostream.h>
#include "ParamRec.h"
#include "ClineFitInitializer.h"
#include "ClineGeneticData.h"
#include "MemoryManager.h"

class ClineDiseqParamRec: virtual public ParamRec{
  public:
	long double lnLDiseq_;
	long double D_;
	
  public:
  	ClineDiseqParamRec(void):lnLDiseq_((long double)0.0),D_((long double)0.0){}
  	ClineDiseqParamRec(ClineFitInitializer& i):
  				lnLDiseq_(0.0),
  				D_(i.D_){
  			lnL_ = 0.0;
  			lnLmaxMinusLnL_=0.0;
//  			SetPointerToDerivedSelf();
  			}  	
  	ClineDiseqParamRec(ClineDiseqParamRec& p):
  				lnLDiseq_(p.lnLDiseq_),
  				D_(p.D_){
  			lnL_ = p.lnL_;
  			lnLmaxMinusLnL_=p.lnLmaxMinusLnL_;
//  			SetPointerToDerivedSelf();
  			}  	
  	ClineDiseqParamRec(ClineDiseqParamRec* p):
  				lnLDiseq_(p->lnLDiseq_),
  				D_(p->D_){
  			lnL_ = p->lnL_;
  			lnLmaxMinusLnL_=p->lnLmaxMinusLnL_;
//  			SetPointerToDerivedSelf();
			}
  	virtual ~ClineDiseqParamRec(void){
//  			pointerToDerivedSelf = 0;//forget this
  			}
  	
  	//OVERRIDE THESE IN ALL DERIVED CLASSES
  	virtual ParamRec* 	Duplicate(void);
//	virtual void 		SetPointerToDerivedSelf(void);
	virtual long 		GetPointerToDerivedSelf(void);
	
	virtual int 		ChangeD(Ran3& ran,double sigma,
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual long double ClineDiseqLikelihood (ClineGeneticData* alloData);
  			long double D(void);//returns D_
  			void 		D(long double dd);//sets D_
			long double lnLDiseq(void);//returns lnLDiseq_
			void 		lnLDiseq(long double lnLval);//sets lnLDiseq_
	virtual void 		Initialize(ClineFitInitializer& i);
	virtual int 		MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void 		MultiplyParam(int /*paramNum*/, long double factor);
	virtual int 		NumParams(void);
	virtual int 		NumDerivedParams(void);
	virtual aString* 	ParamName(int /*paramNum*/);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double ParamValue(int /*paramNum*/);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
	virtual void 		SetParamValue(int /*paramNum*/, long double newVal);//sets value of parameter
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file
	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

			friend int	operator==(ClineDiseqParamRec& p, ClineDiseqParamRec& q);
			friend int	operator!=(ClineDiseqParamRec& p, ClineDiseqParamRec& q);

	};//end base class ClineDiseqParamRec


#endif

