/******************************************************
*
*	ClineMultiCWParamRec.h
*
*	Parameters for a multi-trait tanh cline defined by center and width only.
*	Assumes the y-axis value of the cline varies between 0 and 1.
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/

#ifndef ClineMultiCWParamRech
#define ClineMultiCWParamRech = 1


#include "ParamRec.h"
#include "Hyperbolics.h"
#include "InlineTemplates.h"
#include "ClineFitInitializer.h"
#include "ClineShapeCWParamRec.h"
#include "Element.cpp"
#include "aList.cpp"
#include "MemoryManager.h"

class ClineMultiCWParamRec;
class ClineMultiCWParamRec: virtual public ParamRec{
  public:
	ObjList<ParamRec> traitParamSet_;//holds 
	ObjList<aList<int> > *paramsOnOff_;//links to a common switchboard; never delete within this class
	ObjList<aList<int> > *paramsShared_;//links to a common switchboard; never delete within this class
			//code assumes paramsShared_ stores sorted values
	int varyingParams_;//totaled from paramsOnOff_
  	
  public:
  	ClineMultiCWParamRec(void):ParamRec(),paramsOnOff_(NULL),paramsShared_(NULL),varyingParams_(0){
//  				SetPointerToDerivedSelf();//empty parameter set
  				}
  	ClineMultiCWParamRec(ClineFitInitializer& i):
  					ParamRec(),paramsOnOff_(NULL),paramsShared_(NULL),varyingParams_(0){
//  				SetPointerToDerivedSelf();
  				Initialize(i);
  				}
  	ClineMultiCWParamRec(ClineMultiCWParamRec& p):
  					ParamRec(p),paramsOnOff_(p.paramsOnOff_),paramsShared_(p.paramsShared_),
  					varyingParams_(p.varyingParams_){
//  				SetPointerToDerivedSelf();
  				for(int tr=0;tr<p.traits();++tr){
  					ClineShapeCWParamRec* traitparams = (ClineShapeCWParamRec*) p.traitParamSet_[tr].GetPointerToDerivedSelf();
  					traitParamSet_.Append(NEW ClineShapeCWParamRec(*traitparams));}
				}
  	virtual ~ClineMultiCWParamRec(void){
//  			pointerToDerivedSelf = NULL;//forget
  			paramsOnOff_=NULL;//forget
  			paramsShared_=NULL;//forget
  			traitParamSet_.Clear();
  			}

  	virtual ParamRec* 				Duplicate(void);
  	virtual ClineMultiCWParamRec* 	_Duplicate(void);
//	virtual void 					SetPointerToDerivedSelf(void);//always override in derived
	virtual long 					GetPointerToDerivedSelf(void);//always override in derived
  	


	virtual long double center(int trait);//returns c_ for trait
	virtual long double width(int trait);//returns w_ for trait
	virtual long double leftVal(int trait);//average limiting score on left side
	virtual long double rightVal(int trait);//average limiting score on right side
	virtual long double maxSlope(int trait);//most extreme slope of a cline (positive or negative)

	virtual int 		BaseParams(void){return 2;}
  	
	virtual int 		ChangeCenter(int trait, Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
									ParamRec* paramDeviations,  
									ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=1.0);
	virtual int 		ChangeWidth(int trait, Ran3& ran, double sigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual long double Cline_at_X(long double x, int trait);
	virtual long double ExpectedValue(aList<long double>& coords);
	virtual void 		Initialize(ClineFitInitializer& i);
	virtual int 		MeetsParamCriteria(ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void 		MultiplyParam(int paramNum, long double factor);//use to rescale a param deviation
	virtual int 		NumParams(void);//number of varying parameters
	virtual int 		NumDerivedParams(void);//number of params that depend on varying params
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamCoords(int globalParamNum, int& trait, int& traitParam, int& derived);
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
	virtual void 		SetParamValue(int paramNum, long double newVal);//sets value of parameter
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file
//	virtual int 		TotalParams(void);//number of parameters that could vary, plus derived paramerters
	virtual int 		traits(void);//number of traits
	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

	friend int	operator==(ClineMultiCWParamRec& p, ClineMultiCWParamRec& q);
	friend int	operator!=(ClineMultiCWParamRec& p, ClineMultiCWParamRec& q);

	};//end class ClineMultiCWParamRec



#endif