/******************************************************
*
*	ClineShapeCWParamRec.h
*
*	Parameters for a single-trait tanh cline defined by center and width only.
*	Assumes the y-axis value of the cline varies between 0 and 1.
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/

#ifndef ClineShapeCWParamRech
#define ClineShapeCWParamRech = 1


#include "ParamRec.h"
#include "Hyperbolics.h"
#include "InlineTemplates.h"
#include "ClineFitInitializer.h"
#include "MemoryManager.h"

class ClineShapeCWParamRec;
class ClineShapeCWParamRec: virtual public ParamRec{
  public:
	long double c_;//center
	long double w_;//width
  	
  public:
  	ClineShapeCWParamRec(void):ParamRec(), c_(0.0), w_(1.0){
//  				SetPointerToDerivedSelf();
  				}
  	ClineShapeCWParamRec(long double c, long double w):ParamRec(), c_(c), w_(w){
//  				SetPointerToDerivedSelf();
  				}
  	ClineShapeCWParamRec(ClineFitInitializer& i, int trait_):
  					ParamRec(), c_(i.c_[trait_]), w_(i.w_[trait_]){
//  				SetPointerToDerivedSelf();
  				}
  	ClineShapeCWParamRec(ClineShapeCWParamRec& p):
  					ParamRec(p), 
  					c_(p.c_), 
  					w_(p.w_){
//  				SetPointerToDerivedSelf();
				}
  	virtual ~ClineShapeCWParamRec(void){
 // 			pointerToDerivedSelf = NULL;//forget
  				}

  	virtual ParamRec* 				Duplicate(void);
  	virtual ClineShapeCWParamRec* 	_Duplicate(void);
//	virtual void 					SetPointerToDerivedSelf(void);//always override in derived
	virtual long 					GetPointerToDerivedSelf(void);//always override in derived
  	


			long double center(void);//returns c_
			void 		center(long double c);//sets _
			long double width(void);//returns w_
			void 		width(long double w);//sets w_	
	virtual long double leftVal(int /*trait*/);//average limiting score on left side
	virtual long double rightVal(int /*trait*/);//average limiting score on right side
			long double maxSlope(void);

	virtual long double Cline_at_X(long double x, int trait=0);
  	
	virtual int 		ChangeCenter(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
									ParamRec* paramDeviations,  
									ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=1.0);
	virtual int 		ChangeWidth(Ran3& ran, double sigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual long double ExpectedValue(aList<long double>& coords);
	virtual void 		Initialize(ClineFitInitializer& i, int trait_);
	virtual int 		MeetsParamCriteria(ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void 		MultiplyParam(int paramNum, long double factor);//use to rescale a param deviation
	virtual int 		NumParams(void);//number of varying parameters
	virtual int 		NumDerivedParams(void);//number of params that depend on varying params
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
	virtual void 		SetParamValue(int paramNum, long double newVal);//sets value of parameter
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file
//	virtual int 		TotalParams(void);//number of parameters that could vary, plus derived paramerters

	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

	friend int	operator==(ClineShapeCWParamRec& p, ClineShapeCWParamRec& q);
	friend int	operator!=(ClineShapeCWParamRec& p, ClineShapeCWParamRec& q);

	};//end class ClineShapeCWParamRec



#endif