/******************************************************
*
*	ClineShapeCWTBParamRec.h
*
*	Parameters for a tanh cline defined by center,width, 
*	maximum (top) & minimum (bottom) only.
*	Assumes the y-axis value of the cline varies between top and bottom.
*	There may be as many traits as desired, each with their own top & bottom,
*	but sharing a common center and width.  
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/

#ifndef ClineShapeCWTBParamRech
#define ClineShapeCWTBParamRech = 1


#include "ParamRec.h"
#include "Hyperbolics.h"
#include "InlineTemplates.h"
#include "ClineShapeCWParamRec.h"
#include "ClineFitInitializer.h"
#include "MemoryManager.h"

class ClineShapeCWTBParamRec;
class ClineShapeCWTBParamRec: public ClineShapeCWParamRec{
  public:
	long double pL_;//usually 'bottom'
	long double pR_;//usually 'top'
  	
  public:
  	ClineShapeCWTBParamRec(void);
  	ClineShapeCWTBParamRec(long double c, long double w,
  						long double leftSideScore, long double rightSideScore);
  	ClineShapeCWTBParamRec(ClineFitInitializer& i, int trait_);
  	ClineShapeCWTBParamRec(ClineShapeCWTBParamRec& p);
  	virtual ~ClineShapeCWTBParamRec(void);

  	virtual ParamRec* 					Duplicate(void);
  	virtual ClineShapeCWTBParamRec* 	_Duplicate(void);
//	virtual void 					SetPointerToDerivedSelf(void);//always override in derived
	virtual long 					GetPointerToDerivedSelf(void);//always override in derived
  	

	virtual long double leftVal(int trait=0);//average limiting score on left side
	virtual long double rightVal(int trait=0);//average limiting score on right side
	virtual void 		leftVal(long double newVal, int trait=0);//average limiting score on left side
	virtual void 		rightVal(long double newVal,int trait=0);//average limiting score on right side
//	virtual int 		trait(int paramNum);//nth trait given the parameter number
	virtual int 		traits(void){return 1;}//number of traits scored on the cline

	virtual void 		Clear();
	virtual long double Cline_at_X(long double x, int trait=0); 
  	
	virtual int 		ChangeLeftScore(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=1.0);
	virtual int 		ChangeParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
									ParamRec* paramDeviations,  
									ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=1.0);
	virtual int 		ChangeRightScore(Ran3& ran, ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=1.0);
	virtual long double ExpectedValue(aList<long double>& coords);
	virtual int 		MeetsParamCriteria(ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void 		MultiplyParam(int paramNum, long double factor);//use to rescale a param deviation
	virtual int 		NumParams(void);//number of parameters
	virtual int 		NumDerivedParams(void);//no. params that depend on varying params
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
	virtual void 		SetParamValue(int paramNum, long double newVal);//sets value of parameter
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file
//	virtual int 		TotalParams(void);//number of parameters that could vary, plus derived paramerters

	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

	friend int	operator==(ClineShapeCWTBParamRec& p, ClineShapeCWTBParamRec& q);
	friend int	operator!=(ClineShapeCWTBParamRec& p, ClineShapeCWTBParamRec& q);

	};//end class ClineShapeCWTBParamRec



#endif