/******************************************************
*
*	ClineShapeCWTlTrTBDParamRec.h
*
*	Parameters for a 3-part composite tanh cline defined by center, width,	
*	thetaL, thetaR, zL, zR, and the mean frequencies on each side of the cline.  
*	The latter theta and z parameters describe introgression
*	slopes and intercepts on the Left and Right sides of the cline.
*	The mean y-axis value of the cline varies for each trait on each side.
*	Disequilibrium is also estimated and derived parameters calculated.
*	The widths of each trait's cline are a shared common value and all other
*	parameters vary independenty.
*
*	Use this as a virtual class for derived classes that add parameters.
*
******************************************************/

#ifndef ClineShapeCWTlTrTBDParamRech
#define ClineShapeCWTlTrTBDParamRech = 1


#include "ParamRec.h"
#include "Hyperbolics.h"
#include "InlineTemplates.h"
#include "ClineShapeCWTlTrParamRec.h"
#include "ClineShapeCWTlTrTBParamRec.h"
#include "ClineDiseqParamRec.h"
#include "ClineFitInitializer.h"
#include "MemoryManager.h"

class ClineShapeCWTlTrTBDParamRec;
class ClineShapeCWTlTrTBDParamRec: virtual public ClineDiseqParamRec{
  public:
	//a parameter set for each trait; actually of type ClineShapeCWTlTrTBParamRec
	long double r_;
  	ObjList<ParamRec> traitParamSet_;
  public:
  	ClineShapeCWTlTrTBDParamRec(void);
 	ClineShapeCWTlTrTBDParamRec(ClineFitInitializer& i);
  	ClineShapeCWTlTrTBDParamRec(ClineShapeCWTlTrTBDParamRec& p);
  	virtual ~ClineShapeCWTlTrTBDParamRec(void){
//  			pointerToDerivedSelf = NULL;//forget
  			traitParamSet_.Clear();
  			}

  	virtual ParamRec* 				Duplicate(void);
  	virtual ClineShapeCWTlTrTBDParamRec* 	_Duplicate(void);
//	virtual void 					SetPointerToDerivedSelf(void);//always override in derived
	virtual long 					GetPointerToDerivedSelf(void);//always override in derived
  	


	virtual long double B_L(int trait);
	virtual long double B_R(int trait);
	virtual int 		ChangeParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
									ParamRec* paramDeviations,  
									ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=1.0);
	virtual int 		ChangeWidth(Ran3& ran, double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual long double Cline_at_X(long double x, int trait);//location, trait
	virtual long double ClineDiseqLikelihood (ClineGeneticData* alloData);
			long double D(void);//returns D_
			void 		D(long double d);//sets D_
	virtual double 		DispersalSigma();
	virtual long double distanceBarrierL(int trait);//barrier to introgression from left to right in distance units
	virtual long double distanceBarrierR(int trait);//barrier to introgression from right to left in distance units
	virtual long double ExpectedValue(aList<long double>& coords);
	virtual void 		Initialize(ClineFitInitializer& i);
	virtual long double leftVal(int trait);
			long double lnLDiseq(void);//returns lnLDiseq_
			void 		lnLDiseq(long double lnD);//sets lnLDiseq__
	virtual int 		MeetsParamCriteria(ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void 		MultiplyParam(int paramNum, long double factor);//use to rescale a param deviation
	virtual int 		NumParams(void);//number of parameters
	virtual int 		NumDerivedParams(void);//number of parameters that are functions of varying parameters
	virtual long double NumSelectedLoci(void);//assumes many loci and selection via genetic background
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
			long double r(void);//returns r_
			void 		r(long double rr);//sets r_
	virtual long double rightVal(int trait);
	virtual long double S(void);//effective selection against hybrids in center as if hybrids were single-locus heterozygotes
	virtual void 		SetParamValue(int paramNum, long double newVal);//sets value of parameter
	virtual long double s_perMarkerLocusL(int trait);//ave. direct selection on marker loci in left tail of cline
	virtual long double s_perMarkerLocusR(int trait);//ave. direct selection on marker loci in right tail of cline
	virtual long double s_perReproIsolationLocus(void);//average selection on a locus contributing to reproductive isolation
	
						//indirect & direct selection on marker loci in middle of cline
	virtual long double s_star_vs_Hets(void);//assuming genetic background incompatibilities
	virtual long double s_star_Ecotone(void);//assuming selection across an ecotone
	virtual long double s_star_DomEcotone(void);//assuming selection across an ecotone on a dominant marker

	virtual long double timeBarrierL(int trait);
	virtual long double timeBarrierR(int trait);
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file
	virtual	long double WhybridOverWpure(void);//ratio in cline center
	virtual long double width(void);//returns width

	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

	friend int	operator==(ClineShapeCWTlTrTBDParamRec& p, ClineShapeCWTlTrTBDParamRec& q);
	friend int	operator!=(ClineShapeCWTlTrTBDParamRec& p, ClineShapeCWTlTrTBDParamRec& q);

	};//end class ClineShapeCWTlTrTBDParamRec



#endif